Package: microbiomeExplorer
Type: Package
Title: Microbiome Exploration App
Version: 1.23.0
Date: 2022-09-01
Authors@R: c(
    person("Joseph", "Paulson", , "paulson.joseph@gene.com", role = c("aut")),
    person("Janina", "Reeder", , "reederj1@gene.com", role = c("aut", "cre")),
    person("Mo", "Huang", role = c("aut")),
    person("Genentech", role = c("cph", "fnd"))
    )
Description: The MicrobiomeExplorer R package is designed to facilitate the analysis and visualization of marker-gene survey feature data. 
  It allows a user to perform and visualize typical microbiome analytical workflows either through the command line or an interactive 
  Shiny application included with the package. In addition to applying common analytical workflows the application enables automated 
  analysis report generation.
License: MIT + file LICENSE
Imports: shinyjs (>= 2.0.0), shinydashboard, shinycssloaders,
        shinyWidgets, rmarkdown (>= 1.9.0), DESeq2, RColorBrewer,
        dplyr, tidyr, purrr, rlang, knitr, readr, DT (>= 0.12.0),
        biomformat, tools, stringr, vegan, matrixStats, heatmaply, car,
        broom, limma, reshape2, tibble, forcats, lubridate, methods,
        plotly (>= 4.9.1)
Depends: shiny, magrittr, metagenomeSeq, Biobase
Suggests: V8, testthat (>= 2.1.0)
DeploySubPath: microbiomeExplorer
biocViews: Classification, Clustering, GeneticVariability,
        DifferentialExpression, Microbiome, Metagenomics,
        Normalization, Visualization, MultipleComparison, Sequencing,
        Software, ImmunoOncology
Encoding: UTF-8
RoxygenNote: 7.2.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/microbiomeExplorer
git_branch: devel
git_last_commit: 2398fa4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:46:19 UTC; biocbuild
Author: Joseph Paulson [aut],
  Janina Reeder [aut, cre],
  Mo Huang [aut],
  Genentech [cph, fnd]
Maintainer: Janina Reeder <reederj1@gene.com>

# Generated by roxygen2: do not edit by hand

export(addFeatData)
export(addPhenoData)
export(aggFeatures)
export(aggregationTabUI)
export(calculatePCAs)
export(computeDistMat)
export(corrAnalysisUI)
export(corrFeature)
export(corrPhenotype)
export(dataInputUI)
export(diffAnalysisUI)
export(extendPhenoData)
export(featureAnalysisUI)
export(featureTableUI)
export(filterByPheno)
export(filterMEData)
export(interAnalysisUI)
export(intraAnalysisUI)
export(longAnalysisUI)
export(makeQCPlot)
export(normalizeData)
export(phenotypeTableUI)
export(plotAbundance)
export(plotAlpha)
export(plotAvgAbundance)
export(plotBeta)
export(plotHeatmap)
export(plotLongFeature)
export(plotSingleFeature)
export(plotlyHistogram)
export(plotlySampleBarplot)
export(readData)
export(replaceWithUnknown)
export(reportListUI)
export(rollDownFeatures)
export(runDiffTest)
export(runMicrobiomeExplorer)
import(metagenomeSeq)
import(shiny)
import(shinydashboard)
importFrom(Biobase,"fData<-")
importFrom(Biobase,"featureData<-")
importFrom(Biobase,"pData<-")
importFrom(Biobase,"phenoData<-")
importFrom(Biobase,fData)
importFrom(Biobase,pData)
importFrom(magrittr,"%>%")
importFrom(metagenomeSeq,"normFactors<-")
importFrom(metagenomeSeq,MRcounts)
importFrom(metagenomeSeq,MRtable)
importFrom(metagenomeSeq,aggTax)
importFrom(metagenomeSeq,biom2MRexperiment)
importFrom(metagenomeSeq,cumNorm)
importFrom(metagenomeSeq,cumNormStatFast)
importFrom(metagenomeSeq,fitFeatureModel)
importFrom(metagenomeSeq,loadMeta)
importFrom(metagenomeSeq,loadPhenoData)
importFrom(metagenomeSeq,newMRexperiment)
importFrom(metagenomeSeq,normFactors)
importFrom(methods,is)
importFrom(rlang,":=")

# Microbiome Explorer 
## An R package for the analysis and visualization of microbial communities.

## To install and run Shiny app:

```r
BiocManager::install("zoecastillo/microbiomeExplorer", 
                         ref = "master")
                         
```

## Introduction

The microbiomeExplorer package and the Shiny application contained with it provides methods
and visualizations to explore the results of 16S rRNA amplicon sequencing experiment. The analyses can be performed entirely via the R command line, but the primary intent of the package is to enable access to many of these analyses to non-computational users through the Shiny user interface. 

The interface is launched through the R environment by calling `MicrobiomeExplorerApp()` after loading the package or made available via deployment on a Shiny server. This document is intended to give an introduction of how to use the Shiny application with the addition of R code chunks that a command line user would use to produce the same results.

```r                     
library(microbiomeExplorer)
runMicrobiomeExplorer()
```

## Data upload

#### Microbiome Explorer accepts several different data upload formats

* MRexperiment-class objects stored as RDATA or RDS files
* The Biological Observation Matrix (BIOM) formattes files produced by any program including qiime2 or mothur
* Raw counts files

For further details, please refer to the package vignette.


#### Accessing example data via `metagenomeSeq`

The mouseData example data set is included with the `metagenomeSeq` package on which `microbiomeExplorer` depends. The easiest way to make this data available for the application is to store it as an RDS object on your file system. Then, you can open it via the "Browse" button under *Upload Feature Count Data*.

```r
data("mouseData", package = "metagenomeSeq")
```

## Data QC

Before starting an analysis, it is recommend to review the results of the sequencing experiment and perform quality control. Multiple QC plots can be generated, including those showing the number of unique features in each sample as a barplot or in a scatterplot against the number of reads.

<img src="inst/shiny/www/img/data_input_filter.jpg" width="70%" height="70%" />

## Analysis

The analysis workflow within the application is split into six different sections: intra sample, intra feature, inter sample, correlation, differential and longitudinal. All visualizations are implemented using the plotly R package which provides basic interactivity, including zooming or panning via its modebar. In addition, the user can export the plot in its current state (i.e. showing specific user interactions) as a svg file using the camera icon of the modebar.

### Intra-Sample Analysis

Intra-sample analysis contains functions focus on investigating the microbial composition within a sample or a group of samples.
Relative abundance shows the most abundant feature in a barplot summarized by a user-defined variable across the x-asis. The feature abundance plot shows the individual abundance of a specific feature either as a boxplot or a categorical scatterplot depending on the x-axis variable chosen. Alpha diversity is a measure of the complexity or diversity within a particular sample, eg. habitat or area. Alpha diversity is computed by functions in the vegan package and is visualized as a boxplot using the same input definitions by feature and relative abundance.

<img src="inst/shiny/www/img/analysis_intra_faceted.jpg" width="70%" height="70%" />

### Intra-Feature Analysis

Intra-feature analysis provides a different view on the feature prevalence in the dataset by showing the average abundance for each feature with added error bars indicating the standard deviation across all of the samples in a particular group.

<img src="inst/shiny/www/img/analysis_features.jpg" width="70%" height="70%" />


### Inter-Sample Analysis

Inter-sample analyses focus on differences between samples or groups of samples via feature heatmaps and beta diversity calculations.

<img src="inst/shiny/www/img/analysis_inter_beta.jpg" width="70%" height="70%" />

### Correlation

Correlation allows the user to visualize the relationship between either two features or a feature and a numeric phenotype in a scatterplot enhanced with a linear regression statistic.

<img src="inst/shiny/www/img/analysis_correlation.jpg" width="70%" height="70%" />

### Differential abundance

Differential abundance (DA) analysis is focused on testing the null hypothesis that the mean or mean ranks between groups are the same for a specific feature. DA analysis can help detect changes in feature abundance across two or more different levels of a phenotype. Four different methods can be chosen via the application: DESeq2, Kruskal-Wallis, limma, or a zero-inflated log normal model.

<img src="inst/shiny/www/img/analysis_diff_details.jpg" width="70%" height="70%" />

### Longitudinal

Longitudinal analysis allows the user to generate feature plots with more control over the data shown within the plot. For a specific feature, the user can choose a phenotype and specific levels of that phenotype to show in the plot. The chosen order of the levels will be kept within the visualization which allows sorting by specific dates or tissues among other things.

<img src="inst/shiny/www/img/analysis_longitudinal.jpg" width="70%" height="70%" />

## Reports

Once an analysis is complete, a user might like to share the results with collaborators or download them for further analysis beyond the functionality provided via the Microbiome Explorer. To do this, we are providing the option to include any part of the analysis in a report which is fully reproducible outside of the application. 

<img src="inst/shiny/www/img/report_output.jpg" width="70%" height="70%" />

## Main UI page for Microbiome Explorer
shinyUI(
    dashboardPage(
        title = "Microbiome Explorer",
        ## create the three bars icon square (STACK OVERFLOW)
        dashboardHeader(
            titleWidth = 72,
            title = a(
                href = "#", class = "sidebar-toggle",
                `data-toggle` = "offcanvas", role = "button",
                span(
                    class = "sr-only",
                    "Toggle navigation"
                ),
                icon("bars")
            )
        ),
        ## main sidebar menu items
        dashboardSidebar(
            width = 220, collapsed = TRUE,
            sidebarMenu(
                id = "sidebar_menu",
                menuItem("Data Input",
                    icon = icon("folder"),
                    tabName = "datamenu"
                ),
                menuItem("  Analysis",
                    icon = icon("dna"),
                    tabName = "analysismenu"
                ),
                menuItem("    Report",
                    icon = icon("file"),
                    tabName = "reportmenu"
                ),
                menuItem("User Guide",
                         icon = icon("far fa-hand-point-right"),
                         tabName = "howtomenu"
                ),
                menuItem("     About",
                         icon = icon("fas fa-info-circle"),
                         tabName = "aboutmenu"
                )
            )
        ),
        dashboardBody(
            tags$head(
                tags$link(
                    rel = "stylesheet",
                    type = "text/css",
                    href = "me.css"
                )
            ),
            ## capture interaction on phenorow remove buttons (data input page)
            tags$head(tags$script(HTML(
            "$(document).on('click', '.phenoremoverow .action-button', 
                function () {
                    Shiny.setInputValue('loadnfilter-last_btn', 
                        this.id,{priority: 'event'});
                });"))),
            tags$script(HTML(
            "$('.ellipsis').tooltip({show: {effect:'none', delay:0}});")),


            ## Place the header to the right of the menu button.
            ## From STACK OVERFLOW
            tags$script(HTML('
                    $(document).ready(function() {
                      $("header").find("nav").append(\'<div  class="meIcon"><img src="dishart.png" height = "50"></img><span class="meTitle">Microbiome Explorer</span></div>\');
                    })
                  ')),
            shinyjs::useShinyjs(),
            shinyjs::extendShinyjs(script = "JS/parsePhenoFilters.js",
                                   functions = c("parsePhenoFilters")),
            shinyjs::extendShinyjs(script = "JS/enableTabs.js",
                                   functions = c("disableTab",
                                                 "enableTab",
                                                 "enableAll",
                                                 "collapse")),
            shinyjs::extendShinyjs(script = "JS/traceName.js",
                                   functions = c("getTraceName")),
            shinyjs::extendShinyjs(script = "JS/handleModebar.js",
                                   functions = c("resetAxes")),
            shinyjs::extendShinyjs(script = "JS/changeTable.js",
                                   functions = c("moveButton",
                                                 "removeInputElems")),

            tabItems(
                ##  DATA INPUT PAGES 
                tabItem(
                    tabName = "datamenu",
                    tabBox(
                        id = "data_input_tabbox", 
                        title = "Data Input", width = 12, side = "left",
                        tabPanel(
                            "LOAD & FILTER",
                            dataInputUI("loadnfilter")
                        ),
                        tabPanel(
                            "PHENOTYPE",
                            phenotypeTableUI("phenotab")
                        ),
                        tabPanel(
                            "FEATURES",
                            featureTableUI("featuretab")
                        )
                    )
                ),
                ## ANALYSIS PAGES 
                tabItem(
                    tabName = "analysismenu",
                    column(width = 12,
                           aggregationTabUI("aggregateTab"),
                           tabBox(
                               id = "analysismenu_tabbox", 
                               title = "Analysis", width = 12, side = "left",
                               tabPanel(
                                   "INTRA SAMPLE",
                                   intraAnalysisUI("intraAnalysis")
                               ),
                               tabPanel(
                                   "INTRA FEATURE",
                                   featureAnalysisUI("featureAnalysis")
                               ),
                               tabPanel(
                                   "INTER SAMPLE",
                                   interAnalysisUI("interAnalysis")
                               ),
                               tabPanel(
                                   "CORRELATION",
                                   corrAnalysisUI("corrAnalysis")
                               ),
                               tabPanel(
                                   "DIFFERENTIAL",
                                   diffAnalysisUI("diffAnalysis")
                               ),
                               tabPanel(
                                   "LONGITUDINAL",
                                   longAnalysisUI("longAnalysis")
                               )
                           )
                    )
                ),
                ## REPORT PAGE  
                tabItem(
                    tabName = "reportmenu",
                    tabBox(
                        id = "report_tabbox", title = "Report generation", 
                        width = 12, side = "left",
                        tabPanel(
                            "GENERATE REPORT",
                            reportListUI("reportlist")
                        )
                    )
                ),
                tabItem(
                    tabName = "howtomenu",
                    tabBox(
                        id = "howto_tabbox", title = "User Guide", 
                        width = 12, side = "left",
                        tabPanel(
                            "HOW TO ...",
                            shiny::includeMarkdown("Tutorial.md")
                        )
                    )
                ),
                tabItem(
                    tabName = "aboutmenu",
                    tabBox(
                        id = "about_tabbox", title = "About", 
                        width = 12, side = "left",
                        tabPanel(
                            "PEOPLE",
                            h4("Janina Reeder (Genentech - gRED OMNI Bioinformatics)"),
                            p("Janina developed the Microbiome Explorer under the mentorship of Joseph Paulson from a prototype version by Mo Huang."),
                            h4("Mo Huang (PhD student - Wharton University)"),
                            p("Mo implemented an initial prototype of the application as an intern project under the supervision of Joseph."),
                            h4("Joseph N. Paulson (Genentech - PDBB)"),
                            p("Joseph wrote the metagenomeSeq R package upon which many of the statistical analyses in this application are based.")
                        )
                    )
                )
            )
        )
    )
)

library(shinyjs)
library(shinydashboard)
library(shinycssloaders)
library(shinyWidgets)
library(magrittr)
library(microbiomeExplorer)
library(metagenomeSeq)
library(rmarkdown)
library(DESeq2)

#MAX ALLOWED FILE SIZE for file upload
#modify as needed, but note that the app will become slow with larger data
options(shiny.maxRequestSize = 10 * 1024^2)
options(warn = -1)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----hiddensetup, echo=FALSE, message=FALSE-----------------------------------
library(knitr)
library(rmarkdown)
doctype <- opts_knit$get("rmarkdown.pandoc.to")

## ----setup, echo=TRUE, message = FALSE----------------------------------------

library(microbiomeExplorer)


## ----eval=FALSE,echo=TRUE-----------------------------------------------------
# runMicrobiomeExplorer()

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
data("mouseData", package = "metagenomeSeq")
meData <- filterMEData(mouseData,minpresence = 1, minfeats = 2, minreads = 2)

## ----eval=TRUE, echo=TRUE, warning=FALSE--------------------------------------
makeQCPlot(meData, col_by = "diet",
       log = "none",
       filter_feat = 101,
       filter_read = 511,
       allowWebGL = FALSE)

plotlySampleBarplot(meData,
                    col_by = "diet")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------

meData <- filterMEData(mouseData,minpresence = 1, minfeats = 100, minreads = 500)


## ----eval=TRUE, echo=TRUE-----------------------------------------------------
meData <- normalizeData(meData,norm_method = "Proportion")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
new_pheno <- interaction(pData(meData)[,c("mouseID","relativeTime")])
mutatedRows <- row.names(pData(meData))
mutatedData <- dplyr::mutate(pData(meData), "mouse_time" = new_pheno)
row.names(mutatedData) <- mutatedRows
meData <- addPhenoData(meData,mutatedData)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
bufcolnames <- names(fData(meData))
df <- as.data.frame(t(apply(fData(meData),1, rollDownFeatures)))
names(df) <- bufcolnames
meData <- addFeatData(meData,df)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
aggDat <- aggFeatures(meData, level = "genus")

## ----eval=TRUE, echo=TRUE, warning=FALSE--------------------------------------
plotAbundance(aggDat,
              level = "genus",
              x_var = "diet",
              facet1 = NULL,
              facet2 = NULL,
              ind = 1:10,
              plotTitle = "Top 10 feature percentage at genus level",
              ylab = "Percentage")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
plotSingleFeature(aggDat,
            x_var = "diet",
            ind = 1:10,
            plotTitle = "Percentage of Enterococcus",
            facet1 = NULL,
            facet2 = NULL,
            feature = "Enterococcus",
            ylab = "Percentage",
            log = TRUE,
            showPoints = TRUE)

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
plotAlpha(aggDat,
          level = "genus",
          index = "shannon",
          x_var = "diet",
          facet1 = NULL,
          facet2 = NULL,
          col_by = "mouseID",
          plotTitle = "Shannon diversity index at genus level")

## ----eval=TRUE, echo=TRUE, message = FALSE, warning = FALSE-------------------
distMat <- computeDistMat(aggDat, "bray")
pcaVals <- calculatePCAs(distMat, 
                         c("PC1", "PC2"))
plotBeta(aggDat,
         dist_method = "bray",
         pcas = pcaVals,
         dim = c("PC1", "PC2"),
         col_by = "diet",
         shape_by = NULL,
         plotTitle = "Bray-Curtis diversity at genus level",
         pt_size = "6",
         plotText = "R2: 0.478; Pr(>F): 0.002",
         confInterval = 0.95,
         allowWebGL = FALSE)


## ----eval=TRUE, echo=TRUE, warning = FALSE, fig.width = 8, fig.height = 10----
plotHeatmap(aggDat,
            features = NULL,
            log = TRUE,
            sort_by = "Variance",
            nfeat = 50,
            col_by = c("diet"),
            row_by = "",
            plotTitle = "Top 50 features sorted by Variance at genus level")

## ----eval=TRUE, echo=TRUE, warning = FALSE, message = FALSE-------------------
cf <- corrFeature(aggDat,
                 feat1 = "Bacteroides",
                 feat2 = "Prevotella",
                 log = TRUE,
                 facet1 = "diet",
                 facet2 = NULL,
                 method = "spearman",
                 plotTitle = "Spearman correlation of Bacteroides vs Prevotella split by diet",
                 col_by = "status",
                 allowWebGL = FALSE)


## ----eval=TRUE, echo=TRUE, warning = FALSE, message = FALSE-------------------
diffResults <- runDiffTest(aggDat,
                        level = "genus",
                        phenotype = "diet",
                        phenolevels = c("BK", "Western"),
                        method = "DESeq2")

kable(head(diffResults))

## ----eval=TRUE, echo=TRUE, warning = FALSE, fig.width = 8, fig.height = 10----
plotLongFeature(aggDat,
                x_var = "date",
                id_var = "mouseID",
                plotTitle = "Abundance of Prevotella",
                feature = "Prevotella",
                ylab = "Reads",
                log = TRUE,
                x_levels = c("2007-12-11","2008-01-21","2008-02-11","2008-02-25"))

## Module handling data import/upload and filtering


#' Module handling file upload for the application: UI
#' In a deployed version this module should be replaced with database access
#'
#' @param id module identifier
#' 
#' @return div holding ui elements
#'
#' @author Janina Reeder
fileUploadUI <- function(id) {
  ns <- NS(id)
  
  div(
    ## inputs restricted to certain filetypes defined via global.R
    fileInput(
      ns("datafile"),
      label = "Upload Feature Count Data", 
      accept = getOption("me.filetypes")),
    shinyjs::disabled(
      div(
        id = ns("phenodiv"),
        fileInput(
          ns("phenofile"),
          label = "Link Phenotype Info", 
          accept = getOption("me.delim"))
      )),
    shinyjs::disabled(
      div(
        id = ns("featdiv"),
        fileInput(ns("featfile"),
                  label = "Add Taxonomy Info", 
                  accept = getOption("me.delim"))
      ))
  )
}

#' Module handling file upload for the application: server
#'
#' @param input module input
#' @param output module output
#' @param session app session
#' @param meData main reactive storing the MRexperiment data
#' @param meName main reactive storing the filename uploaded
#' @param initializeData reactiveVal keeping track of new uploads to reset data
#' @param addPheno reactiveVal keeping track of phenodata changes
#' @param dataSource reactive Value storing commands for loading data
#' @param resetFile indicating if module should be reset
#' 
#' @return boolean denoting successful upload of a file
#'
#' @author Janina Reeder
fileUpload <- function(input, output, session, 
                       meData, meName, initializeData, 
                       addPheno, dataSource, 
                       resetFile = reactive(NULL)) {
  ns <- session$ns
  
  fileUploaded <- reactiveVal(NULL)
  
  ## reset all input elements
  observe({
    req(resetFile())
    shinyjs::reset("datafile")
    shinyjs::reset("phenofile")
    shinyjs::reset("featfile")
    fileUploaded(NULL)
  })
  
  observe({
    req(meData())
    shinyjs::enable("phenodiv")
  })
  
  ## loading a data file: uses getFileType & readData from functions.R
  observeEvent(input$datafile, {
    withProgress(message = "Data Uploading", {
      fileext <- tools::file_ext(input$datafile$name)
      filetype <- getFileType(fileext)
      shinyjs::enable("phenodiv")
      meName(input$datafile$name)
      meData(filterMEData(readData(input$datafile$datapath, 
                                   type = filetype), 
                          minpresence = 1, minfeats = 2, minreads = 2))
      dataSource(
        paste0("meData <- filterMEData(readData(\"", 
               input$datafile$datapath, "\", type = \"", 
               filetype,
               "\"),minpresence = 1, minfeats = 2, minreads = 2)", "\n",
               "##Data file name: ",input$datafile$name))
      if (filetype %in% c("TAB", "CSV")) {
        shinyjs::enable("featdiv")
      } else if (filetype == "BIOM" & ncol(fData(meData())) < 2) {
        shinyjs::enable("featdiv")
      } else { 
        shinyjs::disable("featdiv")
      }
      ## UPLOAD COMPLETED
      initializeData(TRUE)
      shinyjs::reset("phenofile")
      shinyjs::reset("featfile")
      fileUploaded(TRUE)
    })
  })
  
  ## handling change in pheno file;
  observeEvent(input$phenofile, {
    if(ncol(pData(meData())) > 1){
      showModal(modalDialog(
        title = "Overwrite phenotype data",
        "Phenotype data is already available for this project.\n
                Do you want to replace or extend the existing phenotype
                data?",
        footer = tagList(actionButton(ns("confirmReplace"), "Replace"),
                         actionButton(ns("confirmExtend"), "Extend"),
                         modalButton("Cancel")
        ),
        easyClose = FALSE))
    } else {
      bufMEDate <- addPhenoData(meData(), input$phenofile$datapath)
      if (is.null(bufMEDate)) {
        showModal(modalDialog(
          title = "Error merging phenotype information",
          "An error occurred when attempting to merge the phenotype 
                information with the counts data.\n
                The phenotable provided must have a row for every sample column 
                in the counts data.",
          easyClose = TRUE))
      } else {
        dataSource(paste0(dataSource(), "\n", 
                          "meData <- addPhenoData(meData, \"", 
                          input$phenofile$datapath, "\")", "\n",
                          "##Pheno file name: ",input$phenofile$name))
        meData(bufMEDate)
        addPheno(TRUE)
      }
    }
  })
  
  observeEvent(input$confirmReplace,{
    removeModal()
    bufMEDate <- addPhenoData(meData(), input$phenofile$datapath)
    if (is.null(bufMEDate)) {
      showModal(modalDialog(
        title = "Error merging phenotype information",
        "An error occurred when attempting to merge the phenotype 
                information with the counts data.\n
                The phenotable provided must have a row for every sample column 
                in the counts data.",
        easyClose = TRUE))
    } else {
      dataSource(paste0(dataSource(), "\n", 
                        "meData <- addPhenoData(meData, \"", 
                        input$phenofile$datapath, "\")", "\n",
                        "##Pheno file name: ",input$phenofile$name))
      meData(bufMEDate)
      addPheno(TRUE)
    }
  })
  
  observeEvent(input$confirmExtend,{
    removeModal()
    bufMEDate <- extendPhenoData(meData(), input$phenofile$datapath)
    if (is.null(bufMEDate)) {
      showModal(modalDialog(
        title = "Error merging phenotype information",
        "An error occurred when attempting to merge the phenotype 
                information with the counts data.\n
                The phenotable provided must have a row for every sample column 
                in the counts data.",
        easyClose = TRUE))
    } else {
      dataSource(paste0(dataSource(), "\n", 
                        "meData <- extendPhenoData(meData, \"", 
                        input$phenofile$datapath, "\")", "\n",
                        "##Pheno file name: ",input$phenofile$name))
      meData(bufMEDate)
      addPheno(TRUE)
    }
  })
  
  
  ## handling change in feature file (taxonomy);
  observeEvent(input$featfile, {
    bufMEDate <- addFeatData(meData(), input$featfile$datapath)
    if (is.null(bufMEDate)) {
      showModal(modalDialog(
        title = "Error merging feature information",
        "An error occurred when attempting to merge the feature 
                information with the counts data.\n
                The feature information provided must have a corresponding row 
                for every row in the counts data.",
        easyClose = TRUE))
    } else {
      dataSource(paste0(dataSource(), "\n", 
                        "meData <- addFeatData(meData, \"", 
                        input$featfile$datapath, "\")", "\n",
                        "##Pheno file name: ",input$featfile$name))
      meData(bufMEDate)
    }
  })
  
  return(fileUploaded)
}

## END UPLOAD DATA FROM FILE


## DATA QC AND FILTERING

#' Main Data input UI where the user selects files to upload to the app or 
#' connects to database
#'
#' @param id module identifier
#' 
#' @return fluidRow holding UI interface
#'
#' @author Janina Reeder
#' 
#' @examples dataInputUI("datainput_id")
#' 
#' @export
dataInputUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    column(
      width = 4, id = ns("datauploadcol"),
      h4("UPLOAD DATA", style = "padding-top: 45px;"),
      fileUploadUI(ns("fileupload")),
      hr(),
      shinyjs::disabled(div(
        id = ns("filterdiv"),
        h4("FILTER DATA", style = "padding-top: 20px;"),
        h5("Features", style = "padding-top:0;"),
        sliderInput(
          ns("featuresams"),
          label = "Minimum Sample Presence",
          value = 1, min = 1, max = 1000, 
          round = TRUE, width = "250px"),
        h5("Samples"),
        sliderInput(
          ns("minfeats"),
          label = "Minimum Number of Features",
          value = 2, min = 2, max = 1000, 
          round = TRUE, width = "250px"),
        sliderInput(
          ns("minreads"),
          label = "Minimum Number of Reads",
          value = 2, min = 2, max = 1000, 
          round = TRUE, width = "250px"),
        box(
          collapsible = TRUE, collapsed = TRUE, width = 10,
          style = "padding:0; margin-top: -40px;",
          numericInput(
            ns("readnum"),
            label = "Set min #reads",
            value = 2, min = 2,
            width = "100px")
        )
      )),
      shinyjs::disabled(
        div(
          id = ns("phenodiv"), style = "margin-top: 100px",
          h5("Phenotypes", 
             style = "padding-top:0; margin-top: -11px;"),
          selectizeInput(
            ns("removepheno"),
            label = "Remove Phenotypes",
            choices = "", 
            multiple = FALSE, 
            options = list(placeholder = 
                             "Select Phenotype"), 
            width = "250px")
        )),
      shinyjs::disabled(
        div(
          id = ns("buttondiv"),
          fluidRow(
            width = 12, id = "actionbuttonrow2",
            ## filter updates meData for further analysis
            actionButton(
              ns("filterbutton"), 
              icon = icon("fas fa-filter"),
              label = HTML("&nbsp;FILTER"), 
              width = "120px"),
            ## resets back to original data 
            actionButton(
              ns("resetbutton"), 
              icon = icon("fas fa-redo-alt"),
              label = HTML("&nbsp;RESET"), 
              width = "120px")
          ),
          hr(),
          h4("OBTAIN DATA", style = "padding-top: 50px;"),
          fluidRow(
            width = 12, id = "actionbuttonrow2",
            ## downloads data to user's computer
            shinyjs::disabled(
              downloadButton(
                ns("savebutton"), 
                label = "GET DATA")),
            ## add QC plots to report
            actionButton(
              ns("reportButton"), 
              label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
              width = "120px")
          )
        ))
    ),
    column(
      width = 8,
      uiOutput(ns("projecthead")),
      fluidRow(
        width = 10, id = "histogramrow",
        column(
          width = 5,
          plotly::plotlyOutput(
            ns("featpres"), 
            width = "200px", 
            height = "200px")
        ),
        column(
          width = 5,
          plotly::plotlyOutput(
            ns("libsize"), 
            width = "200px", 
            height = "200px")
        )
      ),
      shinycssloaders::withSpinner(
        plotly::plotlyOutput(
          ns("dataqcplot"), 
          width = "675px", 
          height = "550px"),
        type = 3, color = "#424242", 
        color.background = "#fdfdfc"),
      uiOutput(ns("plotoptions")),
      fluidRow(
        width = 12, id = "sampleoturow",
        plotly::plotlyOutput(
          ns("sampleotus"), 
          width = "650px", 
          height = "400px")
      ),
      uiOutput(ns("samplebaroptions"))
    )
  )
}


#' Helper function to filter phenodata for interesting phenotypes to be used 
#' for filtering or subsetting
#'
#' @param MRobj the MRexperiment storing the data
#'
#' @author Janina Reeder
#' 
#' @importFrom Biobase pData
#'
#' @return list of named vectors with names being pData column headers and 
#' values being unique entries; columns with only one entry or those with 
#' different values for each samples are omitted
getFilterChoices <- function(MRobj) {
  maxvalues <- nrow(pData(MRobj))
  ## Keeping only those that have more than one unique value,
  ## but less than the number of samples
  allchoices <- lapply(names(pData(MRobj)), function(s) {
    uniqvalues <- unique(pData(MRobj)[[s]])
    if (length(uniqvalues) <= 1 | length(uniqvalues) == maxvalues) {
      return(NULL)
    }
    return(uniqvalues)
  })
  names(allchoices) <- names(pData(MRobj))
  if("SAMPLE_ID" %in% names(allchoices)){
    allchoices[["SAMPLE_ID"]] <- pData(MRobj)[["SAMPLE_ID"]]
  }
  allchoices <- allchoices[vapply(allchoices, function(a) {
    !is.null(a)
  }, logical(1))]
  return(allchoices)
}


#' Main Data input server where the user selects files to upload to the app or 
#' connects to database
#'
#' @param input module input
#' @param output module output
#' @param session app session
#' @param dataSource reactive Value storing commands for loading data
#' @param dataFilterRep reactive Value storing commands for filtering data
#' @param qcRep reactive Value storing commands for producing qc plots
#' @param addPheno reactive boolean keeping track of phenodata changes
#' @param resetReports reactive boolean indicating whether reports need to be reset
#'
#' @author Janina Reeder
#'
#' @return list of reactives containing the uploaded and filtered data as well 
#' as the filterChoices on phenotypes
#'
#' @importFrom metagenomeSeq MRcounts 
#' @importFrom Biobase pData
#' @import shiny
#' @import shinydashboard
dataInput <- function(input, output, session, 
                      dataSource, dataFilterRep, qcRep,
                      addPheno, resetReports) {
  ns <- session$ns
  
  ## main data reactive to be passed to analysis tabs
  meData <- reactiveVal(NULL)
  meName <- reactiveVal("") 
  ## backup of original data: used when resetting
  origData <- reactiveVal(NULL)
  #complete data update
  initializeData <- reactiveVal(FALSE)
  filterChoices <- reactiveVal(NULL)
  remainingFilters <- reactiveVal(NULL)
  facetOptions <- reactiveVal(NULL)
  shapeOptions <- reactiveVal(NULL)
  colorChoice <- reactiveVal("No color")
  logChoice <- reactiveVal("none")
  featureSams <- reactiveVal(1)
  minReads <- reactiveVal(2)
  minFeats <- reactiveVal(2)
  qcCode <- reactiveVal(NULL)
  sortPheno <- reactiveVal(NULL)
  phenoValues <- reactiveVal(NULL)
  
  ## essentially meData, but only updated if plot changes are required
  plotData <- reactiveVal(NULL)
  phenoFilters <- reactiveVal(NULL)
  ## phenotypes selected during filtering steps
  selectedChoices <- reactiveVal(NULL)
  
  callModule(fileUpload, "fileupload", 
             meData = meData, 
             meName = meName,  
             initializeData = initializeData, 
             addPheno = addPheno,
             dataSource = dataSource)
  
  
  observeEvent(filterChoices(),{
    filterNames <- names(filterChoices())
    filterNames <- filterNames[!(filterNames %in% "SAMPLE_ID")]
    if(length(filterNames) == 0){
      shapeOptions(NULL)
      facetOptions(NULL)
    } else {
      shapeIndices <- vapply(filterNames, function(n){
        length(unique(pData(meData())[,n])) <= getOption("me.shapenum")
      }, logical(1))
      if(length(shapeIndices) > 0){
        sO <- filterNames[shapeIndices]
        if(any(!(sO %in% shapeOptions())) |
           any(!(shapeOptions() %in% sO))){
          shapeOptions(sO)
        }
      }
      facetIndices <- vapply(filterNames, function(n){
        length(unique(pData(meData())[,n])) <= getOption("me.facetnum")
      }, logical(1))
      if(length(facetIndices) > 0){
        fO <- filterNames[facetIndices]
        if(any(!(fO %in% facetOptions())) | 
           any(!(facetOptions() %in% fO))){
          facetOptions(fO)
        }
      }
    }
    remainingFilters(filterChoices()[!names(filterChoices()) %in% 
                                       selectedChoices()])
  })
  
  observe({
    updateSelectInput(session, "qc_color",
                      choices = c("No color", facetOptions()),
                      selected = isolate(colorChoice()))
    updateSelectInput(session, "bar_color",
                      choices = c("No color", facetOptions()),
                      selected = isolate(colorChoice()))
    updateSelectInput(session, "sort_pheno",
                      choices = facetOptions())
  })
  
  ## update input elements based on meData
  observeEvent(initializeData(),{
    req(initializeData(), meData())
    shinyjs::enable("filterdiv")
    shinyjs::enable("buttondiv")
    shinyjs::enable("savebutton")
    origData(meData())
    colorChoice("No color")  
    logChoice("none")
    featureSams(1)
    minFeats(2)
    minReads(2)    
    removeUI(".phenoremoverow", multiple = TRUE, immediate = TRUE)
    dataFilterRep(NULL)
    phenoFilters(NULL)  
    filterChoices(NULL)
    remainingFilters(NULL)    
    facetOptions(NULL)
    shapeOptions(NULL)
    plotData(meData())
    resetReports(TRUE)
    shinyjs::js$parsePhenoFilters()
    updateSliderInput(session, "featuresams", 
                      value = 1, min = 1,
                      max = max(rowSums(MRcounts(meData()) > 0)))
    updateSliderInput(session, "minfeats", 
                      value = 2, min = 2,
                      max = max(colSums(MRcounts(meData()) > 0)))
    updateSliderInput(session, "minreads", 
                      value = 2, min = 2,
                      max = max(colSums(MRcounts(meData()))))
    shinyjs::enable("phenodiv")
    filterChoices(getFilterChoices(meData()))
    updateSelectizeInput(session, "removepheno",
                         choices = names(filterChoices()), 
                         selected = "",
                         options = list(placeholder = "Select Phenotype"))
    initializeData(FALSE)
  })
  
  observeEvent(addPheno(),{
    req(addPheno(), meData())
    if (ncol(pData(meData())) > 1) {
      shinyjs::enable("phenodiv")
      filterChoices(getFilterChoices(meData()))
      updateSelectizeInput(session, "removepheno",
                           choices = names(filterChoices()), 
                           selected = "",
                           options = list(placeholder = "Select Phenotype")) 
      isolate({
        if(!colorChoice() %in% names(filterChoices()))
          colorChoice("No color")
      })
      plotData(meData())
    } else {
      shinyjs::disable("phenodiv")
    }
    addPheno(FALSE)
  })
  
  ## workaround to allow the user to select a specific value
  observeEvent(input$readnum,{
    updateSliderInput(session, "minreads", value = input$readnum)
  })
  
  ## show the qc data in log scale
  observeEvent(input$logoptions, {
    prevLog <- logChoice()
    minf <- input$minfeats
    minr <- input$minreads
    
    if(input$logoptions == "none"){
      if(prevLog == "x axis"){
        minr <- exp(minr)
      } else if(prevLog == "y axis"){
        minf <- exp(minf)
      } else if(prevLog == "both"){
        minr <- exp(minr)
        minf <- exp(minf)
      }
      updateSliderInput(session, "minfeats", min = minFeats(),
                        max = max(colSums(MRcounts(meData()) > 0)),
                        value = minf)
      updateSliderInput(session, "minreads", min = minReads(),
                        max = max(colSums(MRcounts(meData()))),
                        value = minr)
    } else if(input$logoptions == "x axis"){
      if(prevLog == "none"){
        minr <- log(minr)
      } else if(prevLog == "y axis"){
        minf <- exp(minf)
        minr <- log(minr)
      } else if(prevLog == "both"){
        minf <- exp(minf)
      }
      updateSliderInput(session, "minfeats", min = minFeats(),
                        max = max(colSums(MRcounts(meData()) > 0)),
                        value = minf)
      updateSliderInput(session, "minreads", min = log(minReads()),
                        max = log(max(colSums(MRcounts(meData())))),
                        value = minr)
      
    } else if(input$logoptions == "y axis"){     
      if(prevLog == "none"){
        minf <- log(minf)
      } else if(prevLog == "x axis"){
        minr <- exp(minr)
        minf <- log(minf)
      } else if(prevLog == "both"){
        minr <- exp(minr)
      }
      updateSliderInput(session, "minfeats", min = log(minFeats()),
                        max = log(max(colSums(MRcounts(meData()) > 0))),
                        value = minf)
      updateSliderInput(session, "minreads", min = minReads(),
                        max = max(colSums(MRcounts(meData()))),
                        value = minr)
    } else if(input$logoptions == "both"){
      if(prevLog == "none"){
        minf <- log(minf)
        minr <- log(minr)
      }
      if(prevLog == "x axis"){
        minf <- log(minf)
      }
      if(prevLog == "y axis"){
        minr <- log(minr)
      }
      updateSliderInput(session, "minfeats", min = log(minFeats()),
                        max = log(max(colSums(MRcounts(meData()) > 0))),
                        value = minf)
      updateSliderInput(session, "minreads", min = log(minReads()),
                        max = log(max(colSums(MRcounts(meData())))),
                        value = minr)
    }
    logChoice(input$logoptions)
  }, ignoreInit = TRUE)
  
  
  ## add pheno filter UI element
  observeEvent(input$removepheno, {
    req(input$removepheno,remainingFilters())
    insertUI(
      selector = paste0("#", ns("removepheno")),
      where = "afterEnd",
      ui = fluidRow(
        class = "phenoremoverow", 
        id = ns(paste0("ui_", input$removepheno)),
        column(width = 1,
               actionLink(ns(paste0("rmbut_", input$removepheno)),
                          label = "", icon = icon("times-circle-o"),
                          style = paste0("color: darkred"), class = "rmpclass")
        ),
        column(
          width = 10, id = ns("checkboxcol"),
          selectInput(
            inputId = ns(paste0("rp_", input$removepheno)),
            label = paste0(input$removepheno, 
                           ": Select values to remove"),
            multiple = TRUE,
            choices = as.character(
              remainingFilters()[[input$removepheno]]),
          )
        )
      ),
      immediate = TRUE
    )
    selectedChoices(c(selectedChoices(), input$removepheno))
    shinyjs::js$parsePhenoFilters()
  })
  
  observeEvent(selectedChoices(),{
    remainingFilters(filterChoices()[!names(filterChoices()) %in% 
                                       selectedChoices()])
    updateSelectizeInput(session, "removepheno", 
                         choices = c(names(remainingFilters())), 
                         selected = "",
                         options = list(placeholder = "Select Phenotype"))
  })
  
  ## observes remove button clicks handled via javascript in UI
  observe({
    ## last button is changed via Shiny.setInputValue within 'on' click 
    ## function (see UI)
    req(input$last_btn)
    isolate({
      sc <- selectedChoices()
      selectedChoices(sc[!sc %in% gsub(".*rmbut_", "", input$last_btn)])
    })
    ## jquery (used by removeUI) can't handle dots in names, 
    ##so we need to explicitely include control characters
    removeUI(paste0("#", gsub("rmbut", "ui", 
                              gsub("\\.", "\\\\.", input$last_btn))), 
             immediate = TRUE)
    shinyjs::js$parsePhenoFilters()
  })
  
  
  ## process filtering operation on data: if origData hasn't been saved yet, 
  ##store current meData as origData
  observeEvent(input$filterbutton, {
    mustabort <- FALSE
    if (is.na(input$featuresams)) {
      showModal(modalDialog(
        title = "Input Error",
        "No value entered for minimum sample presence.\n 
                Please revise.",
        easyClose = TRUE
      ))
    } else {
      ## parse input elements for max allowed values: shiny doesn't 
      ## control text inputs.
      if (input$featuresams > max(rowSums(MRcounts(meData()) > 0))) {
        mustabort <- TRUE
      }
      
      if (input$minfeats > max(colSums(MRcounts(meData()) > 0))) {
        mustabort <- TRUE
      }
      
      if (input$minreads > max(colSums(MRcounts(meData())))) {
        mustabort <- TRUE
      }
      if (mustabort == TRUE) {
        showModal(modalDialog(
          title = "Filtering Error",
          "One of the filter values chosen is larger than the overall 
                    maximum. All data will be removed.\n Please revise.",
          easyClose = TRUE
        ))
      } else {
        ## PERFORM FILTERING BASED ON FEATURES AND READS
        prevfeatsams <- featureSams()
        prevminfeats <- minFeats()
        prevminreads <- minReads()
        featureSams(input$featuresams)
        minFeats(input$minfeats)
        minReads(input$minreads)
        minf <- minFeats()
        minr <- minReads()
        if(logChoice() %in% c("both","x axis"))
          minr <- exp(minr)
        if(logChoice() %in% c("both","y axis"))
          minf <- exp(minf)
        if(prevfeatsams != featureSams() | 
           prevminfeats != minFeats() | 
           prevminreads != minReads()){
          bufData <- filterMEData(meData(), 
                                  minpresence = featureSams(), 
                                  minfeats = minf,
                                  minreads = minr)
          if (nrow(pData(bufData)) == 0) {
            showModal(modalDialog(
              title = "Error filtering data",
              "No data remains when applying size filters. Please revise ...",
              easyClose = TRUE
            ))
          } else {
            meData(bufData) 
            plotData(meData())
            updateSliderInput(session, "featuresams", 
                              min = input$featuresams)
            updateSliderInput(session, "minfeats", 
                              min = input$minfeats)
            updateSliderInput(session, "minreads", 
                              min = input$minreads)
            dataFilterRep(
              paste0(dataFilterRep(),
                     "\nmeData <- filterMEData(meData,minpresence = ", 
                     featureSams(),
                     ", minfeats = ", minf, ", minreads = ", 
                     minr, ")"))
          }
        }
        ## FILTER ON PHENOTABLE
        ## go through all open filters and adjust meData
        if (!is.null(input$parsedFilters)) {
          phenovalstoremove <- lapply(
            input$parsedFilters, 
            function(f) {
              selectedFilterVals <- input[[gsub("loadnfilter-ui", "rp", f)]]
              selectedFilterVals[selectedFilterVals == ""] <- "NA"
              selectedFilterVals
            }
          )
          names(phenovalstoremove) <- gsub("loadnfilter-ui_", "", 
                                           input$parsedFilters)
          bufData <- filterByPheno(meData(), phenovalstoremove)
          if (nrow(pData(bufData)) == 0) {
            showModal(
              modalDialog(
                title = "Error filtering data",
                "No data remains when applying phenotype filters. Please revise ...",
                easyClose = TRUE
              ))
          } else {
            removeUI(".phenoremoverow", multiple = TRUE, 
                     immediate = TRUE)
            selectedChoices(NULL)
            meData(bufData)
            filterChoices(getFilterChoices(meData()))
            remainingFilters(filterChoices())
            updateSelectizeInput(session, "removepheno",
                                 choices = names(filterChoices()), 
                                 selected = "",
                                 options = list(placeholder = "Select Phenotype"))
            isolate({
              if(!colorChoice() %in% names(pData(meData())))
                colorChoice("No color")
            })
            plotData(meData())
            #Adjust code for reports
            phenoFilters(c(phenoFilters(),phenovalstoremove))
            duplPF <- duplicated(phenoFilters())
            phenoFilters(phenoFilters()[!duplPF])
            
            dataFilterRep(
              paste0(dataFilterRep(),
                     "\nfiltList <- ",
                     paste0("list(",
                            paste0('if'(lengths(phenoFilters()) == 1,"\"",""),
                                   phenoFilters(), 
                                   'if'(lengths(phenoFilters()) == 1,"\"",""),
                                   collapse= ","),")"),
                     "\nnames(filtList) <- ",
                     paste0("c(",
                            paste0("\"",
                                   names(phenoFilters()),
                                   "\"",
                                   collapse=", "),
                            ")", collapse = ", "),
                     "\nmeData <- filterByPheno(meData, filtList)"
              )
            )
          }
        }
      }
    }
  })
  
  ## reset any filtering operations: check to proceed
  observeEvent(input$resetbutton, {
    showModal(modalDialog(
      tagList(
        p("Reset will set data back to last saved version. All 
                  modifications will be lost.\nPlease confirm reset!")
      ),
      title = "Resetting Data",
      footer = tagList(
        actionButton(ns("confirmReset"), "Reset"),
        modalButton("Cancel")
      )
    ))
  })
  
  ## proceed approved: perform reset
  observeEvent(input$confirmReset, {
    colorChoice("No color")
    logChoice("none")
    minFeats(2)
    minReads(2)
    featureSams(1)
    qcCode(NULL)
    updateSliderInput(session, "featuresams", value = 1, min = 1)
    updateSliderInput(session, "minfeats", value = 2, min = 2)
    updateSliderInput(session, "minreads", value = 2, min = 2)
    removeUI(".phenoremoverow", multiple = TRUE, immediate = TRUE)
    dataFilterRep(NULL)
    phenoFilters(NULL)
    meData(origData())
    filterChoices(getFilterChoices(meData()))
    remainingFilters(filterChoices())
    updateSelectizeInput(session, "removepheno",
                         choices = names(filterChoices()), 
                         selected = "",
                         options = list(placeholder = "Select Phenotype"))
    updateSelectInput(session, "qc_color", selected = "No color")
    plotData(meData())
    removeModal()
  })
  
  ## Render header
  output$projecthead <- renderUI({
    if(!is.null(meData())){
      div(
        h2(id = "projecttitle", meName()),
        br(),
        div(
          p("Performing quality control (QC) is an important step prior to downstream analyses. 
          The following QC plots highlight the distribution of present features and library sizes 
          as well as the relationship between these two. 
          Colors can be added based on phenotype information, e.g. sequencing or library preparation batches.
          Data can be subset based on phenotype information or filtered to feature/library constraints.
          It is important for the user to consider the appropriate filtering approaches. Downstream analyses 
          require normalization to account for sequencing variability. Currently, 
          proportions and cumulative sum scaling (CSS) are implemented (Paulson, 2013).
          To enable inputs in the analysis sections, aggregate to a specific taxonomy level. If no feature data
          is available, choose \"unavailable\" which will allow analysis at the raw counts level.")
        ),
        h4("QC OVERVIEW")
      )
    } else{
      div(id = "datahelp",
          h2("Data input"),
          div(
            h5("Microbiome Explorer accepts several different data upload formats:"),
            tags$li("MRexperiment-class objects stored as RDATA or RDS files"),
            tags$li("The Biological Observation Matrix (BIOM) formatted files"),
            tags$li("Raw counts files"),
            p(""),
            p("A ",strong("counts"), " file is required and can be uploaded in delimited form (csv, tsv). The required format
             is such that each ",strong("sample is a column"), " and each unique ",strong("feature is a row"), " in the data set.
            The counts matrix should have no row names and store the information on the features/OTUs in its first columns. 
            All other columns names should correspond to sample names."),
            p("A tabular ", strong("phenotype"), "file can be linked to the counts data. Each ", strong("sample is a row"),
              "with the names of the rows in the phenotype data corresponding to the names of the columns in the count
            data. Appending several phenotype files by subsequent uploads is possible.
            If no phenotype file is given, the names of the columns of the count data are
            used as the phenotype data."),
            p("A ", strong("feature"), " data file must be provided if aggregation to a particular phylogenetic level is desired.
            Each unique ",strong("feature is a row"), " which must correspond to the ones in the counts data.
            Each column is a taxonomy level.
            If the feature data file is omitted, analysis can only be done at the raw counts level."),
            p("An MRexperiment-class object from metagenomeSeq already combines these different data files into one data structure based on 
           an extended eSet-class.") 
          )
      )
    }
  })
  
  ## render plot options
  output$plotoptions <- renderUI({
    req(plotData())
    selection <- isolate('if'(colorChoice() != "", 
                              colorChoice(), "No Color"))
    shinyWidgets::dropdownButton(
      tags$h3("Plot Options"),
      selectInput(
        ns("qc_color"),
        label = "Color By",
        choices = c("No color", isolate(facetOptions())),
        selected = selection,
        multiple = FALSE,
        selectize = FALSE
      ),      
      sliderInput(
        inputId = ns("plotWidth"),
        label = "Adjust plot width",
        value = 550,
        min = 250,
        max = 1600,
        round = TRUE
      ),
      shinyWidgets::radioGroupButtons(
        inputId = ns("logoptions"),
        label = "Log scale on",
        choices = c("none", 
                    "x axis", 
                    "y axis",
                    "both"),
        status = "primary",
        checkIcon = list(
          yes = icon("ok", 
                     lib = "glyphicon"),
          no = icon("remove",
                    lib = "glyphicon"))
      ),
      circle = FALSE, status = "danger", icon = icon("gear"), 
      width = "315px",
      label = "Plot Options"
    )
  })
  
  ## adjust color of plot
  observeEvent(input$qc_color, {
    colorChoice(input$qc_color)
    updateSliderInput(session, "minreads",
                      value = input$minreads - 1)
  }, ignoreInit = TRUE)
  
  ## store qc code in reactive Value
  observeEvent(input$reportButton, {
    qcRep(qcCode())
  })
  
  ## update plot based on min features or min reads
  observeEvent(c(input$minfeats, input$minreads), {
    req(plotData(), input$minfeats, input$minreads)
    if (nrow(plotData()) > 0) {
      minf <- input$minfeats
      minr <- input$minreads
      featthresh <- 2
      readthresh <- 2
      if(logChoice() %in% c("both","x axis"))
        readthresh <- log(2)
      if(logChoice() %in% c("both","y axis"))
        featthresh <- log(2)
      featalpha <- 0
      featalphafill <- 0
      if (minf > featthresh) {
        featalpha <- 1
        featalphafill <- 0.2
      }
      readalpha <- 0
      readalphafill <- 0
      if (minr > readthresh) {
        readalpha <- 1
        readalphafill <- 0.2
      }
      
      plotly::plotlyProxy("dataqcplot", session) %>%
        plotly::plotlyProxyInvoke("deleteTraces", 0)
      plotly::plotlyProxy("dataqcplot", session) %>%
        plotly::plotlyProxyInvoke(
          "addTraces",
          list(
            x = c(0, max(colSums(MRcounts(plotData())))),
            y = c(minf, minf),
            type = "scatter",
            mode = "lines",
            line = list(
              color = paste0("rgba(119, 0, 0,",
                             featalpha, ")"),
              width = 1, dash = "dash"
            ),
            fill = "tozeroy",
            fillcolor = paste0("rgba(219,219,219,",
                               featalphafill, ")")
          ),
          0
        )
      
      plotly::plotlyProxy("dataqcplot", session) %>%
        plotly::plotlyProxyInvoke("deleteTraces", 1)
      plotly::plotlyProxy("dataqcplot", session) %>%
        plotly::plotlyProxyInvoke(
          "addTraces",
          list(
            y = c(0, max(colSums(MRcounts(plotData()) > 0))),
            x = c(minr, minr),
            type = "scatter",
            mode = "lines",
            line = list(
              color = paste0("rgba(119, 0, 0,",
                             readalpha, ")"),
              width = 1, dash = "dash"
            ),
            fill = "tozerox",
            fillcolor = paste0("rgba(219,219,219,",
                               readalphafill, ")")
          ),
          1
        )
    }
  }, ignoreInit = TRUE)
  
  observeEvent(input$plotWidth, {
    req(plotData())
    req(nrow(plotData()) > 0)
    plotly::plotlyProxy("dataqcplot", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = input$plotWidth)
      )
  })
  
  observeEvent(input$barWidth, {
    req(plotData())
    req(nrow(plotData()) > 0)
    plotly::plotlyProxy("sampleotus", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = input$barWidth)
      )
  })
  
  
  ## render the QC plot
  output$dataqcplot <- plotly::renderPlotly({
    req(plotData())
    req(nrow(plotData()) > 0)
    minf <- minFeats()
    minr <- minReads()
    featthresh <- 2
    readthresh <- 2
    if(logChoice() %in% c("both","x axis"))
      readthresh <- log(2)
    if(logChoice() %in% c("both","y axis"))
      featthresh <- log(2)
    if (minf <= featthresh) {
      minf <- 0
    }
    if (minr <= readthresh) {
      minr <- 0
    }
    qcCode(paste(
      paste0("#' ## QC plots"),
      paste0("", "#+ qcplot, fig.width = 7"),
      paste0("makeQCPlot(meData, col_by = \"", colorChoice(), "\","),
      paste0("\tlog = \"", logChoice(),"\","),
      paste0("\tfilter_feat = ", minf, ","),
      paste0("\tfilter_read = ", minr, ","),
      paste0("\tallowWebGL = FALSE)"),
      paste0("\n"),            
      paste0("", "#+ qcsample, fig.width = 7"),
      paste0("plotlySampleBarplot(meData,"),
      paste0("\tcol_by = \"",input$bar_color, "\","),
      paste0("\tpwidth = 600,"),
      paste0("\tsortbyfreq = ",input$sortbyfreq, ","),
      paste0("\tpheno_sort = ",
             'if'(is.null(sortPheno()),"NULL",
                  paste0("\"",sortPheno(),"\"")), ","),
      paste0("\tx_levels = ",
             'if'(is.null(phenoValues()),"NULL",
                  paste0("c(", paste0("\"",
                                      phenoValues(), 
                                      "\"",
                                      collapse = ","),")")),
             ")\n"),
      paste0("#'", ""),
      paste0(""),
      paste0("", "#+ histogram1, fig.width = 3, fig.height = 3"),
      paste0("plotlyHistogram(histvalue = colSums(MRcounts(meData)>0),"),
      paste0("\tplotTitle = \"No. Present Features\","),
      paste0("\txaxisTitle = \"features\","),
      paste0("\tyaxisTitle = \"frequency\")"),
      paste0("\n"),
      paste0("", "#+ histogram2, fig.width = 3, fig.height = 3"),
      paste0("plotlyHistogram(histvalue = colSums(MRcounts(meData)),"),
      paste0("\tplotTitle = \"Library Size\","),
      paste0("\txaxisTitle = \"Library Size\","),
      paste0("\tyaxisTitle = \"frequency\")"),
      paste0("#'", ""),
      paste0(""),
      sep = "\n"))
    
    
    microbiomeExplorer::makeQCPlot(plotData(),
                                   col_by = colorChoice(),
                                   log = logChoice(),
                                   filter_feat = minf,
                                   filter_read = minr,
                                   pwidth = isolate(input$plotWidth))
    
  })
  
  
  ## renders present features histogram; updates with meData
  output$featpres <- plotly::renderPlotly({
    req(plotData())
    if (nrow(plotData()) > 0) {
      microbiomeExplorer::plotlyHistogram(
        histvalue = colSums(MRcounts(plotData()) > 0),
        plotTitle = "Feature distribution",
        xaxisTitle = "features",
        yaxisTitle = "frequency"
      )
    }
  })
  
  ## renders library size histogram; updates with meData
  output$libsize <- plotly::renderPlotly({
    req(plotData())
    if (nrow(plotData()) > 0) {
      microbiomeExplorer::plotlyHistogram(
        histvalue = colSums(MRcounts(plotData())),
        plotTitle = "Read distribution",
        xaxisTitle = "library size",
        yaxisTitle = "frequency"
      )
    }
  })
  
  ## render plot options
  output$samplebaroptions <- renderUI({
    req(plotData())
    selection <- isolate('if'(colorChoice() != "", 
                              colorChoice(), "No Color"))
    shinyWidgets::dropdownButton(
      tags$h3("Plot Options"),
      selectInput(
        ns("bar_color"),
        label = "Color By",
        choices = c("No color", isolate(facetOptions())),
        selected = selection,
        multiple = FALSE,
        selectize = FALSE
      ),      
      sliderInput(
        inputId = ns("barWidth"),
        label = "Adjust plot width",
        value = 550,
        min = 250,
        max = 1600,
        round = TRUE
      ),
      shinyWidgets::switchInput(
        inputId = ns("sortbyfreq"),
        label = "Sort by frequency",
        size = "mini",
        value = FALSE,
        labelWidth = "100px"
      ),
      shinyWidgets::switchInput(
        inputId = ns("sortbypheno"),
        label = "Sort by phenotype",
        size = "mini",
        value = FALSE,
        labelWidth = "100px"
      ),
      shinyjs::hidden(selectInput(
        ns("sort_pheno"),
        label = "Sort By",
        choices = isolate(facetOptions()),
        multiple = FALSE,
        selectize = FALSE
      )),
      shinyjs::hidden(selectizeInput(
        ns("pheno_values"),
        label = "in order of",
        choices = "",
        options = list(placeholder = "Select levels"),
        selected = "",
        multiple = TRUE
      )),
      shinyjs::hidden(actionButton(
        ns("changeFactorOrder"), 
        label = "reorder", 
        width = "100px")),
      circle = FALSE, status = "danger", icon = icon("gear"), 
      width = "315px",
      label = "Plot Options"
    )
  })
  
  output$sampleotus <- plotly::renderPlotly({
    req(plotData())
    
    if(nrow(plotData()) > 0){
      microbiomeExplorer::plotlySampleBarplot(
        plotData(),
        col_by = input$bar_color,
        pwidth = 600,
        sortbyfreq = input$sortbyfreq,
        pheno_sort = sortPheno(),
        x_levels = phenoValues())
    }
  })
  
  observe({
    if(isTRUE(input$sortbyfreq)){
      shinyjs::hide("sort_pheno")
      shinyjs::hide("pheno_values")
      shinyjs::hide("changeFactorOrder")
      shinyWidgets::updateSwitchInput(session, "sortbypheno", value = FALSE)
    }
    else{
      if(isTRUE(input$sortbypheno)){
        shinyjs::show("sort_pheno")
        shinyjs::show("pheno_values")
        shinyjs::show("changeFactorOrder")
      } else {
        shinyjs::hide("sort_pheno")
        shinyjs::hide("pheno_values")
        shinyjs::hide("changeFactorOrder")
      }
    }
  })
  
  observeEvent(input$sort_pheno,{
    updateSelectizeInput(session,"pheno_values", 
                         choices = unique(plotData()[[input$sort_pheno]]))
  }, ignoreNULL = TRUE)
  
  observeEvent(input$changeFactorOrder,{
    sortPheno('if'(length(input$sort_pheno) > 0, input$sort_pheno,NULL))
    phenoValues('if'(length(input$pheno_values) > 0, input$pheno_values,NULL))
  })
  
  observeEvent(input$sortbypheno,{
    if(isFALSE(input$sortbypheno)){
      sortPheno(NULL)
      phenoValues(NULL)
    } else {
      shinyWidgets::updateSwitchInput(session, "sortbyfreq", value = FALSE)
    }
  })
  
  ## download modified data as RDS
  output$savebutton <- downloadHandler(
    filename = function() {
      fext <- tools::file_ext(meName())
      if(fext == ""){
        paste0(meName(),".rds")
      } else {
        gsub(fext, "rds", meName())
      }
    },
    content = function(file) {
      saveRDS(meData(), file)
    }
  )
  
  adonisOptions <- reactive({
    fC <- filterChoices()
    fC[!(names(fC) %in% "SAMPLE_ID")]
  })
  
  # current data after filtering is returned and passed to other tabs
  return(list(meData = meData, 
              facetOptions = facetOptions, 
              shapeOptions = shapeOptions,
              adonisOptions = adonisOptions,
              qcCode = qcCode, 
              dataName = meName))
}

### NON-PLOTTING, DATA-PROCESSING AND HELPER FUNCTIONS


#' Main function to start the Microbiome Explorer Shiny app via a command line
#' call
#'
#' @return the shiny application
#' 
#'
#' @export
runMicrobiomeExplorer <- function(){
  shiny::runApp(appDir = system.file("shiny", package = "microbiomeExplorer"))
}


## DATA PROCESSING ############################################################

#' Add phenotype data to object.
#'
#' This function adds phenotype data to the phenoData slot in an MRexperiment
#' object.
#'
#' @param MRobj An MRexperiment object.
#' @param phenodata Phenotype data frame or file path.
#'
#' @return An updated MRexperiment object.
#'
#' @importFrom metagenomeSeq loadPhenoData MRcounts
#' @importFrom Biobase phenoData<- pData<-
#'
#' @export
addPhenoData <- function(MRobj, phenodata = NULL) {
  if (is.null(phenodata)) {
    return(MRobj)
  }
  if (is.character(phenodata)) {
    phenodata <- utils::read.delim(phenodata, 
                               sep = 'if'(
                                 tools::file_ext(phenodata) == "csv", 
                                 ",", "\t"),
                               row.names = 1)
  }
  ord <- match(colnames(MRcounts(MRobj)), rownames(phenodata))
  if (any(is.na(ord))) {
    return(NULL)
  }
  phenodata <- phenodata[ord, ,drop = FALSE]
  phenoData(MRobj) <- Biobase::AnnotatedDataFrame(phenodata)
  if (!"SAMPLE_ID" %in% names(pData(MRobj))) {
    pData(MRobj) <- data.frame(SAMPLE_ID = rownames(pData(MRobj)),
                               pData(MRobj))
  }
  return(MRobj)
}

#' Extends existing phenodata for an object
#'
#' This function adds phenotype data to the phenoData slot in an MRexperiment
#' object.
#'
#' @param MRobj An MRexperiment object.
#' @param phenodata Phenotype data frame or file path.
#'
#' @return An updated MRexperiment object.
#'
#' @importFrom metagenomeSeq loadPhenoData MRcounts
#' @importFrom Biobase phenoData<-
#'
#' @export
extendPhenoData <- function(MRobj, phenodata = NULL) {
  if (is.null(phenodata)) {
    return(MRobj)
  }
  if (is.character(phenodata)) {
    phenodata <- as.data.frame(
      utils::read.delim(phenodata, 
                    sep = 'if'(
                      tools::file_ext(phenodata) == "csv", 
                      ",", "\t"),
                    row.names = 1))
  }
  if (!"SAMPLE_ID" %in% names(phenodata)) {
    phenodata <- data.frame(SAMPLE_ID = rownames(phenodata),
                            phenodata)
  }
  phenodata <- dplyr::left_join(pData(MRobj),phenodata)
  rownames(phenodata) <- phenodata$SAMPLE_ID
  phenoData(MRobj) <- Biobase::AnnotatedDataFrame(phenodata)
  
  return(MRobj)
}


#' Add feature data to MRobj.
#'
#' This function adds feature data to the featureData slot in an MRexperiment
#' object.
#'
#' @param MRobj An MRexperiment object.
#' @param featdata Feature data frame or file path.
#'
#' @return An updated MRexperiment object.
#'
#' @import metagenomeSeq 
#' @importFrom Biobase featureData<-
#'
#' @export
addFeatData <- function(MRobj, featdata = NULL) {
  if (is.null(featdata)) {
    return(MRobj)
  }
  if (is.character(featdata)) {
    featdata <- utils::read.delim(featdata,
                           sep = 'if'(
                             tools::file_ext(featdata) == "csv", ",", "\t"), 
                           row.names = 1,
                           stringsAsFactors = FALSE
    )
  }
  ord <- match(rownames(MRcounts(MRobj)), rownames(featdata))
  if (sum(is.na(ord)) > 0) {
    return(NULL)
  }
  featdata <- featdata[ord, ]
  featureData(MRobj) <- Biobase::AnnotatedDataFrame(featdata)
  return(MRobj)
}


#' Reads in data
#'
#' This function reads in an MRexperiment object saved as an RDS file, a Biom
#' file, or a tab - delimited count matrix with features as rows
#' and samples as columns.
#'
#' @param filepath Relative or absolute file path of data object.
#' @param type The type of file to be read; default is "RDS", other options are
#'  "RDATA", "BIOM", "TAB", "CSV"
#'
#' @return An MRexperiment object.
#'
#' @importFrom metagenomeSeq biom2MRexperiment loadMeta newMRexperiment
#' @importFrom Biobase pData<- fData<-
#' @importFrom methods is
#'
#' @export
readData <- function(filepath, type = "RDS") {
  if (type == "RDS") {
    bufobj <- readr::read_rds(filepath)
    MRobj <- newMRexperiment(counts = MRcounts(bufobj))
    pData(MRobj) <- pData(bufobj)
    fData(MRobj) <- fData(bufobj)
    if (!is.null(shiny::getDefaultReactiveDomain())) {
      incProgress(0.9)
    }
  } else if (type == "RDATA") {
    bufobj <- get(load(filepath))
    MRobj <- newMRexperiment(counts = MRcounts(bufobj))
    pData(MRobj) <- pData(bufobj)
    fData(MRobj) <- fData(bufobj)
    if (!is.null(shiny::getDefaultReactiveDomain())) {
      incProgress(0.9)
    }
  } else if (type == "BIOM") {
    if (!is.null(shiny::getDefaultReactiveDomain())) {
      incProgress(message = "Reading biom file", 
                  detail = "\nConversion in progress ...")
    }
    Bobj <- biomformat::read_biom(filepath)
    if (!is.null(shiny::getDefaultReactiveDomain())) {
      incProgress(0.3)
    }
    MRobj <- biom2MRexperiment(Bobj)
    if (!is.null(shiny::getDefaultReactiveDomain())) {
      incProgress(0.3)
    }
    featdata <- biomformat::observation_metadata(Bobj)
    if (!is.null(shiny::getDefaultReactiveDomain())) {
      incProgress(0.3)
    }
  } else if (type == "TAB") {
    countdata <- loadMeta(filepath)
    MRobj <- newMRexperiment(countdata$counts)
  } else if (type == "CSV") {
    countdata <- loadMeta(filepath, sep = ",")
    MRobj <- newMRexperiment(countdata$counts)
  } else {
    return(NULL)
  }
  if(!is(MRobj, "MRexperiment")){
    return(NULL)
  } 
  ## add required columns if they aren't available yet
  if (!"SAMPLE_ID" %in% names(pData(MRobj))) {
    pData(MRobj) <- data.frame(SAMPLE_ID = rownames(pData(MRobj)),
                               pData(MRobj))
  }
  
  if(!is.null(pData(MRobj))){
    pData(MRobj) <- pData(MRobj) %>%
      tibble::rownames_to_column("samname") %>%
      dplyr::mutate_if(is.character, as.factor) %>%
      tibble::column_to_rownames("samname")
  }
  
  if(!is.null(fData(MRobj))){
    fData(MRobj) <- fData(MRobj) %>%
      tibble::rownames_to_column("samname") %>%
      dplyr::mutate_if(is.character, as.factor) %>%
      tibble::column_to_rownames("samname")
  }
  
  return(MRobj)
}

#' Helper function assigning different file extensions to specific short texts 
#' identifying the types
#'
#' @param fileext the file extension found after '.'
#'
#' @author Janina Reeder
#'
#' @return character string for the filetype
getFileType <- function(fileext) {
  switch(tolower(fileext),
         "rds" = "RDS",
         "rda" = "RDATA",
         "rdata" = "RDATA",
         "tsv" = "TAB",
         "txt" = "TAB",
         "csv" = "CSV",
         "csv2" = "CSV",
         "biom" = "BIOM",
         "biom2" = "BIOM"
  )
}



#' Function to filter the MRexperiment data by numerical parameters
#'
#' @param MRobj MRExperiment object to filter
#' @param minpresence minimum sample presence per feature
#' @param minfeats minimum number of features per sample
#' @param minreads minimum number of reads per sample
#'
#' @author Janina Reeder
#'
#' @return the filtered MRobj
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' filterMEData(MRobj = mouseData, minpresence = 4, minfeats = 300)
#' 
#' @export
filterMEData <- function(MRobj, minpresence = 1, 
                         minfeats = 2, minreads = 2) {
  filteredObj <- MRobj
  while(TRUE){
    if(nrow(filteredObj) == 0 || ncol(filteredObj) == 0) 
      break
    rowsToRemove <- rowSums(MRcounts(filteredObj) > 0) < minpresence
    colsToRemove <- colSums(MRcounts(filteredObj) > 0) < minfeats |
      colSums(MRcounts(filteredObj)) < minreads
    if(any(rowsToRemove) || any(colsToRemove)){
        filteredObj <- filteredObj[!rowsToRemove,!colsToRemove]
    } else {
      break
    }
  }

  return(filteredObj)
}



#' Function to filter the MRexperiment by certain phenotype values
#'
#' @param MRobj the MRexperiment to subset
#' @param rm_phenovalues list of named vectors with names corresponding to 
#' column names in pData and values representing phenotypes within the column
#'
#' @author Janina Reeder
#'
#' @return the filtered MRobj
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' filterByPheno(MRobj = mouseData, 
#'   rm_phenovalues = list("diet" = c("BK"),"mouseID" = c("PM1","PM10")))
#' 
#' @export
filterByPheno <- function(MRobj, rm_phenovalues) {
  samstoremove <- rowSums(
    dplyr::bind_cols(lapply(names(rm_phenovalues), 
                            function(r) {
                              naind <- which(rm_phenovalues[[r]] == "NA")
                              rm_phenovalues[[r]][naind] <- NA
                              pData(MRobj)[, r] %in% rm_phenovalues[[r]]
                            })
    )
  )
  return(MRobj[, !samstoremove])
}


## PHENOTABLE 


#' Helper function used to build a correct interactionName based on the chosen 
#' columns
#'
#' @param interactionName as chosen by user. This may not be good to store 
#' internally
#'
#' @return updated interactionName or warning/error string
parseInteractionName <- function(interactionName) {
  if (is.null(interactionName) | interactionName == "") {
    return("ERROR: No interaction name specified")
  }
  if (interactionName != make.names(interactionName)) {
    return(paste0("WARNING: ", 
                  gsub("\\.+", "_", make.names(interactionName))))
  }
  return(interactionName)
}


#' Makes header for R script
#'
#' This function makes the header for the report R script to be rendered by
#' knitr into Rmarkdown and rendered into HTML, PDF, or Word.
#'
#' This was adapted from https://yihui.name/knitr/demo/stitch/
#'
#' @param title Title of the report.
#' @param author Author of the report.
#' @param date Date of the report.
#' @param data.source R code used to obtain the dataset
#' @param output Output of Rmarkdown file. 
#' @param toc Table of contents. Default is TRUE.
#'
#' @return A character vector where each element is a line in the R script.
createHeader <- function(title = "MicrobiomeExplorer Report", 
                         author = "",
                         date = "",
                         data.source = "",
                         output = getOption("me.reportformat"),
                         toc = TRUE) {
  cat(file = stderr(), "-- creating header --\n")
  
  if (is.null(output)) {
    output <- "html_document"
  }
  # output <- match.arg(output)
  output <- c(output, "md_document")
  
  cat(file = stderr(), "-- setting toc ---")
  if (toc) {
    output2 <- unlist(lapply(output, function(o){
      paramlist <- c(paste0(o, ":"),
                     "  toc: true")
      if(o %in% "powerpoint_presentation"){
        paramlist <- c(paramlist,"  slide_level: 3")
      }
      return (paramlist)
    }))
  } else {
    output2 <- paste0(output, ": default")
  }
  
  cat(file = stderr(), "-- data processing ---")
  
  ep_project <- 'if'(stringr::str_starts(data.source, "proj"), TRUE, FALSE)
  
  header <- c(
    "---",
    paste0("title: ", title),
    paste0("author: ", author),
    paste0("date: ", date),
    paste0("output:"),
    paste0("  ", output2),
    paste0("always_allow_html: true"),
    paste0("font-size: 11pt"),
    "---"
  )
  
  header <- paste0("#' ", header)
  header <- c(
    header, "", "#+ setup, include=FALSE",
    "knitr::opts_chunk$set(echo=FALSE, message=FALSE,
              warning=FALSE, cache=FALSE)"
  )
  header <- c(
    header, "#'", "",
    "#+ load_package, cache=FALSE",
    "library(knitr)",
    "library(microbiomeExplorer)",
    "library(magrittr)",
    "library(metagenomeSeq)",
    "library(shiny)",
    'if'(ep_project, "library(DataSetDB)", ""),
    "#'", "",
    "",
    "",
    "#' This report has been generated from the Microbiome Explorer app",
    "#' Data is stored externally and must be available to render", 
    "#' the Rmarkdown document\n\n",
    "#'", "",
    "doctype <- opts_knit$get(\"rmarkdown.pandoc.to\")",
    "options(me.round_digits = 3)",
    "options(me.modebar = list(list(\"toImage\", \"zoom2d\", \"pan2d\",", 
    "\t\"select2d\", \"lasso2d\", \"resetScale2d\")))"
  )
  header
}

#' Generates report
#'
#' This function generates the pieces of the report, which includes the R
#' script, Rmarkdown file, and any Rmarkdown outputs.
#'
#' Adapted from https://yihui.name/knitr/demo/stitch/
#'
#' @param rcode A named list where each element corresponds to a different
#' analysis (Alpha diversity, Beta diversity). The name of the list is used to
#' denote the first part of the code chunks in each analysis section
#' (alpha, beta). Each element is itself a list of R commands corresponding to
#' a code chunk. 
#' @param filename Name of output files. Default is "report".
#' @param dir Directory of output. Default is "out".
#' @param title Title of the report.
#' @param author Author of the report.
#' @param date Date of the report.
#' @param data.source R code used to obtain the dataset
#' @param output Output of Rmarkdown file. Options defined in global.R
#' @param toc Table of contents. Default is TRUE.
#' @param intro_text Introductory text to include with the report (optional)
#'
#' @return A character vector where each element is a line in the R script.
generateReport <- function(rcode,
                           filename = "report",
                           dir = "out",
                           title = "MicrobiomeExplorer Report",
                           author = "",
                           date = "`r format(Sys.time(), '%d %B, %Y')`",
                           data.source = "",
                           output = c("html_document"),
                           toc = TRUE,
                           intro_text = NULL) {
  cat(file = stderr(), "-- entering my report generator --\n")
  
  ## Generate body code and combine with header
  header <- createHeader(title, author, date, data.source, 
                         output, toc)
  cat(file = stderr(), "-- header created! --\n")
  
  if (!is.null(intro_text) && length(intro_text) > 0) {
    intro_text <- paste("#' ## Introduction\n", intro_text, "\n\n  ")
    intro_text <- vapply(intro_text, stringr::str_replace_all,
                         pattern = "\n", replacement = "\n#' ", 
                         FUN.VALUE = character(1))
  } else {
    intro_text <- NULL
  }
  
  cat(file = stderr(), "-- build intro text --\n")
  
  ep_project <- 'if'(stringr::str_starts(data.source, "proj"), TRUE, FALSE)
  dataloading <- c("#' ## Data loading and preparation",
                   data.source)
  
  report <- c(header, intro_text, dataloading, 
              unlist(rcode, use.names = FALSE))
  
  cat(file = stderr(), "-- merged code --\n")
  
  ## handle files, processing and output
  rscript <- paste0(dir, "/", filename, ".R")
  cat(report, sep = "\n", file = rscript)
  knitr::spin(rscript, knit = FALSE)
  rmarkdown::render(rscript, output_format = output)
  out.ext <- c("Rmd", 
               gsub("powerpoint", "pptx", 
                    gsub("word", "docx", 
                         vapply(strsplit(output, split = "_"), `[[`, 1,
                                FUN.VALUE = character(1)))))
  print(c(rscript, paste(paste(dir, filename, sep = "/"), 
                         out.ext, sep = ".")))
}


## HELPER FUNCTIONS

#' Sets up a dataframe used by several plotting functions by joining
#' the required data with relevant phenotype data
#'
#' @param df dataframe storing plotting data values
#' @param phenoTable pData of the MRexperiment; all following parameters must be 
#' a column of the phenoTable
#' @param x_var main plotting variable
#' @param facet1 column-based faceting (can be NULL)
#' @param facet2 row-based faceting (can be NULL)
#' @param col_by coloring factor (can be NULL)
#' @param col_name character to be used as name for col_by
#' @param id_var variable used to connect samples longitudinally (can be NULL)
#'
#' @return dataframe obtained by joining df and relevant columns of phenoTable
buildPlottingDF <- function(df, phenoTable, x_var = NULL, 
                            facet1 = NULL, facet2 = NULL, 
                            col_by = NULL, col_name = col_by,
                            id_var = NULL){
  sample_data <- data.frame(
    samname = rownames(phenoTable),
    X1 = phenoTable[, x_var],
    X2 = phenoTable[, facet1],
    X3 = phenoTable[, facet2],
    X4 = phenoTable[, col_by],
    X5 = phenoTable[, id_var]
  )
  colnames(sample_data) <- c("samname", x_var, facet1, facet2, col_name, id_var)
  sample_data[] <- lapply(sample_data, function(s) gsub("^$","NA",s))
  sample_data[] <- lapply(sample_data, factor, exclude = NULL)
  sample_data[] <- lapply(sample_data, 
                          forcats::fct_explicit_na, na_level = "NA")
  suppressMessages(dplyr::left_join(df, sample_data))
}




#' Helper function to account for issues plotly has with very small widths
#' (these end up being 1 and cover the entire plotting area)
#'
#' @param df2 dataframe storing plotting data
#' @param facets column facets
#' @param x_var x variable
#' @param drop passed on as .drop to dplyr::group_by
#'
#' @return widths for each facet
getWidths <- function(df2, facets, x_var, drop = TRUE){
  numinfacet <- df2 %>%
    dplyr::group_by_at(c(facets, x_var), .drop = drop)  %>%
    dplyr::tally() %>%
    dplyr::group_by_at(facets) %>%
    dplyr::summarise(maxnum = length(x_var))
  totalwidth <- sum(numinfacet$maxnum)
  widths <- numinfacet$maxnum/totalwidth
  widths <- ifelse(widths < 0.025, 0.025, widths)
  if(sum(widths) > 1){
    adj_ind <- which(widths > 0.025)
    diff <- (sum(widths) - 1) / length(adj_ind)
    widths[adj_ind] <- widths[adj_ind] - diff
  }
  widths
}

#' Function to find a non-empty facet in the last row. This will be the
#' one to be connected to the plot legend to avoid duplicates within
#'
#' @param df2 plotting data frame
#' @param facets column facets
#' @param facet2s row facets
#'
#' @return the name of the column-based facet which can be used as legend
getLegendLevel <- function(df2, facets, facet2s){
  legendlevel <- "nofacets"
  grouptally <- df2 %>%
    dplyr::group_by_at(c(facets, facet2s)) %>% 
    dplyr::tally() 
  if(nrow(grouptally) > 0)
    legendlevel <- grouptally[nrow(grouptally),"facets"]
  legendlevel
}


#' Creates an empty plotly plot using the given labels on the x and y axis
#'
#' @param xaxis_text x axis label
#' @param ylab y axis label
#'
#' @return call to plotly_empty
buildEmptyPlotlyPlot <- function(xaxis_text, ylab){
  plotly::plotly_empty(type = "scatter", 
                       mode = "markers") %>%
    plotly::layout(
      xaxis = list(
        title = list(
          text = xaxis_text,
          font = list(
            size = 12
          )
        ),
        showgrid = FALSE,
        visible = TRUE,
        tickfont = list(
          size = 9,
          color = "#5b5b5b"
        )
      ),
      yaxis = list(
        title = list(
          text = ylab,
          font = list(
            size = 12
          )
        ),
        visible = TRUE,
        showgrid = TRUE,
        showticklabels = TRUE,
        tickfont = list(
          size = 9,
          color = "#5b5b5b"
        )
      )
    )
}

#' Adds a layout call based on plotly::layout
#'
#' @param plotTitle plot title to use
#' @param xaxis_text x axis label to use
#' @param ylab y axis label to use
#' @param .data plotly data object to apply the layout call to
#'
#' @return plotly::layout call
add_plotly_layout <- function(.data, plotTitle, xaxis_text, ylab){
  plotly::layout(.data,
    #barmode = "stack",
    title = list(
      text = plotTitle,
      font = list(
        size = 14
      )
    ),
    xaxis = list(
      title = list(
        text = xaxis_text,
        font = list(
          size = 12
        )
      ),
      showgrid = FALSE,
      visible = TRUE,
      tickangle = 45,
      tickfont = list(
        size = 9,
        color = "#5b5b5b"
      )
    ),
    yaxis = list(
      title = list(
        text = ylab,
        font = list(
          size = 12
        )
      ),
      visible = TRUE,
      showgrid = TRUE,
      showticklabels = TRUE,
      tickfont = list(
        size = 9,
        color = "#5b5b5b"
      )
    ),
    margin = list(
      b = 150,r = 50, t = 25, l = 50
    ),
    legend = list(
      font = list(
        family = "sans-serif",
        size = 11,
        color = "#000"
      ),
      bordercolor = "#E2E2E2",
      borderwidth = 2
    )
  )
}

#' Adds a config call based on plotly::config
#'
#' @param .data plotly data object to apply the config call to
#'
#' @return plotly::config call
add_plotly_config <- function(.data){
  plotly::config(.data,
    toImageButtonOptions = list(
      format = "svg"
    ),
    displaylogo = FALSE,
    modeBarButtons = list(list("toImage", "zoom2d", "pan2d", 
                               "select2d", "lasso2d", "resetScale2d"))
  )
}

data("mouseData")
pData(mouseData) <- data.frame(SAMPLE_ID = rownames(pData(mouseData)),
                           pData(mouseData))
md_counts <- system.file("extdata", "md_counts.tsv", package = "microbiomeExplorer")
md_feats <- system.file("extdata", "md_feats.tsv", package = "microbiomeExplorer")
md_pheno <- system.file("extdata", "md_pheno.tsv", package = "microbiomeExplorer")
pheno_add <- system.file("extdata", "add_pheno.tsv", package = "microbiomeExplorer")
mr_base <- readData(md_counts, type = "TAB")
df <- reshape2::melt(MRcounts(mouseData), varnames = c("OTU", "samname"), 
                     value.name = "Reads")

test_that("adding pheno data", {
  mr_pheno <- addPhenoData(mouseData,pheno_add)
  expect_equal(ncol(pData(mr_pheno)),2)
  expect_equal(nrow(pData(mr_pheno)),139)
  expect_equal(names(pData(mr_pheno)),c("SAMPLE_ID","rand_num"))
})

test_that("extending pheno data", {
  mr_pheno <- extendPhenoData(mouseData,pheno_add)
  expect_equal(ncol(pData(mr_pheno)),7)
  expect_equal(nrow(pData(mr_pheno)),139)
  expect_equal(names(pData(mr_pheno)),c("SAMPLE_ID","mouseID", "date", "diet", "relativeTime", "status", "rand_num"))
})

test_that("reading data", {
  expect_equal(ncol(pData(mr_base)),1)
  expect_equal(nrow(pData(mr_base)),139)
  expect_equal(names(pData(mr_base)),"SAMPLE_ID")
  expect_equal(ncol(fData(mr_base)),0)
  expect_equal(nrow(fData(mr_base)),10172)
  expect_equal(sum(MRcounts(mr_base)),315593)
  expect_equal(row.names(MRcounts(mr_base))[1:5],c("Prevotellaceae:1", "Lachnospiraceae:1", "Unclassified-Screened:1",
                                                   "Clostridiales:1", "Clostridiales:2"))
})

test_that("adding feat data", {
  mr_feats <- addFeatData(mr_base, featdata = md_feats)
  expect_equal(ncol(pData(mr_feats)),1)
  expect_equal(nrow(pData(mr_feats)),139)
  expect_equal(names(pData(mr_feats)),"SAMPLE_ID")
  expect_equal(ncol(fData(mr_feats)),7)
  expect_equal(nrow(fData(mr_feats)),10172)
  expect_equal(sum(MRcounts(mr_feats)),315593)
  expect_equal(names(fData(mr_feats)), c("superkingdom", "phylum", "class", "order", "family", "genus", "OTU" ))
  expect_equal(row.names(fData(mr_feats))[1:5],c("Prevotellaceae:1", "Lachnospiraceae:1", "Unclassified-Screened:1",
                                                   "Clostridiales:1", "Clostridiales:2"))
})

test_that("filtering min presence", {
  expect_equal(dim(filterMEData(mouseData, minpresence = 4))[[1]], 2248)
  expect_equal(dim(filterMEData(mouseData, minpresence = 10))[[1]], 1063)
  expect_equal(dim(filterMEData(mouseData, minpresence = 10))[[2]], 139)
})

test_that("filtering min feats", {
  expect_equal(dim(filterMEData(mouseData, minfeats = 500))[[1]],3309)
  expect_equal(dim(filterMEData(mouseData, minfeats = 500))[[2]],13)
})

test_that("filtering min reads", {
  expect_equal(dim(filterMEData(mouseData, minreads = 1500))[[1]],9628)
  expect_equal(dim(filterMEData(mouseData, minreads = 1500))[[2]],129)
})

test_that("filtering combined", {
  test <- filterMEData(mouseData, minpresence = 2, minfeats = 200, minreads = 1500)
  expect_false(any(rowSums(MRcounts(test) > 0) < 2))
  expect_false(any(colSums(MRcounts(test) > 0) < 200))
  expect_false(any(colSums(MRcounts(test)) < 1500))
  expect_equal(dim(test)[[1]], 4031)
  expect_equal(dim(test)[[2]], 127)
})

test_that("filter by pheno", {
  expect_equal(dim(filterByPheno(mouseData, rm_phenovalues = list("diet" = "Western")))[[1]], 10172)
  expect_equal(dim(filterByPheno(mouseData, rm_phenovalues = list("diet" = "Western")))[[2]], 85)
})

## Main module for long sample analysis

#'Long Analysis Module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the ui code
#' 
#' @examples longAnalysisUI("longanalysis_id")
#' 
#' @export
longAnalysisUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    width = 12,
    column(
      width = 3,
      longInputUI(ns("longInput"))
    ),
    column(
      width = 9,
      fluidRow(
        width = 11,
        box(
          width = 10,
          p("Longitudinal analysis provides a module to compare the microbial 
            composition across time points or conditions (e.g. tissues).
            The order of the phenotype levels is preserved in the visualization.
            Use the optional phenotype ID parameter to add connections 
            between IDs over all selected time points/conditions.
            Individual lines can be highlighted via mouse clicks or
            the input area with option to choose colors. Select multiple
            connections by holding the \"Shift\" key. Double clicks
            near the edge of the plot remove selections.")
        ),
        longResultsUI(ns("longResults"))
      )
    )
  )
}


#' long Analysis Module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param data the main data object returned from data_input_module
#' @param levelOpts available levels to aggregate on (depends on input data)
#' @param chosenLevel previously selected level (passed from longerent instance)
#' @param resetInput reactive boolean determining if reset is required
#' @param aggData the aggregated MRExperiment object
#' @param normalizedData boolean indicating if normalization was done
#' 
#' @author Janina Reeder
#' 
#'
#' @return reactive holding code to be used in reports
longAnalysis <- function(input, output, session, data, levelOpts,
                         chosenLevel, resetInput, aggData, normalizedData) {
  ns <- session$ns
  
  longSettings <- callModule(longInput,
                             "longInput",
                             meData = data$meData,
                             facetOptions = data$facetOptions,
                             reset = resetInput,
                             aggDat = reactive(aggData$mrobj)
  )
  
  longResultsRep <- callModule(longResults,
                               "longResults",
                               aggDat = reactive(aggData$mrobj),
                               featLevel = chosenLevel,
                               longSettings = longSettings,
                               normalizedData = normalizedData,
                               reset = resetInput
  )
  
  return(longResultsRep)
  
}

#' Differential Analysis module UI
#'
#' @param id namespace identifier
#' 
#' @author Janina Reeder
#' 
#'
#' @return row containing the UI elements
diffTableUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    box(
      title = "DIFFERENTIAL ANALYSIS", solidHeader = TRUE, 
      collapsible = TRUE, width = 12,
      fluidRow(
        column(width = 1),
        column(
          width = 10,
          div(
            id = ns("downloaddiv"),
            shinyjs::hidden(
              downloadButton(
                ns("download_button"),
                "Download")),
            DT::DTOutput(
              ns("diffdatatable"), width = "100%")
          )
        )
      )
    ),
    box(width = 12,
        fluidRow(
          column(width = 1),
          column(
            width = 10,
            plotly::plotlyOutput(
              ns("clickedFeature"), 
              width = "100%", height = "100%"),
            br(),
            shinyjs::disabled(
              shinyWidgets::dropdownButton(
                tags$h3("Plot Options"),
                shinyWidgets::switchInput(
                  inputId = ns("sP"),
                  label = "Show Points",
                  size = "mini",
                  labelWidth = "80px"
                ),
                shinyWidgets::switchInput(
                  inputId = ns("logS"),
                  label = "Log Scale",
                  value = TRUE,
                  size = "mini",
                  labelWidth = "80px"
                ),
                circle = FALSE, 
                status = "danger", 
                icon = icon("gear"), 
                width = "300px",
                label = "Plot Options",
                inputId = ns("optionbutton")
              ))
          )
        )
    )
  )
}

#' Differential analysis module server code
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param diffSettings reactive storing values selected in analysis 
#' input interface
#' @param reset boolean reactive which resets the module if TRUE
#' @param normalized boolean reactive indicating if data has been normalized
#' 
#' @author Janina Reeder
#' 
#' @import metagenomeSeq
#' 
#' @return list containing R code for analysis and for feature plots
diffTable <- function(input, output, session, 
                      aggDat, featLevel, diffSettings, 
                      reset, normalized) {
  ns <- session$ns
  
  ## reactive storing analysis code for reports
  repCode <- reactiveVal(NULL)
  ## reactive storing feature plot code for reports
  featListRep <- reactiveVal(NULL)
  normalizeCode <- reactiveVal(NULL)
  diffResults <- reactiveVal(NULL)
  plotHeight <- reactiveVal(350)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    diffResults(NULL)
  })
  
  diffAggData <- reactive({
    req(aggDat(), featLevel())
    # FORCE NORMALIZATION IF NOT PERFORMED YET
    if (isFALSE(normalized())) {
      diffAggDat <- aggDat()
      ## need trycatch for only one feature level remaining
      p <- tryCatch({
        cumNormStatFast(diffAggDat)
      },
      error = function(e){
        return(0.5)
      })
      if(is.na(sum(normFactors(aggDat()))))
        diffAggDat <- cumNorm(diffAggDat, p)
      normalizeCode(paste0(c("diffAggDat <- aggDat",
                             "p <- tryCatch({",
                             "\tcumNormStatFast(diffAggDat)",
                             "},",
                             "error = function(e){",
                             "\treturn(0.5)",
                             "})",
                             "if(is.na(sum(normFactors(diffAggDat))))",
                             "\tdiffAggDat <- cumNorm(diffAggDat, p)",
                             "diffResults <- runDiffTest(diffAggDat,"),
                           collapse = "\n"))
      diffAggDat
    } else {
      normalizeCode("diffResults <- runDiffTest(aggDat,")
      aggDat()
    }
  })
  
  # performs the differential analysis based on given data, level and settings
  observe({
    req(diffAggData(), featLevel(), 
        diffSettings()$comparison, 
        diffSettings()$method,
        diffSettings()$phenolevel1,
        diffSettings()$phenolevel2)
    
    if(diffSettings()$method == "DESeq2"){
      if(make.names(diffSettings()$phenolevel1) ==
         make.names(diffSettings()$phenolevel2)){
        showModal(modalDialog(
          title = "DeSeq2 Issue",
          "DeSeq2 uses make.names() on the comparison phenotypes.\n 
           The chosen values lead to non-unique values. \n
           Please update the phenotable for this project in order to run DeSeq2 
           differential analysis using the selected comparison levels.",
          easyClose = TRUE
        ))
        return(NULL)
      }
    }
    
    tableCaption <- paste0(diffSettings()$method,
                           " comparison of ",
                           diffSettings()$comparison,
                           ": ",
                           diffSettings()$phenolevel1,
                           " vs ",
                           diffSettings()$phenolevel2)
    
    repCode(paste(
      paste0("#' ### Differential analysis"),
      paste0("#' #### ", diffSettings()$method,": ",
             diffSettings()$phenolevel1, " vs ",
             diffSettings()$phenolevel2),
      paste0("", "#- fig.width = 8"),
      paste0(normalizeCode()),
      paste0("\tlevel = \"", featLevel(), "\","),
      paste0("\tphenotype = \"", 
             diffSettings()$comparison, "\","),
      paste0(
        "\tphenolevels = c(\"",
        diffSettings()$phenolevel1,
        "\", \"",
        diffSettings()$phenolevel2, "\"),"
      ),
      paste0("\tmethod = \"", 
             diffSettings()$method, "\")"),
      paste0("\n"),
      paste0("if(doctype == \"html\"){"),
      paste0("\tDT::datatable(data = diffResults, 
                   class = \"stripe hover cell-border order-column\","),
      paste0("\t\tfilter = \"none\", style = \"bootstrap\","),
      paste0("\t\tcaption = \"",tableCaption,"\","),
      paste0("\t\toptions = list(scrollX = TRUE,paging = TRUE, 
                   digits = 4,dom = \"<Bftp>\"),"),
      paste0("\t\trownames = FALSE)"),
      paste0("} else {"),
      paste0("\tprint(\"data too large to display in pdf\")"),
      paste0("}"),
      sep = "\n"
    ))
    
    withProgress({
      diffResults(runDiffTest(
        aggdat = diffAggData(),
        level = featLevel(),
        phenotype = diffSettings()$comparison,
        phenolevels = c(diffSettings()$phenolevel1, 
                        diffSettings()$phenolevel2),
        method = diffSettings()$method
      ))
    }, message = "Calculating ...")
  })
  
  ## render the table showing the results; this needs to be client side, 
  ## so sorting doesn't remove selections
  output$diffdatatable <- DT::renderDT({
    req(diffResults())
    shinyjs::js$moveButton(ns("downloaddiv"),ns("download_button"))
    tableCaption <- isolate(paste0(diffSettings()$method,
                                   " comparison of ",
                                   diffSettings()$comparison,
                                   ": ",
                                   diffSettings()$phenolevel1,
                                   " vs ",
                                   diffSettings()$phenolevel2))
    shinyjs::show("download_button")
    DT::datatable(
      data = diffResults(), 
      class = "stripe hover cell-border order-column",
      filter = "none", style = "bootstrap",
      callback = DT::JS("$('div.dwnld_diff').append($('#diffAnalysis-differentialTable-download_button'));"),
      caption = tableCaption,
      extensions = c("ColReorder", "Buttons", "Select"),
      options = list(
        scrollX = TRUE,
        paging = TRUE,
        colReorder = TRUE,
        stateSave = TRUE,
        stateLoadParams = DT::JS("function (settings, data) 
                                         {return false;}"),
        digits = 4,
        columnDefs = list(list(className = "select-checkbox", 
                               targets = 0)),
        select = list(style = "multi", selector = "tr"),
        buttons = list(
          list(
            extend = "colvis",
            text = "Select columns"
          )),
        dom = '<"dwnld_diff"B>ftlp'
      ),
      rownames = "",
      selection = "none")
  }, server = FALSE)
  
  output$download_button <- downloadHandler(
    filename = function() {
      gsub(" ","",paste0(diffSettings()$phenolevel1,
                         "_vs_",
                         diffSettings()$phenolevel2, "_", Sys.Date(), ".csv"))
    },
    content = function(file) {
      readr::write_csv(diffResults(), file)
    }
  )
  
  ## add/remove points in feature plots
  observeEvent(input$sP, {
    req(input$diffdatatable_rows_selected, diffResults(), aggDat(), 
        featLevel(), diffSettings())
    featListRep(vapply(featListRep(), 
                       function(f){
                         gsub("showPoints = [A-Z].+",
                              paste0("showPoints = ", input$sP, ")"),f)
                       }, character(1))
                
    )
    showPoints <- 'if'(input$sP, "all", FALSE)
    plotly::plotlyProxy("clickedFeature", session) %>%
      plotly::plotlyProxyInvoke(
        "restyle",
        list(boxpoints = showPoints)
      )
  })
  
  
  observeEvent(plotHeight(), {
    req(input$diffdatatable_rows_selected, diffResults(), aggDat(), 
        featLevel(), diffSettings())
    plotly::plotlyProxy("clickedFeature", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(height = plotHeight())
      )
  },ignoreInit = TRUE)
  
  
  ## renders a plot if datatable rows is clicked and stores results for report
  output$clickedFeature <- plotly::renderPlotly({
    req(input$diffdatatable_rows_selected, diffResults(), 
        aggDat(), featLevel(), diffSettings())
    isolate(featListRep(NULL))
    shinyjs::enable("optionbutton_state")
    ## determine how many rows the plots should be distributed upon
    numofrows <- ceiling(length(input$diffdatatable_rows_selected) / 2)
    
    plotList <- lapply(input$diffdatatable_rows_selected, function(i) {
      isolate({
        featListRep(c(
          featListRep(),
          paste(
            paste0("#' ### Feature plot"),
            paste0("", "#- fig.width = 7"),
            paste0("plotSingleFeature(aggDat,"),
            paste0("\tplotTitle = \"",diffResults()[i, featLevel()], "\","),
            paste0("\tfeature = \"", 
                   diffResults()[i, featLevel()], "\","),
            paste0("\tx_var = \"", 
                   diffSettings()$comparison, "\","),
            paste0("\tylab = \"Reads\","), 
            paste0("\tlog = ", input$logS, ","),
            paste0("\tshowPoints = ", input$sP, ")\n\n"),
            sep = "\n"
          )
        ))
      })
      
      plotSingleFeature(
        aggdat = aggDat(),
        feature = diffResults()[i, featLevel()],
        x_var = diffSettings()$comparison,
        ylab = "Reads",
        xlab = diffResults()[i, featLevel()],
        log = input$logS,
        showPoints = isolate(input$sP),
        fixedHeight = numofrows * 350,
        x_levels = c(diffSettings()$phenolevel1, 
                     diffSettings()$phenolevel2)
      )
    })
    
    plotly::subplot(plotList, nrows = numofrows, 
                    margin = 0.1, titleY = TRUE, titleX = TRUE) %>%
      plotly::layout(showlegend = FALSE)
  })
  
  return(list(reptable = repCode, repplot = featListRep))
}

## Modules for intra sample analysis

#' Relative abundance plot module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return box containing the ui code
relAbundanceUI <- function(id) {
  ns <- NS(id)
  
  box(
    title = "RELATIVE ABUNDANCE", solidHeader = TRUE, 
    collapsible = TRUE, width = 12,
    fluidRow(
      column(
        width = 11,
        shinycssloaders::withSpinner(
          plotly::plotlyOutput(
            ns("relabplot"), 
            width = "auto", 
            height = "auto"),
          type = 3, color = "#424242", color.background = "#fdfdfc"),
        br(),
        shinyjs::disabled(
          shinyWidgets::dropdownButton(
            tags$h3("Plot Options"),
            shinyWidgets::radioGroupButtons(
              inputId = ns("relAbChoice"),
              label = "Y axis",
              choices = c("Percentage", "Reads"),
              individual = TRUE,
              checkIcon = list(
                yes = tags$i(
                  class = "fa fa-circle",
                  style = "color: steelblue"
                ),
                no = tags$i(
                  class = "fa fa-circle-o",
                  style = "color: steelblue"
                )
              )
            ),
            div(id=ns("pspan"), class = "percentage",
                p("Normalization is required to show percentage")
            ),
            span(
              numericInput(
                inputId = ns("numofFeatures"),
                label = "Max number of Features to show",
                value = 10,
                min = 1
              ),
              sliderInput(
                inputId = ns("plotWidth"),
                label = "Adjust plot width",
                value = 650,
                min = 250,
                max = 1600,
                round = TRUE
              )
            ),
            circle = FALSE, status = "danger",
            icon = icon("gear"), width = "300px",
            label = "Plot Options",
            inputId = ns("optionbutton")
          ))
      )
    )
  )
}

#' Relative abundance plot module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param intraSettings analysis input settings passed over to this module
#' @param normalizedData boolean indicating whether data has been normalized
#' @param reset boolean reactive which resets the module if TRUE
#' 
#'
#' @return list storing plot clicks and number of features displayed 
#' (passed to feature plot module) as well as the R code to make plot
relAbundance <- function(input, output, session, 
                         aggDat, 
                         featLevel, 
                         intraSettings,
                         normalizedData,
                         reset) {
  ns <- session$ns
  
  ## stores the plotly plot (needed to set source correctly)
  paPlot <- reactiveVal(NULL)
  ## stores the R code needed to build the relative abundance plot
  repCode <- reactiveVal(NULL)
  width <- reactiveVal(650)
  
  clickedFeature <- reactive(input$clickedFeature)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    shinyjs::enable("relAbChoice")
    paPlot(NULL)
    repCode(NULL)
  })
  
  observe({
    width(input$plotWidth)
  })
  
  
  ## debouncing number of features to avoid multiple redraws
  nof <- reactive(input$numofFeatures)
  numOfFeats <- debounce(nof,1000)
  
  
  ## calls the plotting function
  observe({
    req(aggDat(), intraSettings(),numOfFeats())
    ylab <- "Reads"
    if(isFALSE(normalizedData())){
      shinyWidgets::updateRadioGroupButtons(session = session,
                                            inputId = "relAbChoice",
                                            selected = "Reads")
      shinyjs::removeClass(id = "pspan", class = "hide")
      shinyjs::disable("relAbChoice")
    } else {
      ylab <- input$relAbChoice
      shinyjs::addClass(id = "pspan", class = "hide")
      shinyjs::enable("relAbChoice")
    }
    facet1 <- intraSettings()$facetby
    facet2 <- intraSettings()$facetby2
    plotTitle <- paste0("Top ",
                        numOfFeats(),
                        " feature ", 
                        tolower(gsub("Reads","abundance",ylab)),
                        " at ",
                        featLevel()," level")
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    pa <- plotAbundance(aggDat(),
                        level = featLevel(),
                        x_var = intraSettings()$xvariable,
                        facet1 = facet1,
                        facet2 = facet2,
                        ind = seq_len(numOfFeats()), 
                        plotTitle = plotTitle,
                        ylab = ylab
    ) 
    pa$x$source <- "relabplot"
    paPlot(pa)
    shinyjs::enable("optionbutton_state")
  })
  
  output$relabplot <- plotly::renderPlotly({
    paPlot()
  })
  
  observeEvent(width(), {
    req(paPlot())
    plotly::plotlyProxy("relabplot", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = width())
      )
  })
  
  
  ## reactive stores plotly click events
  observe({
    pe <- plotly::event_data("plotly_click", source = "relabplot")
    ## get the name of the trace that was clicked (= feature name)
    shinyjs::js$getTraceName(isolate(paPlot()), pe$curveNumber)
  })
  
  ## update R code if inputs change
  observe({
    req(aggDat(), intraSettings())
    ylab <- input$relAbChoice
    facet1 <- intraSettings()$facetby
    facet2 <- intraSettings()$facetby2
    plotTitle <- paste0("Top ",
                        numOfFeats(),
                        " feature ", 
                        tolower(gsub("Reads","abundance",ylab)),
                        " at ",
                        featLevel()," level")
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    repCode(paste(
      paste0("#' ### Relative Abundance"),
      paste0("", "#- fig.width = 10"),  
      paste0("plotAbundance(aggDat,"),
      paste0("\tlevel = \"", featLevel(), "\","),
      paste0("\tx_var = \"", 
             intraSettings()$xvariable, "\","),
      paste0(
        "\tfacet1 = ", 
        'if'(is.null(facet1),"NULL",paste0("\"", facet1, "\"")),
        ","
      ),
      paste0(
        "\tfacet2 = ", 
        'if'(is.null(facet2),"NULL",paste0("\"",facet2, "\"")),
        ","
      ),
      paste0("\tind = 1:", numOfFeats(), ","),
      paste0("\tplotTitle = \"",plotTitle, "\","),
      paste0("\tylab = \"", ylab, "\")\n\n"),
      sep = "\n"
    ))
  })
  
  return(list(featName = clickedFeature, 
              ylabMode = reactive(input$relAbChoice),
              numOfFeats = numOfFeats, 
              repCode = repCode))
}


#' Feature plot module - UI
#'
#' @param id namespace identifier
#'
#' @return box holding the UI code
featAbundanceUI <- function(id) {
  ns <- NS(id)
  
  box(title = "FEATURE PLOT", solidHeader = TRUE, 
      collapsible = TRUE, width = 12,
      fluidRow(
        column(
          width = 11,
          shinycssloaders::withSpinner(
            plotly::plotlyOutput(ns("featplot"), 
                                 width = "auto", 
                                 height = "auto"),
            type = 3, color = "#424242", 
            color.background = "#fdfdfc"),
          br(),
          shinyjs::disabled(
            shinyWidgets::dropdownButton(
              tags$h3("Plot Options"),
              shinyWidgets::radioGroupButtons(
                inputId = ns("featChoice"),
                label = "Y axis",
                choices = c("Reads", "Percentage"),
                individual = TRUE,
                checkIcon = list(
                  yes = tags$i(
                    class = "fa fa-circle",
                    style = "color: steelblue"
                  ),
                  no = tags$i(
                    class = "fa fa-circle-o",
                    style = "color: steelblue"
                  )
                )
              ),          
              div(id=ns("pspan"), class = "percentage",
                  p("Normalization is required to show percentage")
              ),
              shinyWidgets::switchInput(
                inputId = ns("sP"),
                label = "Show Points",
                value = TRUE,
                size = "mini",
                labelWidth = "80px"
              ),
              shinyWidgets::switchInput(
                inputId = ns("logS"),
                label = "Log Scale",
                value = TRUE,
                size = "mini",
                labelWidth = "80px"
              ),
              sliderInput(
                inputId = ns("plotWidth"),
                label = "Adjust plot width",
                value = 650,
                min = 250,
                max = 1600,
                round = TRUE
              ),
              circle = FALSE, status = "danger", 
              icon = icon("gear"), width = "300px",
              label = "Plot Options",
              inputId = ns("optionbutton")
            ))
        )
      )
  )
}




#' Feature plot module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param intraSettings analysis settings passed over from analysis input module
#' @param selectedFeat feature selected via drop down element of analysis input
#' @param featName plotly click event passed via relative abundance
#' @param numOfFeats number of features shown in relative abundance plot 
#' (affects plotly click data)
#' @param ylabMode character indication if raw \"Reads\" or \"Percentage\" 
#' should be shown
#' @param normalizedData boolean indicating whether data has been normalized
#' @param reset boolean reactive which resets the module if TRUE
#' 
#'
#' @author Janina Reeder
#' 
#' @return R code needed to build the feature plot
featAbundance <- function(input, output, session, 
                          aggDat, 
                          featLevel, 
                          intraSettings, 
                          selectedFeat, 
                          featName, 
                          numOfFeats,
                          ylabMode,
                          normalizedData,
                          reset) {
  ns <- session$ns
  
  ## selected feature for plot is updated both via dropdown (selectedFeat) 
  ## as well as plot click (featName)
  selectedFeature <- reactiveVal(NULL)
  ## stores R code needed to make the feature Plot
  repCode <- reactiveVal(NULL)
  width <- reactiveVal(650)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    shinyjs::enable("featChoice")
    selectedFeature(NULL)
    repCode(NULL)
  })
  
  observe({
    width(input$plotWidth)
  })
  
  
  observe({
    req(ylabMode())
    shinyWidgets::updateRadioGroupButtons(
      session,"featChoice", selected = ylabMode())
  })
  
  observe({
    req(selectedFeat())
    selectedFeature(selectedFeat())
  })
  
  observe({
    selectedFeature(featName())
  })
  
  ## add/remove points using plotlyProxy to avoid redrawing plot
  observeEvent(input$sP, {
    req(aggDat(), intraSettings()$xvariable, 
        featLevel(), selectedFeature(), numOfFeats())
    show_points <- 'if'(input$sP, "all", FALSE)
    plotly::plotlyProxy("featplot", session) %>%
      plotly::plotlyProxyInvoke(
        "restyle",
        list(boxpoints = show_points)
      )
  })
  
  observeEvent(width(), {
    req(aggDat(), intraSettings()$xvariable, 
        featLevel(), selectedFeature(), numOfFeats())
    plotly::plotlyProxy("featplot", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = width())
      )
  })
  
  
  ## calls function to do a feature abundance plot
  output$featplot <- plotly::renderPlotly({
    req(aggDat(), intraSettings()$xvariable, 
        featLevel(), selectedFeature(), numOfFeats())
    shinyjs::enable("optionbutton_state")
    ylab <- "Reads"
    if(!normalizedData()){
      shinyWidgets::updateRadioGroupButtons(session = session,
                                            inputId = "featChoice",
                                            selected = "Reads")
      shinyjs::removeClass(id = "pspan", class = "hide")
      shinyjs::disable("featChoice")
    } else {
      ylab <- input$featChoice
      shinyjs::addClass(id = "pspan", class = "hide")
      shinyjs::enable("featChoice")
    }
    facet1 <- intraSettings()$facetby
    facet2 <- intraSettings()$facetby2
    plotTitle <- paste0(gsub("Reads","Abundance",ylab),
                        " of ",
                        selectedFeature())
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    plotSingleFeature(aggDat(),
                      x_var = intraSettings()$xvariable,
                      ind = seq_len(numOfFeats()),
                      plotTitle = plotTitle,
                      facet1 = intraSettings()$facetby,
                      facet2 = intraSettings()$facetby2,
                      feature = selectedFeature(),
                      ylab = ylab,
                      log = input$logS,
                      showPoints = isolate(input$sP)
    )
  })
  
  ## update R code needed to make plot
  observe({
    req(aggDat(), intraSettings()$xvariable, 
        featLevel(), selectedFeature(), numOfFeats())
    facet1 <- intraSettings()$facetby
    facet2 <- intraSettings()$facetby2
    plotTitle <- paste0(gsub("Reads","Abundance",input$featChoice),
                        " of ",
                        selectedFeature())
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    repCode(paste(
      paste0("#' ### Feature plot"),
      paste0("", "#- fig.width = 8"),
      paste0("plotSingleFeature(aggDat,"),
      paste0("\tx_var = \"", 
             intraSettings()$xvariable, "\","),
      paste0("\tind = 1:", numOfFeats(), ","),
      paste0("\tplotTitle = \"",plotTitle, "\","),
      paste0(
        "\tfacet1 = ", 
        'if'(is.null(intraSettings()$facetby),
             "NULL",
             paste0("\"", 
                    intraSettings()$facetby, "\"")
        ),
        ","
      ),
      paste0(
        "\tfacet2 = ", 
        'if'(is.null(intraSettings()$facetby2),
             "NULL",
             paste0("\"", 
                    intraSettings()$facetby2, "\"")
        ),
        ","
      ),
      paste0("\tfeature = \"", selectedFeature(), "\","),
      paste0("\tylab = \"", input$featChoice, "\","),
      paste0("\tlog = ", 
             'if'(is.null(input$logS), "TRUE", input$logS), ","),
      paste0("\tshowPoints = ", input$sP, ")\n\n"),
      sep = "\n"
    ))
  })
  
  return(repCode)
}

#' Alpha Diversity module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return box holding the UI code
alphaDiversityUI <- function(id) {
  ns <- NS(id)
  
  box(title = "ALPHA DIVERSITY", solidHeader = TRUE, 
      collapsible = TRUE, width = 12,
      fluidRow(
        column(
          width = 11,
          shinycssloaders::withSpinner(
            plotly::plotlyOutput(
              ns("alphaDiv"), 
              width = "auto", 
              height = "auto"),
            type = 3, color = "#424242", 
            color.background = "#fdfdfc"),
          br(),
          shinyjs::disabled(
            shinyWidgets::dropdownButton(
              tags$h3("Plot Options"),
              selectInput(
                inputId = ns("alphaInd"),
                label = "Index",
                choices = c(
                  "Shannon", "Simpson", "Inverse Simpson",
                  "Richness"
                )
              ),
              selectInput(
                inputId = ns("alphacol"),
                label = "Color by",
                choices = ""
              ),
              sliderInput(
                inputId = ns("plotWidth"),
                label = "Adjust plot width",
                value = 650,
                min = 250,
                max = 1600,
                round = TRUE
              ),
              circle = FALSE, status = "danger", 
              icon = icon("gear"), width = "300px",
              label = "Plot Options",
              inputId = ns("optionbutton")
            ))
        )
      )
  )
}

#' Alpha Diversity module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param intraSettings analysis settings as passed over from analysis 
#' input module
#' @param colorOptions phenotype selections: used for color choices
#' @param reset boolean reactive which resets the module if TRUE
#'
#' @author Janina Reeder
#' 
#' @return R code used to make the alpha diversity plot
alphaDiversity <- function(input, output, session, 
                           aggDat, 
                           featLevel, 
                           intraSettings, 
                           colorOptions,
                           reset) {
  ns <- session$ns
  
  width <- reactiveVal(650)
  ## stores index to pass on to alpha diversity calculation
  chosenIndex <- reactive({
    switch(input$alphaInd,
           "Shannon" = "shannon",
           "Simpson" = "simpson",
           "Inverse Simpson" = "invsimpson",
           "Richness" = "richness"
    )
  })
  
  ## handles color selection
  colorChoice <- reactiveVal("No Color")
  ## stores R code needed to build the plot
  repCode <- reactiveVal(NULL)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    repCode(NULL)
    colorChoice("No Color")
  })
  
  observe({
    width(input$plotWidth)
  })
  
  ## update color options available
  observe({
    updateSelectInput(session, "alphacol", 
                      choices = c("No Color", colorOptions()))
  })
  
  observeEvent(input$alphacol, {
    colorChoice(input$alphacol)
  })
  
  observeEvent(width(), {
    req(aggDat(), featLevel(), chosenIndex(), 
        intraSettings(), colorChoice())
    plotly::plotlyProxy("alphaDiv", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = width())
      )
  })
  
  ## calls the alpha diversity plotting function
  output$alphaDiv <- plotly::renderPlotly({
    req(aggDat(), featLevel(), chosenIndex(), 
        intraSettings(), colorChoice())
    shinyjs::enable("optionbutton_state")
    facet1 <- intraSettings()$facetby
    facet2 <- intraSettings()$facetby2
    plotTitle <- paste0(stringr::str_to_sentence(chosenIndex()),
                        " diversity index at ", 
                        featLevel()," level")
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    color <- as.character(colorChoice())
    if (stringr::str_detect(color, "No Color")) {
      color <- NULL
    }
    plotAlpha(aggDat(),
              level = featLevel(),
              index = chosenIndex(),
              x_var = intraSettings()$xvariable,
              facet1 = intraSettings()$facetby,
              facet2 = intraSettings()$facetby2,
              col_by = color,
              plotTitle = plotTitle
    )
  })
  
  ## update R code based on input choices
  observe({
    req(aggDat(), featLevel(), chosenIndex(), 
        intraSettings(), colorChoice())
    color <- as.character(colorChoice())
    if (stringr::str_detect(color, "No Color")) {
      color <- NULL
    }   
    facet1 <- intraSettings()$facetby
    facet2 <- intraSettings()$facetby2
    plotTitle <- paste0(stringr::str_to_sentence(chosenIndex()),
                        " diversity index at ", 
                        featLevel()," level")
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    color <- as.character(colorChoice())
    if (stringr::str_detect(color, "No Color")) {
      color <- NULL
    }
    repCode(paste(
      paste0("#' ### Alpha diversity"),
      paste0("", "#- fig.width = 7"),
      paste0("plotAlpha(aggDat,"),
      paste0("\tlevel = \"", featLevel(), "\","),
      paste0("\tindex = \"", chosenIndex(), "\","),
      paste0("\tx_var = \"", 
             intraSettings()$xvariable, "\","),
      paste0(
        "\tfacet1 = ", 
        'if'(is.null(intraSettings()$facetby),
             "NULL",
             paste0("\"", 
                    intraSettings()$facetby, "\"")
        ),
        ","
      ),
      paste0(
        "\tfacet2 = ", 
        'if'(is.null(intraSettings()$facetby2),
             "NULL",
             paste0("\"", 
                    intraSettings()$facetby2, "\"")
        ),
        ","
      ),
      paste0("\tcol_by = ", 
             'if'(is.null(color), "NULL", 
                  paste0("\"", color, "\"")), ","),
      paste0("\tplotTitle = \"",plotTitle, "\")\n\n"),
      sep = "\n"
    ))
  })
  
  return(repCode)
}

## MAIN SERVER

#' Set up the shiny server
#'
#' @param session 
#' @param input 
#' @param output 
#' 
#' @author Janina Reeder
#'
#' @import shiny
#'
#' @export
shinyServer(function(session, input, output) {
  
  
  ## tabs are initially disabled
  js$disableTab("PHENOTYPE")
  js$disableTab("FEATURES")
  js$disableTab("INTRA SAMPLE")
  js$disableTab("INTRA FEATURE")
  js$disableTab("INTER SAMPLE")
  js$disableTab("CORRELATION")
  js$disableTab("DIFFERENTIAL")
  js$disableTab("LONGITUDINAL")
  
  ## reactive steering reset of all analysis parts of the app
  resetInput <- reactiveVal(NULL)
  ## reactive steering reset of reports
  resetReports <- reactiveVal(FALSE)
  #pheno data modified/added
  addPheno <- reactiveVal(FALSE)
  
  ## reactives storing code to be passed to report generation tab
  dataSource <- reactiveVal(NULL)
  dataFilterRep <- reactiveVal(NULL)
  qcRep <- reactiveVal(NULL)
  phenoModRep <- reactiveVal(NULL)
  featureModRep <- reactiveVal(NULL)
  analysisRep <- reactiveVal(NULL)
  
  levelOpts <- reactiveVal(NULL)
  ## reactive storing aggregation level
  chosenLevel <- reactiveVal(NULL)
  
  ## reactive values storing aggregated data and aggregation level
  aggData <- reactiveValues(
    mrobj = NULL,
    level = NULL
  )
  normalizedData <- reactiveVal(FALSE)
  
  ## DATA FILTERING AND PREPROCESSING (SUBSETTING, NORMALIZATION)
  
  data <- callModule(
    microbiomeExplorer:::dataInput, "loadnfilter",
    dataSource = dataSource,
    dataFilterRep = dataFilterRep,
    qcRep = qcRep,
    addPheno = addPheno,
    resetReports = resetReports
  )
  
  ## enable tabs once data is available
  observeEvent(data$meData(),{
    resetInput(NULL)
    js$enableTab("PHENOTYPE")
    js$enableTab("FEATURES")
    js$enableTab("INTRA SAMPLE")
    js$enableTab("INTRA FEATURE")
    js$enableTab("INTER SAMPLE")
    js$enableTab("CORRELATION")
    js$enableTab("DIFFERENTIAL")
    js$enableTab("LONGITUDINAL")
    disable("intraAnalysis-intraInput-analysisbox")
    disable("featureAnalysis-featureInput-analysisbox")
    disable("interAnalysis-betaInput-analysisbox")
    disable("interAnalysis-heatmapInput-analysisbox")
    disable("corrAnalysis-fCorrInput-analysisbox")
    disable("corrAnalysis-pCorrInput-analysisbox")
    disable("diffAnalysis-diffInput-analysisbox")
    disable("longAnalysis-longInput-analysisbox")
    featName <- colnames(fData(data$meData()))
    if(length(colnames(fData(data$meData()))) == 0){
      featName <- "unavailable"
    } else {
      featName <- featName[!(featName %in% getOption("me.featurenames"))]
      if(length(featName) > 0){
        safeIndex <- sapply(featName, function(f){
          length(levels(as.factor(fData(data$meData())[[f]]))) > 1
        })
        featName <- featName[safeIndex]
      } else {
        featName <- colnames(fData(data$meData()))[1]
      }
    }
    if(!identical(levelOpts(),featName)){
      genusindex <- grepl("enus",featName)
      if(sum(genusindex) == 1){
        chosenLevel(featName[genusindex])
      } else if(length(featName) > 1){
        chosenLevel(featName[length(featName)-1])
      } else {
        chosenLevel(featName[1])
      }
      levelOpts(featName)
    }
    resetInput(TRUE)
  }, ignoreNULL = TRUE, ignoreInit = TRUE)
  
  
  ## PHENOTABLE MODULE
  callModule(
    microbiomeExplorer:::phenotypeTable, 
    "phenotab", data$meData, phenoModRep, addPheno)
  ## FEATURE TABLE MODULE
  callModule(
    microbiomeExplorer:::featureTable, 
    "featuretab", data$meData, featureModRep)
  
  ## store all data loading, filtering and modifications for report
  preprocessRep <- reactive({
    req(data$meData())
    c(
      phenoModRep(), featureModRep(), dataFilterRep(),
      paste(paste0("", "#+ mrdata, echo=TRUE"),
            paste0("\nmeData\n"),
            sep = "\n")
    )
  })
  
  
  ## AGGREGATION
  
  aggResults <- callModule(
    microbiomeExplorer:::aggregationTab, 
    "aggregateTab",
    resetInput, levelOpts, chosenLevel, data$meData)
  
  observe({
    aggData$mrobj <- aggResults$mrobj()
    aggData$level <- chosenLevel()
    normalizedData(aggResults$normalizedData())
  })
  
  observeEvent(aggResults$aggCode(),{
    analysisRep(unlist(c(analysisRep(),aggResults$aggCode())))
  })
  
  observe({
    req(aggData$mrobj)
    # enable analysis input modules when data is available
    enable("intraAnalysis-intraInput-analysisbox")
    enable("featureAnalysis-featureInput-analysisbox")
    enable("interAnalysis-betaInput-analysisbox")
    enable("interAnalysis-heatmapInput-analysisbox")
    enable("corrAnalysis-fCorrInput-analysisbox")
    enable("corrAnalysis-pCorrInput-analysisbox")
    enable("diffAnalysis-diffInput-analysisbox")
    enable("longAnalysis-longInput-analysisbox")
    js$enableAll()
  })
  
  ## INTRA SAMPLE ANALYSIS
  intraRep <- callModule(
    microbiomeExplorer:::intraAnalysis, "intraAnalysis", data, levelOpts,
    chosenLevel, resetInput, aggData, normalizedData)
  
  ## INTRA FEATURE ANALYSIS
  featureRep <- callModule(
    microbiomeExplorer:::featureAnalysis, "featureAnalysis", data,
    resetInput, aggData, normalizedData)
  
  ## INTER SAMPLE ANALYSIS
  interRep <- callModule(
    microbiomeExplorer:::interAnalysis, "interAnalysis", data, levelOpts,
    chosenLevel, resetInput, aggData)
  
  ## CORRELATION ANALYSIS
  corrRep <- callModule(
    microbiomeExplorer:::corrAnalysis, 
    "corrAnalysis", data, levelOpts,
    chosenLevel, resetInput, aggData)
  
  ## DIFFERENTIAL ANALYSIS
  diffRep <- callModule(
    microbiomeExplorer:::diffAnalysis, "diffAnalysis", data, levelOpts,
    chosenLevel, resetInput, aggData, normalizedData)
  
  ## DIFFERENTIAL ANALYSIS
  longRep <- callModule(
    microbiomeExplorer:::longAnalysis, "longAnalysis", data, levelOpts,
    chosenLevel, resetInput, aggData, normalizedData)
  
  ## ANALYSIS REPORTS
  
  ## capture report button clicks via reactive value (multiple buttons)
  reportClick <- reactiveVal(NULL)
  onclick("intraAnalysis-intraInput-reportButton", reportClick("intra"))
  onclick("featureAnalysis-featureInput-reportButton", reportClick("feature"))
  onclick("interAnalysis-betaInput-reportButton", reportClick("beta"))
  onclick("interAnalysis-heatmapInput-reportButton", reportClick("heatmap"))
  onclick("corrAnalysis-fCorrInput-reportButton", reportClick("fcorr"))
  onclick("corrAnalysis-pCorrInput-reportButton", reportClick("pcorr"))
  onclick("diffAnalysis-diffInput-reportButton", reportClick("diff"))
  onclick("longAnalysis-longInput-reportButton", reportClick("long"))
  
  ## reactive storing which code chunks contain aggregation code
  aggIndex <- reactiveVal(NULL)
  
  ## handle report button clicks: add new code to analysisRep
  observeEvent(reportClick(), {
    analysisRep(unlist(
      c(
        analysisRep(),
        switch(reportClick(),
               "intra" = intraRep(),
               "feature" = featureRep(),
               "beta" = interRep()$beta,
               "heatmap" = interRep()$abheat,
               "fcorr" = corrRep()$fcor,
               "pcorr" = corrRep()$pcor,
               "diff" = diffRep(),
               "long" = longRep()
        )
      )
    ))
    analysisRep(analysisRep()[!is.null(analysisRep())])
    ## capture aggregation chunks (cannot be deselected)
    aggIndex(stringr::str_starts(analysisRep(), "aggDat|meData"))
    ## omit duplicated code
    duplicatedEntries <- duplicated(analysisRep())
    analysisRep(analysisRep()[aggIndex() | !duplicatedEntries])
    reportClick(NULL)
  })
  
  observe({
    req(resetReports())
    dataFilterRep(NULL)
    qcRep(NULL)
    phenoModRep(NULL)
    featureModRep(NULL)
    analysisRep(NULL)
    resetReports(FALSE)
  })
  
  
  ## REPORT TAB
  callModule(
    microbiomeExplorer:::reportList, "reportlist",
    dataSource = dataSource,
    preprocessRep = preprocessRep,
    qcRep = qcRep,
    analysisRep = analysisRep,
    aggIndex = aggIndex,
    reset = resetInput
  )
})

## Main module for diff sample analysis

#'Diff Analysis Module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the ui code
#' 
#' @examples diffAnalysisUI("diffanalysis_id")
#' 
#' @export
diffAnalysisUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
        width = 12,
        column(
            width = 3,
            diffInputUI(ns("diffInput"))
        ),
        column(
            width = 9,
            fluidRow(
                width = 11,
                box(width = 10,
                    p("Differential abundance analysis allows to compare the microbiome composition
                    across certain conditions.")
                ),
                diffTableUI(ns("differentialTable"))
            )
        )
    )
}


#' diff Analysis Module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param data the main data object returned from data_input_module
#' @param levelOpts available levels to aggregate on (depends on input data)
#' @param chosenLevel previously selected level (passed from different instance)
#' @param resetInput reactive boolean determining if reset is required
#' @param aggData the aggregated MRExperiment object
#' @param normalizedData boolean indicating if normalization was done
#' 
#' @author Janina Reeder
#' 
#'
#' @return reactive holding code to be used in reports
diffAnalysis <- function(input, output, session, data, levelOpts,
                          chosenLevel, resetInput, aggData, normalizedData) {
  ## diff SAMPLE ANALYSIS
  ns <- session$ns
  
  diffSettings <- callModule(diffInput,
                             "diffInput",
                             meData = data$meData,
                             facetOptions = data$facetOptions,
                             reset = resetInput
  )
  
  diffTableRep <- callModule(diffTable,
                             "differentialTable",
                             aggDat = reactive(aggData$mrobj),
                             featLevel = chosenLevel,
                             diffSettings = diffSettings,
                             reset = resetInput,
                             normalized = normalizedData
  )
  
  ## store differential analysis code for reports
  diffRep <- reactive({
    list(
      "table" = diffTableRep$reptable(),
      "plot" = diffTableRep$repplot()
    )
  })
  
  return(diffRep)
  
}

#' Report row module consisting of a checkbox, image and description/R code area
#'
#' @param id namespace identifier
#' @param type boolean indicating if a selector checkbox should be added
#' 
#' @author Janina Reeder
#' @return div holding the UI code
reportRowUI <- function(id, type) {
    ns <- NS(id)
    
    div(
        fluidRow(
            width = 11, id = "reportrow",
            column(
                width = 1,
                if (!type) {
                    shinyWidgets::prettyCheckbox(
                        inputId = ns("includeCheck"),
                        label = "",
                        value = TRUE,
                        icon = icon("check"),
                        status = "success",
                        animation = "rotate",
                        outline = TRUE
                    )
                }
            ),
            column(
                width = 4,
                plotOutput(
                    ns("reportImage"), height = "175px", width = "175px")
            ),
            column(width = 1),
            column(
                width = 6,
                uiOutput(ns("reportText"), height = "200px")
            )
        ),
        hr()
    )
}


#' Report Row
#'
#' @param input module input
#' @param output module output
#' @param session app session
#' @param type boolean indicating whether checkbox should be included
#' @param content R code to show
#' 
#' @author Janina Reeder
#' @return reactive boolean indicating whether row is selected
reportRow <- function(input, output, session, type, content) {
    ns <- session$ns
    
    ## preprocess code to only display relevant parts
    codecontent <- reactive({
        req(content())
        cc <- lapply(stringr::str_split(content(), "\n")[[1]], function(c) {
            'if'(stringr::str_starts(c, "#"), "", c)
        })
        cc[cc != ""]
    })
    
    ## determine which icon to show
    imagename <- reactive({
        req(codecontent())
        if (stringr::str_starts(codecontent()[1], 
                                "makeQCPlot")) {
            "qcpic"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "plotSingleFeature")) {
            "featplot"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "plotAbundance")) {
            "relAb"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "plotAlpha")) {
            "alphadiv"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "plotHeatmap")) {
            "heatmap"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "distMat")) {
            "betadiv"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "c[fp]")) {
            "correlation"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "diff")) {
            "difftest"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "plotLongFeature")) {
            "longitudinal"
        } else if (stringr::str_starts(codecontent()[1], 
                                       "plotAvgAbundance")) {
            "intrafeature"
        } else {
            "blank"
        }
    })
    
    ## render the icon
    output$reportImage <- renderImage({
        req(imagename())
        filename <- normalizePath(file.path(
            "www/",
            paste(imagename(), ".png", sep = "")
        ))
        
        # Return a list containing the filename
        list(
            src = filename,
            width = "175px",
            height = "175px"
        )
    }, deleteFile = FALSE)
    
    ## render the code sections
    output$reportText <- renderUI({
        req(codecontent())
        div(pre(id = "rcode", codecontent()),
            class = "reportDivBox"
        )
    })
    
    return(input$includeCheck)
}

#' Main diffanalysis input module.
#' Set up to handle diff analysis tabs in the app depending on given parameters
#'
#' @param id element identifier - namespace
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
diffInputUI <- function(id) {
  ns <- NS(id)
  
  box(width = 12, id = ns("analysisbox"),
      h4("ANALYSIS PARAMETERS"),
      selectInput(
        ns("method"),
        label = "Method", 
        choices = getOption("me.diffmethods"), 
        multiple = FALSE, 
        selectize = FALSE, 
        width = "250px"
      ),
      div(
        selectInput(
          ns("comparison"),
          label = "Comparison phenotype", choices = "", 
          multiple = FALSE, selectize = FALSE, width = "250px"
        ),
        selectInput(
          ns("phenolevel1"),
          label = "Comparison level 1",
          choices = "", multiple = FALSE, width = "250px"
        ),
        selectInput(
          ns("phenolevel2"),
          label = "Comparison level 2",
          choices = "", multiple = FALSE, width = "250px"
        )
      ),
      ## buttons are used to submit events. This ensures plots are not redrawn 
      ## while user still adjusts parameters
      div(
        id = ns("buttondiv"),
        fluidRow( 
          width = 12, id = "actionbuttonrow",
          ## update analysis outputs (plots/tables)
          actionButton(
            ns("updatebutton"), 
            icon = icon("far fa-chart-bar"),
            label = HTML("&nbsp;UPDATE"), width = "120px"),
          actionButton(
            ns("reportButton"), 
            label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
            width = "120px")
        )
      )
  )
}


#' Server side for the analysis input module handling analysis control
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment object storing all data
#' @param facetOptions named vector of available facet choices
#' @param reset reactive boolean determining if all inputs should be reset
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
diffInput <- function(input, output, session, 
                      meData, facetOptions = NULL,
                      reset) {
  ns <- session$ns
  
  ## reactive value storing choices made in the UI
  chosenValues <- reactiveVal(NULL)
  ## reactive storing all available phenotype levels
  phenoOpts <- reactiveVal(NULL)
  
  ## reset all entries
  observe({
    req(reset())
    chosenValues(NULL)
    phenoOpts(NULL)
  })
  
  ## initialize input elements
  observe({
    updateSelectInput(session, "comparison", 
                      choices = c("", facetOptions()))
  })
  
  observe({
    req(input$comparison)
    req(input$comparison %in% names(pData(meData())))
    phenoOpts(levels(
      forcats::fct_explicit_na(
        factor(pData(meData())[, input$comparison]),
        na_level = "NA"))
    )
    updateSelectInput(session, "phenolevel1",
                      choices = phenoOpts(),
                      selected = phenoOpts()[1])
    updateSelectInput(session, "phenolevel2",
                      choices = phenoOpts(),
                      selected = phenoOpts()[2])
  }) 
  
  observe({
    req(!is.null(input$comparison))
    req(input$comparison == "")
    updateSelectInput(session, "phenolevel1",
                      choices = "")
    updateSelectInput(session, "phenolevel2",
                      choices = "")
  })
  
  ## react to selecting first phenolevel (second must be different)
  observeEvent(input$phenolevel1, {
    req(input$phenolevel1 != "", input$phenolevel1 == input$phenolevel2)
    updateSelectInput(session, "phenolevel2",
                      selected = phenoOpts()[!(phenoOpts() %in% 
                                                 input$phenolevel1)][1])
  })
  
  ## react to selecting second phenolevel (first must be different)
  observeEvent(input$phenolevel2, {
    req(input$phenolevel2 != "", input$phenolevel1 == input$phenolevel2)
    updateSelectInput(session, "phenolevel1",
                      selected = phenoOpts()[!(phenoOpts() %in% 
                                                 input$phenolevel2)][1])
  })
  
  ## main control button: store input choices in chosenValues
  observeEvent(input$updatebutton, {
    comparison <- NULL
    if(input$comparison != "")
      comparison <- input$comparison
    cV <- list("method" = input$method,
               "comparison" = comparison,
               "phenolevel1" = input$phenolevel1,
               "phenolevel2" = input$phenolevel2)
    chosenValues(cV)
  })
  
  
  return(chosenValues)
}

## QC PLOTS ###################################################################


#' Plots sequencing statistics scatterplot
#'
#' This function makes a scatterplot of read and feature counts
#' for each sample. It was adjusted based on original work by Mo Huang
#'
#' @param MRobj metagenomeSeq object to be plotted
#' @param col_by factor by which to color the points
#' @param log character indicating which (if any) axes should be shown as log
#' @param filter_feat Numeric Y-coordinate to draw horizontal dashed line to
#' indicate feature filtering. If 0 (default), no line is drawn.
#' @param filter_read Numeric X-coordinate to draw vertical dashed line to
#' indicate read count filtering. If 0 (default), no line is drawn.
#' @param allowWebGL boolean indicating if webGL should be added
#' @param pwidth overall plot width; default is 550 (125 are added for legend)
#' @param pheight overall plot height; default is 550
#'
#' @author Janina Reeder
#'
#' @importFrom metagenomeSeq MRcounts
#'
#' @return the plotly QC plot
#' 
#' @examples
#' data("mouseData", package = "metagenomeSeq")
#' makeQCPlot(mouseData)
#'
#' @export
makeQCPlot <- function(MRobj, col_by = NULL, log = "none",
                       filter_feat = 0, 
                       filter_read = 0,
                       allowWebGL = TRUE, 
                       pwidth = 550, pheight = 550) {
  cts <- MRcounts(MRobj)
  ## how many features are present
  nfeatures <- colSums(cts > 0)
  ## how many reads are present 
  nreads <- colSums(cts)
  legendtitle <- ""
  
  if (!is.null(col_by) && col_by != "No color") {
    group <- forcats::fct_explicit_na(
      factor(pData(MRobj)[, which(colnames(pData(MRobj)) == col_by)]),
      na_level = "NA")
    ## adjust overall plotted area if legend is shown
    pwidth <- pwidth + 125
    legendtitle <- list(yref = "paper",xref = "paper",
                        y=1.06,x=1.2, 
                        text = col_by,showarrow = FALSE)
  } else {
    group <- I("black")
    legend <- FALSE
  }
  
  if(length(log) > 0){
    if(log == "x axis"){
      nreads <- log(nreads + 1)
    } else if(log == "y axis"){
      nfeatures <- log(nfeatures + 1)
    } else if(log == "both"){
      nreads <- log(nreads + 1)
      nfeatures <- log(nfeatures + 1)
    }
  }
  
  
  plotdata <- tibble::tibble(
    text = paste0(paste0("<b>", colnames(cts), "</b>"), 
                  paste0("<br>Reads: ", nreads), 
                  paste0("<br>Features: ", nfeatures)),
    x = nreads, y = nfeatures, color = group
  )
  
  useWebGL <- FALSE
  if(nrow(plotdata) > getOption("me.minwebgl") && allowWebGL)
    useWebGL <- TRUE
  
  ## ensure viewport shows everything
  max_x <- max(plotdata$x)
  max_x <- max_x + 0.05 * max_x
  min_x <- 0 - 0.05 * max_x
  max_y <- max(plotdata$y)
  max_y <- max_y + 0.05 * max_y
  min_y <- 0 - 0.05 * max_y
  
  ## make lines invisible if no filters are applied
  featalpha <- 0
  featalphafill <- 0
  if (filter_feat != 0) {
    featalpha <- 1
    featalphafill <- 0.2
  }
  readalpha <- 0
  readalphafill <- 0
  if (filter_read != 0) {
    readalpha <- 1
    readalphafill <- 0.2
  }
  
  
  ## generate actual plot
  p <- plotly::plot_ly(
    plotdata, hoverinfo = "text", 
    height = pheight, width = pwidth,
    colors = getOption("me.colorvalues")[seq(1, length(levels(group)))]) %>%
    plotly::add_segments(
      x = 0, y = filter_feat, xend = max_x, yend = filter_feat,
      line = list(color = paste0("rgba(119, 0, 0,", featalpha, ")"), 
                  width = 1, dash = "dash"), showlegend = FALSE,
      fill = "tozeroy", 
      fillcolor = paste0("rgba(219,219,219,", featalphafill, ")")
    ) %>%
    plotly::add_segments(
      x = filter_read, y = 0, xend = filter_read, yend = max_y,
      line = list(color = paste0("rgba(119, 0, 0,", readalpha, ")"), 
                  width = 1, dash = "dash"), showlegend = FALSE,
      fill = "tozerox", 
      fillcolor = paste0("rgba(219,219,219,", readalphafill, ")")
    ) %>%
    plotly::add_markers(
      x = ~x,
      y = ~y,
      color = ~color,
      text = ~text
    ) 
  if(useWebGL)
    p <- p %>% plotly::toWebGL() 
  
  p <- p %>% plotly::config(
    toImageButtonOptions = list(
      format = "svg"
    ),
    displaylogo = FALSE,
    modeBarButtons = getOption("me.modebar")
  ) %>%
    plotly::layout(
      xaxis = list(title = "&nbsp;\nNumber of reads", 
                   tickangle = 45,
                   range = c(min_x, max_x)),
      yaxis = list(title = paste0("Number of features\n&nbsp;"), 
                   range = c(min_y, max_y)),
      margin = list(l = 75, b = 75, t = 50, r = 50),
      annotations = legendtitle
    ) 
  return(p)
}


#' Function plotting a plotly histogram on the given histvalue
#'
#' @param histvalue the value to plot as a histogram
#' @param plotTitle title of the plot
#' @param xaxisTitle name of xaxis; default is ""
#' @param yaxisTitle name of yaxis; default is ""
#' @param pwidth overall plot width; default is 200 
#' @param pheight overall plot height; default is 200
#'
#' @return plotly plot object
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' plotlyHistogram(histvalue = colSums(MRcounts(mouseData) > 0),
#'   plotTitle = "Feature distribution",
#'   xaxisTitle = "features", yaxisTitle = "frequency")
#' 
#' @export
plotlyHistogram <- function(histvalue, plotTitle, 
                            xaxisTitle = "", yaxisTitle = "",
                            pwidth = 200, pheight = 200) {
  ## ensure viewport includes 0
  xminus <- 0 - 0.01 * max(histvalue)
  
  p <- plotly::plot_ly(x = ~histvalue,
                       width = pwidth,
                       height = pheight) %>%
    plotly::add_histogram(
      marker = list(
        color = "rgba(219, 219, 219,0.8)",
        line = list(
          color = "black",
          width = 1
        )
      ),
      hovertemplate = "Frequency of <br> (%{x}) <br><b> %{y}</b>"
    ) %>%
    plotly::add_segments(
      x = xminus, y = 0, xend = xminus, yend = 0,
      line = list(color = "#444", width = 1), 
      showlegend = FALSE, hoverinfo = "none"
    ) %>%
    plotly::layout(
      title = list(
        text = plotTitle,
        font = list(
          size = 14
        )
      ),
      xaxis = list(
        title = list(
          text = xaxisTitle,
          font = list(
            size = 12
          )
        ),
        showgrid = FALSE,
        tickfont = list(
          size = 9
        )
      ),
      yaxis = list(
        title = list(
          text = yaxisTitle,
          font = list(
            size = 12
          )
        ),
        visible = TRUE,
        showgrid = FALSE,
        tickfont = list(
          size = 9
        )
      ),
      margin = list(
        pad = 1, t = 20,b = 20,l = 20,r = 20
      )
    ) %>%
    plotly::config(
      toImageButtonOptions = list(
        format = "svg"
      ),
      displaylogo = FALSE,
      modeBarButtons = list(list("toImage"))
    )
  p
}

#' Function plotting a barplot showing number of OTUs per samples
#'
#' @param MRobj containing data to plot
#' @param col_by phenotype to color bars by; default is NULL
#' @param xaxisTitle name of xaxis; default is ""
#' @param yaxisTitle name of yaxis; default is ""
#' @param pwidth overall plot width; default is 600 
#' @param pheight overall plot height; default is 450
#' @param sortbyfreq boolean determining if bars should be sorted by frequency; 
#' default is FALSE
#' @param pheno_sort order of pheno levels to sort by; 
#' ignored if sortbyfreq is TRUE
#' @param x_levels character vector holding x values in order to be shown
#' 
#' @importFrom metagenomeSeq MRcounts
#'
#' @return plotly plot object
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' plotlySampleBarplot(mouseData)
#' 
#' @export
plotlySampleBarplot <- function(MRobj, col_by = NULL,
                                xaxisTitle = "", yaxisTitle = "",
                                pwidth = 600, pheight = 450,
                                sortbyfreq = FALSE,
                                pheno_sort = NULL,
                                x_levels = NULL) {
  
  cts <- MRcounts(MRobj)
  numofotus <- colSums(cts > 0)
  showlegend <- FALSE
  
  if (!is.null(col_by) && col_by != "No color") {
    group <- forcats::fct_explicit_na(
      factor(pData(MRobj)[, which(colnames(pData(MRobj)) == col_by)]),
      na_level = "NA")
    showlegend <- TRUE
    pwidth <- pwidth + 100
  } else {
    group <- I("rgba(219, 219, 219,0.8)")
  }
  
  plotdata <- tibble::tibble(
    text = paste0(paste0("<b>", colnames(cts), "</b>"), 
                  paste0("<br>Number of unique features: ", numofotus)),
    x = colnames(cts), y = numofotus, color = group
  )
  if(isTRUE(sortbyfreq)){
    plotdata[["x"]] <- factor(
      plotdata[["x"]], 
      levels = unique(plotdata[["x"]])[order(plotdata[["y"]], 
                                             decreasing = TRUE)])
  } else if(!is.null(pheno_sort) && !is.null(x_levels)){
    x_factor <- forcats::fct_explicit_na(
      factor(pData(MRobj)[, which(colnames(pData(MRobj)) == pheno_sort)]),
      na_level = "NA")
    plotdata$x_factor <- forcats::fct_relevel(x_factor,as.character(x_levels))
    plotdata[["x"]] <- factor(
      plotdata[["x"]], 
      levels = plotdata[order(plotdata$x_factor), ][["x"]])
  }
  
  p <- plotly::plot_ly(
    plotdata, hoverinfo = "text",
    height = pheight, width = pwidth,
    colors = getOption("me.colorvalues")[seq(1, 
                                             length(levels(group)))]) %>%
    plotly::add_bars(
      x = ~x,
      y = ~y,
      color = ~color,
      text = ~text,
      showlegend = showlegend
    )  %>%
    plotly::layout(
      xaxis = list(title = "&nbsp;\nSamples",
                   tickfont = list(
                     size = 9
                   )),
      yaxis = list(title = "Number of features\n&nbsp;"),
      margin = list(l = 75, b = 120, t = 50, r = 50)
    ) %>%
    plotly::config(
      toImageButtonOptions = list(
        format = "svg"
      ),
      displaylogo = FALSE,
      modeBarButtons = getOption("me.modebar")
    )
  
  p
}

## Main module for corr sample analysis

#'corr Analysis Module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the ui code
#' 
#' @examples corrAnalysisUI("coranalysis_id")
#' 
#' @export
corrAnalysisUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    width = 12,
    column(
      width = 3,
      corrInputUI(ns("fCorrInput"), type = "feature"),
      corrInputUI(ns("pCorrInput"), type = "pheno")
    ),
    column(
      width = 9,
      fluidRow(
        width = 11,
        box(width = 10,
            p("The correlation page contains methods to investigate how specific
            taxa are correlated with each other or to numeric phenotypes.")
        ),
        featureCorrUI(ns("fCorrelation")),
        phenotypeCorrUI(ns("pCorrelation"))
      )
    )
  )
}


#' corr Analysis Module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param data the main data object returned from data_input_module
#' @param levelOpts available levels to aggregate on (depends on input data)
#' @param chosenLevel previously selected level (passed from different instance)
#' @param resetInput reactive boolean determining if reset is required
#' @param aggData the aggregated MRExperiment object
#'
#' @return reactive holding code to be used in reports
corrAnalysis <- function(input, output, session, data, levelOpts,
                          chosenLevel, resetInput, aggData) {
  ## corr SAMPLE ANALYSIS
  ns <- session$ns
  
  fCorrSettings <- callModule(corrInput,
                              "fCorrInput",
                              type = reactive("feature"),
                              meData = data$meData,
                              facetOptions = data$facetOptions,
                              reset = resetInput,
                              aggDat = reactive(aggData$mrobj)
  )
  
  pCorrSettings <- callModule(corrInput,
                              "pCorrInput",
                              type = reactive("pheno"),
                              meData = data$meData,
                              facetOptions = data$facetOptions,
                              reset = resetInput,
                              aggDat = reactive(aggData$mrobj)
  )
  
  ## reactive values storing correlation input settings
  fCorrFeatBase <- reactiveVal("")
  pCorrFeatBase <- reactiveVal("")
  fCorrMethod <- reactiveVal("")
  pCorrMethod <- reactiveVal("")

  ## update base feature for comparisons only if changed
  observe({
      req(fCorrFeatBase() !=
              fCorrSettings()$featurecorr1)
      fCorrFeatBase(fCorrSettings()$featurecorr1)
  }, priority = 20)
  
  ## update base feature for comparisons only if changed
  observe({
    req(pCorrFeatBase() !=
          pCorrSettings()$featurecorr1)
    pCorrFeatBase(pCorrSettings()$featurecorr1)
  }, priority = 20)

  ## update correlation method only if changed
  observe({
      req(fCorrMethod() != fCorrSettings()$method)
      fCorrMethod(fCorrSettings()$method)
  }, priority = 20)

  ## update correlation method only if changed
  observe({
    req(pCorrMethod() != pCorrSettings()$method)
    pCorrMethod(pCorrSettings()$method)
  }, priority = 20)
  

  fCorrRep <- callModule(featureCorr, "fCorrelation",
      aggDat = reactive(aggData$mrobj),
      colorOptions = data$facetOptions,
      corFeatBase = fCorrFeatBase,
      corFeat2 = reactive(fCorrSettings()$featurecorr2),
      corFacet1 = reactive(fCorrSettings()$facetby),
      corFacet2 = reactive(fCorrSettings()$facetby2),
      corMethod = fCorrMethod,
      reset = resetInput
  )

  pCorrRep <- callModule(phenotypeCorr, "pCorrelation",
                        aggDat = reactive(aggData$mrobj),
                        colorOptions = data$facetOptions,
                        corFeatBase = pCorrFeatBase,
                        corPheno = reactive(pCorrSettings()$phenoselect),
                        corFacet1 = reactive(pCorrSettings()$facetby),
                        corFacet2 = reactive(pCorrSettings()$facetby2),
                        corMethod = pCorrMethod,
                        reset = resetInput
  )


  ## store correlation analysis code for reports
  corrRep <- reactive({
      list(
          "fcor" = fCorrRep(),
          "pcor" = pCorrRep()
      )
  })
  
  return(corrRep)
  
}

#' Helper function to replace any un-annotated features with the term unknown
#'
#' @param featcol vector of entried to be replaced where needed (fData column)
#'
#' @author Janina Reeder
#'
#' @return modified featcol
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' featcol <- fData(mouseData)[["genus"]]
#' featcol[featcol == "NA"] <- NA
#' replaceWithUnknown(featcol)
#' 
#' @export
replaceWithUnknown <- function(featcol) {
    featcol <- stringr::str_replace_na(featcol, replacement = "unknown")
    featcol <- stringr::str_replace_all(featcol, "^[kpcofgs]{1}__$", "unknown")
    featcol[stringr::str_length(featcol) == 0] <- "unkown"
    return(featcol)
}

#' Helper function which rolls down annotated from closest higher order with 
#' annotation
#'
#' @param featrow vector of entries to be replaced where needed (fData row)
#'
#' @author Janina Reeder
#'
#' @return modified featurerow
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' featrow <- fData(mouseData)[5,]
#' rollDownFeatures(featrow)
#' 
#' @export
rollDownFeatures <- function(featrow) {
    featrow <- stringr::str_replace_na(t(featrow), replacement = "NA")
    featrow[stringr::str_length(featrow) == 0] <- "NA"
    rdlocs <- stringr::str_detect(featrow, "^[kpcofgs]{1}__$|^NA$")
    if (sum(rdlocs) > 0) {
        ri <- min(which(rdlocs))
        featrow[rdlocs] <- paste0("unknown_", featrow[ri - 1])
    }
    return(featrow)
}

splitColumns <- function(featData){
    if(is.null(featData)){
        return(NULL)
    }
    if(length(featData) == 0){
        return(NULL)
    }
    
    featList <- lapply(featData, function(f){
        stringr::str_trim(
            stringr::str_split(f,";")[[1]]
        )
    })
    numofcols <- max(lengths(featList))
    featList %>% purrr::map_dfr(~ as.data.frame(t(.)))
}


#' Feature table UI module
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the UI code for feature tables
#' 
#' @examples featureTableUI("feature_id")
#' 
#' @export
featureTableUI <- function(id) {
    ns <- NS(id)
    
    fluidRow(
        column(
            width = 3, id = ns("featannocol"),
            shinyjs::hidden(
                div(
                    id = ns("splitdiv"),
                    h4("SPLIT COLUMNS"),
                    selectInput(
                        ns("splittaxonomy"),
                        label = "Choose taxonomy column",
                        choices = "", 
                        multiple = FALSE, selectize = FALSE, width = "250px"),
                    actionButton(
                        ns("splitbutton"), 
                        icon = icon("fas fa-angle-double-right"),
                        label = HTML("&nbspSPLIT"), 
                        width = "100px")
                )
            ),
            h4("ANNOTATE BLANK VALUES"),
            selectInput(
                ns("featureanno"),
                label = "Method",
                choices = c("Roll down taxonomy", "Mark as unknown"), 
                multiple = FALSE, selectize = FALSE, width = "250px"),
            div(
                id = ns("buttondiv"),
                fluidRow(
                    width = 12, id = "actionbuttonrow",
                    actionButton(
                        ns("annobutton"), 
                        icon = icon("fas fa-angle-double-right"),
                        label = HTML("&nbsp;ASSIGN"), 
                        width = "100px"),
                    actionButton(
                        ns("resetbutton"),
                        icon = icon("fas fa-redo-alt"),
                        label = HTML("&nbsp;RESET"), 
                        width = "100px")
                ),
                fluidRow(
                    width = 12, id = "actionbuttonrow2",
                    shinyjs::disabled(
                        actionButton(
                            ns("savebutton"), 
                            icon = icon("far fa-save"),
                            label = HTML("&nbsp;SAVE"),
                            width = "100px"))
                )
            )
        ),
        column(
            width = 9,
            box(
                width = 10,
                h2("FEATURE OVERVIEW"),
                p("Available feature taxonomy for the counts data.  
                Table settings allow paging through sections of the data, 
                choosing how many entries to display
                 or searching for specific entries.
                Unannotated features can be marked as \"Unknown\" or obtain
                annotation via the next available higher taxonomy level in a 
                roll down mechanism. Modifications must be saved in order
                  to be available in the analysis sections.")
            ),
            box(
                width = 12,
                div(
                    id = ns("downloaddiv"),
                    downloadButton(ns("download_button"),"Download"),
                    DT::DTOutput(ns("featuredatatable"), width = "100%")
                )
            )
        )
    )
}

#' Helper function returning the fData modifications as strings for 
#' report generation
#'
#' @param featureanno type of feature annotation; values are "Mark unknown" or 
#' "Roll down"
#'
#' @return String containing R code performing the modification
getFeatModCode <- function(featureanno) {
    if (featureanno == "Mark as unknown") {
        paste(paste0("bufrownames <- row.names(fData(meData))"),
              paste0("df <- as.data.frame(apply(fData(meData),2, 
                   replaceWithUnknown))"),
              paste0("rownames(df) <- bufrownames"),
              paste0("meData <- addFeatData(meData,df)"),
              sep = "\n"
        )
    } else if (featureanno == "Roll down taxonomy") {
        paste(paste0("bufcolnames <- names(fData(meData))"),
              paste0("df <- as.data.frame(t(apply(fData(meData),1, 
                   rollDownFeatures)))"),
              paste0("names(df) <- bufcolnames"),
              paste0("meData <- addFeatData(meData,df)"),
              sep = "\n"
        )
    }
}

#' Helper function returning the fData modifications as strings for 
#' report generation
#'
#' @param splittaxonomy name of column to split on
#'
#' @return String containing R code performing the modification
getFeatSplitCode <- function(splittaxonomy) {
    paste(paste0("bufrownames <- row.names(fData(meData))"),
          paste0("df <- splitColumns(fData(meData)[[\"",
                 splittaxonomy,"\"]])"),
          paste0("rownames(df) <- bufrownames"),
          paste0("meData <- addFeatData(meData,df)"),
          sep = "\n"
    )
}

#' Feature table module server code
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment storing the data
#' @param featureModRep reactiveValue storing modifications performed on fData
#'
#' @return feature table server fragment - no return value
#'
#' @author Janina Reeder
#' 
#' @importFrom Biobase fData
featureTable <- function(input, output, session, meData, featureModRep) {
    ns <- session$ns
    
    ## stores the fData of the given MRExperiment
    featFrame <- reactiveVal(NULL)
    ## keeps track of whether annotation was performed
    annotated <- reactiveVal(FALSE)
    taxsplit <- reactiveVal(FALSE)
    
    ## initialize featFrame when meData becomes available
    observe({
        req(meData())
        isolate(featFrame(fData(meData())))
    })
    
    observe({
        req(featFrame())
        if(ncol(featFrame()) <= 2){
            shinyjs::disable("annobutton")
        } else {
            shinyjs::enable("annobutton")
        }
        if(nrow(featFrame()) > 0 && ncol(featFrame()) > 0){
            semicolon_index <- sapply(
                seq(1,ncol(featFrame())), 
                function(i) any(grepl(";", featFrame()[[i]]))
            )
            if(any(semicolon_index)){
                shinyjs::show("splitdiv")
                updateSelectInput(session,"splittaxonomy",
                                  choices = names(featFrame())[semicolon_index])
            } else {
                shinyjs::hide("splitdiv")
            }
        } else {
            shinyjs::hide("splitdiv")
        }
    })
    
    observeEvent(input$splitbutton, {
        req(input$splittaxonomy %in% names(featFrame()))
        bufrownames <- row.names(featFrame())
        shinyjs::addClass("featuredatatable", "transparent")
        df <- splitColumns(featFrame()[[input$splittaxonomy]])
        rownames(df) <- bufrownames
        shinyjs::removeClass("featuredatatable", "transparent")
        ## update stored feature frame
        featFrame(df)
        taxsplit(TRUE)
        shinyjs::enable("savebutton")
    })
    
    ## perform annotation op
    observeEvent(input$annobutton, {
        if (input$featureanno == "Mark as unknown") {
            ## everything will just be unknown
            bufrownames <- row.names(featFrame())
            df <- as.data.frame(apply(featFrame(), 2, replaceWithUnknown))
            rownames(df) <- bufrownames
            ## update stored feature frame
            featFrame(df)
            ## mark as annotated
            annotated(TRUE)
        } else if (input$featureanno == "Roll down taxonomy") {
            ## use annotation of parent
            bufcolnames <- names(featFrame())
            ## Mark datatable as tranparent to show the user the roll down is 
            ## in progress (css switch)
            shinyjs::addClass("featuredatatable", "transparent")
            ## roll down features row by row; used stringr, so fairly fast. 
            df <- as.data.frame(t(apply(featFrame(), 1, rollDownFeatures)))
            ## Return to normal
            shinyjs::removeClass("featuredatatable", "transparent")
            names(df) <- bufcolnames
            ## update stored feature frame
            featFrame(df)
            ## mark as annotated
            annotated(TRUE)
        }
        shinyjs::enable("savebutton")
    })
    
    ## revert annotation changes
    observeEvent(input$resetbutton, {
        req(meData())
        if (annotated() || taxsplit()) {
            ## go back to original dataset
            featFrame(fData(meData()))
            annotated(FALSE)
            taxsplit(FALSE)
            featureModRep(NULL)
            shinyjs::disable("savebutton")
        }
    })
    
    ## make changes permanent
    observeEvent(input$savebutton, {
        req(meData())
        if (annotated() || taxsplit()) {
            ## adjust original dataset
            meData(addFeatData(meData(), featFrame()))
            if(taxsplit()){
                featureModRep(getFeatSplitCode(input$splittaxonomy))
                taxsplit(FALSE)
            }
            if(annotated()){
                featureModRep(paste(featureModRep(),
                                    getFeatModCode(input$featureanno),
                                    sep = "\n"))
                annotated(FALSE)
            }
            shinyjs::disable("savebutton")
        }
    })
    
    ## render table showing feature data
    output$featuredatatable <- DT::renderDT({
        req(meData())
        shinyjs::js$moveButton(ns("downloaddiv"),ns("download_button"))
        DT::datatable(
            data = featFrame(), class = "stripe hover cell-border order-column",
            filter = "none", style = "bootstrap",
            callback = DT::JS("$('div.dwnld_feat').append($('#featuretab-download_button'));"),
            extensions = c("FixedColumns", "ColReorder", "Buttons"),
            options = list(
                scrollX = TRUE,
                paging = TRUE,
                colReorder = TRUE,
                stateSave = TRUE,
                stateLoadParams = DT::JS("function (settings, data) {
                                         return false;}"),
                buttons = list(
                    list(
                        extend = "colvis",
                        text = "Select columns"
                    )),
                dom = '<"dwnld_feat"B>ftlp'
            )
        )
    })
    
    output$download_button <- downloadHandler(
        filename = function() {
            paste0("featuredata-", Sys.Date(), ".csv")
        },
        content = function(file) {
            readr::write_csv(featFrame(), file)
        }
    )
}

## INTRA SAMPLE ANALYSIS 

#' Plot relative abundance
#'
#' This function plots the relative abundance of the top abundant features.
#'
#' @param aggdat aggregated MRExperiment object
#' @param level Feature level.
#' @param x_var Phenotype to aggregate over on X-axis. Default by "SAMPLE_ID".
#' @param ind Indices of top abundant features to plot. Rest of features are
#' aggregated and displayed as "other".
#' @param plotTitle Plot title. Default shows no title.
#' @param ylab Y-axis label. Default is "Reads"
#' @param facet1 Phenotype for facet 1.
#' @param facet2 Phenotype for facet 2.
#' @param source name of the plot (needed for event handling); default is "A"
#' @param pwidth overall plot width; default is 650 
#' @param pheight overall plot height; default is 150
#'
#' @author Janina Reeder
#'
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#'
#' @return plotly plot
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotAbundance(aggdat, level = "genus", x_var = "diet")
#'
#' @export
plotAbundance <- function(aggdat, level, x_var = "SAMPLE_ID",
                          ind = seq_len(10), plotTitle = "", ylab = "Reads", 
                          facet1 = NULL, facet2 = NULL, source = "A",
                          pwidth = 650, pheight = 150) {
  facets <- NULL
  facet2s <- NULL 
  yval <- NULL
  Reads <- NULL
  norm <- (ylab == "Percentage")
  aggmat <- MRcounts(aggdat, norm = norm)
  phenoTable <- pData(aggdat)
  ordmat <- aggmat
  xlab <- as.character(x_var)
  
  numofcols <- nrow(ordmat) - 1
  ## combine all other features as "other"
  if (nrow(aggmat) > (max(ind) + 1)) {
    ordmat <- rbind(aggmat[ind, ], 
                    other = colSums(aggmat[seq((max(ind) + 1),nrow(aggmat))
                                           , ]))
    numofcols <- max(ind)
  } 
  
  ## prepare datastructure for plotting: join with relevant phenodata
  df <- reshape2::melt(ordmat, varnames = c(level, "samname"), 
                       value.name = "Reads")
  df[, level] <- as.factor(df[, level])
  df[, "samname"] <- as.factor(df[, "samname"])
  
  df2 <- buildPlottingDF(df = df, 
                         phenoTable = phenoTable,
                         x_var = x_var, 
                         facet1 = facet1,
                         facet2 = facet2)
  
  ## find facet levels or add "nofacets"
  if (!is.null(facet1)) {
    facetvals <- levels(df2[, facet1])
    xlab <- paste0("", "<br>", xlab, "<br>", 
                   paste(facet1, facetvals, sep = " "))
  } else {
    facet1 <- "nofacet"
    df2$nofacet <- "nofacets"
    facetvals <- "nofacets"
  }
  
  ## find facet levels or add "nofacets:
  if (!is.null(facet2)) {
    facetvals2 <- levels(df2[, facet2])
  } else {
    facet2 <- "nofacet2"
    df2$nofacet2 <- "nofacets"
    facetvals2 <- "nofacets"
  }
  
  ## define color palette
  pal <- grDevices::colorRampPalette(
    c(RColorBrewer::brewer.pal(min(numofcols, 12), "Paired")))
  colvalues <- c(pal(numofcols), "gray")
  
  ## get yval for every group to be shown (by feature, x_var and both 
  ## facets as needed)
  ## mean (of x_var group) used for numbers

  df2 <- df2 %>%
    dplyr::group_by(Family = get(level), x_var = get(x_var), 
                    facets = get(facet1), facet2s = get(facet2)) %>%
    dplyr::summarise(yval = mean(Reads, na.rm = TRUE))
  
  collevels <- levels(df2[,"Family"])
  
  ## percentage (of x_var group) for percentage
  if (ylab == "Percentage") {
    dftotal <- df2 %>%
      dplyr::group_by(x_var, facets, facet2s) %>%
      dplyr::summarise(total = sum(yval))
    df2 <- suppressMessages(dplyr::left_join(df2, dftotal))
    df2$yval <- ifelse(df2$total == 0, 0, (df2$yval / df2$total) * 100)
  }
  
  
  ## get number of values for each facet element
  ## this determines how much of the total width each facet gets
  ## determine number of elements in each facets to set widths
  widths <- getWidths(df2, facets = "facets", x_var = "x_var")
  
  df2$text <- paste0(
    paste0("<b>", df2$Family, "</b>"),
    paste0("<br>", x_var, ": ", df2$x_var),
    paste0(
      "<br>", ylab, ": ",
      round(df2$yval, digits = getOption("me.round_digits")),
      'if'(ylab == "Percentage", "%", "")
    )
  )
  
  xaxis_text <- ""
  maxj <- length(facetvals2)
  totalheight <- pheight + 300 * length(facetvals2)
  
  legendlevel <- getLegendLevel(df2, facets = "facets", facet2s = "facet2s")
  
  
  ## iterate over facet2 around facet1
  ## for each facetvalue in facet2 we need a row
  ## for each facetvalue in facet1 we need a column
  plotlist <- lapply(facetvals2, function(fv2) {
    # group and filter for each facetvalue in facet2
    j <- which(facetvals2 %in% fv2)
    sdf <- df2 %>%
      dplyr::group_by(facet2s) %>%
      dplyr::filter(facet2s %in% fv2) %>%
      droplevels()
    
    if (facet2 != "nofacet2") {
      ylab <- 'if'(is.na(fv2),
                   paste0("NA<br>", ylab),
                   paste0(facet2, " ", fv2, "<br>", ylab)
      )
    }
    
    ## for each facetvalue in facet1, we need a column
    p <- lapply(facetvals, function(f) {
      ## group and filter for each facetvalue in facet1 given a specific 
      ## facet2
      i <- which(facetvals %in% f)
      sp <- sdf %>%
        dplyr::group_by(facets) %>%
        dplyr::filter(facets %in% f)
      
      showL <- FALSE
      ## We need to add xaxis labels in the last row only
      if (j == maxj) {
        xaxis_text <- xlab[i]
        if (f == legendlevel) {
          ##need to find first non-empty facet
          showL <- TRUE
        }
      }
      
      ## no values available for this combination of facet2 and facet1:
      ## draw empty plot
      if (nrow(sp) == 0) {
        return(buildEmptyPlotlyPlot(xaxis_text, ylab))
      }
      
      ## something to plot is available; drop all other levels
      sp <- sp %>% droplevels()
      ## make stacked barplot for this specific combination of facet2 
      ## and facet1
      sp <- sp %>%
        plotly::plot_ly(
          x = ~x_var, y = ~yval, hoverinfo = "text", 
          height = totalheight, width = pwidth,
          color = ~Family, showlegend = showL, 
          legendgroup = ~Family
        ) %>%
        plotly::add_bars(
          colors = colvalues,
          text = ~text,
          hoverlabel = list(
            font = list(
              color = "black"
            ),
            bordercolor = "black",
            borderwidth = 2
          )
        ) %>%
        plotly::layout(barmode = "stack") %>%
        add_plotly_layout(plotTitle = plotTitle, 
                          xaxis_text = xaxis_text, 
                          ylab = ylab)
      sp
    })
    ## combine all columns in one row
    plotly::subplot(p, nrows = 1, shareY = TRUE, 
                    titleX = TRUE, widths = widths)
  })
  
  p <- plotly::subplot(plotlist, nrows = length(plotlist), 
                       shareY = FALSE, titleX = TRUE, titleY = TRUE) %>%
    add_plotly_config()
  
  return(p)
}




#' Plot features
#'
#' This function plots the reads of a particular feature or set of features.
#'
#' @param aggdat aggregated MRExperiment
#' @param feature Feature to plot.
#' @param x_var Phenotype to aggregate over on X-axis. Default by "SAMPLE_ID".
#' @param ind Indices of top abundant features to plot. Needed to determine 
#' appropriate color
#' @param plotTitle Plot title. Default shows no title.
#' @param ylab Y-axis label. Default is "Reads"
#' @param xlab X-axis label. If NULL, x_var will be used as label.
#' @param facet1 Phenotype for facet 1.
#' @param facet2 Phenotype for facet 2.
#' @param log Log2 transform data. Default is FALSE.
#' @param showPoints add points for each sample on plot
#' @param fixedHeight sets a specific plot height (differential analysis)
#' @param x_levels restrict to specific levels of x_var (differential analysis)
#' @param pwidth overall plot width; default is 650 
#' 
#' @return plotly plot object
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotSingleFeature(aggdat, feature = "Prevotella", x_var = "diet")
#'
#' @export
plotSingleFeature <- function(aggdat, feature = "other", x_var = "SAMPLE_ID",
                              ind = seq_len(10), plotTitle = NULL, 
                              ylab = "Reads",
                              xlab = NULL,
                              facet1 = NULL, facet2 = NULL,
                              log = FALSE, showPoints = FALSE,
                              fixedHeight = NULL, x_levels = NULL,
                              pwidth = 500) {
  facets <- NULL
  facet2s <- NULL
  norm <- (ylab == "Percentage")
  aggmat <- MRcounts(aggdat, norm = norm)
  phenoTable <- pData(aggdat)
  if (log == TRUE) {
    aggmat <- log2(aggmat + 1)
    ylab <- paste0("Log ", ylab)
  }
  
  numofcols <- min(nrow(aggmat),max(ind))
  if ((feature == "other") && (nrow(aggmat) > (max(ind) + 1))) {
    aggmat <- rbind(aggmat[ind, ], 
                    other = colSums(aggmat[seq((max(ind) + 1),
                                               nrow(aggmat))
                                           , ]))
    numofcols <- max(ind) 
  }
  
  feat_pos <- match(feature, rownames(aggmat))
  ## feature not found; this happens when data is reaggregated in App
  if (is.na(feat_pos)) {
    return(NULL)
  }
  
  df <- data.frame(samname = colnames(aggmat), 
                   yval = aggmat[feat_pos, ],
                   total = colSums(aggmat))
  
  df2 <- buildPlottingDF(df = df, 
                         phenoTable = phenoTable,
                         x_var = x_var, 
                         facet1 = facet1,
                         facet2 = facet2)
  
  if(!is.null(x_levels)){
    df2 <- df2 %>%
      dplyr::filter(get(x_var) %in% x_levels)
  }
  
  if(is.null(xlab))
    xlab <- as.character(x_var)
  
  ## get facetvalues or set as "nofacets"
  if (!is.null(facet1)) {
    facetvals <- levels(df2[, facet1])
    xlab <- paste0("", "<br>", xlab, "<br>", 
                   paste(facet1, facetvals, sep = " "))
  } else {
    facet1 <- "nofacet"
    df2$nofacet <- "nofacets"
    facetvals <- "nofacets"
  }
  
  ## get facetvalues or set as "nofacets"
  if (!is.null(facet2)) {
    facetvals2 <- levels(df2[, facet2])
  } else {
    facet2 <- "nofacet2"
    df2$nofacet2 <- "nofacets"
    facetvals2 <- "nofacets"
  }
  
  
  ##TODO: fix colors here
  ## set up color palette (same as relative abundance for color consistency)
  pal <- grDevices::colorRampPalette(
    c(RColorBrewer::brewer.pal(min(numofcols, 12), "Paired")))
  colvalues <- c(pal(numofcols), "gray")
  
  if (feat_pos <= max(ind)) {
    colvalues <- colvalues[feat_pos]
  } else {
    colvalues <- "gray"
  }
  
  ## group data by given settings: x_var, facet1 and facet2
  df2 <- df2 %>%
    dplyr::group_by(x_var = get(x_var), 
                    facets = get(facet1), facet2s = get(facet2))
  
  ## percentage (of x_var group) for percentage
  if (ylab == "Percentage") {
    df2$yval <- ifelse(df2$total == 0, 0, (df2$yval / df2$total) * 100)
  }
  
  ## add name of feature dataframe columns
  df2$Feature <- feature
  df2$text <- paste0("<b>",df2$samname,"</b><br />",
                     df2$Feature,": ",
                     round(df2$yval, digits = getOption("me.round_digits")))
  
  ## determine number of elements in each facets to set widths
  drop <- TRUE
  if(!is.null(x_levels))
    drop <- FALSE
  widths <- getWidths(df2, facets = "facets", x_var = "x_var", drop = drop)
  
  if (showPoints) {
    showPoints <- "all"
  }
  
  xaxis_text <- ""
  maxj <- length(facetvals2)
  totalheight <- 120 + 300 * length(facetvals2)
  if(!is.null(fixedHeight)){
    totalheight <- fixedHeight
  }
  
  ## iterate over facet2 around facet1
  ## for each facetvalue in facet2 we need a row
  plotlist <- lapply(facetvals2, function(fv2) {
    j <- which(facetvals2 %in% fv2)
    sdf <- df2 %>%
      dplyr::group_by(facet2s) %>%
      dplyr::filter(facet2s %in% fv2) %>%
      droplevels()
    
    if (facet2 != "nofacet2") {
      ylab <- 'if'(is.na(fv2),
                   paste0("NA<br>", ylab),
                   paste0(facet2, " ", fv2, "<br>", ylab)
      )
    }
    ## iterating over facet1: a column for each
    p <- lapply(facetvals, function(f) {
      i <- which(facetvals %in% f)
      sp <- sdf %>%
        dplyr::group_by(facets) %>%
        dplyr::filter(facets %in% f)
      ## add xaxis labels in last row
      if (j == maxj) {
        xaxis_text <- xlab[i]
      }
      ## no entries: return empty plot
      if (nrow(sp) == 0) {
        return(buildEmptyPlotlyPlot(xaxis_text, ylab))
      }
      sp <- sp %>% droplevels()
      sp <- sp %>%
        plotly::plot_ly(
          x = ~x_var, y = ~yval, color = ~Feature, 
          hoverinfo = "text", height = totalheight, width = pwidth,
          showlegend = FALSE, legendgroup = ~Feature
        ) %>%
        plotly::add_boxplot(
          colors = colvalues,
          boxpoints = showPoints,
          text = ~text,
          pointpos = 0
        ) %>%
        add_plotly_layout(plotTitle = plotTitle, 
                          xaxis_text = xaxis_text, 
                          ylab = ylab)
      sp
    })
    plotly::subplot(p, nrows = 1, shareY = TRUE, 
                    titleX = TRUE, widths = widths)
  })
  
  ## nrows will be determined by levels of facet2
  p <- plotly::subplot(plotlist, nrows = length(plotlist), 
                       shareY = FALSE, titleX = TRUE, titleY = TRUE) %>%
    plotly::layout(showlegend = TRUE) %>%
    add_plotly_config()
  
  return(p)
}


#' Plot alpha diversity
#'
#' This function plots the alpha diversity. See ?vegan::diversity for details
#' on the available index
#'
#' @param aggdat aggregated MRExperiment
#' @param level Feature level
#' @param index Diversity index, one of "shannon", "simpson", "invsimpson" or 
#' "richness" (=number of features). Default is "shannon".
#' @param x_var Phenotype to aggregate over on X-axis. Default by "SAMPLE_ID".
#' @param ylab Y-axis label. Default is "Reads".
#' @param col_by Phenotype for coloring.
#' @param facet1 Phenotype for facet 1.
#' @param facet2 Phenotype for facet 2. 
#' @param plotTitle Plot title. By default, no title is used.
#' @param pwidth overall plot width; default is 650 
#' @param pheight overall plot height; default is 150
#'
#' @return plotly plot object
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotAlpha(aggdat, level = "genus", index = "shannon", x_var = "diet")
#'
#' @export
plotAlpha <- function(aggdat, level,
                      index = c("shannon", "simpson", 
                                "invsimpson", "richness"),
                      x_var = "SAMPLE_ID", ylab = index,
                      col_by = NULL, facet1 = NULL, facet2 = NULL,
                      plotTitle = "", pwidth = 500, pheight = 150) {
  facets <- NULL
  facet2s <- NULL
  aggmat <- MRcounts(aggdat)
  phenoTable <- pData(aggdat)
  
  ## DETERMINE ALPHA DIVERSITY 
  if (index == "richness") {
    mat <- colSums(aggmat > 0)
    Index <- "Num of Features Observed"
  } else {
    mat <- vegan::diversity(aggmat, index = index, MARGIN = 2)
    Index <- paste0(stringr::str_to_title(index), "_Diversity")
  }
  
  df <- data.frame(names(mat), mat)
  
  colnames(df) <- c("samname", "Diversity")
  legendtitle <- ""
  
  if (!is.null(col_by)) {
    legendtitle <- list(yref = "paper",xref = "paper",
                        y=1.06,x=1.2, 
                        text = col_by,showarrow = FALSE)
    col_name <- paste0(col_by, "_color")
    pwidth <- pwidth + 150
  } else {
    col_name <- NULL
  }
  xlab <- as.character(x_var)
  
  df2 <- buildPlottingDF(df = df, 
                         phenoTable = phenoTable,
                         x_var = x_var, 
                         facet1 = facet1,
                         facet2 = facet2,
                         col_by = col_by,
                         col_name = col_name)
  col_by <- col_name
  
  ## set up for facets and color
  if (!is.null(facet1)) {
    facetvals <- levels(df2[, facet1])
    xlab <- paste0("", "<br>", xlab, "<br>", 
                   paste(facet1, facetvals, sep = " "))
  } else {
    facet1 <- "nofacet"
    df2$nofacet <- "nofacets"
    facetvals <- "nofacets"
  }
  
  if (!is.null(facet2)) {
    facetvals2 <- levels(df2[, facet2])
  } else {
    facet2 <- "nofacet2"
    df2$nofacet2 <- "nofacets"
    facetvals2 <- "nofacets"
  }
  
  collevels <- levels(df2[,col_by])
  ## will be zero if col_by is NULL
  numofcols <- length(collevels)
  colvalues <- getOption("me.colorvalues")[seq(1, numofcols)]
  if (is.null(col_by)) {
    col_by <- "nocolor"
    collevels <- ""
    df2$nocolor <- index
    colvalues <- "#a5a39f"
  } 
  
  df2$text <- paste0(
    "<b>",df2$samname,"</b><br />",
    round(df2$Diversity, digits = getOption("me.round_digits")))

  df2 <- df2 %>%
    dplyr::group_by(x_var = get(x_var), 
                    facets = get(facet1), facet2s = get(facet2))
  
  ## determine number of elements in each facets to set widths
  widths <- getWidths(df2, facets = "facets", x_var = "x_var")
  
  xaxis_text <- ""
  yaxis_text <- Index
  maxj <- length(facetvals2)
  totalheight <- pheight + 300 * length(facetvals2)
  
  ## iterate over facet2 around facet1
  plotlist <- lapply(facetvals2, function(fv2) {
    j <- which(facetvals2 %in% fv2)
    sdf <- df2 %>%
      dplyr::group_by(facet2s) %>%
      dplyr::filter(facet2s %in% fv2)
    
    
    if (facet2 != "nofacet2") {
      yaxis_text <- 'if'(is.na(fv2),
                         paste0("NA<br>", Index),
                         paste0(facet2, " ", fv2, "<br>", Index)
      )
    }
    ## iterating over facet1
    p <- lapply(facetvals, function(f) {
      i <- which(facetvals %in% f)
      sp <- sdf %>%
        dplyr::group_by(facets) %>%
        dplyr::filter(facets %in% f)
      if (j == maxj) {
        xaxis_text <- xlab[i]
      }
      if (nrow(sp) == 0) {
        return(buildEmptyPlotlyPlot(xaxis_text, ylab))
      }
      sp <- sp %>%
        dplyr::mutate(colorfact = as.factor(get(col_by)))
      present_levels <- unique(sp$colorfact)
      showL <- FALSE
      if(any(present_levels %in% collevels))
        showL <- TRUE
      sp <- sp %>%
        plotly::plot_ly(
          x = ~x_var, y = ~Diversity, color = ~colorfact, 
          hoverinfo = "text", height = totalheight, width = pwidth,
          showlegend = showL, legendgroup = ~colorfact
        ) %>%
        plotly::add_boxplot(colors = colvalues, 
                            boxpoints = "all", 
                            text = ~text,
                            pointpos = 0) %>%
        add_plotly_layout(plotTitle = plotTitle, 
                          xaxis_text = xaxis_text, 
                          ylab = yaxis_text)
      ## using global environment assignment as we are substituting in place 
      collevels <<- collevels[!collevels %in% present_levels]
      sp
    })
    plotly::subplot(p, nrows = 1, shareY = TRUE, 
                    titleX = TRUE, widths = widths)
  })
  
  p <- plotly::subplot(plotlist, nrows = length(plotlist), 
                       shareY = FALSE, titleX = TRUE, titleY = TRUE) %>%
    plotly::layout(showlegend = TRUE,
                   colorway = getOption("me.colorvalues")) %>%
    add_plotly_config()
  
  return(p)
}

## DIFFERENTIAL ANALYSIS


#' Produce design matrix of pairwise comparisons
#'
#' This function takes in the levels of a factor phenotype and outputs a
#' design matrix of all pairwise comparisons.
#'
#' @param levels Character vector of the levels of a factor phenotype
#'
#' @return A model matrix
designPairs <- function(levels) {
  n <- length(levels)
  design <- matrix(0, n, choose(n, 2))
  rownames(design) <- levels
  colnames(design) <- seq_len(choose(n, 2))
  k <- 0
  for (i in seq_len(n - 1)) {
    for (j in (i + 1):n) {
      k <- k + 1
      design[j, k] <- 1
      design[i, k] <- -1
      colnames(design)[k] <- paste(levels[j], "-", levels[i], sep = "")
    }
  }
  design
}


#' Performs differential abundance testing
#'
#' This function performs differential abundance testing between groups of a
#' specified phenotype. Four methods are available: limma, Kruskal-Wallis, 
#' ZILN and DESeq2 (see details).
#'
#' limma is a differential expression tool for microarray data using
#' linear models. It can also be applied to microbiome data.
#'
#' The Kruskal-Wallis test is a non-parametric rank test which examines if 
#' groups come from the same distribution. A significant result indicates at 
#' least one group is distributionally different than another group.
#'
#' ZILN is a zero-inflated log-normal model implemented in
#' \code{\link[metagenomeSeq]{fitFeatureModel}()} of the \code{metagenomeSeq}
#' package.
#' 
#' DeSeq2 performs differential gene expression analysis based on the negative 
#' binomial distribution
#'
#' @param aggdat aggregated MRExperiment
#' @param level Feature level.
#' @param phenotype Phenotype to test.
#' @param phenolevels levels of the phenotype to restrict the comparison to
#' @param log Log2 transform data. Default is TRUE.
#' @param method Differential testing method. One of "limma" (default),
#' "Kruskal-Wallis", or "DESeq2".
#' @param coef Numeric which indicates which pairwise comparison to analyze
#' when there are more than two groups. Corresponds to the column number of the
#' model matrix produced by \code{\link{designPairs}()}. If NULL, a test of any
#' difference between all groups is performed.
#'
#' @importFrom metagenomeSeq fitFeatureModel MRtable MRcounts
#' @importFrom Biobase pData
#' 
#' @return data.frame holding results of the differential analysis
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' runDiffTest(aggdat = aggdat,level = "genus", 
#'             phenotype = "diet", method = "Kruskal-Wallis")
#'
#' @export
runDiffTest <- function(
  aggdat, level, phenotype, phenolevels = NULL,
                        log = TRUE, coef = NULL,
                        method = c("limma", 
                                   "Kruskal-Wallis", 
                                   #"ZILN", 
                                   "DESeq2")) {
  phenoTable <- pData(aggdat)
  method <- match.arg(method)
  
  phenoTypeDF <- phenoTable %>% dplyr::select(phenotype)
  pd <- forcats::fct_explicit_na(
    factor(phenoTypeDF[,phenotype]),
    na_level = "NA")
  chosenlevels <- pd
  na <- integer(0)
  if (!is.null(phenolevels)) {
    if (length(phenolevels) < 2) {
      stop("Only one phenolevel specified.")
    }
    chosenlevels <- pd %in% phenolevels
    na <- which(is.na(pd) | !chosenlevels)
    pd <- factor(pd[pd %in% phenolevels])
  } else if (length(levels(pd)) < 2) {
    stop("Phenotype only contains one group.")
  }
  
  if(length(levels(pd)) > 2)
    stop("Please restrict to two phenolevels for differential testing")
  
  norm <- (method != "DESeq2")
  aggmat <- MRcounts(aggdat, norm = norm)
  
  if (length(na) > 0) {
    aggmat <- aggmat[, -na]
    phenoTypeDF <- phenoTypeDF %>%
      dplyr::filter(!dplyr::row_number() %in% na)
  }
  
  if (log && !(method %in% "DESeq2")) {
    aggmat <- log2(aggmat + 1)
  }
  
  design <- stats::model.matrix(~ 0 + pd)
  colnames(design) <- levels(pd)
  
  if(method == "DESeq2"){
    mod <- stats::as.formula(paste("~",phenotype,sep = ""))
    dds <- DESeq2::DESeqDataSetFromMatrix(countData = aggmat, 
                                          colData = phenoTypeDF, 
                                          design = mod)
    dds <- DESeq2::estimateSizeFactors(object = dds, type = "poscounts")
    dds <- DESeq2::DESeq(
      dds, 
      fitType = "parametric", 
      test = "Wald", 
      quiet = TRUE)
    out <- DESeq2::results(
      dds, 
      independentFiltering = FALSE, 
      cooksCutoff = FALSE)
    #removing ZILN until lmFit adjusts error check back to ==
  # } else if (method == "ZILN") {
  #   good.ind <- which(rowSums(design[, c(1,2)]) == 1)
  #   ## app requires selection of specific phenolevels instead
  #   if (length(levels(pd)) > 2) {
  #     warning("Only using first two levels of phenotype")
  #   }
  #   design2 <- stats::model.matrix(~pd)[good.ind, c(1,2)]
  #   names(dim(design2)) <- c("Samples","")
  #   fit <- fitFeatureModel(aggdat[, good.ind],
  #                          mod = design2,
  #                          spos = FALSE
  #   )
  #   out <- MRtable(fit, group = 3, number = Inf)
  } else if (method == "limma") {
    fit <- limma::lmFit(as.data.frame(aggmat), design)
    contrast.matrix <- designPairs(colnames(design))
    fit2 <- limma::contrasts.fit(fit, contrast.matrix)
    fit2 <- limma::eBayes(fit2)
    out <- limma::topTable(fit2, coef = coef, adjust = "BH", number = Inf)
    
    if (is.null(coef)) {
      colnames(out)[seq_len(ncol(contrast.matrix))] <- 
        colnames(contrast.matrix)
    } else {
      colnames(out)[1] <- colnames(contrast.matrix)[coef]
    }
    
  } else if (method == "Kruskal-Wallis"){ 
    fit <- limma::lmFit(as.data.frame(aggmat), design)
    contrast.matrix <- designPairs(colnames(design))
    fit2 <- limma::contrasts.fit(fit, contrast.matrix)
    fit2 <- limma::eBayes(fit2)
    out <- limma::topTable(fit2, coef = coef, adjust = "BH", number = Inf)
    
    colnames(out)[seq_len(ncol(contrast.matrix))] <- colnames(contrast.matrix)
    kw <- base::apply(
      aggmat, 1, function(x) broom::tidy(stats::kruskal.test(x, pd)))
    ## bind together and add row names
    kw.df <- dplyr::bind_rows(kw)
    kw.df$names <- names(kw)
    kw.df <- kw.df %>% tibble::column_to_rownames("names")
    matched <- match(rownames(out), rownames(kw.df))
    if ("F" %in% colnames(out)) {
      out$F <- kw.df$statistic[matched]
      colnames(out)[colnames(out) == "F"] <- "KW-Statistic"
    } else {
      out$t <- kw.df$statistic[matched]
      colnames(out)[colnames(out) == "t"] <- "KW-Statistic"
    }
    out$P.Value <- kw.df$p.value[matched]
    out$adj.P.Val <- stats::p.adjust(out$P.Value, method = "BH")
    out <- out[order(out$adj.P.Val), ]
  }
  out2 <- data.frame(rownames(out), out, check.names = FALSE) %>%
    dplyr::mutate_if(is.numeric, round, digits = getOption("me.round_digits"))
  colnames(out2)[1] <- level
  return(out2)
}

data("mouseData")
aggdat <- aggFeatures(mouseData, level = "genus")

test_that("run differential test - limma", {
  limMD <- runDiffTest(aggdat = aggdat,
              level = "genus",
              phenotype = "diet",
              method = "limma")
  expect_equal(names(limMD),c("genus", "Western-BK", "AveExpr", "t", "P.Value", "adj.P.Val", "B"))
  limMD <- limMD[order(limMD$`Western-BK`),]
  expect_equal(as.character(limMD[1,"genus"]),"Prevotella")
  expect_equal(as.character(limMD[nrow(limMD),"genus"]),"Enterococcus")
})

test_that("run differential test - Kruskal-Wallis", {
  kwMD <- runDiffTest(aggdat = aggdat,
                       level = "genus",
                       phenotype = "diet",
                       method = "Kruskal-Wallis")
  expect_equal(names(kwMD),c("genus", "Western-BK", "AveExpr", "KW-Statistic", "P.Value", "adj.P.Val", "B"))
  kwMD <- kwMD[order(kwMD$`Western-BK`),]
  expect_equal(as.character(kwMD[1,"genus"]),"Prevotella")
  expect_equal(as.character(kwMD[nrow(kwMD),"genus"]),"Enterococcus")
  expect_equal(as.character(kwMD[1,"KW-Statistic"]),"92.951")
})

#ZILN IS NOT AVAILABLE ATM DUE TO ISSUE WITH LIMMA UPDATE IN METAGENOMESEQ
# test_that("run differential test - ZILN", {
#   expect_warning(runDiffTest(aggdat = aggdat,
#                              level = "genus",
#                              phenotype = "diet",
#                              method = "ZILN"))
#   zilnMD <- suppressWarnings(runDiffTest(aggdat = aggdat,
#                       level = "genus",
#                       phenotype = "diet",
#                       method = "ZILN"))
#   expect_equal(names(zilnMD),c("genus", "+samples in group 0", "+samples in group 1", "counts in group 0", "counts in group 1",
#                                "logFC", "se", "pvalues", "adjPvalues"))
#   zilnMD <- zilnMD[order(zilnMD$logFC),]
#   expect_equal(as.character(zilnMD[1,"genus"]),"Prevotella")
#   expect_equal(as.character(zilnMD[2,"genus"]),"Ruminococcus")
#   expect_equal(as.character(zilnMD[nrow(zilnMD),"genus"]),"Enterococcus")
#   expect_equal(as.character(zilnMD[1,"logFC"]),"-4.916")
# })

test_that("run differential test - DeSeq2", {
  skip_on_cran()
  deseq <- suppressWarnings(runDiffTest(aggdat = aggdat,
                      level = "genus",
                      phenotype = "diet",
                      method = "DESeq2"))
  expect_equal(names(deseq),c("genus", "baseMean", "log2FoldChange", "lfcSE", "stat", "pvalue", "padj"))
  deseq <- deseq[order(deseq$log2FoldChange),]
  expect_equal(as.character(deseq[1,"genus"]),"Prevotella")
  expect_equal(as.character(deseq[2,"genus"]),"Dorea")
  expect_equal(as.character(deseq[nrow(deseq),"genus"]),"Enterococcus")
  expect_equal(as.character(deseq[3,"log2FoldChange"]),"-4.664")
})

test_that("run differential test - phenolevels", {
  leveledDiff <- suppressWarnings(runDiffTest(aggdat = aggdat,
                                        level = "genus",
                                        phenotype = "mouseID",
                                        phenolevels = c("PM2","PM5"),
                                        method = "limma"))
  leveledDiff <- leveledDiff[order(leveledDiff$`PM5-PM2`),]
  expect_equal(as.character(leveledDiff[1,"genus"]),"Prevotella")
  expect_equal(as.character(leveledDiff[2,"genus"]),"Faecalibacterium")
  expect_equal(as.character(leveledDiff[nrow(leveledDiff),"genus"]),"Enterococcus")
  expect_equal(as.character(leveledDiff[5,"PM5-PM2"]),"-2.467")
})

#' Plot longitudinal features
#'
#' This function plots the reads of a particular feature over different time points.
#'
#' @param aggdat aggregated MRExperiment
#' @param feature Feature to plot.
#' @param x_var Phenotype to show along on X-axis.
#' @param id_var phenotype used to connect data points. Default is "SAMPLE_ID"
#' @param plotTitle Plot title. Default shows no title.
#' @param ylab Y-axis label. Default is "Reads"
#' @param log Log2 transform data. Default is FALSE.
#' @param showLines add lines between the points
#' @param fixedHeight sets a specific plot height (differential analysis)
#' @param x_levels restrict to specific levels of x_var (differential analysis)
#' @param pwidth overall plot width; default is 650 
#' 
#' @return plotly object holding long feature plot
#'
#' @author Janina Reeder, Mo Huang
#'
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotLongFeature(aggdat, feature = "Prevotella", x_var = "diet", 
#'                 id_var = "mouseID")
#'
#' @export
plotLongFeature <- function(aggdat, feature, 
                            x_var, id_var = "SAMPLE_ID",
                            plotTitle = NULL, ylab = "Reads",
                            log = FALSE, showLines = TRUE,
                            fixedHeight = NULL, x_levels = NULL,
                            pwidth = 650) {
  yval <- NULL
  norm <- (ylab == "Percentage")
  aggmat <- MRcounts(aggdat, norm = norm)
  phenoTable <- pData(aggdat)
  if (log == TRUE) {
    aggmat <- log2(aggmat + 1)
    ylab <- paste0("Log ", ylab)
  }
  
  feat_pos <- match(feature, rownames(aggmat))
  ## feature not found; this happens when data is reaggregated in App
  if (is.na(feat_pos)) {
    return(NULL)
  }
  
  df <- data.frame(samname = colnames(aggmat), 
                   yval = aggmat[feat_pos, ],
                   total = colSums(aggmat))
  
  df2 <- buildPlottingDF(df = df, 
                         phenoTable = phenoTable,
                         x_var = x_var, 
                         id_var = id_var)
  
  if(!is.null(x_levels)){
    df2 <- df2 %>%
      dplyr::filter(get(x_var) %in% x_levels)
  }
  
  ## set up color palette (same as relative abundance for color consistency)
  pal <- grDevices::colorRampPalette(c(RColorBrewer::brewer.pal(12, "Paired")))
  colvalues <- c(pal(12), "gray")
  
  if (feat_pos <= 12) {
    colvalues <- colvalues[feat_pos]
  } else {
    colvalues <- "gray"
  }
  
  ## group and summarize data by given settings: x_var, id_var  
  ## percentage (of x_var group) for percentage
  if (ylab == "Percentage") {
    df2$yval <- ifelse(df2$total == 0, 0, (df2$yval / df2$total) * 100)
  }
  
  df2 <- df2 %>%
    dplyr::group_by(x_var = get(x_var),
                    id_var = get(id_var)) %>%
    dplyr::mutate(id_mean = mean(yval, na.rm = TRUE))
  
  
  if(!is.null(x_levels)){
    df2$x_var <- forcats::fct_relevel(df2$x_var,as.character(x_levels))
  }
  
  ## add name of feature dataframe columns
  df2$Feature <- feature
  df2$text <- paste0("<b>",df2$samname,"</b><br />",
                     df2$Feature,": ",
                     round(df2$yval, digits = getOption("me.round_digits")))
  
  xaxis_text <- ""
  totalheight <- 420
  if(!is.null(fixedHeight)){
    totalheight <- fixedHeight
  }
  
  df2 <- df2 %>% 
    droplevels() %>%
    dplyr::group_by(id_var)
  
  p <- df2 %>%
    plotly::highlight_key(~id_var, group = id_var) %>%
    plotly::plot_ly(
      x = ~x_var, y = ~id_mean, color = ~Feature, 
      hoverinfo = "text", height = totalheight, width = pwidth
    ) %>%
    plotly::add_boxplot(
      colors = colvalues,
      pointpos = 0,
      showlegend = FALSE
    ) 
  if(showLines){
    p <- p %>% plotly::add_lines(
      color = I("black"),
      opacity = 0.2,
      name = id_var,
      text = ~id_var,
      showlegend = TRUE
    )
  }
  p <- p %>% plotly::add_markers(
    text = ~text,
    showlegend = FALSE
  ) %>%
    add_plotly_layout(plotTitle = plotTitle, 
                      xaxis_text = xaxis_text, 
                      ylab = ylab) %>%
    plotly::highlight(on = "plotly_click", 
                      off = "plotly_doubleclick",
                      selectize = TRUE,
                      dynamic = TRUE,
                      color = getOption("me.colorvalues"),
                      opacityDim = 0.4,
                      selected = plotly::attrs_selected(name = p$key))
  
  p <- p %>%
    plotly::layout(showlegend = TRUE)  %>%
    add_plotly_config()
  
  return(p)
}

## AVERAGE FEATURE ABUNDANCE ANALYSIS 

#' Plot average relative abundance
#'
#' This function plots the average relative abundance of the top abundant features.
#'
#' @param aggdat aggregated MRExperiment object
#' @param level Feature level.
#' @param ind Indices of top abundant features to plot. Rest of features are
#' aggregated and displayed as "other".
#' @param plotTitle Plot title. Default shows no title.
#' @param ylab Y-axis label. Default is "Reads"
#' @param facet1 Phenotype for facet 1.
#' @param facet2 Phenotype for facet 2.
#' @param source name of the plot (needed for event handling); default is "A"
#' @param pwidth overall plot width; default is 500
#' @param pheight overall plot height; default is 150
#'
#' @author Janina Reeder
#'
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#'
#' @return plotly plot
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotAvgAbundance(aggdat, level = "genus")
#'
#' @export
plotAvgAbundance <- function(aggdat, level, ind = seq_len(10), plotTitle = "", 
                          ylab = "Reads", facet1 = NULL, facet2 = NULL, 
                          source = "A", pwidth = 500, pheight = 150) {
  facets <- NULL
  facet2s <- NULL 
  yval <- NULL
  Reads <- NULL
  norm <- (ylab == "Percentage")
  aggmat <- MRcounts(aggdat, norm = norm)
  phenoTable <- pData(aggdat)
  ordmat <- aggmat
  
  ## combine all other features as "other"
  if (nrow(aggmat) > (max(ind) + 1)) {
    ordmat <- rbind(aggmat[ind, ], 
                    other = colSums(aggmat[seq((max(ind) + 1),nrow(aggmat))
                                           , ]))
  }
  
  
  df <- as.data.frame(t(ordmat))
  
  df$samname <- rownames(df)
  df2 <- buildPlottingDF(df, phenoTable, facet1 = facet1, facet2 = facet2)
  
  ## find facet levels or add "nofacets"
  if (!is.null(facet1)) {
    facetvals <- levels(df2[, facet1])
  } else {
    facet1 <- "nofacet"
    df2$nofacet <- "nofacets"
    facetvals <- "nofacets"
  }

  ## find facet levels or add "nofacets:
  if (!is.null(facet2)) {
    facetvals2 <- levels(df2[, facet2])
  } else {
    facet2 <- "nofacet2"
    df2$nofacet2 <- "nofacets"
    facetvals2 <- "nofacets"
  }
  
  ## define color palette
  pal <- grDevices::colorRampPalette(
    c(RColorBrewer::brewer.pal(min(length(ind), 12), "Paired")))
  colvalues <- c(pal(max(ind)), "gray")
  
  # percentage 
  if (ylab == "Percentage") {
    rs <- rowSums(df2[,seq(1,ncol(df2)-3)])
    ## we should not reach this. Remove samples that are all 0 to avoid NaN.
    df2 <- df2[rs != 0,]
    df2[,seq(1,ncol(df2)-3)] <- df2[,seq(1,ncol(df2)-3)]/rs * 100
  }

  
  meanvals <- stats::aggregate(
    df2[,seq(1,ncol(df2)-3)],by = list(df2[[facet1]], df2[[facet2]]), mean)
  sdvals <- stats::aggregate(
    df2[,seq(1,ncol(df2)-3)],by = list(df2[[facet1]], df2[[facet2]]), 
    function(x) stats::sd(x)/sqrt(length(x)))
  meanvals <- reshape2::melt(
    meanvals, id.vars = c("Group.1","Group.2"), value.name = "mean")
  sdvals <- reshape2::melt(
    sdvals, id.vars = c("Group.1","Group.2"), value.name = "sd")
  plotData <- dplyr::full_join(meanvals, sdvals)
  names(plotData) <- c("facets", "facet2s", "feature", "mean", "sd")

  maxj <- length(facetvals2)
  maxi <- length(facetvals)
  totalwidth <- pwidth + 200 * length(facetvals)
  totalheight <- pheight + 300 * length(facetvals2)
  xaxis_text <- ""
  yaxis_text <- ""

  ## iterate over facet2 around facet1
  ## for each facetvalue in facet2 we need a row
  ## for each facetvalue in facet1 we need a column
  plotlist <- lapply(facetvals2, function(fv2) {
    # group and filter for each facetvalue in facet2
    j <- which(facetvals2 %in% fv2)
    sdf <- plotData %>%
      dplyr::group_by(facet2s) %>%
      dplyr::filter(facet2s %in% fv2) 

    if (facet2 != "nofacet2") {
      yaxis_text <- 'if'(is.na(fv2),"",
                   paste0(facet2, " ", fv2)
      )
    }
    
    ## for each facetvalue in facet1, we need a column
    p <- lapply(facetvals, function(f) {
      ## group and filter for each facetvalue in facet1 given a specific 
      ## facet2
      i <- which(facetvals %in% f)
      sp <- sdf %>%
        dplyr::group_by(facets) %>%
        dplyr::filter(facets %in% f) 
      
      showL <- FALSE
      ## We need to add xaxis labels in the last row only
      if (j == maxj) {
        xaxis_text <- f
        if(i == maxi){
          showL <- TRUE
        }
      }
  
      ## no values available for this combination of facet2 and facet1:
      ## draw empty plot
      if (nrow(sp) == 0) {
        return(buildEmptyPlotlyPlot(xaxis_text, ylab))
      }
      
      sp <- sp %>% plotly::plot_ly(
                           x = ~mean,
                           y = ~feature,
                           color = ~feature,
                           colors = colvalues,
                           width = totalwidth,
                           height = totalheight,
                           legendgroup = ~feature) %>%
        plotly::add_bars(error_x = ~list(array = sd,
                                         color = '#000000'),
                         showlegend = showL) %>%
        add_plotly_layout(plotTitle = plotTitle, 
                          xaxis_text = xaxis_text,
                          ylab = yaxis_text)
      sp
    })
    ## combine all columns in one row
    plotly::subplot(p, nrows = 1, shareY = TRUE, 
                    shareX = TRUE, titleX = TRUE)
  })
  
  p <- plotly::subplot(plotlist, nrows = length(plotlist), 
                       shareY = FALSE, titleX = TRUE, titleY = TRUE,
                       margin = c(0.02,0.02,0.05,0.05)) %>%
      add_plotly_config()
  
  return(p)
}

## Main module for intra sample analysis

#' Intra Analysis Module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the ui code
#' 
#' @examples intraAnalysisUI("intraanalysis_id")
#' 
#' @export
intraAnalysisUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    width = 12,
    column(
      width = 3,
      intraInputUI(ns("intraInput"))
    ),
    column(
      width = 9,
      fluidRow(
        width = 11,
        box(width = 10,
            p("The intra sample page contains methods to analyze the microbial 
              composition and diversity within a sample")
        ),
        relAbundanceUI(ns("abundancePlot")),
        featAbundanceUI(ns("featurePlot")),
        alphaDiversityUI(ns("alphaDiv"))
      )
    )
  )
}


#' Intra Analysis Module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param data the main data object returned from data_input_module
#' @param levelOpts available levels to aggregate on (depends on input data)
#' @param chosenLevel previously selected level (passed from different instance)
#' @param resetInput reactive boolean determining if reset is required
#' @param aggData the aggregated MRExperiment object
#' @param normalizedData boolean indicating if normalization was done
#' 
#' @author Janina Reeder
#' 
#'
#' @return reactive holding code to be used in reports
intraAnalysis <- function(input, output, session, data, levelOpts,
                          chosenLevel, resetInput, aggData, normalizedData) {
  ## INTRA SAMPLE ANALYSIS
  ns <- session$ns
  
  
  intraSettings <- callModule(intraInput,
                              "intraInput",
                              meData = data$meData,
                              facetOptions = data$facetOptions,
                              reset = resetInput,
                              aggDat = reactive(aggData$mrobj)
  )
  
  
  relAbReact <- callModule(relAbundance,
                           "abundancePlot",
                           aggDat = reactive(aggData$mrobj),
                           featLevel = reactive(aggData$level),
                           intraSettings = intraSettings$chosenValues,
                           normalizedData = normalizedData,
                           reset = resetInput
  )
  
  featAbRep <- callModule(featAbundance,
                          "featurePlot",
                          aggDat = reactive(aggData$mrobj),
                          featLevel = reactive(aggData$level),
                          intraSettings = intraSettings$chosenValues,
                          selectedFeat = intraSettings$selectedFeat,
                          featName = relAbReact$featName,
                          numOfFeats = relAbReact$numOfFeats,
                          ylabMode = relAbReact$ylabMode,
                          normalizedData = normalizedData,
                          reset = resetInput
  )
  
  alphDivRep <- callModule(alphaDiversity,
                           "alphaDiv",
                           colorOptions = data$facetOptions,
                           aggDat = reactive(aggData$mrobj),
                           featLevel = reactive(aggData$level),
                           intraSettings = intraSettings$chosenValues,
                           reset = resetInput
  )
  
  ## store intra analysis code for reports
  intraRep <- reactive({
    list(
      "relab" = relAbReact$repCode(),
      "featab" = featAbRep(),
      "alpha" = alphDivRep()
    )
  })
  
  return(intraRep)
  
}

#' Main intra analysis input module.
#' Set up to handle all analysis tabs in the app depending on given parameters
#'
#' @param id element identifier - namespace
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
intraInputUI <- function(id) {
  ns <- NS(id)
  
  box(
    width = 12, id = ns("analysisbox"),
    h4("ANALYSIS PARAMETERS"),
    
    selectInput(
      ns("xvariable"),
      label = "Phenotype", choices = "", 
      multiple = FALSE, 
      selectize = FALSE, width = "250px"
    ),
    div(
      selectInput(
        ns("facetby"),
        label = "Facet columns by", choices = "", 
        multiple = FALSE, 
        selectize = FALSE, width = "250px"
      ),
      selectInput(
        ns("facetby2"),
        label = "Facet rows by", choices = "", 
        multiple = FALSE, 
        selectize = FALSE, width = "250px"
      )
    ),
    selectizeInput(
      ns("featureselect"),
      label = "Select Feature", choices = "", 
      multiple = FALSE, 
      width = "250px"
    ),
    ## buttons are used to submit events. This ensures plots are not redrawn 
    ## while user still adjusts parameters
    div(
      id = ns("buttondiv"),
      fluidRow( 
        width = 12, id = "actionbuttonrow",
        ## update analysis outputs (plots/tables)
        actionButton(
          ns("updatebutton"), 
          icon = icon("far fa-chart-bar"),
          label = HTML("&nbsp;UPDATE"), 
          width = "120px"),
        actionButton(
          ns("reportButton"), 
          label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
          width = "120px")
      )
    )
  )
}


#' Server side for the intra analysis input module
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment object storing all data
#' @param facetOptions named vector of available facet choices
#' @param reset reactive boolean determining if all inputs should be reset 
#' @param aggDat aggregated MRExperiment object (default is NULL)
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
intraInput <- function(input, output, session, 
                       meData, facetOptions = NULL, 
                       reset, aggDat = reactive(NULL)) {
  ns <- session$ns
  
  ## reactive value storing choices made in the UI
  chosenValues <- reactiveVal(NULL)
  ## specific reactive for just the feature selection
  ## this is required to only update the feature plot upon click/selection
  selectedFeat <- reactiveVal(NULL)
  ## reactive storing sorted features at given level
  aggFeatures <- reactiveVal(NULL)
  
  observe({
    req(aggDat())
    aggFeatures(rownames(MRcounts(aggDat())))
    shinyjs::enable("savebutton")
  })
  
  ## reset all entries
  observe({
    req(reset())
    chosenValues(NULL)
    selectedFeat(NULL)
    aggFeatures(NULL)
    shinyjs::disable("savebutton")
  })
  
  
  ## initialize input elements
  observe({
    req(meData())
    updateSelectInput(session, "xvariable", 
                      choices = names(pData(meData())),
                      selected = chosenValues()$xvariable)
    updateSelectInput(session, "facetby", 
                      choices = c("", facetOptions()), 
                      selected = chosenValues()$facetby)
    updateSelectInput(session, "facetby2", 
                      choices = c("", facetOptions()), 
                      selected = chosenValues()$facetby2)
  })
  
  ## react to main phenotype variable (facets should be distinct)
  observeEvent(input$xvariable, {
    req(facetOptions())
    facetopts <- facetOptions()
    facetopts <- facetopts[!facetopts %in% input$xvariable]
    updateSelectInput(session, "facetby", choices = c("", facetopts))
    updateSelectInput(session, "facetby2", choices = c("", facetopts))
  }, priority = 15, ignoreInit = TRUE)
  
  ## react to first facet variable (phenotype variables should be distinct)
  observeEvent(input$facetby, {
    req(facetOptions())
    facetopts <- facetOptions()
    facetopts <- facetopts[!facetopts %in% input$facetby]
    xopts <- names(pData(meData()))
    xopts <- xopts[!xopts %in% input$facetby]
    facetopts <- facetopts[!facetopts %in% input$xvariable]
    updateSelectInput(session, "xvariable", choices = xopts, 
                      selected = input$xvariable)
    updateSelectInput(session, "facetby2", choices = c("", facetopts), 
                      selected = input$facetby2)
  }, priority = 15, ignoreInit = TRUE)
  
  ## react to second facet variable (phenotype variables should be distinct)
  observeEvent(input$facetby2, {
    req(facetOptions())
    facetopts <- facetOptions()
    facetopts <- facetopts[!facetopts %in% input$facetby2]
    xopts <- names(pData(meData()))
    xopts <- xopts[!xopts %in% input$facetby2]
    facetopts <- facetopts[!facetopts %in% input$xvariable]
    updateSelectInput(session, "xvariable", choices = xopts, 
                      selected = input$xvariable)
    updateSelectInput(session, "facetby", choices = c("", facetopts), 
                      selected = input$facetby)
  }, priority = 15, ignoreInit = TRUE)
  
  ## update any inputs based on feature values
  observeEvent(aggFeatures(),{
    req(aggFeatures())
    featdata  <- data.frame(value = aggFeatures(), 
                            label = aggFeatures())
    selfeat <- ""
    if(!is.null(chosenValues()$featureselect))
      selfeat <- chosenValues()$featureselect
    updateSelectizeInput(session, "featureselect",
                         choices = featdata,
                         selected = selfeat,
                         options = list(placeholder = "Select Feature"),
                         server = TRUE)
  })
  
  ## main control button: store input choices in chosenValues
  observeEvent(input$updatebutton, {
    facet1 <- NULL
    if(!is.null(input$facetby)){
      if(input$facetby != "")
        facet1 <- input$facetby
    }
    facet2 <- NULL
    if(!is.null(input$facetby2)){
      if(input$facetby2 != "")
        facet2 <- input$facetby2
    }
    cV <- list("xvariable" = input$xvariable,
               "facetby" = facet1,
               "facetby2" = facet2)
    chosenValues(cV)
  })
  
  
  
  ## selected feature is stored specifically to update feature plot
  observeEvent(input$featureselect, {
    selectedFeat(input$featureselect)
  }, ignoreInit = TRUE)
  
  return(list(chosenValues = chosenValues, selectedFeat = selectedFeat))
}

## INTER SAMPLE ANALYSIS 


#' Function to compute the distance matrix using vegdist from the vegan package
#'
#' @param aggdat aggregated MRExperiment
#' @param dist_method distance method from vegan package (See ?vegan::vegdist for details)
#' @param log transform count matrix to log2; default is TRUE
#' @param nfeatures number of features to use; default is all
#'
#' @return distance as dist
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' computeDistMat(aggdat, dist_method = "bray")
#' 
#' @export
computeDistMat <- function(aggdat, dist_method, 
                           log = TRUE, nfeatures = nrow(aggmat)){
  aggmat <- MRcounts(aggdat, norm = TRUE)
  mat <- aggmat[order(matrixStats::rowSds(aggmat), decreasing = TRUE), ]
  mat <- mat[seq(1,nfeatures), ]
  if (log == TRUE) {
    mat <- log2(mat + 1)
  }
  
  mat <- t(mat)
  vegan::vegdist(mat, method = tolower(dist_method))
}


#' Function to compute the PCAs for a given distance matrix
#'
#' @param distmat the distance matrix
#' @param pcas 2-element vector of PCAs to include in results
#'
#' @return the x slot limited to pcas after calling stats::prcomp on distmat
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' distmat <- computeDistMat(aggdat, dist_method = "bray")
#' calculatePCAs(distmat, c(1,2))
#' 
#' @export
calculatePCAs <- function(distmat, pcas){
  pcaRes <- stats::prcomp(distmat)
  pcaRes$x[,pcas]
}


#' Plot beta diversity
#'
#' This functions plots the beta diversity as a PCoA plot.
#'
#' @param aggdat aggregated MRExperiment
#' @param dim Vector of length 2 specifying which dimensions to plot.
#' @param log Log2 transform data. Default is TRUE.
#' @param dist_method Which distance method to use. See ?vegan::vegdist for more
#' \code{\link[vegan]{vegdist}()} for options. Default is "bray".
#' @param pcas precalculated pcas to avoid recalculation via CalcPCs
#' @param nfeatures Number of top features in terms of standard deviation.
#' Default is all.
#' @param col_by Phenotype for coloring.
#' @param shape_by Phenotype for shape. 
#' @param plotTitle Plot title. By default, becomes PCoA (code{dist.method}).
#' @param xlab X-axis label. By default, shows dimension and percent variance
#' explained.
#' @param ylab Y-axis label. By default, shows dimension and percent variance
#' explained.
#' @param pt_size the size of the markers
#' @param plotText adonis text to be added to plot
#' @param confInterval numeric value indicating confidence level for ellipses
#' @param allowWebGL boolean indicating if WebGL should be used 
#' @param pwidth overall plot width; default is 550 (125 are added for legend)
#' @param pheight overall plot height; default is 550
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#'
#' @return plotly plot object
#' 
#' @examples
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotBeta(aggdat)
#'
#' @export
plotBeta <- function(aggdat, dim = c(1,2),
                     log = TRUE, dist_method = "bray", pcas = NULL,
                     nfeatures = nrow(aggdat), col_by = NULL, shape_by = NULL,
                     plotTitle = "", xlab = NULL, ylab = NULL, pt_size = 8,
                     plotText = NULL, confInterval = NULL, allowWebGL = TRUE,
                     pwidth = 550, pheight = 550) {
  color <- NULL
  aggmat <- MRcounts(aggdat, norm = TRUE)
  phenoTable <- pData(aggdat)
  
  legendtitle <- ""
  
  if(is.null(pcas)){
    distmat <- computeDistMat(aggdat,
                              dist_method = dist_method,
                              log = log,
                              nfeatures = nfeatures)
    pcaRes <- stats::prcomp(distmat)
    pcas <- pcaRes$x[, dim]
  }
  
  legend <- FALSE
  testellipses <- NULL
  if (!is.null(col_by)) {
    colgroup <- forcats::fct_explicit_na(
      factor(phenoTable[, which(colnames(phenoTable) == col_by)]),
      na_level = "NA")
    if(!is.null(confInterval)){
      phenoTable[, which(colnames(phenoTable) == col_by)] <- colgroup
      ## need to through out single levels
      dup <- unique(colgroup[duplicated(colgroup)])
      elligroup <- factor(colgroup[colgroup %in% dup])
      ellipseData <- as.data.frame(pcas)
      ellipseData$SAMPLE_ID <- rownames(ellipseData)
      ellipseData <- dplyr::left_join(ellipseData,
                                      phenoTable[,c("SAMPLE_ID",col_by)])
      ellipseData <- ellipseData[ellipseData[[col_by]] %in% dup,]
      ## dataEllipse requires a plot even if draw = FALSE
      graphics::plot.new()
      testellipses <- car::dataEllipse(
        x = ellipseData[,dim[1]],
        y = ellipseData[,dim[2]],
        groups = factor(ellipseData[[col_by]]),
        draw = FALSE, 
        levels = confInterval,
        col = getOption("me.colorvalues")[seq(1, length(levels(elligroup)))])
      grDevices::dev.off()
      keepindices <- vapply(levels(elligroup), function(c){
        subEllipse <- ellipseData[ellipseData[[col_by]] == c,]
        if(sum(abs(stats::cor(subEllipse[,c(1,2)]))) == 4){
          return(FALSE)
        }
        return(TRUE)
      }, logical(1))
      testellipses <- testellipses[keepindices]
    }
    
    pwidth <- pwidth + 125
    legendtitle <- list(yref = "paper",xref = "paper",
                        y=1.06,x=1.15, 
                        text = col_by, showarrow = FALSE)
    legend <- TRUE
  } else {
    colgroup <- I("black")
  }
  
  if (!is.null(shape_by)) {
    shapegroup <- forcats::fct_explicit_na(
      factor(phenoTable[, which(colnames(phenoTable) == shape_by)]),
      na_level = "NA")
    if(legend){
      legendtitle <- list(yref = "paper",xref = "paper",
                          y=1.06, x=1.15, 
                          text = paste0(col_by," - ",shape_by), 
                          showarrow = FALSE)
    } else {
      legendtitle <- list(yref = "paper",xref = "paper",
                          y=1.06,x=1.15, 
                          text = shape_by, showarrow = FALSE)
    }
    legend <- TRUE
    if(is.null(col_by))
      pwidth <- pwidth + 125
  } else {
    shapegroup <- I("circle")
  }
  
  
  plotdata <- tibble::tibble(
    text = paste0(
      paste0("<b>", colnames(aggmat), "</b>"),
      paste0("<br>", dim[1], ": ", 
             round(pcas[, 1], digits = getOption("me.round_digits"))),
      paste0("<br>", dim[2], ": ", 
             round(pcas[, 2], digits = getOption("me.round_digits")))
    ),
    SAMPLE_ID = colnames(aggmat), 
    x = pcas[, 1], 
    y = pcas[, 2], 
    color = colgroup,
    shape = shapegroup,
    adonis = "TEST"
  ) %>% dplyr::arrange(color)
  
  useWebGL <- FALSE
  if(nrow(plotdata) > getOption("me.minwebgl") && allowWebGL)
    useWebGL <- TRUE
  
  max_y <- max(plotdata$y)
  max_y <- max_y + 0.05 * max_y
  min_x <- min(plotdata$x) - 0.05 * min(plotdata$x)
  max_x <- max(plotdata$x)
  max_x <- max_x + 0.05 * max_x
  center_x <- (max_x + min_x)/2
  if(is.null(plotText))
    plotText <- ""
  
  plot_xlab <- 'if'(is.null(xlab), colnames(pcas)[1], xlab)
  plot_ylab <- 'if'(is.null(ylab), colnames(pcas)[2], ylab)
  
  p <- plotly::plot_ly(
    plotdata, hoverinfo = "text", 
    height = pheight, width = pwidth,
    colors = 
      getOption("me.colorvalues")[seq(1,length(levels(colgroup)))]) 
  if(!is.null(testellipses)){
    res <- lapply(seq(1,length(testellipses)), function(t){
      p <<- p %>% plotly::add_polygons(
        x = testellipses[[t]][,1], 
        y = testellipses[[t]][,2],
        fillcolor = 'transparent',
        name = names(testellipses)[[t]],
        line = list(color = getOption("me.colorvalues")[t]))
    })
  }
  p <- p %>%
    plotly::add_text(
      x = center_x,
      y = max_y,
      text = plotText,
      showlegend = FALSE
    ) %>%
    plotly::add_markers(
      x = ~x,
      y = ~y,
      color = ~color,
      symbol = ~shape,
      text = ~text,
      marker = list(size = pt_size),
      customdata = ~SAMPLE_ID,
      showlegend = legend
    ) 
  if(useWebGL)
    p <- p %>% plotly::toWebGL()
  p <- p %>% 
    add_plotly_config() %>%
    plotly::layout(
      title = list(
        text = plotTitle,
        font = list(
          size = 14
        )),
      xaxis = list(title = paste0("&nbsp;\n", plot_xlab),
                   tickangle = 45),
      yaxis = list(title = paste0(plot_ylab, "\n&nbsp;")),
      margin = list(l = 75, b = 75, t = 50, r = 50),
      annotations = legendtitle
    )
  
  return(p)
}



#' Plot heatmap
#'
#' This function plots a heatmap of feature abundance.
#'
#' @param aggdat aggregated MRExperiment
#' @param features Vector of features to plot. If NULL, the top `nfeat`
#' features in terms of `sort_by` will be plotted.
#' @param log Log2 transform data. Default is TRUE.
#' @param sort_by Dispersion measure to sort features, one of "Fano", "MAD",
#' and "Variance"
#' @param nfeat Number of features to display. Default is 50.
#' @param col_by Vector of phenotypes for coloring.
#' @param row_by Name of feature level for coloring.
#' @param plotTitle Plot title. By default, no title.
#'
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#'
#' @return plotly heatmap
#' 
#' @examples
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' plotHeatmap(aggdat, sort_by = "Fano")
#' 
#' @export
plotHeatmap <- function(aggdat, features = NULL,
                        log = TRUE, sort_by = c("Fano", "MAD", "Variance"),
                        nfeat = 50, col_by = NULL, 
                        row_by = NULL, plotTitle = "") {
  aggmat <- MRcounts(aggdat, norm = TRUE)
  phenoTable <- pData(aggdat)
  
  if (is.null(nfeat)) {
    nfeat <- 50
  }
  
  ## show only specified features or return NULL if not found
  if (!is.null(features)) {
    aggmat <- aggmat[rownames(aggmat) %in% features, ]
    if (nrow(aggmat) == 0) {
      return(NULL)
    }
    aggmat <- aggmat[seq(1,min(length(features), nfeat)), ]
  } else {
    ind <- switch(sort_by,
                  "Fano" = matrixStats::rowVars(sqrt(aggmat)) / 
                    matrixStats::rowMeans2(sqrt(aggmat)), 
                  "MAD" = matrixStats::rowMads(log2(aggmat + 1)), 
                  "Variance" = matrixStats::rowVars(log2(aggmat + 1))
    ) 
    ind <- order(ind, decreasing = TRUE)[seq(1,min(length(ind), nfeat))]
    aggmat <- aggmat[ind, ]
  }
  
  if (log) {
    aggmat <- log2(aggmat + 1)
  }
  
  col_group <- NULL
  if (!is.null(col_by)) {
    col_group <- dplyr::select(phenoTable, col_by)
  }
  
  row_group <- NULL
  if (!is.null(row_by)) { 
    if (!row_by == "") {
      ## assign each feature to its group: rownames of aggregated fData 
      ## always correspond to aggregation level
      featData <- dplyr::select(fData(aggdat), row_by)
      rownames(featData)[rownames(featData) == ""] <- "unnamed"
      ## select the correct higher feature level based on rowname order
      row_group <- data.frame(featData[rownames(aggmat), ])
      names(row_group) <- row_by
    }
  }
  
  aggmat <- round(aggmat, digits = getOption("me.round_digits"))
  
  pal <- grDevices::colorRampPalette(
    rev(RColorBrewer::brewer.pal(11, "RdYlBu")))
  colvalues <- pal(100)
  minval <- min(aggmat)
  maxval <- max(aggmat)
  
  if (is.null(col_group)) {
    if (is.null(row_group)) {
      hm <- heatmaply::heatmaply(aggmat,
                                 cexRow = 0.9,
                                 dendrogram = "column",
                                 col = colvalues,
                                 limits = c(minval, maxval),
                                 Rowv = FALSE,
                                 show_legend = FALSE,
                                 branches_lwd = 0.3,
                                 plot_method = "plotly"
      ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.2, which = 1
        )
    } else {
      hm <- heatmaply::heatmaply(aggmat,
                                 cexRow = 0.9,
                                 dendrogram = "column",
                                 col = colvalues,
                                 limits = c(minval, maxval),
                                 Rowv = FALSE,
                                 row_side_colors = row_group,
                                 show_legend = FALSE,
                                 branches_lwd = 0.3,
                                 plot_method = "plotly"
      ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.2, which = 1
        ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.3, which = 2
        )
    }
  } else {
    if (is.null(row_group)) {
      hm <- heatmaply::heatmaply(aggmat,
                                 cexRow = 0.9,
                                 dendrogram = "column",
                                 col = colvalues,
                                 limits = c(minval, maxval),
                                 Rowv = FALSE,
                                 col_side_colors = col_group,
                                 show_legend = FALSE,
                                 branches_lwd = 0.3,
                                 plot_method = "plotly"
      ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.3, which = 1
        ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.2, which = 2
        )
    } else {
      hm <- heatmaply::heatmaply(aggmat,
                                 cexRow = 0.9,
                                 dendrogram = "column",
                                 col = colvalues,
                                 limits = c(minval, maxval),
                                 Rowv = FALSE,
                                 col_side_colors = col_group,
                                 row_side_colors = row_group,
                                 show_legend = FALSE,
                                 branches_lwd = 0.3,
                                 plot_method = "plotly"
      ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.3, which = 1
        ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.2, which = 2
        ) %>%
        plotly::colorbar(
          tickfont = list(size = 10), titlefont = list(size = 10),
          lenmode = "fraction", len = 0.3, y = 0.5, which = 3
        )
    }
  }
  
  hm <- hm %>%
    plotly::layout(
      title = list(
        text = plotTitle,
        font = list(
          size = 14
        )),
      margin = list(l = 75, b = 75, t = 75, r = 50),
      showlegend = FALSE) %>%
    add_plotly_config()
  
  return(hm)
}

data("mouseData")


test_that("compute distance matrix", {
  aggdat <- aggFeatures(mouseData, level = "order")
  distmat <- computeDistMat(aggdat, dist_method = "bray")
  expect_equal(round(sum(distmat)), 1740)
  distmat <- computeDistMat(aggdat, dist_method = "euclidean")
  expect_equal(round(sum(distmat)), 89262)
})

#' Longitudinal Analysis module UI
#'
#' @param id namespace identifier
#' 
#' @author Janina Reeder
#' 
#' @return row containing the UI elements
longResultsUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    box(
      title = "LONGITUDINAL ANALYSIS", solidHeader = TRUE, 
      collapsible = TRUE, width = 12,
      fluidRow(
        column(
          width = 11, id = ns("longplot_column"),
          shinycssloaders::withSpinner(
            plotly::plotlyOutput(ns("longplot"), 
                                 width = "auto", 
                                 height = "auto"),
            type = 3, color = "#424242", 
            color.background = "#fdfdfc"),
          br(),
          shinyjs::disabled(
            shinyWidgets::dropdownButton(
              tags$h3("Plot Options"),
              shinyWidgets::radioGroupButtons(
                inputId = ns("longChoice"),
                label = "Y axis",
                choices = c("Reads", "Percentage"),
                individual = TRUE,
                checkIcon = list(
                  yes = tags$i(
                    class = "fa fa-circle",
                    style = "color: steelblue"
                  ),
                  no = tags$i(
                    class = "fa fa-circle-o",
                    style = "color: steelblue"
                  )
                )
              ),        
              div(id=ns("pspan"), class = "percentage",
                  p("Normalization is required to show percentage")
              ),
              shinyWidgets::switchInput(
                inputId = ns("logS"),
                label = "Log Scale",
                value = TRUE,
                size = "mini",
                labelWidth = "80px"
              ),
              sliderInput(
                inputId = ns("plotWidth"),
                label = "Adjust plot width",
                value = 650,
                min = 250,
                max = 1600,
                round = TRUE
              ),
              circle = FALSE, status = "danger", 
              icon = icon("gear"), width = "300px",
              label = "Plot Options",
              inputId = ns("optionbutton")
            ))
        )
      )
    )
  )
}

#' Longitudinal analysis module server code
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param longSettings reactive storing values selected in analysis 
#' input interface
#' @param normalizedData reactive boolean indicating if data has been normalized
#' @param reset boolean reactive which resets the module if TRUE
#' 
#' @author Janina Reeder
#' 
#' @return list containing R code for analysis and for feature plots
longResults <- function(input, output, session, 
                        aggDat, 
                        featLevel, 
                        longSettings, 
                        normalizedData,
                        reset) {
  ns <- session$ns
  
  repCode <- reactiveVal(NULL)
  width <- reactiveVal(650)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    shinyjs::enable("longChoice")
    shinyjs::js$removeInputElems()
    repCode(NULL)
  })
  
  observe({
    width(input$plotWidth)
  })
  
  
  observeEvent(width(), {
    req(aggDat(), longSettings()$featureselect)
    plotly::plotlyProxy("longplot", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = width())
      )
  })
  
  
  ## calls function to do a feature abundance plot
  output$longplot <- plotly::renderPlotly({
    req(aggDat(), longSettings()$featureselect, longSettings()$comparison,
        longSettings()$phenolevels)
    shinyjs::js$removeInputElems()
    shinyjs::enable("optionbutton_state")
    ylab <- "Reads"
    if(isFALSE(normalizedData())){
      shinyWidgets::updateRadioGroupButtons(session = session,
                                            inputId = "longChoice",
                                            selected = "Reads")
      shinyjs::removeClass(id = "pspan", class = "hide")
      shinyjs::disable("longChoice")
    } else {
      ylab <- input$longChoice
      shinyjs::addClass(id = "pspan", class = "hide")
      shinyjs::enable("longChoice")
    }
    plotTitle <- paste0(gsub("Reads","Abundance",ylab),
                        " of ",
                        longSettings()$featureselect)
    phenoId <- longSettings()$phenoid
    showLines <- TRUE
    if(phenoId == ""){
      phenoId <- "SAMPLE_ID"
      showLines <- FALSE
    }
    
    plotLongFeature(aggDat(),
                    x_var = longSettings()$comparison,
                    id_var = phenoId,
                    plotTitle = plotTitle,
                    feature = longSettings()$featureselect,
                    ylab = ylab,
                    log = input$logS,
                    showLines = showLines,
                    x_levels = longSettings()$phenolevels
    ) 
  })
  
  ## update R code needed to make plot
  observe({
    req(aggDat(), longSettings()$featureselect)
    plotTitle <- paste0(gsub("Reads","Abundance",input$longChoice),
                        " of ",
                        longSettings()$featureselect)
    
    repCode(paste(
      paste0("#' ### Feature plot"),
      paste0("", "#- fig.width = 8"),
      paste0("plotLongFeature(aggDat,"),
      paste0("\tx_var = \"", longSettings()$comparison, "\","),
      'if'(longSettings()$phenoid != "",
           paste0("\tid_var = \"", longSettings()$phenoid, "\","),
           paste0("\tshowLines = FALSE,")),
      paste0("\tplotTitle = \"",plotTitle, "\","),
      paste0("\tfeature = \"", longSettings()$featureselect, "\","),
      paste0("\tylab = \"", input$longChoice, "\","),
      paste0("\tlog = ", 
             'if'(is.null(input$logS), "TRUE", input$logS), ","),
      paste0("\tx_levels = c(", paste0("\"",
                                       longSettings()$phenolevels, 
                                       "\"",
                                       collapse = ","),
             "))\n\n"),
      sep = "\n"
    ))
  })
  
  return(repCode)
  
}

## CORRELATION ANALYSIS 


#' Helper function to calculate the confidence interval for a cor.test
#'
#' @param num number of samples
#' @param mS results of cor.test
#' @param method statistical method used for cor.test
#'
#' @return named vector holding lower and upper thresholds
computeCI_Interval <- function(num, mS, method){
  if(num <= 4)
    return(c("lower" = NA, "upper" = NA))
  if(method == "spearman"){
    rho <- mS$estimate
    stderr <- 1.0 / sqrt(num - 3)
    delta <- 1.96 * stderr
    lower <- tanh(atanh(rho) - delta)
    upper <- tanh(atanh(rho) + delta)
  } else if(method == "pearson"){
    lower <- mS$conf.int[1]
    upper <- mS$conf.int[2]
  } else if(method == "kendall"){
    tau <- mS$estimate
    z_tau <- 0.5 * log((1+tau)/(1-tau))
    lower <- z_tau - 1.959964 * sqrt(0.437/(num - 4))
    upper <- z_tau + 1.959964 * sqrt(0.437/(num - 4))
  }
  ci_interval <- c(lower, upper)
  names(ci_interval) <- c("lower","upper")
  return(ci_interval)
}


#' Scatterplot of two features
#'
#' This function plots a scatterplot of two features along with sample
#' correlation statistics.
#'
#' @param aggdat aggregated MRExperiment
#' @param feat1 Feature 1.
#' @param feat2 Feature 2.
#' @param log Log2 transform data. Default is TRUE.
#' @param method Correlation coefficient. One of "spearman" (default),
#' "pearson", or "kendall".
#' @param addRegression boolean parameter indicating whether linear regression
#' line should be drawn; default: TRUE
#' @param col_by Phenotype for coloring.
#' @param facet1 Phenotype for facet 1.
#' @param facet2 Phenotype for facet 2. 
#' @param plotTitle Plot title. Default is no title.
#' @param xlab X-axis label. Default is \code{feat1}.
#' @param ylab Y-axis label. Default is \code{feat2}.
#' @param allowWebGL boolean indicating if WebGL should be used for large data
#' @param pwidth overall plot width; default is 550 
#' @param pheight overall plot height; default is 200
#'
#' @return list holding plotly plot and lm fit
#'
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#' @importFrom magrittr %>%
#' 
#' @examples
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' corrFeature(aggdat, feat1 = "Bacteroides", feat2 = "Prevotella")
#'
#' @export
corrFeature <- function(aggdat, feat1, feat2,
                        log = TRUE, method = c("spearman", "pearson", "kendall"),
                        addRegression = TRUE,
                        col_by = NULL, facet1 = NULL, facet2 = NULL,
                        plotTitle = "", xlab = NULL, ylab = NULL, 
                        allowWebGL = TRUE, pwidth = 550, pheight = 200) {
  facet2s <- NULL
  facets <- NULL
  aggmat <- MRcounts(aggdat, norm = TRUE)
  phenoTable <- pData(aggdat)
  
  if (is.null(feat1) || is.null(feat2)) {
    return(NULL)
  }
  
  method <- match.arg(method)
  if (log) {
    aggmat <- log2(aggmat + 1)
  }
  x <- aggmat[which(rownames(aggmat) == feat1), ]
  y <- aggmat[which(rownames(aggmat) == feat2), ]
  
  ## feature not found: return immediately
  if (length(x) == 0 || length(y) == 0) {
    return(list(plot = NULL, stats = NULL))
  }
  
  df <- data.frame(colnames(aggmat), x, y)
  colnames(df) <- c("samname", "feat1", "feat2")
  showlegend <- TRUE
  
  legendtitle <- ""
  addwidth <- 0
  
  legendtitle <- ""
  
  if (!is.null(col_by)) {
    group <- forcats::fct_explicit_na(
      factor(phenoTable[, which(colnames(phenoTable) == col_by)]),
      na_level = "NA")
    addwidth <- 200
    legendtitle <- list(yref = "paper",xref = "paper",
                        y=1.06,x=1.1,
                        text = col_by,showarrow = FALSE)
    col_name <- paste0(col_by, "_color")
    
    pwidth <- pwidth + 150
  } else {
    col_name <- NULL
    group <- I("black")
    showlegend <- FALSE
  }
  
  df2 <- buildPlottingDF(
    df = df, 
    phenoTable = phenoTable,
    facet1 = facet1,
    facet2 = facet2,
    col_by = col_by,
    col_name = col_name)
  col_by <- col_name
  
  logtext <- ""
  if (log == TRUE)
    logtext <- "Log "
  
  if(is.null(xlab))
    xlab <- feat1
  
  if (!is.null(facet1)) {
    facetvals <- levels(df2[, facet1])
    xlab <- paste0(xlab, "<br>", paste(facet1, facetvals, sep = " "))
    pwidth <- length(facetvals) * 300
  } else {
    facet1 <- "nofacet"
    df2$nofacet <- "nofacets"
    facetvals <- "nofacets"
  }
  
  if (!is.null(facet2)) {
    facetvals2 <- levels(df2[, facet2])
  } else {
    facet2 <- "nofacet2"
    df2$nofacet2 <- "nofacets"
    facetvals2 <- "nofacets"
  }
  
  df2$group <- group
  
  df2 <- df2 %>%
    dplyr::group_by(facets = get(facet1), facet2s = get(facet2))
  
  useWebGL <- FALSE
  if(nrow(df2) > getOption("me.minwebgl") && allowWebGL)
    useWebGL <- TRUE
  
  xaxis_text <- ""
  
  if(is.null(ylab))
    ylab <- feat2
  yaxis_text <- ylab
  
  maxj <- length(facetvals2)
  totalheight <- pheight + 400 * length(facetvals2)
  pwidth <- pwidth + addwidth
  
  fitlist <- list()
  
  ## iterate over facet2 around facet1
  plotlist <- lapply(facetvals2, function(fv2) {
    j <- which(facetvals2 %in% fv2)
    sdf <- df2 %>%
      dplyr::group_by(facet2s) %>%
      dplyr::filter(facet2s %in% fv2)
    
    if (facet2 != "nofacet2") {
      yaxis_text <- 'if'(is.na(fv2),
                         paste0(ylab, "<br>NA<br> "),
                         paste0(ylab, "<br>", facet2, " ", fv2, "<br> ")
      )
    }
    
    ## iterating over facet1
    plist <- lapply(facetvals, function(f) {
      i <- which(facetvals %in% f)
      sp <- sdf %>%
        dplyr::group_by(facets) %>%
        dplyr::filter(facets %in% f)
      
      sp <- sp[!is.na(sp$feat1) & !is.na(sp$feat2),]
      
      if (j == maxj) {
        xaxis_text <- paste0(logtext, xlab[i])
      }
      if (nrow(sp) == 0) {
        return(buildEmptyPlotlyPlot(xaxis_text, paste0(logtext, yaxis_text)))
      }
      
      facetgroup <- sp$group
      if (is.null(col_by)) {
        facetgroup <- I("black")
      }
      
      ## PERFORM CORRELATION 
      fit <- stats::lm(feat2 ~ feat1, data = sp, na.action = stats::na.exclude)
      mS_label <- tryCatch({
        mS <- stats::cor.test(sp$feat2,sp$feat1, method = method)
        ci_interval <- computeCI_Interval(
          num = nrow(sp), 
          mS = mS, 
          method = method)
        mS$lower <- round(ci_interval[["lower"]], getOption("me.round_digits"))
        mS$upper <- round(ci_interval[["upper"]], getOption("me.round_digits"))

        ## store fit in fitlist
        fitlist[[gsub("<br>", "_", 
                      paste0(yaxis_text, " <br/> ", xlab[i]))]] <<- mS
        paste0("\n", names(mS$estimate)," = ",
               round(mS$estimate,getOption("me.round_digits")),
               "; p = ",round(mS$p.value,getOption("me.round_digits")),
               "\nlower CI = ", mS$lower,
               "\nupper CI = ", mS$upper)
      }, error = function(e){
        ""
      })
      
      plotdata <- tibble::tibble(
        samname = sp$samname,
        x = sp$feat1,
        y = sp$feat2,
        color = facetgroup,
        text = paste0(
          paste0("<b>", sp$samname, "</b>"), 
          paste0("<br>", feat1, ": ",
                 round(sp$feat1, digits = getOption("me.round_digits"))), 
          paste0("<br>", feat2, ": ", 
                 round(sp$feat2, digits = getOption("me.round_digits"))))
      ) 
      
      fitlabel <- paste(
        "Adj R2: ", signif(summary(fit)$adj.r.squared, 5),
        "<br>Intercept: ", signif(fit$coef[[1]], 5),
        "<br>Slope: ", signif(fit$coef[[2]], 5),
        "<br>P: ", 'if'(nrow(summary(fit)$coef) < 2 || 
                          ncol(summary(fit)$coef < 4),
                        NA,
                        signif(summary(fit)$coef[2, 4], 5)
        )
      )
      
      p <- plotdata %>%
        plotly::plot_ly(x = ~x, hoverinfo = "text", 
                        width = pwidth, height = totalheight) %>%
        plotly::add_markers(
          y = ~y,
          color = ~color,
          colors = getOption("me.colorvalues")[seq(1, length(levels(group)))],
          text = ~text,
          showlegend = showlegend
        ) %>%
        plotly::add_lines(
          y = ~x,
          line = list(
            dash = "dash",
            color = "rgba(220,220,220,1)"
          ),
          name = "onetoone",
          text = "",
          showlegend = FALSE
        )
      if(addRegression){
        p <- p %>%
          plotly::add_lines(
            y = stats::predict(fit),
            line = list(
              color = "rgba(119,0,0, 1)",
              shape = "spline"
            ),
            name = "Linear Regression",
            text = fitlabel,
            showlegend = FALSE
          ) %>%
          plotly::add_ribbons(
            data = broom::augment(fit, se_fit = TRUE),
            x = ~feat1,
            ymin = ~ .fitted - 1.96 * .se.fit,
            ymax = ~ .fitted + 1.96 * .se.fit,
            line = list(color = "rgba(219,219,219, 0.35)"),
            fillcolor = "rgba(219,219,219, 0.3)",
            name = "Standard Error",
            showlegend = FALSE
          ) 
      }
      if(useWebGL)
        p <- p %>% plotly::toWebGL()
      p <- p %>% 
        add_plotly_layout(plotTitle = plotTitle, 
                          xaxis_text = paste0(xaxis_text, mS_label), 
                          ylab = paste0(logtext, yaxis_text)) %>%
        plotly::layout(margin = list(b = 150))
      p
    })
    plotly::subplot(plist, nrows = 1, shareY = TRUE, titleX = TRUE)
  })
  
  
  p <- plotly::subplot(plotlist, nrows = length(plotlist),   
               shareY = FALSE, titleX = TRUE, titleY = TRUE) %>%

  add_plotly_config()
  
  return(list(plot = p, stats = fitlist))
}

#' Scatterplot of a feature and a phenotype
#'
#' This function plots a scatterplot of a feature and a phenotype along with 
#' sample correlation statistics.
#'
#' @param aggdat aggregated MRExperiment
#' @param feature Feature input.
#' @param phenotype Phenotype input (must be numeric)
#' @param log Log2 transform data. Default is TRUE.
#' @param method Correlation coefficient. One of "spearman" (default),
#' "pearson", or "kendall".
#' @param addRegression boolean parameter indicating whether linear regression
#' line should be drawn; default: TRUE
#' @param col_by Phenotype for coloring.
#' @param facet1 Phenotype for facet 1.
#' @param facet2 Phenotype for facet 2. (WIP/TODO)
#' @param plotTitle Plot title. Default is no title.
#' @param xlab X-axis label. Default is \code{feat1}.
#' @param ylab Y-axis label. Default is \code{feat2}.
#' @param allowWebGL boolean indicating if WebGL should be used for large data
#' @param pwidth overall plot width; default is 550 
#' @param pheight overall plot height; default is 200
#'
#' @return list holding plotly plot and lm fit
#'
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData
#' 
#' @examples
#' data("mouseData", package = "metagenomeSeq")
#' aggdat <- aggFeatures(mouseData, level = "genus")
#' corrPhenotype(aggdat, feature = "Bacteroides", phenotype = "relativeTime")
#'
#' @export
corrPhenotype <- function(aggdat, feature, phenotype, log = TRUE, 
                          method = c("spearman", "pearson", "kendall"),
                          addRegression = TRUE,
                          col_by = NULL, facet1 = NULL, facet2 = NULL,
                          plotTitle = "", xlab = NULL, ylab = NULL,
                          allowWebGL = TRUE, pwidth = 550, pheight = 200) {
  facet2s <- NULL
  facets <- NULL
  aggmat <- MRcounts(aggdat, norm = TRUE)
  phenoTable <- pData(aggdat)
  
  if (is.null(feature) | is.null(phenotype)) {
    return(NULL)
  }
  
  method <- match.arg(method)
  if (log) {
    aggmat <- log2(aggmat + 1)
  }
  
  phenoval <- phenoTable[, phenotype]
  
  featureval <- aggmat[which(rownames(aggmat) == feature), ]
  if (length(featureval) == 0) {
    return(list(plot = NULL, stats = NULL))
  }
  
  df <- data.frame(colnames(aggmat), featureval, phenoval)
  colnames(df) <- c("samname", "feature", "phenotype")
  showlegend <- TRUE
  
  legendtitle <- ""
  addwidth <- 0
  
  if (!is.null(col_by)) {
    group <- forcats::fct_explicit_na(
      factor(phenoTable[, which(colnames(phenoTable) == col_by)]),
      na_level = "NA")
    addwidth <- 200
    legendtitle <- list(yref = "paper",xref = "paper",
                        y=1.06,x=1.1,
                        text = col_by,showarrow = FALSE)
    col_name <- paste0(col_by, "_color")
    
    pwidth <- pwidth + 150
  } else {
    col_name <- NULL
    group <- I("black")
    showlegend <- FALSE
  }
  
  df2 <- buildPlottingDF(df = df, 
                         phenoTable = phenoTable,
                         facet1 = facet1,
                         facet2 = facet2,
                         col_by = col_by,
                         col_name = col_name)
  col_by <- col_name
  
  logtext <- ""
  if(log == TRUE)
    logtext <- "Log "
  
  if(is.null(xlab))
    xlab <- feature
  
  if (!is.null(facet1)) {
    facetvals <- levels(df2[, facet1])
    xlab <- paste0(xlab, "<br>", paste(facet1, facetvals, sep = " "))
    pwidth <- length(facetvals) * 300
  } else {
    facet1 <- "nofacet"
    df2$nofacet <- "nofacets"
    facetvals <- "nofacets"
  }
  
  
  if (!is.null(facet2)) {
    facetvals2 <- levels(df2[, facet2])
  } else {
    facet2 <- "nofacet2"
    df2$nofacet2 <- "nofacets"
    facetvals2 <- "nofacets"
  }
  
  df2$group <- group
  df2 <- df2 %>%
    dplyr::group_by(facets = get(facet1), facet2s = get(facet2))
  
  useWebGL <- FALSE
  if(nrow(df2) > getOption("me.minwebgl") && allowWebGL)
    useWebGL <- TRUE
  
  xaxis_text <- ""
  if(is.null(ylab))
    ylab <- phenotype
  yaxis_text <- ylab
  
  maxj <- length(facetvals2)
  totalheight <- pheight + 400 * length(facetvals2)
  pwidth <- pwidth + addwidth
  
  fitlist <- c()
  ## iterate over facet2 around facet1
  plotlist <- lapply(facetvals2, function(fv2) {
    j <- which(facetvals2 %in% fv2)
    sdf <- df2 %>%
      dplyr::group_by(facet2s) %>%
      dplyr::filter(facet2s %in% fv2)
    
    if (facet2 != "nofacet2") {
      yaxis_text <- 'if'(is.na(fv2),
                         paste0(phenotype, "<br>NA<br> "),
                         paste0(phenotype, "<br>", facet2, " ", fv2, "<br> ")
      )
    }
    # iterating over facet1
    plist <- lapply(facetvals, function(f) {
      i <- which(facetvals %in% f)
      sp <- sdf %>%
        dplyr::group_by(facets) %>%
        dplyr::filter(facets %in% f)
      
      sp <- sp[!is.na(sp$feature) & !is.na(sp$phenotype),]
      
      if (j == maxj) {
        xaxis_text <- paste0(logtext, xlab[i])
      }
      
      emptyplot <- FALSE
      if (nrow(sp) == 0) {
        emptyplot <- TRUE
      } else if (all(is.na(sp$phenotype))) {
        emptyplot <- TRUE
      }
      if (emptyplot) {
        return(buildEmptyPlotlyPlot(xaxis_text, paste0(logtext, yaxis_text)))
      }
      
      facetgroup <- sp$group
      if (is.null(col_by)) {
        facetgroup <- I("black")
      }
      
      ## PERFORM CORRELATION 
      fit <- stats::lm(phenotype ~ feature, data = sp, 
                       na.action = stats::na.exclude)       
      mS_label <- tryCatch({
        mS <- stats::cor.test(sp$phenotype, sp$feature, method = method)
        ci_interval <- computeCI_Interval(
          num = nrow(sp), 
          mS = mS, 
          method = method)
        mS$lower <- round(ci_interval[["lower"]], getOption("me.round_digits"))
        mS$upper <- round(ci_interval[["upper"]], getOption("me.round_digits"))
        ## store fit in fitlist
        fitlist[[gsub("<br>", "_", 
                      paste0(yaxis_text, " <br/> ", xlab[i]))]] <<- mS
        paste0("\n", names(mS$estimate)," = ",
               round(mS$estimate,getOption("me.round_digits")),
               "; p = ",round(mS$p.value,getOption("me.round_digits")),
               "\nlower CI = ", mS$lower,
               "\nupper CI = ", mS$upper)
      }, error = function(e){
        ""
      })
      
      pt <- f
      if (is.na(pt)) {
        pt <- "NA"
      } else if (pt == "nofacets") {
        pt <- ""
      }
      
      plotdata <- tibble::tibble(
        samname = sp$samname,
        x = sp$feature,
        y = sp$phenotype,
        color = facetgroup,
        text = paste0(
          paste0("<b>", sp$samname, "</b>"), 
          paste0("<br>", feature, ": ", 
                 round(sp$feature, 
                       digits = getOption("me.round_digits"))), 
          paste0("<br>", phenotype, ": ", 
                 round(sp$phenotype, 
                       digits = getOption("me.round_digits"))))
      )
      
      fitlabel <- paste(
        "Adj R2: ", signif(summary(fit)$adj.r.squared, 5),
        "<br>Intercept: ", signif(fit$coef[[1]], 5),
        "<br>Slope: ", signif(fit$coef[[2]], 5),
        "<br>P: ", 'if'(nrow(summary(fit)$coef) < 2 || 
                          ncol(summary(fit)$coef < 4),
                        NA,
                        signif(summary(fit)$coef[2, 4], 5)
        )
      )
      
      p <- plotdata %>%
        plotly::plot_ly(x = ~x, hoverinfo = "text", 
                        width = pwidth, height = totalheight) %>%
        plotly::add_markers(
          y = ~y,
          color = ~color,
          colors = getOption("me.colorvalues")[seq(1, length(levels(group)))],
          text = ~text,
          showlegend = showlegend
        ) %>%
        plotly::add_lines(
          y = ~x,
          line = list(
            dash = "dash",
            color = "rgba(220,220,220,1)"
          ),
          name = "onetoone",
          text = "",
          showlegend = FALSE
        ) 
      if(addRegression){
        p <- p %>%
          plotly::add_lines(
            y = stats::predict(fit),
            line = list(
              color = "rgba(119,0,0, 1)",
              shape = "spline"
            ),
            name = "Linear Regression",
            text = fitlabel,
            showlegend = FALSE
          ) %>%
          plotly::add_ribbons(
            data = broom::augment(fit, se_fit = TRUE),
            x = ~feature,
            ymin = ~ .fitted - 1.96 * .se.fit,
            ymax = ~ .fitted + 1.96 * .se.fit,
            line = list(color = "rgba(219,219,219, 0.35)"),
            fillcolor = "rgba(219,219,219, 0.3)",
            name = "Standard Error",
            showlegend = FALSE
          ) 
      }
      if(useWebGL)
        p <- p %>% plotly::toWebGL() 
      p <- p %>%
        add_plotly_layout(plotTitle = plotTitle, 
                          xaxis_text = paste0(xaxis_text, mS_label),  
                          ylab = paste0(logtext, yaxis_text)) %>%
        plotly::layout(margin = list(b = 150))
      p
    })
    plotly::subplot(plist, nrows = 1, shareY = TRUE, titleX = TRUE)
  })
  
  p <- plotly::subplot(plotlist, nrows = length(plotlist), 
               shareY = FALSE, titleX = TRUE, titleY = TRUE) %>%
    add_plotly_config()
  
  return(list(plot = p, stats = fitlist))
}

data("mouseData")
aggdat <- aggFeatures(mouseData, level = "genus")
aggmat <- MRcounts(aggdat, norm = TRUE)
aggmat <- log2(aggmat + 1)
feat1 <- "Bacteroides"
feat2 <- "Prevotella"
x <- aggmat[which(rownames(aggmat) == feat1), ]
y <- aggmat[which(rownames(aggmat) == feat2), ]
df <- data.frame(colnames(aggmat), x, y)
colnames(df) <- c("samname", "feat1", "feat2")

test_that("compute confidence interval - spearman", {
  mS <- suppressWarnings(stats::cor.test(df$feat2,df$feat1, method = "spearman"))
  ci_interval <- computeCI_Interval(num = nrow(df), mS = mS, method = "spearman")
  expect_equal(as.character(ci_interval),c("0.171106874573396", "0.46912194979915"))
})

test_that("compute confidence interval - pearson", {
  mS <- suppressWarnings(stats::cor.test(df$feat2,df$feat1, method = "pearson"))
  ci_interval <- computeCI_Interval(num = nrow(df), mS = mS, method = "pearson")
  expect_equal(as.character(ci_interval),c("0.265416496698058", "0.542758284278827"))
})

test_that("compute confidence interval - kendall", {
  mS <- suppressWarnings(stats::cor.test(df$feat2,df$feat1, method = "kendall"))
  ci_interval <- computeCI_Interval(num = nrow(df), mS = mS, method = "kendall")
  expect_equal(as.character(ci_interval),c("0.0930203803066753", "0.316044546486128"))
})

test_that("check error for small numbers", {
  mS <- suppressWarnings(stats::cor.test(df$feat2,df$feat1, method = "spearman"))
  ci_interval <- computeCI_Interval(num = 4, mS = mS, method = "spearman")
  expect_true(is.na(ci_interval[["lower"]]))
  expect_true(is.na(ci_interval[["upper"]]))
})

## Main module for inter sample analysis

#'inter Analysis Module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the ui code
#' 
#' @examples interAnalysisUI("interanalysis_id")
#' 
#' @export
interAnalysisUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    width = 12,
    column(
      width = 3,
      betaInputUI(ns("betaInput")),
      heatmapInputUI(ns("heatmapInput"))
    ),
    column(
      width = 9,
      fluidRow(
        width = 11,
        box(
          width = 10,
          p("The inter sample page contains methods to analyze the microbial 
           composition and diversity between samples using PCA and interactive 
           heatmaps.")
        ),
        betaDiversityUI(ns("betaDiv")),
        abundanceHeatmapUI(ns("heatmap"))
      )
    )
  )
}


#' inter Analysis Module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param data the main data object returned from data_input_module
#' @param levelOpts available levels to aggregate on (depends on input data)
#' @param chosenLevel previously selected level (passed from different instance)
#' @param resetInput reactive boolean determining if reset is required
#' @param aggData the aggregated MRExperiment object
#'
#' @return reactive holding code to be used in reports
interAnalysis <- function(input, output, session, data, levelOpts,
                          chosenLevel, resetInput, aggData) {
  ## inter SAMPLE ANALYSIS
  ns <- session$ns
  
  betaSettings <- callModule(betaInput,
                             "betaInput",
                             meData = data$meData,
                             adonisOptions = data$adonisOptions,
                             reset = resetInput
  )
  
  ## reactive storing beta distance settings
  betaDist <- reactiveVal("")  
  ## reactives storing heatmap settings
  hmSort <- reactiveVal("")
  hmFeatList <- reactiveVal(NULL)
  
  ## update betaDist if a new measure was selected
  observe({
    req(betaDist() != betaSettings()$distance)
    betaDist(betaSettings()$distance)
  })
  
  observe({
    req(resetInput())
    betaDist("")
    hmSort("")
    hmFeatList(NULL)
  })
  
  betaDivRep <- callModule(betaDiversity,
                           "betaDiv",
                           aggDat = reactive(aggData$mrobj),
                           aggLevel = reactive(aggData$level),
                           colorOptions = data$facetOptions,
                           shapeOptions = data$shapeOptions,
                           betadistance = betaDist,
                           betaSettings = betaSettings,
                           reset = resetInput
  )
  
  
  heatmapSettings <- callModule(heatmapInput,
                                "heatmapInput",
                                meData = data$meData,
                                reset = resetInput,
                                aggDat = reactive(aggData$mrobj)
  )
  
  
  
  ## update sorting method (only) if a new value was selected
  observe({
    req(hmSort() != heatmapSettings()$sorting)
    hmSort(heatmapSettings()$sorting)
  }, priority = 20)
  
  ## stores selected features
  observe({
    hmFeatList(heatmapSettings()$featureselect)
  }, priority = 20)
  
  abHeatRep <- callModule(abundanceHeatmap,
                          "heatmap",
                          aggDat = reactive(aggData$mrobj),
                          featLevel = chosenLevel,
                          colorOptions = data$facetOptions,
                          levelOpts = levelOpts,
                          hmSort = hmSort,
                          hmFeatList = hmFeatList,
                          reset = resetInput
  )
  
  interRep <- reactive({
    list(
      "beta" = betaDivRep(),
      "abheat" = abHeatRep()
    )
  })
  
  
  return(interRep)
  
}

## Modules for inter sample analysis tab

#' Abundance Heatmap module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return box holding the UI code
abundanceHeatmapUI <- function(id) {
  ns <- NS(id)
  
  box(
    title = "ABUNDANCE HEATMAP", solidHeader = TRUE, 
    collapsible = TRUE, width = 12,
    fluidRow(
      column(
        width = 11,
        shinycssloaders::withSpinner(
          plotly::plotlyOutput(ns("abhmplot"), 
                               width = "auto", 
                               height = "800px"),
          type = 3, color = "#424242", 
          color.background = "#fdfdfc"),
        br(),
        shinyjs::disabled(
          shinyWidgets::dropdownButton(
            tags$h3("Plot Options"),
            numericInput(
              inputId = ns("hmfeats"),
              label = "Number of features",
              value = 50, min = 1
            ),
            selectInput(
              inputId = ns("hmcol"),
              label = "Phenotype annotations",
              choices = "",
              multiple = TRUE
            ),
            selectInput(
              inputId = ns("rowcol"),
              label = "Feature annotation",
              choices = "",
              multiple = FALSE
            ),
            shinyWidgets::switchInput(
              inputId = ns("hmlog"),
              label = "Log Scale",
              size = "mini",
              value = TRUE,
              labelWidth = "80px"
            ),
            actionButton(
              ns("changeHeatmapSettings"), 
              label = "GO", width = "50px"),
            circle = FALSE, status = "danger", 
            icon = icon("gear"), width = "300px",
            label = "Plot Options",
            inputId = ns("optionbutton")
          ))
      )
    )
  )
}

#' Abundance Heatmap module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param colorOptions reactive storing filters selected via data input
#' @param levelOpts all available level choices for this dataset
#' @param hmSort reactive storing sorting method for heatmap
#' @param hmFeatList reactive storing list of features to include in heatmap
#' @param reset boolean reactive which resets the module if TRUE
#'
#' @author Janina Reeder
#' 
#' @return R code needed to generate the heatmap
abundanceHeatmap <- function(input, output, session, 
                             aggDat, 
                             featLevel, 
                             colorOptions, 
                             levelOpts, 
                             hmSort, 
                             hmFeatList,
                             reset) {
  ns <- session$ns
  
  ## update with available phenotype annotation options
  observe({
    if(is.null(colorOptions())){
      updateSelectInput(session, "hmcol", choices = "")
    } else {
      updateSelectInput(session, "hmcol", choices = c("",colorOptions()))
    }
  })
  
  ## update with available feature level annotation options
  observe({
    req(levelOpts(), featLevel())
    uptoind <- which(levelOpts() == featLevel())
    updateSelectInput(session, "rowcol", 
                      choices = c("", levelOpts()[seq_len(uptoind)]))
  })
  
  ## reatives storing plot options and whether plot update is required
  hmColors <- reactiveVal(NULL)
  rowColors <- reactiveVal(NULL)
  numOfFeats <- reactiveVal(50)
  logScale <- reactiveVal(TRUE)
  updatePlot <- reactiveVal(TRUE)
  ## stores the R code needed to build the heatmap
  repCode <- reactiveVal(NULL)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    hmColors(NULL)
    rowColors(NULL)
    numOfFeats(50)
    updatePlot(TRUE)
    repCode(NULL)
    shinyjs::reset("abhmplot")
  })
  
  
  ## update plot options based on GO click
  observeEvent(input$changeHeatmapSettings,{
    changed <- FALSE
    updatePlot(FALSE)
    if(!is.null(input$hmcol) && !input$hmcol %in% hmColors()){
      hmColors(input$hmcol)
      changed <- TRUE
    }
    if(!is.null(input$rowcol) && !input$rowcol %in% rowColors()){
      rowColors(input$rowcol)
      changed <- TRUE
    }
    if(!is.null(input$hmfeats) && !input$hmfeats %in% numOfFeats()){
      numOfFeats(input$hmfeats)
      changed <- TRUE
    }
    if(!is.null(input$hmlog) && !input$hmlog %in% logScale()){
      logScale(input$hmlog)
      changed <- TRUE
    }
    updatePlot(changed)
  })
  
  ## render the heatmap
  output$abhmplot <- plotly::renderPlotly({
    req(aggDat(), hmSort(), updatePlot())
    shinyjs::enable("optionbutton_state")
    plotTitle <- 'if'(is.null(hmFeatList()),
                      paste0("Top ", numOfFeats(),
                             " features sorted by ",
                             hmSort()),
                      paste0("Selected features"))
    plotTitle <- paste0(plotTitle,
                        " at ",
                        featLevel(),
                        " level")
    repCode(paste(
      paste0("#' ### Feature Heatmap"),
      paste0("", "#- fig.width = 7, fig.height = 9"),
      paste0("plotHeatmap(aggDat,"),
      paste0("\tfeatures = ", 
             'if'(is.null(hmFeatList()), "NULL", 
                  paste0("c(",
                         paste0("\"", hmFeatList(), "\"",
                                collapse = ", "), ")")),
             ","),
      paste0("\tlog = ", logScale(), ","),
      paste0("\tsort_by = \"", hmSort(), "\","),
      paste0("\tnfeat = ", numOfFeats(), ","),
      paste0("\tcol_by = ", 
             'if'(is.null(hmColors()), "NULL", 
                  paste0("c(",
                         paste0("\"", hmColors(), "\"", 
                                collapse = ", "),")")),
             ","),
      paste0("\trow_by = ", 
             'if'(is.null(rowColors()), "NULL", 
                  paste0("\"", rowColors(), "\"")), ","),
      paste0("\tplotTitle = \"",plotTitle,"\")\n\n"),
      sep = "\n"
    ))
    
    plotHeatmap(
      aggdat = aggDat(),
      features = hmFeatList(),
      log = logScale(),
      sort_by = hmSort(),
      nfeat = isolate(numOfFeats()),
      col_by = isolate(hmColors()),
      row_by = isolate(rowColors()),
      plotTitle = plotTitle
    )
  })
  
  return(repCode)
}



#' Beta Diversity module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return box holding the ui code
betaDiversityUI <- function(id) {
  ns <- NS(id)
  
  box(
    title = "BETA DIVERSITY", solidHeader = TRUE, 
    collapsible = TRUE, width = 12,
    fluidRow(
      column(
        width = 11,
        shinycssloaders::withSpinner(
          plotly::plotlyOutput(ns("betaDiv"), 
                               width = "auto",
                               height = "auto"), 
          type = 3, color = "#424242", 
          color.background = "#fdfdfc"),
        shinyjs::disabled(
          shinyWidgets::dropdownButton(
            tags$h3("Plot Options"),
            selectInput(
              inputId = ns("Xbeta"),
              label = "X",
              choices = c("PC1", "PC2", "PC3", "PC4", 
                          "PC5", "PC6", "PC7", "PC8"),
              selected = "PC1"
            ),
            selectInput(
              inputId = ns("Ybeta"),
              label = "Y",
              choices = c("PC1", "PC2", "PC3", "PC4", 
                          "PC5", "PC6", "PC7", "PC8"),
              selected = "PC2"
            ),
            selectInput(
              inputId = ns("betacol"),
              label = "Color by",
              choices = ""
            ),
            checkboxInput(inputId = ns("confEllipse"),
                          label = "Add confidence ellipse",
                          value = FALSE),
            sliderInput(inputId = ns("confLevel"),
                        label = "Confidence Level",
                        min = 0.01, max = 0.99,
                        step = 0.01, value = 0.95),
            selectInput(inputId = ns("betashape"),
                        label = "Shape by",
                        choices = ""
            ),
            numericInput(inputId = ns("betasize"),
                         label = "Point size",
                         value = 8, min = 1
            ),
            sliderInput(
              inputId = ns("plotWidth"),
              label = "Adjust plot width",
              value = 650,
              min = 250,
              max = 1600,
              round = TRUE
            ),
            actionButton(ns("changeBetaSettings"), 
                         label = "GO", width = "50px"),
            circle = FALSE, status = "danger", 
            icon = icon("gear"), width = "300px",
            label = "Plot Options",
            inputId = ns("optionbutton")
          ))
      )
    ),
    fluidRow(
      column(width = 1),
      column(width = 10, class = "statsrow",
             DT::DTOutput(ns("statsdatatable"), 
                          width = "90%", height = "auto")
      )
    )
  )
}


#' Beta Diversity module - server 
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat MRExperiment storing data
#' @param aggLevel aggregation level
#' @param colorOptions phenotype selection options for color 
#' @param shapeOptions phenotype selection options for shape
#' @param betadistance distance measured used for beta diversity analysis
#' @param betaSettings input choices for beta diversity
#' @param reset boolean reactive which resets the module if TRUE
#'
#' @author Janina Reeder
#'
#' @return R code needed to generate the beta diversity plot
betaDiversity <- function(input, output, session, 
                          aggDat, 
                          aggLevel,
                          colorOptions, 
                          shapeOptions,
                          betadistance,
                          betaSettings,
                          reset) {
  ns <- session$ns
  
  ## reatives storing plot options and whether plot update is required
  shapeChoice <- reactiveVal("None")
  colorChoice <- reactiveVal("No Color")
  sizeChoice <- reactiveVal(8)
  xbeta <- reactiveVal("PC1")
  ybeta <- reactiveVal("PC2")
  updatePlot <- reactiveVal(FALSE)
  ## stores the R code needed to build the plot
  repCode <- reactiveVal(NULL)
  ## reactive storing distance matrices (to be pulled from DB)
  distMat <- reactiveVal(NULL)
  pcaVals <- reactiveVal(NULL)
  
  adonisVar <- reactiveVal(NULL)
  adonisStrata <- reactiveVal(NULL)
  adonisText <- reactiveVal(NULL)
  adonisCode <- reactiveVal(NULL)
  plotComplete <- reactiveVal(FALSE)
  confInterval <- reactiveVal(NULL)
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    shapeChoice("None")
    colorChoice("No Color")
    sizeChoice(8)
    xbeta("PC1")
    ybeta("PC2")
    updatePlot(FALSE)
    repCode(NULL)
    distMat(NULL)
    pcaVals(NULL)
    adonisVar(NULL)
    adonisStrata(NULL)
    adonisText(NULL)
    adonisCode(NULL)
    plotComplete(FALSE)
  })
  
  ## update available choices for color and shape
  observe({
    updateSelectInput(session, "betacol", 
                      choices = c("No Color", colorOptions()))
    updateSelectInput(session, "betashape", 
                      choices = c("None",shapeOptions()))
  })
  
  observeEvent(input$betacol,{
    if(input$betacol == "No Color"){
      shinyjs::disable("confEllipse")
      shinyjs::disable("confLevel")
    } else {
      shinyjs::enable("confEllipse")
    }
  })
  
  observeEvent(input$optionbutton_state,{
    req(input$betacol == "No Color")
    shinyjs::disable("confEllipse")
    shinyjs::disable("confLevel")
  })
  
  observe({
    req(input$optionbutton_state)
    if(input$confEllipse){
      shinyjs::enable("confLevel")
    } else {
      shinyjs::disable("confLevel")
    }
  })
  
  observe({
    req(input$betacol != "No Color", input$confEllipse)
    confInterval(input$confLevel)
  })
  
  ## update adonisVar and adonisStrata if inputs change
  observeEvent(betaSettings()$adonisvar,{
    req(aggDat())
    if(!is.null(betaSettings()$adonisvar) && betaSettings()$adonisvar != ""){
      phenoData <- pData(aggDat())
      ## check if columns contains NA
      varNA <- (sum(is.na(
        phenoData[,betaSettings()$adonisvar])) |
          sum(phenoData[,betaSettings()$adonisvar] %in% "")) > 0
      strataNA <- FALSE
      if(!is.null(betaSettings()$adonisstrata)){
        strataNA <- (sum(is.na(
          phenoData[,betaSettings()$adonisstrata])) |
            sum(phenoData[,betaSettings()$adonisstrata] %in% "")) > 0
      }
      ## in case NAs are present, we need to check in with user
      if(varNA | strataNA){
        showModal(modalDialog(
          title = "Adonis Variables contains NA",
          paste0("NA values are present in the adonis variables:\n\n",
                 'if'(
                   varNA, 
                   paste0("\n\t",
                          betaSettings()$adonisvar),
                   ""),
                 'if'(
                   strataNA,
                   paste0("\n\t",
                          betaSettings()$adonisstrata),
                   ""),
                 "\nTo compute adonis statistics, all NAs need to be removed
                 or treated as \"NA\" strings. \n\n We advise to use adonis
                variables not containing NA values.\n\n
                Do you want to replace NA with \"NA\" or abort to review?"),
          footer = tagList(actionButton(ns("confirmReplace"), 
                                        "Replace"),
                           actionButton(ns("confirmReview"), 
                                        "Abort")
          ),
          easyClose = FALSE))
      } else {
        if(!varNA){
          adonisVar(phenoData[,betaSettings()$adonisvar]) 
        }
        if(!strataNA){
          if(!is.null(betaSettings()$adonisstrata)){
            adonisStrata(
              phenoData[,betaSettings()$adonisstrata])
          } else {
            adonisStrata(NULL)
          }
        }
      }
    } else {
      adonisText(NULL)
      adonisVar(NULL)
      adonisStrata(NULL)
      adonisCode(NULL)
    }
  })
  
  observeEvent(input$confirmReview,{
    removeModal()
    adonisVar(NULL)
    adonisStrata(NULL)
  })    
  
  
  observeEvent(input$confirmReplace,{
    removeModal()
    phenoData <- pData(aggDat())
    adonisVar('if'(
      is.na(phenoData[,betaSettings()$adonisvar]),
      "NA",
      phenoData[,betaSettings()$adonisvar]))
    
    if(!is.null(betaSettings()$adonisstrata)){
      adonisStrata('if'(
        is.na(
          phenoData[,betaSettings()$adonisstrata]),
        "NA",
        phenoData[,betaSettings()$adonisstrata]))
    }
  })
  
  observeEvent(c(aggDat(), betadistance()),{
    req(aggDat(), betadistance() != "")
    if(!(betadistance() %in% names(distMat()))){
      withProgress({
        newDist <- list(computeDistMat(aggDat(),betadistance()))
        incProgress(0.5)
        oldDists <- distMat()
        oldDists[[betadistance()]] <- newDist
        distMat(oldDists)
      }, message = "Calculating distance matrix and PCAs")
    }
    pcaVals(calculatePCAs(distMat()[[betadistance()]][[1]], 
                          c(xbeta(),ybeta())))
  }, priority = 20)
  
  output$statsdatatable <- DT::renderDT({
    req(aggDat(),distMat(), adonisVar())
    if(!is.null(betaSettings()$adonisvar) && betaSettings()$adonisvar != ""){ 
      beta_dis <- distMat()[[betadistance()]][[1]]
      req(labels(beta_dis) == rownames(pData(aggDat())))
      phenoData <- pData(aggDat())
      phenoData[,betaSettings()$adonisvar] <-
        adonisVar()
      
      formula <- stats::as.formula(paste("beta_dis ~ ",
                                         betaSettings()$adonisvar))
      x <- tryCatch(
        vegan::adonis(formula = formula, 
                      data = phenoData, 
                      strata = isolate(adonisStrata()),
                      permutations=500, 
                      parallel = 1),
        error = function(e){
          showModal(modalDialog(
            title = "Error running adonis",
            e,
            easyClose = TRUE
          ))
          return(NULL)
        }
      )
      
      tableCaption <- paste0("Adonis variance of ",
                             isolate(betaSettings()$adonisvar))
      if(!is.null(adonisStrata()))
        tableCaption <- paste0(tableCaption,
                               " with strata ",
                               isolate(betaSettings()$adonisstrata))
      
      adonisCode(paste(
        paste0("\nphenoData <- pData(aggDat)"),
        paste0("phenoData[,\"",betaSettings()$adonisvar,
               "\"] <- c(", paste0(
                 paste0("\"",adonisVar(),"\""),
                 collapse = ","),")"),
        paste0("formula <- as.formula(distMat ~ ",
               betaSettings()$adonisvar,")\n"),
        paste0("x <- vegan::adonis(formula = formula,"), 
        paste0("\tdata = phenoData,"), 
        paste0("\tstrata = ", 'if'(is.null(adonisStrata()), "NULL",
                                   paste0("c(",paste0(
                                     paste0("\"",adonisStrata(),"\""),
                                     collapse = ","),")")),","),
        paste0("\tpermutations=500,"), 
        paste0("\tparallel = 1)"),
        paste0("adonisData <- as.data.frame(x$aov.tab)"),
        paste0("adonisData[] <- sapply(adonisData, as.numeric)"),
        paste0("adonisData[] <- sapply(adonisData, round, digits = getOption(\"me.round_digits\"))"),
        paste0("\n\nif(doctype == \"html\"){"),
        paste0("\tDT::datatable(data = adonisData, 
                   class = \"stripe hover cell-border order-column\","),
        paste0("\t\tfilter = \"none\", style = \"bootstrap\","),
        paste0("\t\tcaption = \"",tableCaption,"\","),
        paste0("\t\toptions = list(scrollX = TRUE, paging = FALSE, 
                   digits = 4,dom = \"<tp>\"),"),
        paste0("\t\tescape = FALSE)"),
        paste0("} else {"),
        paste0("\tkable(adonisData)"),
        paste0("}\n\n"),
        sep = "\n"))
      
      
      adonisData <- as.data.frame(x$aov.tab)
      adonisData[] <- vapply(adonisData, as.numeric, numeric(3))
      adonisData[] <- vapply(adonisData, round, 
                             digits = getOption("me.round_digits"),
                             numeric(3))
      
      adonisText(paste0("R2: ", adonisData[1,"R2"],
                        "; Pr(>F): ", adonisData[1,"Pr(>F)"]))
      
      
      DT::datatable(
        data = adonisData, 
        class = "stripe hover cell-border order-column",
        filter = "none", style = "bootstrap",
        caption = tableCaption,
        options = list(
          scrollX = TRUE,
          paging = FALSE,
          digits = 4,
          dom = "<tp>"
        ),
        escape = FALSE
      )
    } else {
      adonisText(NULL)
      return (NULL)
    }
  })
  
  ## update plot options based on GO click
  observeEvent(input$changeBetaSettings,{
    changed <- FALSE
    updatePlot(FALSE)
    if(!input$Xbeta %in% xbeta()){
      xbeta(input$Xbeta)
      changed <- TRUE
    }
    if(!input$Ybeta %in% ybeta()){
      ybeta(input$Ybeta)
      changed <- TRUE
    }
    if(changed){
      req(distMat(), betadistance() != "")
      pcaVals(calculatePCAs(distMat()[[betadistance()]][[1]], 
                            c(xbeta(),ybeta())))
    }
    if(!input$betacol %in% colorChoice()){
      colorChoice(input$betacol)
      changed <- TRUE
    }
    if(!input$betashape %in% shapeChoice()){
      shapeChoice(input$betashape)
      changed <- TRUE
    }
    if(!input$betasize %in% sizeChoice()){
      sizeChoice(input$betasize)
      changed <- TRUE
    }
    if(input$confEllipse && !(input$confLevel != confInterval())){
      confInterval(input$confLevel)
      changed <- TRUE
    }
    if(!input$confEllipse && !is.null(confInterval())){
      confInterval(NULL)
      changed <- TRUE
    }
    updatePlot(changed)
  })
  
  observe({
    req(plotComplete())
    aT <- adonisText()
    if(is.null(aT))
      aT <- ""
    plotly::plotlyProxy("betaDiv", session) %>%
      plotly::plotlyProxyInvoke(
        "restyle",
        list(text = aT),
        0
      )
    shinyjs::js$resetAxes()
  })
  
  observeEvent(input$plotWidth, {
    req(pcaVals(), plotComplete())
    plotly::plotlyProxy("betaDiv", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = input$plotWidth)
      )
  })
  
  ## calls beta diversity plotting function
  output$betaDiv <- plotly::renderPlotly({
    req(pcaVals())
    shinyjs::enable("optionbutton_state")
    updatePlot()
    plotComplete(FALSE)
    plotTitle <- paste0(betadistance(),
                        " diversity at ", 
                        aggLevel()," level")
    color <- as.character(isolate(colorChoice()))
    if (stringr::str_detect(color, "No Color")) {
      color <- NULL
    }
    shape <- as.character(isolate(shapeChoice()))
    if (stringr::str_detect(shape, "None")) {
      shape <- NULL
    }
    
    
    pb <- plotBeta(
      aggdat = aggDat(),
      dist_method = tolower(betadistance()),
      pcas = isolate(pcaVals()),
      dim = isolate(c(xbeta(), ybeta())),
      col_by = color,
      shape_by = shape,
      plotTitle = plotTitle,
      pt_size = isolate(sizeChoice()),
      plotText = isolate(adonisText()),
      confInterval = isolate(confInterval())
    )
    shinyjs::delay(500, plotComplete(TRUE))
    pb
  })
  
  ## update the stored R code based on input choices
  observe({
    req(pcaVals())
    color <- as.character(colorChoice())
    if (stringr::str_detect(color, "No Color")) {
      color <- NULL
    }
    shape <- as.character(shapeChoice())
    if (stringr::str_detect(shape, "None")) {
      shape <- NULL
    }
    plotTitle <- paste0(betadistance(),
                        " diversity at ", 
                        aggLevel()," level")
    
    repCode(paste(
      paste0("#' ### Beta diversity"),
      paste0("", "#- fig.width = 7"),
      paste0("distMat <- computeDistMat(aggDat, \"", 
             tolower(betadistance()), "\")"),
      paste0("pcaVals <- calculatePCAs(distMat, 
                      c(\"", xbeta(), "\", \"", ybeta(), "\"))\n"),
      paste0("plotBeta(aggDat,"),
      paste0("\tdist_method = \"", tolower(betadistance()), "\","),
      paste0("\tpcas = pcaVals,"),
      paste0("\tdim = c(\"", xbeta(), "\", \"", ybeta(), "\"),"),
      paste0("\tcol_by = ", 
             'if'(is.null(color), "NULL", 
                  paste0("\"", color, "\"")), ","),
      paste0("\tshape_by = ", 
             'if'(is.null(shape), "NULL", 
                  paste0("\"", shape, "\"")), ","),
      paste0("\tplotTitle = \"",plotTitle, "\","),
      paste0("\tpt_size = \"",sizeChoice(), "\","),
      paste0("\tplotText = \"",adonisText(), "\","),
      paste0("\tconfInterval = ",confInterval(), ","),
      paste0("\tallowWebGL = FALSE)\n\n"),
      adonisCode(),"\n\n",
      sep = "\n"
    ))
  })
  
  return(repCode)
}

#' Main correlation analysis input module.
#' Handles correlation analysis tab in the app
#'
#' @param id element identifier - namespace
#' @param type determines if 'feature' or 'pheno' correlation
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
corrInputUI <- function(id, type) {
  ns <- NS(id)
  
  box(width = 12, id = ns("analysisbox"),
      h4("ANALYSIS PARAMETERS"),
      div(
        h5('if'(type == "feature", 
                "Feature vs Feature",
                "Feature vs Phenotype")),
        selectizeInput(
          ns("featurecorr1"),
          label = "Feature correlation (base)", choices = "", 
          multiple = FALSE, width = "250px"
        ),
        if(type == "feature") {
          selectizeInput(
            ns("featurecorr2"),
            label = "Feature correlation 2", choices = "", 
            multiple = FALSE, width = "250px"
          )
        } else {
          selectInput(
            ns("phenoselect"),
            label = "Phenotype correlation", choices = "", 
            multiple = FALSE, selectize = TRUE, width = "250px"
          )
        }
      ),
      div(
        selectInput(
          ns("facetby"),
          label = "Facet columns by", choices = "", multiple = FALSE, 
          selectize = FALSE, width = "250px"
        ),
        selectInput(
          ns("facetby2"),
          label = "Facet rows by", choices = "", multiple = FALSE, 
          selectize = FALSE, width = "250px"
        )
      ),
      selectInput(
        ns("method"),
        label = "Method", choices = getOption("me.corrmethods"), 
        multiple = FALSE, 
        selectize = FALSE, width = "250px"
      ),
      ## buttons are used to submit events. This ensures plots are not redrawn 
      ## while user still adjusts parameters
      div(id = ns("buttondiv"),
          fluidRow( 
            width = 12, id = "actionbuttonrow",
            ## update analysis outputs (plots/tables)
            actionButton(
              ns("updatebutton"), 
              icon = icon("far fa-chart-bar"),
              label = HTML("&nbsp;UPDATE"), width = "120px"),
            actionButton(
              ns("reportButton"), 
              label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
              width = "120px")
          )
      )
  )
}


#' Server side for the analysis input module handling analysis control
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param type of the correlation (feature vs phenotype)
#' @param meData MRExperiment object storing all data
#' @param facetOptions named vector of available facet choices
#' @param reset reactive boolean determining if all inputs should be reset 
#' @param aggDat aggregated MRExperiment object (default is NULL)
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
corrInput <- function(input, output, session, type,
                      meData, facetOptions = NULL,
                      reset, aggDat = reactive(NULL)) {
  ns <- session$ns
  
  ## reactive value storing choices made in the UI
  chosenValues <- reactiveVal(NULL)
  ## reactive storing sorted features at given level
  aggFeatures <- reactiveVal(NULL)
  
  observe({
    req(aggDat())
    aggFeatures(rownames(MRcounts(aggDat())))
  })
  
  ## check which phenotypes are numeric
  numericPheno <- reactive({
    req(meData())
    names(pData(meData()))[vapply(pData(meData()), is.numeric, logical(1))]
  })
  
  ## reset all entries
  observe({
    req(reset())
    chosenValues(NULL)
    aggFeatures(NULL)
  })
  
  
  ## initialize input elements
  observe({
    req(meData())
    if(!is.null(chosenValues()$method))
      updateSelectInput(session, "method", 
                        selected = chosenValues()$method)
    updateSelectInput(session, "facetby", 
                      choices = c("", facetOptions()), 
                      selected = chosenValues()$facetby)
    updateSelectInput(session, "facetby2", 
                      choices = c("", facetOptions()), 
                      selected = chosenValues()$facetby2)
  })
  
  observe({
    req(meData(),numericPheno(), type() == "pheno")
    updateSelectInput(session, "phenoselect", 
                      choices = c("", numericPheno()), 
                      selected = chosenValues()$phenoselect)
  })
  
  # ## react to phenotype selection variable (facets should be distinct)
  observeEvent(input$phenoselect, {
    req(facetOptions())
    facetopts <- facetOptions()
    facetopts <- facetopts[!facetopts %in% input$phenoselect]
    updateSelectInput(session, "facetby", choices = c("", facetopts))
    updateSelectInput(session, "facetby2", choices = c("", facetopts))
  }, priority = 15, ignoreInit = TRUE)
  
  ## react to first facet variable (phenotype variables should be distinct)
  observeEvent(input$facetby, {
    req(facetOptions(), type())
    facetopts <- facetOptions()
    facetopts <- facetopts[!facetopts %in% input$facetby]
    if (type() == "pheno") {
      req(numericPheno())
      numPheno <- numericPheno()
      numPheno <- numPheno[!numPheno %in% input$facetby]
      facetopts <- facetopts[!facetopts %in% input$phenoselect]
      updateSelectInput(session, "phenoselect", choices = c("", numPheno), 
                        selected = input$phenoselect)
    }
    updateSelectInput(session, "facetby2", choices = c("", facetopts), 
                      selected = input$facetby2)
  }, priority = 15, ignoreInit = TRUE)
  
  ## react to second facet variable (phenotype variables should be distinct)
  observeEvent(input$facetby2, {
    req(facetOptions(), type())
    facetopts <- facetOptions()
    facetopts <- facetopts[!facetopts %in% input$facetby2]
    if (type() == "pheno") {
      req(numericPheno())
      numPheno <- numericPheno()
      numPheno <- numPheno[!numPheno %in% input$facetby2]
      facetopts <- facetopts[!facetopts %in% input$phenoselect]
      updateSelectInput(session, "phenoselect", choices = c("", numPheno),
                        selected = input$phenoselect)
    }
    updateSelectInput(session, "facetby", choices = c("", facetopts), 
                      selected = input$facetby)
  }, priority = 15, ignoreInit = TRUE)
  
  ## update any inputs based on feature values
  observeEvent(aggFeatures(),{
    req(aggFeatures(), type())
    corrdata  <- data.frame(value = aggFeatures(), 
                            label = aggFeatures())
    selcorr <- corrdata[1,]
    updateSelectizeInput(session, "featurecorr1",
                         choices = corrdata,
                         selected = selcorr,
                         server = TRUE)
    selcorr <- ""
    if(type() == "feature"){
      if(!is.null(chosenValues()$featurecorr2))
        selcorr <- chosenValues()$featurecorr2
      updateSelectizeInput(session, "featurecorr2",
                           choices = corrdata,
                           selected = selcorr,
                           server = TRUE,
                           options = list(placeholder = "Select feature"))
    }
  })
  
  
  ## main control button: store input choices in chosenValues
  observeEvent(input$updatebutton, {
    req(type())
    facet1 <- NULL
    if(!is.null(input$facetby)){
      if(input$facetby != "")
        facet1 <- input$facetby
    }
    facet2 <- NULL
    if(!is.null(input$facetby2)){
      if(input$facetby2 != "")
        facet2 <- input$facetby2
    }
    if(type() == "feature"){
      cV <- list("featurecorr1" = input$featurecorr1,
                 "featurecorr2" = input$featurecorr2,
                 "facetby" = facet1,
                 "facetby2" = facet2,
                 "method" = input$method)
    } else {
      cV <- list("featurecorr1" = input$featurecorr1,
                 "phenoselect" = input$phenoselect,
                 "facetby" = facet1,
                 "facetby2" = facet2,
                 "method" = input$method)
    }
    chosenValues(cV)
  })
  
  return(chosenValues)
}

## Module handling aggregating the MRExperiment data

#' Aggregation module ui function
#'
#' @param id namespace identifier
#' @author Janina Reeder
#' 
#' @return box holding aggregation input elements
#' 
#' @examples aggregationTabUI("atu_id")
#' 
#' @export
aggregationTabUI <- function(id) {
    ns <- NS(id)
    
    box(
        id = ns("aggregation_box"),
        style = "padding-top: 0; padding-left: 20px;",
        width = 12,
        title = textOutput(ns("agg_title")),
        collapsible = TRUE,
        collapsed = FALSE,
        
        
        fluidRow(
            width = 12,
            shinyjs::disabled(
                div(
                    id = ns("normaggdiv"),
                    column(
                        width = 3,
                        selectInput(
                            ns("normalizedata"),
                            label = "Normalize Data",
                            choices = c("Proportion", "CSS", 
                                        "Do not normalize"),
                            multiple = FALSE, 
                            selectize = FALSE,
                            width = "150px")
                    ),
                    column(
                        width = 3,
                        selectInput(
                            ns("featurelevel"),
                            label = "Feature level", 
                            choices = "",
                            multiple = FALSE, 
                            selectize = FALSE, 
                            width = "250px"
                        )
                    ),
                    column(
                        width = 2,
                        div(style = "margin-top: 24px;",
                            actionButton(
                                ns("aggregatebutton"), 
                                icon = icon("fas fa-compress"),
                                label = HTML("&nbsp;AGGREGATE"),
                                width = "120px"))
                    ),
                    column(
                        width = 2,
                        div(style = "margin-top: 24px;",
                            shinyjs::disabled(
                                downloadButton(
                                    ns("savebutton"),
                                    label = "GET DATA",
                                    width = "120px"))
                        )
                    )
                )
            )
        )    
    )
}

#' Aggregation module server function
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param resetInput boolean updated to TRUE if new data is available
#' @param levelOpts available levels to aggregate on (depends on input data)
#' @param chosenLevel previously selected level (passed from different instance)
#' @param meData the main MRexperiment object
#' 
#' @author Janina Reeder
#' 
#' @return reactive list holding aggregated object, aggregation code and 
#' boolean on normalization
aggregationTab <- function(
    input, output, session, 
    resetInput, levelOpts, chosenLevel, meData) {
    
    ns <- session$ns
    
    aggCode <- reactiveVal(NULL)
    aggMRobj <- reactiveVal(NULL)
    normalizedData <- reactiveVal(FALSE)
    normUpdate <- reactiveVal(FALSE)
    
    observeEvent(
        meData(),{
            if(isFALSE(normUpdate())){
                if(is.null(fData(meData()))){
                    shinyjs::disable("normaggdiv")
                } else {
                    shinyjs::enable("normaggdiv")
                }
                aggCode(NULL)
                aggMRobj(NULL)
                normalizedData(FALSE)
            } else {
                normUpdate(FALSE)
            }
        }, ignoreNULL = TRUE)
    
    observeEvent(levelOpts(),{
        updateSelectInput(session, "featurelevel", 
                          choices = levelOpts(), 
                          selected = chosenLevel())
    })
    
    
    observeEvent(input$aggregatebutton, {
        req(meData())
        aggC <- ""
        if(input$normalizedata %in% c("Proportion", "CSS")){
            normUpdate(TRUE)
            meData(normalizeData(meData(), norm_method = input$normalizedata))
            aggC <- paste0("\nmeData <- normalizeData(meData, norm_method = \"",
                           input$normalizedata, "\")")
            normalizedData(TRUE)
        } else if(all(is.na(normFactors(meData())))){
            normalizedData(FALSE)
        } else {
            normalizedData(TRUE)
        }
        aggMRobj(aggFeatures(meData(), level = input$featurelevel))
        chosenLevel(input$featurelevel)
        aggCode(
            paste0(
                aggC,"\naggDat <- aggFeatures(meData, level = \"",
                input$featurelevel, "\")\n"))
        shinyjs::js$collapse(ns("aggregation_box"))
    })
    
    output$agg_title <- renderText({
        if (is.null(aggMRobj())) {
            return("Aggregation")
        } else {
            return(paste0("Aggregation level: ",chosenLevel()))
        }
    })
    
    observe({
        if(!is.null(aggMRobj())){
            shinyjs::enable("savebutton")
        } else {
            shinyjs::disable("savebutton")
        }
    })
    
    ## download aggregated counts as csv
    output$savebutton <- downloadHandler(
        filename = function() {
            paste0("featureCounts.csv")
        },
        content = function(file) {
            rawcounts <- as.data.frame(MRcounts(aggMRobj()))
            rawcounts <- rawcounts %>%
                tibble::rownames_to_column(var = "Features")
            readr::write_csv(rawcounts, file)
        }
    )
    
    return(list(aggCode = aggCode,
                mrobj = aggMRobj,
                normalizedData = normalizedData))
    
}

#' Main diffanalysis input module.
#' Set up to handle diff analysis tabs in the app depending on given parameters
#'
#' @param id element identifier - namespace
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
longInputUI <- function(id) {
  ns <- NS(id)
  
  box(
    width = 12, id = ns("analysisbox"),
    h4("ANALYSIS PARAMETERS"),
    selectizeInput(
      ns("featureselect"),
      label = "Select Feature", choices = "", 
      multiple = FALSE, 
      width = "250px"
    ),
    div(
      selectInput(
        ns("comparison"),
        label = "Longitudinal phenotype", choices = "", 
        multiple = FALSE, selectize = FALSE, width = "250px"
      ),
      selectizeInput(
        ns("phenolevels"),
        label = "Phenotype level order",
        choices = "", multiple = TRUE, width = "250px"
      ),
      selectInput(
        ns("phenoid"),
        label = "ID phenotype (optional)", choices = "", 
        multiple = FALSE, selectize = FALSE, width = "250px"
      )
    ),
    ## buttons are used to submit events. This ensures plots are not redrawn 
    ## while user still adjusts parameters
    div(
      id = ns("buttondiv"),
      fluidRow( 
        width = 12, id = "actionbuttonrow",
        ## update analysis outputs (plots/tables)
        actionButton(
          ns("updatebutton"), 
          icon = icon("far fa-chart-bar"),
          label = HTML("&nbsp;UPDATE"), width = "120px"),
        actionButton(
          ns("reportButton"), 
          label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
          width = "120px")
      )
    )
  )
}


#' Server side for the analysis input module handling analysis control
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRexperiment object storing all data
#' @param facetOptions named vector of available facet choices
#' @param reset reactive boolean determining if all inputs should be reset
#' @param aggDat aggregated MRexperiment
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
longInput <- function(input, output, session, 
                      meData, facetOptions = NULL,
                      reset, aggDat = reactive(NULL)) {
  ns <- session$ns
  
  ## reactive value storing choices made in the UI
  chosenValues <- reactiveVal(NULL)
  ## reactive storing all available phenotype levels
  phenoOpts <- reactiveVal(NULL)
  aggFeatures <- reactiveVal(NULL)
  
  observe({
    req(aggDat())
    aggFeatures(rownames(MRcounts(aggDat())))
  })
  
  ## reset all entries
  observe({
    req(reset())
    chosenValues(NULL)
    phenoOpts(NULL)
    aggFeatures(NULL)
  })
  
  ## update any inputs based on feature values
  observeEvent(aggFeatures(),{
    req(aggFeatures())
    featdata  <- data.frame(value = aggFeatures(), 
                            label = aggFeatures())
    selfeat <- ""
    if(!is.null(chosenValues()$featureselect))
      selfeat <- chosenValues()$featureselect
    updateSelectizeInput(session, "featureselect",
                         choices = featdata,
                         selected = selfeat,
                         options = list(placeholder = "Select Feature"),
                         server = TRUE)
  })
  
  ## initialize input elements
  observe({
    req(meData())
    updateSelectInput(session, "comparison", 
                      choices = c("", facetOptions()))
    updateSelectInput(session, "phenoid", 
                      choices = c("", names(pData(meData()))))
  })
  
  observe({
    req(input$comparison)
    req(input$comparison %in% names(pData(meData())))
    phenoOpts(levels(
      forcats::fct_explicit_na(
        factor(pData(meData())[, input$comparison]),
        na_level = "NA"))
    )
    updateSelectizeInput(session, "phenolevels",
                         choices = phenoOpts(),
                         options = list(placeholder = "Select levels"),
                         selected = "")
    remainingIds <- names(pData(meData()))[!(names(pData(meData())) 
                                             %in% input$comparison)]
    updateSelectInput(session, "phenoid", 
                      choices = c("", remainingIds))
  }) 
  
  observe({
    req(!is.null(input$comparison))
    req(input$comparison == "")
    updateSelectizeInput(session, "phenolevels",
                         choices = "",
                         options = list(placeholder = "Select levels"),
                         selected = "")
  })
  
  ## main control button: store input choices in chosenValues
  observeEvent(input$updatebutton, {
    comparison <- NULL
    if(input$comparison != "")
      comparison <- input$comparison
    cV <- list("featureselect" = input$featureselect,
               "comparison" = comparison,
               "phenolevels" = input$phenolevels,
               "phenoid" = input$phenoid)
    chosenValues(cV)
  })
  
  
  return(chosenValues)
}

## Main module for feature sample analysis

#' feature Analysis Module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow containing the ui code
#' 
#' @examples featureAnalysisUI("featureanalysis_id")
#' 
#' @export
featureAnalysisUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    width = 12,
    column(
      width = 3,
      featureInputUI(ns("featureInput"))
    ),
    column(
      width = 9,
      fluidRow(
        width = 11,
        box(width = 10,
            p("The feature sample page contains methods to analyze the microbial composition 
              and diversity within a sample")
        ),
        avgAbundanceUI(ns("avgFeaturePlot")),
      )
    )
  )
}


#' feature Analysis Module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param data the main data object returned from data_input_module
#' @param resetInput reactive boolean determining if reset is required
#' @param aggData the aggregated MRExperiment object
#' @param normalizedData boolean indicating if normalization was done
#' 
#' @author Janina Reeder
#' 
#'
#' @return reactive holding code to be used in reports
featureAnalysis <- function(input, output, session, data,
                          resetInput, aggData, normalizedData) {
  ## feature SAMPLE ANALYSIS
  ns <- session$ns
  
  
  featureSettings <- callModule(featureInput,
                              "featureInput",
                              meData = data$meData,
                              facetOptions = data$facetOptions,
                              reset = resetInput,
                              aggDat = reactive(aggData$mrobj)
  )
  
  
  featureRep <- callModule(avgAbundance,
                         "avgFeaturePlot",
                         aggDat = reactive(aggData$mrobj),
                         featLevel = reactive(aggData$level),
                         featureSettings = featureSettings$chosenValues,
                         normalizedData = normalizedData,
                         reset = resetInput
  )
  
  return(featureRep)
  
}

#' Phenotype table UI module
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return fluidRow holding the ui code
#' 
#' @examples phenotypeTableUI("phenotype_id")
#' 
#' @export
phenotypeTableUI <- function(id) {
  ns <- NS(id)
  
  fluidRow(
    column(width = 3, id = ns("phenointeractioncol"),
           h4("COMBINE PHENOTYPE COLUMNS"),
           selectInput(
             ns("pheno1"),
             label = "Phenotype 1",
             choices = "", multiple = FALSE, selectize = FALSE, 
             width = "250px"),
           selectInput(
             ns("pheno2"),
             label = "Phenotype 2",
             choices = "", multiple = FALSE, selectize = FALSE, 
             width = "250px"),
           textInput(
             ns("interactionname"),
             label = "Column Name",
             value = "", width = "250px"),
           fluidRow(
             width = 12, id = "actionbuttonrow",
             actionButton(
               ns("createbutton"), 
               icon = icon("fas fa-plus"),
               label = HTML("&nbsp;ADD"), width = "100px"),
             actionButton(
               ns("savebutton"), 
               icon = icon("far fa-save"),
               label = HTML("&nbsp;SAVE"), width = "100px")
           ),
           br(),
           box(
             width = 12,
             title = "ADJUST DATATYPES (optional)",
             selectInput(
               ns("phenotype"),
               label = "Adjust Phenotype",
               choices = "", multiple = FALSE, selectize = FALSE, 
               width = "250px"),
             selectInput(
               ns("phenodatatype"),
               label = "Change data type",
               choices = "", multiple = FALSE, selectize = FALSE, 
               width = "250px"),
             fluidRow(
               width = 12, id = "actionbuttonrow2",
               actionButton(
                 ns("changebutton"), 
                 icon("fas fa-exchange-alt"),
                 label = HTML("&nbsp;ADJUST"), width = "90px"),
               actionButton(
                 ns("savebutton2"),
                 icon = icon("far fa-save"),
                 label = HTML("&nbsp;SAVE"), width = "90px")
             ),
             collapsed = TRUE,
             collapsible = TRUE
           )
    ),
    column(
      width = 9,
      box(
        width = 10,
        h2("PHENOTYPE INFORMATION"),
        p("Create new interaction phenotypes by combining two columns and adjust  
         column data types if needed. New phenotype information can be added  
         via \"Load & Filter\" tab. Table settings allow paging through sections
          of the data, choosing how many entries to display or searching for 
         specific entries.Modifications must be saved in order to be available 
         in the analysis sections.")
      ),
      box(
        width = 12, 
        div(
          id = ns("downloaddiv"),
          downloadButton(ns("download_button"),"Download"),
          DT::DTOutput(ns("phenodatatable"), width = "100%")
        )
      )
    )
  )
}

#' Helper function returning the code used to modify the phenotable as a string
#'
#' @param name interaction name
#' @param pheno1 first interaction phenotype
#' @param pheno2 second interaction phenotype
#' 
#' @author Janina Reeder
#'
#' @return String storing code to perform modification
getPhenoModCode <- function(name, pheno1, pheno2) {
  paste(paste0("\nnew_pheno <- interaction(pData(meData)[,c(\"",
               pheno1, "\",\"", pheno2, "\")])"),
        paste0("mutatedRows <- row.names(pData(meData))"),
        paste0("mutatedData <- dplyr::mutate(pData(meData), \"", 
               name, "\" = new_pheno)"),
        paste0("row.names(mutatedData) <- mutatedRows"),
        paste0("meData <- addPhenoData(meData,mutatedData)"),
        sep = "\n"
  )
}

adjustPhenoColCode <- function(selectedCols){
  paste(paste0("\nmutatedRows <- row.names(pData(meData))"),
        paste0("mutatedData <- pData(meData)[, ",
               paste0("c(",paste0(selectedCols, collapse=", "),
                      ")", collapse = ", "), "]"),
        paste0("rownames(mutatedData) <- mutatedRows"),
        paste0("meData <- addPhenoData(meData,mutatedData)"),
        sep = "\n")
}

#' Helper function returning the code used to modify the data types of the
#' pheno table
#'
#' @param phenotype name of the phenotype column header
#' @param datatype variable type to assign to column
#' 
#' @author Janina Reeder
#'
#' @return String storing code to perform modification
getPhenoChanges <- function(phenotype, datatype){
  paste(paste0("\nnew_pheno <- pData(meData)"),
        paste0("pfdata <- new_pheno[,\"",phenotype,"\"]"),
        paste0("pfdata <- switch(\"",datatype,"\","),
        paste0("\t\"factor\" = as.factor(pfdata),"),
        paste0("\t\"integer\" = as.integer(pfdata),"),
        paste0("\t\"numeric\" = as.numeric(pfdata),"),
        paste0("\t\"logical\" = as.logical(pfdata))"),
        paste0("new_pheno[,\"",phenotype,"\"] <- pfdata"),
        paste0("meData <- addPhenoData(meData,new_pheno)\n\n"),
        sep = "\n"
  )
}



#' Phenotype table server module
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment storing the data
#' @param phenoModRep reactive Value storing any phenotable modifications made
#' @param addPheno reactive boolean keeping track of pheno data modifications
#'
#' @return phenotype table server fragment - no return value
#'
#' @author Janina Reeder
#' 
#' @importFrom Biobase pData
#' @importFrom rlang :=
phenotypeTable <- function(input, output, session, meData, 
                           phenoModRep, addPheno) {
  ns <- session$ns
  variable <- NULL
  ## stores the currently shown phenotype table
  phenoFrame <- reactiveVal(NULL)
  colTypes <- reactiveVal(NULL)
  ## captures whether any changes were made
  mutated <- reactiveVal(FALSE)
  phenoChangeCode <- reactiveVal(NULL)
  phenoInteractCode <- reactiveVal(NULL)
  phenoColChangeCode <- reactiveVal(NULL)
  phenoChanged <- reactiveVal(FALSE)
  
  ## initializes the local pheno frame and input fields
  observe({
    req(meData())
    isolate({
      phenoFrame(pData(meData()))
      colTypes(tibble::as_tibble(phenoFrame()) %>%
                 dplyr::mutate_if(lubridate::is.POSIXt, as.Date) %>%
                 dplyr::summarise_all(class) %>% 
                 tidyr::gather(variable, class))
    })
    updateSelectInput(session, "pheno1", choices = names(pData(meData())))
    updateSelectInput(session, "pheno2", choices = names(pData(meData())))
    updateSelectInput(session, "phenotype", 
                      choices = names(pData(meData())))
  })
  
  observeEvent(input$phenotype,{
    req(colTypes())
    df <- colTypes()
    updateSelectInput(session, "phenodatatype",
                      choices = c("factor","integer","numeric","logical", 
                                  "Date"),
                      selected = df[df$variable %in% input$phenotype,
                                    "class"])
  })
  
  observeEvent(input$changebutton,{
    shinyjs::disable("savebutton2")
    df <- phenoFrame()
    pfdata <- phenoFrame()[,input$phenotype]
    pfdata <- switch(input$phenodatatype,
                     "factor" = as.factor(pfdata),
                     "integer" = as.integer(pfdata),
                     "numeric" = as.numeric(pfdata),
                     "logical" = as.logical(pfdata),
                     "Date" = tryCatch(as.Date(pfdata), 
                                       error = function(e) pfdata))
    df[,input$phenotype] <- pfdata
    phenoFrame(df)
    phenoChangeCode(c(phenoChangeCode(),
                      getPhenoChanges(phenotype = input$phenotype,
                                      datatype = input$phenodatatype)))
    shinyjs::enable("savebutton2")
  })
  
  ## add new interaction column to phenotable
  observeEvent(input$createbutton, {
    req(phenoFrame())
    shinyjs::disable("savebutton")
    mustabort <- FALSE
    
    if (is.null(input$pheno1) | is.null(input$pheno2) | 
        input$pheno1 == input$pheno2) {
      showModal(modalDialog(
        title = "Error Creating New Phenotype",
        "Two different phenotypes must be selected in order to create 
                a new one. \nPlease revise.",
        easyClose = TRUE))
      mustabort <- TRUE
    }
    interactname <- parseInteractionName(input$interactionname)
    if (startsWith(interactname, "ERROR")) {
      showModal(modalDialog(
        title = "Error Creating New Phenotype",
        paste0("The interaction name is not valid: ", 
               gsub("ERROR: ", "", interactname), "\nPlease revise."),
        easyClose = TRUE))
      mustabort <- TRUE
    }
    if (startsWith(interactname, "WARNING")) {
      showModal(modalDialog(
        title = "Warning",
        paste0("The interaction name contains invalid characters which 
                       have been adjusted: ", 
               gsub("WARNING: ", "", interactname)),
        easyClose = TRUE))
    }
    if (!mustabort) {
      new_pheno <- interaction(phenoFrame()[, c(input$pheno1, 
                                                input$pheno2)])
      ## store old row names to copy over to modified frame
      mutatedRows <- rownames(phenoFrame())
      mutatedData <- dplyr::mutate(phenoFrame(), 
                                   !!interactname := new_pheno)
      rownames(mutatedData) <- mutatedRows
      phenoInteractCode(getPhenoModCode(interactname,input$pheno1,input$pheno2))
      ## mark that table has been modified
      mutated(TRUE)
      ## display modified phenotype table
      phenoFrame(mutatedData)
    }
    shinyjs::enable("savebutton")
  })
  
  ## make changes permanent
  observeEvent(c(input$savebutton, input$savebutton2), {
    req(meData())
    selectedCols <- vapply(
      seq_along(input$phenodatatable_state$columns),
      function(i) input$phenodatatable_state$columns[[i]]$visible,
      logical(1))
    selectedCols <- selectedCols[-1]
    if (sum(selectedCols) != ncol(phenoFrame())) {
      ## store rownames to pass over to modified frame
      mutatedRows <- rownames(phenoFrame())
      mutatedData <- phenoFrame()[, selectedCols]
      rownames(mutatedData) <- mutatedRows
      phenoColChangeCode(adjustPhenoColCode(selectedCols))
      ## mark as modified
      mutated(TRUE)
      ## store modification
      phenoFrame(mutatedData)
    }
    if (mutated()) {
      phenoChanged(TRUE)
      phenoModRep(paste0(c(phenoModRep(),
                           phenoInteractCode()),
                         collapse = "\n"))
      phenoInteractCode(NULL)
      mutated(FALSE)
    }
    if (!is.null(phenoChangeCode())) {
      phenoChanged(TRUE)
      phenoModRep(paste0(c(phenoModRep(),
                           phenoChangeCode()),
                         collapse = "\n"))
      phenoChangeCode(NULL)
    }
    if (!is.null(phenoColChangeCode())) {
      phenoChanged(TRUE)
      phenoModRep(paste0(c(phenoModRep(),
                           phenoColChangeCode()),
                         collapse = "\n"))
      phenoColChangeCode(NULL)
    }
    if(phenoChanged()){
      ## pass modifications over to base data structure
      meData(addPhenoData(meData(), phenoFrame()))
      addPheno(TRUE)
      phenoChanged(FALSE)
    }
  })
  
  ## render pheno data as a table
  output$phenodatatable <- DT::renderDT({
    req(phenoFrame())
    shinyjs::js$moveButton(ns("downloaddiv"),ns("download_button"))
    DT::datatable(
      data = phenoFrame(), 
      class = "stripe hover cell-border order-column",
      filter = "none", style = "bootstrap",
      callback = DT::JS("$('div.dwnld_pheno').append($('#phenotab-download_button'));"),
      extensions = c("FixedColumns", "ColReorder", "Buttons"),
      plugins = "ellipsis",
      options = list(
        scrollX = TRUE,
        fixedColumns = list(leftColumns = 1, rightColumns = 0),
        paging = TRUE,
        colReorder = TRUE,
        stateSave = TRUE,
        stateLoadParams = DT::JS("function (settings, data) 
                                         {return false;}"),
        columnDefs = list(list(
          targets = "_all",
          render = DT::JS("$.fn.dataTable.render.ellipsis( 50 , false)"))),
        buttons = list(
          list(
            extend = "colvis",
            text = "Select columns"
          )), 
        dom = '<"dwnld_pheno"B>ftlp'
      )
    )
  })
  
  
  
  
  output$download_button <- downloadHandler(
    filename = function() {
      paste0("phenodata-", Sys.Date(), ".csv")
    },
    content = function(file) {
      readr::write_csv(phenoFrame(), file)
    }
  )
}

#' report tab ui
#'
#' @param id namespace identifier
#' 
#' @return fluidRow holding ui elements
#'
#' @author Janina Reeder
#' 
#' @examples reportListUI("reportlist_id")
#' 
#' @export
reportListUI <- function(id) {
    ns <- NS(id)
    
    fluidRow(
        column(
            width = 4,
            h4("REPORT SETTINGS"),
            textInput(
                ns("repfile"),
                label = "File name",
                value = "me_report"
            ),
            textInput(
                ns("reptitle"),
                label = "Report title",
                value = "MicrobiomeExplorer Report"
            ),
            textInput(
                ns("repauthor"),
                label = "Author",
                value = ""
            ),
            textAreaInput(
                ns("intro_text"), "Introductory text (optional)",
                placeholder = "Any text entered here will be presented at the 
                top of the generated report"
            ),
            shinyWidgets::prettyCheckbox(
                inputId = ns("repcontents"),
                label = "Include table of contents",
                value = TRUE,
                icon = icon("check"),
                status = "success",
                animation = "rotate",
                outline = TRUE
            ),
            h4("REPORT FORMAT", style = "padding-bottom: 0;"),
            shinyWidgets::prettyCheckboxGroup(
                inputId = ns("repformat"),
                label = "",
                shape = "square",
                icon = icon("check"),
                choices = getOption("me.reportformatshort"),
                selected = "HTML",
                status = "success",
                animation = "rotate",
                outline = TRUE
            ),
            fluidRow(
                width = 12,
                actionButton(
                    ns("generatebutton"), 
                    icon = icon("fas fa-file-alt"),
                    label = HTML("&nbsp;GENERATE"), 
                    width = "110px"),
                shinyjs::disabled(
                    actionButton(ns("exportbutton"), 
                                 icon = icon("fas fa-download"),
                                 label = HTML("&nbsp;EXPORT"), 
                                 width = "90px"))
            ),
            fluidRow(
                width = 12,
                downloadButton(ns("downloadbutton"), 
                               label = "Download", 
                               style = "visibility: hidden;")
            )
        ),
        column(
            width = 8,
            h2("REPORT CONTENTS"),
            uiOutput(ns("dataRowBox"), width = "90%"),
            uiOutput(ns("qcRowBox"), width = "90%"),
            uiOutput(ns("reportListBox"), width = "90%")
        )
    )
}

#' Report tab module server
#'
#' @param input module input
#' @param output module output
#' @param session app session
#' @param dataSource R code to obtain data for rendering
#' @param preprocessRep R code containing preprocessing steps of data
#' @param qcRep R Code to generate QC plots
#' @param analysisRep R Code to generate all analyses saved to reports
#' @param aggIndex boolean value representing aggregation steps in analysisRep
#' @param reset boolean reactive which resets the module if TRUE
#' 
#' @return report list server fragment - no return value
#' 
#' @author Janina Reeder
reportList <- function(input, output, session, 
                       dataSource,
                       preprocessRep, 
                       qcRep, 
                       analysisRep, 
                       aggIndex,
                       reset) {
    ns <- session$ns
    
    ## stores info on whether qc plots should be included in final report
    qcSelected <- reactiveVal(FALSE)
    ## stores info on which analysis rows should be included in final report
    selectedRows <- reactiveVal(NULL)
    
    
    dataCode <- reactive({
        req(preprocessRep(), dataSource())
        paste0(dataSource(),
               paste0(preprocessRep(), collapse = "\n"),
               sep = "\n\n")
    })
    
    ## server code for data row module
    observe({
        req(dataCode())
        callModule(reportRow, "datarow", type = TRUE, content = dataCode)
    })
    
    ## server code for qc plot module
    observe({
        req(qcRep())
        qcSelected(callModule(reportRow, "qcrow", 
                              type = FALSE, content = qcRep))
    })
    
    ## ui call for data row
    output$dataRowBox <- renderUI({
        req(preprocessRep())
        div(
            box(
                width = 11,
                reportRowUI(ns("datarow"), type = TRUE)
            )
        )
    })
    
    ## ui call for qc plot row
    output$qcRowBox <- renderUI({
        req(qcRep())
        div(
            box(
                width = 11,
                reportRowUI(ns("qcrow"), type = FALSE)
            )
        )
    })
    
    ## calling server rows for all analysis code parts
    observe({
        req(aggIndex(), analysisRep())
        selectedRows(lapply(
            seq_len(length(analysisRep())),
            function(i) callModule(reportRow, paste0("rr", i), 
                                   type = aggIndex()[i], 
                                   content = reactive(analysisRep()[i]))
        ))
    })
    
    ## calling ui code for all analysis code parts
    output$reportListBox <- renderUI({
        req(aggIndex(), analysisRep())
        div(box(
            width = 11,
            lapply(seq_len(length(analysisRep())), 
                   function(i) reportRowUI(ns(paste0("rr", i)), 
                                           type = aggIndex()[i]))
        ))
    })
    
    ## get a temp directory to use for rendering reports
    report_dir <- tempdir()
    ## reactive keeping track of whether the report is ready
    report_ready <- reactiveVal(FALSE)
    fileName <- reactive(gsub(" ","",input$repfile))
    
    ## prepare R code for rendering
    myreport <- reactive({
        mr <- list(unname(unlist(preprocessRep())))
        if (qcSelected()) {
            mr <- append(mr, qcRep())
        }
        if (!is.null(selectedRows())) {
            mr <- append(
                mr, 
                lapply(
                    seq_len(length(selectedRows())), 
                    function(si) {
                        if (is.null(selectedRows()[[si]])) {
                            unlist(analysisRep()[si])
                        } else if (isTRUE(selectedRows()[[si]])) {
                            unlist(analysisRep()[si])
                        } else {
                            NULL
                        }
                    }))
        }
        mr <- mr[vapply(mr, function(m) !is.null(m), logical(1))]
        labelchunks <- vapply(mr, function(m) {
            sum(stringr::str_count(m, pattern = "#-"))
        }, numeric(1))
        sumofchunks <- sum(labelchunks > 0)
        chunklocs <- mr[labelchunks == 1]
        ## each chunk should have a unique id
        mr[labelchunks == 1] <- vapply(seq_len(sumofchunks), function(i) {
            stringr::str_replace(chunklocs[i], "#-", 
                                 paste0("#+ codechunk", i, ","))
        }, character(1))
        mr
    })
    
    chosenFormat <- reactive({
        vapply(input$repformat,
               switch,
               "HTML" = "html_document",
               "PDF" = "pdf_document",
               "DOC" = "word_document",
               "PPT" = "powerpoint_presentation",
               FUN.VALUE = character(1))
    })
    
    ## start building the reports
    observeEvent(input$generatebutton, {
        report_ready(FALSE)
        req(chosenFormat())
        shinyjs::disable("exportbutton")
        
        showModal(modalDialog(
            title = "Creating Report",
            p("Report is rendering, this may take a moment. To download 
              generated report, click Export when report is ready")
        ))
        
        cat(file = stderr(), paste("-- Sending report to", report_dir, "--\n"))
        
        file.remove(list.files(report_dir, paste0(fileName(), "[.]"),
                               full.names = TRUE
        ))
        try(
            file.remove(
                file.path(report_dir, paste0(fileName(), c(".Rmd", ".html")))
            ),
            silent = TRUE
        )
        cat(file = stderr(), paste("-- Removing", 
                                   file.path(report_dir, 
                                             paste0(fileName(), 
                                                    ".Rmd", "--\n"))))
        
        ## main function generating report
        x <- try(generateReport(myreport(),
                                filename = fileName(),
                                dir = report_dir,
                                title = input$reptitle,
                                author = input$repauthor,
                                date = format(Sys.time(), "%d %B, %Y"),
                                data.source = dataSource(),
                                output = chosenFormat(),
                                intro_text = input$intro_text,
                                toc = input$repcontents),
                 silent = TRUE
        )
        
        cat(file = stderr(), "gen report try value:", class(x))
        removeModal()
        if (is(x, "try-error")) {
            showNotification(duration = 3, type = "error", 
                             ui = "Report could not be generated")
            validate(
                need(!is(x, "try-error"), 
                     message = "Report could not be generated")
            )
        } else {
            report_ready(TRUE)
            shinyjs::enable("exportbutton")
        }
    })
    
    ## downloding generated report (this button is invisible)
    output$downloadbutton <- downloadHandler(
        filename = function() {
            paste0(fileName(), ".zip")
        },
        content = function(file) {
            paths <- c(
                file.path(report_dir, paste0(fileName(), ".Rmd")),
                vapply(
                    chosenFormat(), switch,
                    "html_document" = file.path(
                        report_dir, 
                        paste0(fileName(), ".html")),
                    "pdf_document" = file.path(
                        report_dir, 
                        paste0(fileName(), ".pdf")),
                    "word_document" = file.path(
                        report_dir, 
                        paste0(fileName(), ".docx")),
                    "powerpoint_presentation" = file.path(
                        report_dir, 
                        paste0(fileName(), ".pptx")),
                    FUN.VALUE = character(1)
                )
            )
            paths <- c(paths, dir(file.path(report_dir, 
                                            paste0(fileName(), "_files"), 
                                            "figure-markdown_strict"),
                                  pattern = ".png",
                                  full.names = TRUE
            ))
            utils::zip(file, paths)
        }
    )
    
    ## call invisible download button
    observeEvent(input$exportbutton, {
        path <- file.path(report_dir, paste0(fileName(), ".Rmd"))
        if (!file.exists(path)) {
            showNotification(ui = "Could not download report", 
                             type = "error", duration = 3)
            stop()
        }
        req(file.exists(path))
        shinyjs::runjs("document.getElementById('reportlist-downloadbutton').
                       click();")
    })
}

.onLoad <- function(libname, pkgname){
  
  op <- options()
  
  op.microbiomeExplorer <- list(
    ## ALLOWED FILE EXTENSIONS (letter case is ignored)
    me.filetypes = c(
      ".biom", ".biom2", ".txt", ".tsv", ".csv",
      ".RDS", ".rds", ".rda", ".Rdata"
    ),
    me.delim = c(".tsv", ".csv", ".txt"),
    ## used to filter out columns in the feature annotation section
    ## that cannot be used for aggregation
    ## adjust as needed
    me.featurenames = c("Taxonomy","taxonomy","TAXONOMY",
                          "confidence","Confidence","CONFIDENCE",
                          "clusterCenter","clustercenter","CLUSTERCENTER"),
    me.corrmethods = c("spearman", "pearson", "kendall"),
    #removing ZILN as an option to avoid error in lmFit asking 
    #for identical ncol and nrow instead of equal
    me.diffmethods = c("DESeq2", "Kruskal-Wallis", "limma"),
    me.facetnum = 15,
    me.shapenum = 6,
    me.minwebgl = 500,
    ## COLOR PALETTE: Adjust as needed for different color options
    ## first should be the neutral color used if no color is specified
    me.colorvalues = c("#0C1707", "#9B110E", "#899DA4", "#FDD262", "#046C9A",
                    "#C7B19C", "#9986A5", "#4E2A1E", "#DC863B", "#273046", 
                    "#F4B5BD", "#446455", "#EBCC2A",  "#0B775E", "#C93312", 
                    "#9C964A"),
    me.round_digits = 3,
    me.modebar = list(list("toImage", "zoom2d", "pan2d", 
                                "select2d", "lasso2d", "resetScale2d")),
    ## AVAILABLE REPORT FORMATS (depends on pandoc version and pdflatex)
    me.reportformat = c("html_document", 
                       "pdf_document", 
                       "word_document",
                       "powerpoint_presentation"),
    me.reportformatshort = c("HTML", 
                             "PDF", 
                             "DOC", 
                             "PPT")
  )
  
  toset <- !(names(op.microbiomeExplorer) %in% names(op))
  if (any(toset)) options(op.microbiomeExplorer[toset])
  
  invisible()
}

#' Main beta analysis input module.
#' Set up to handle all analysis tabs in the app depending on given parameters
#'
#' @param id element identifier - namespace
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
betaInputUI <- function(id) {
  ns <- NS(id)
  
  box(width = 12, id = ns("analysisbox"),
      h4("ANALYSIS PARAMETERS"),
      
      div(
        h5("Beta Diversity"),
        selectInput(
          ns("distance"),
          label = "Distance (beta)",
          choices = c(
            "Bray", "Canberra", "Jaccard", "Euclidean",
            "Manhattan", "Clark", "Kulczynski", "Gower",
            "altGower", "Morisita", "Horn", "Mountford",
            "Raup", "Binomial", "Chao", "Cao", "Mahalanobis"
          ),
          selected = "Bray", multiple = FALSE,
          selectize = FALSE, width = "250px"
        ),
        box(
          collapsible = TRUE, collapsed = TRUE, width = 11,
          style = "padding:0;",
          title = "*",
          p("Adonis refers to \"permutational multivariate analysis of variance 
             using distance matrices\" from the vegan package. The adonis 
             variable specifies the column of the pheno data holding the 
             independant variable whereas strata (optional) defines the groups 
             within which to constrain permutations. For more details and 
             descriptions of the specific dissimilarity matrices, please refer 
             to the vegan package.")
        ),
        selectInput(
          ns("adonisvar"),
          label = "Adonis variable (Optional)*",
          choices = "",
          selected = "", multiple = FALSE,
          selectize = FALSE, width = "250px"
        ),
        selectInput(
          ns("adonisstrata"),
          label = "Adonis strata (Optional)*",
          choices = "",
          selected = "", multiple = FALSE,
          selectize = FALSE, width = "250px"
        )
      ),
      ## buttons are used to submit events. This ensures plots are not redrawn 
      ## while user still adjusts parameters
      div(
        id = ns("buttondiv"),
        fluidRow(
          width = 12, id = "actionbuttonrow",
          ## update analysis outputs (plots/tables)
          actionButton(
            ns("updatebutton"), 
            icon = icon("far fa-chart-bar"),
            label = HTML("&nbsp;UPDATE"), 
            width = "120px"),
          actionButton(
            ns("reportButton"), 
            label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
            width = "120px")
        )
      )
  )
}


#' Server side for the analysis input module handling analysis control
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment object storing all data
#' @param adonisOptions phenodata colums ready for adonis analysis
#' @param reset reactive boolean determining if all inputs should be reset 
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
betaInput <- function(input, output, session, 
                      meData, adonisOptions, reset) {
  ns <- session$ns
  
  ## reactive value storing choices made in the UI
  chosenValues <- reactiveVal(NULL)
  
  ## reset all entries
  observe({
    req(reset())
    chosenValues(NULL)
  })
  
  ## initialize input elements
  observe({
    req(meData(), adonisOptions())
    isolate({
      updateSelectInput(session, "adonisvar", 
                        choices = c("", names(adonisOptions())), 
                        selected = chosenValues()$adonisvar)
      updateSelectInput(session, "adonisstrata", 
                        choices = c("", names(pData(meData()))), 
                        selected = chosenValues()$adonisstrata)
    })
  })
  
  ## main control button: store input choices in chosenValues
  observeEvent(input$updatebutton, {
    adonisvar <- NULL
    if(input$adonisvar != "")
      adonisvar <- input$adonisvar
    adonisstrata <- NULL
    if(input$adonisstrata != "")
      adonisstrata <- input$adonisstrata
    cV <- list("distance" = input$distance,
               "adonisvar" = adonisvar,
               "adonisstrata" = adonisstrata)
    chosenValues(cV)
  })
  
  return(chosenValues)
}


#' Heatmap analysis input module.
#' Set up to handle all analysis tabs in the app depending on given parameters
#'
#' @param id element identifier - namespace
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
heatmapInputUI <- function(id) {
  ns <- NS(id)
  
  box(
    width = 12, id = ns("analysisbox"), style = "padding-top: 0;",
    div(
      h5("Heatmap"),
      selectInput(
        ns("sorting"),
        label = "Top features sorted by",
        choices = c("Variance", "Fano", "MAD"),
        selected = "Variance", multiple = FALSE,
        selectize = FALSE, width = "250px"
      )
    ),
    selectizeInput(
      ns("featureselect"),
      label = "Feature selection (optional)", choices = "", 
      multiple = TRUE, 
      width = "250px"
    ),
    ## buttons are used to submit events. This ensures plots are not redrawn 
    ## while user still adjusts parameters
    div(
      id = ns("buttondiv"),
      fluidRow( 
        width = 12, id = "actionbuttonrow",
        ## update analysis outputs (plots/tables)
        actionButton(
          ns("updatebutton"), 
          icon = icon("far fa-chart-bar"),
          label = HTML("&nbsp;UPDATE"), 
          width = "120px"),
        actionButton(
          ns("reportButton"), 
          label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
          width = "120px")
      )
    )
  )
}


#' Server side for the analysis input module handling analysis control
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment object storing all data
#' @param reset reactive boolean determining if all inputs should be reset 
#' @param aggDat aggregated MRExperiment object (default is NULL)
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
heatmapInput <- function(input, output, session, 
                         meData, reset, aggDat = reactive(NULL)) {
  ns <- session$ns
  
  ## reactive value storing choices made in the UI
  chosenValues <- reactiveVal(NULL)
  ## reactive storing sorted features at given level
  aggFeatures <- reactiveVal(NULL)
  
  observe({
    req(aggDat())
    aggFeatures(rownames(MRcounts(aggDat())))
  })
  
  ## reset all entries
  observe({
    req(reset())
    chosenValues(NULL)
    aggFeatures(NULL)
  })
  
  ## update any inputs based on feature values
  observeEvent(aggFeatures(),{
    req(aggFeatures())
    selfeat <- ""
    if(!is.null(chosenValues()$featureselect))
      selfeat <- chosenValues()$featureselect
    corrdata  <- data.frame(value = aggFeatures(), 
                            label = aggFeatures())
    updateSelectizeInput(session, "featureselect",
                         choices = corrdata,
                         selected = selfeat,
                         options = list(placeholder = "Select Feature"),
                         server = TRUE)
  })
  
  
  ## main control button: store input choices in chosenValues
  observeEvent(input$updatebutton, {
    cV <- list("sorting" = input$sorting,
               "featureselect" = input$featureselect)
    chosenValues(cV)
  })
  
  return(chosenValues)
}

#' Calls appropriate normalization functions depending on input parameter
#' The two available methods included in the package are based on either 
#' calculating proportions or by using cumulative sum scaling (CSS), 
#' Paulson, et al. Nat Meth 2013.
#'
#' @param MRobj the MRexperiment
#' @param norm_method method to use for normalization; CSS or Proportional
#'
#' @importFrom metagenomeSeq cumNorm cumNormStatFast normFactors
#'
#' @return the normalized MRobj
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' normalizeData(mouseData, norm_method = "CSS")
#' 
#' @export
normalizeData <- function(MRobj, norm_method) {
  if (norm_method == "CSS") {
    MRobj <- cumNorm(MRobj, suppressMessages(cumNormStatFast(MRobj)))
  } else if (norm_method == "Proportion") {
    normFactors(MRobj) <- colSums(MRcounts(MRobj))
  }
  MRobj
}


## AGGREGATION 


#' Aggregates counts by level
#'
#' This function aggregates counts by a level specified in the featureData slot
#' of the MRexperiment object.
#'
#' @param MRobj An MRexperiment object.
#' @param level Level to aggregate over. If NULL, no aggregation occurs.
#' @param sort boolean determining if resulting aggregated MRexperiment should 
#' be sorted based on rowSums; default is TRUE
#'
#' @return Aggregated MRexperiment object or matrix depending on \code{out}.
#'
#' @importFrom metagenomeSeq aggTax MRcounts normFactors normFactors<-
#' @importFrom Biobase pData<- fData<-
#' 
#' @examples 
#' data("mouseData", package = "metagenomeSeq")
#' aggFeatures(mouseData, level = "genus")
#'
#' @export
aggFeatures <- function(MRobj, level = NULL, sort = TRUE) {
  if (!is.null(level)) {
    ## account for absent feature table
    if (level == "unavailable") {
      aggobj <- MRobj
    } else {
      aggobj <- aggTax(MRobj, lvl = level)
      if(is(normFactors(MRobj), "numeric"))
        normFactors(aggobj) <- normFactors(MRobj)
    }
  } else {
    aggobj <- MRobj
  }
  aggcts <- MRcounts(aggobj)
  ## account for absent feature table
  if(ncol(fData(aggobj)) > 0){
    ## replace empty string with unnamed
    rownames(aggcts)[rownames(aggcts) %in% ""] <- "unnamed"
    rownames(fData(aggobj))[rownames(fData(aggobj)) %in% ""] <- "unnamed"
    levels(fData(aggobj)[, level]) <- gsub("^$", "unnamed", 
                                           levels(fData(aggobj)[, level]))
  }
  aggobj@assayData$counts <- aggcts
  if (sort) {
    aggobj <- aggobj[order(rowSums(MRcounts(aggobj)), decreasing = TRUE), ]
  }
  return(aggobj)
}

data("mouseData")

test_that("normalize data methods", {
  proportionMR <- normalizeData(mouseData, norm_method = "Proportion")
  expect_equal(sum(normFactors(proportionMR)),315593)
  cssMR <- normalizeData(mouseData, norm_method = "CSS")
  expect_equal(sum(normFactors(cssMR)),33914)
})

test_that("filter and normalize", {
  initialNormFactors <- normFactors(mouseData)
  filteredMR <- filterMEData(mouseData, minpresence = 2, minfeats = 10, minreads = 1500)
  filteredNormFactors <- normFactors(normalizeData(filteredMR, norm_method = "CSS"))
  expect_failure(expect_equal(filteredNormFactors, normFactors(filteredMR)))
  expect_equal(sum(filteredNormFactors),26488)
  filteredNormFactors <- normFactors(normalizeData(filteredMR, norm_method = "Proportion"))
  expect_failure(expect_equal(sum(filteredNormFactors),33914))
})

test_that("aggregate features", {
  genusMD <- aggFeatures(mouseData, level = "genus")
  expect_equal(ncol(genusMD), ncol(mouseData))
  expect_failure(expect_equal(nrow(genusMD),nrow(mouseData)))
  expect_equal(as.numeric(nrow(genusMD)), 61)
})

## Modules for feature sample analysis

#' Relative abundance plot module - UI
#'
#' @param id namespace identifier
#'
#' @author Janina Reeder
#'
#' @return box containing the ui code
avgAbundanceUI <- function(id) {
  ns <- NS(id)
  
  box(
    title = "AVERAGE RELATIVE ABUNDANCE", solidHeader = TRUE, 
    collapsible = TRUE, width = 12,
    fluidRow(
      column(
        width = 11,
        shinycssloaders::withSpinner(
          plotly::plotlyOutput(ns("avgabplot"), 
                               width = "auto", 
                               height = "auto"),
          type = 3, color = "#424242", color.background = "#fdfdfc"),
        br(),
        shinyjs::disabled(shinyWidgets::dropdownButton(
          tags$h3("Plot Options"),
          shinyWidgets::radioGroupButtons(
            inputId = ns("avgAbChoice"),
            label = "Y axis",
            choices = c("Percentage", "Reads"),
            individual = TRUE,
            checkIcon = list(
              yes = tags$i(
                class = "fa fa-circle",
                style = "color: steelblue"
              ),
              no = tags$i(
                class = "fa fa-circle-o",
                style = "color: steelblue"
              )
            )
          ),
          div(id=ns("pspan"), class = "percentage",
               p("Normalization is required to show percentage")
          ),
          span(
            numericInput(
              inputId = ns("numofFeatures"),
              label = "Max number of Features to show",
              value = 10,
              min = 1
            ),
            sliderInput(
              inputId = ns("plotWidth"),
              label = "Adjust plot width",
              value = 650,
              min = 250,
              max = 1600,
              round = TRUE
            )
          ),
          circle = FALSE, status = "danger",
          icon = icon("gear"), width = "300px",
          label = "Plot Options",
          inputId = ns("optionbutton")
        ))
      )
    )
  )
}

#' Relative abundance plot module - server
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param featLevel chosen feature level (aggregation level)
#' @param featureSettings analysis input settings passed over to this module
#' @param normalizedData boolean indicating whether data has been normalized
#' @param reset boolean reactive which resets the module if TRUE
#' 
#'
#' @return list storing plot clicks and number of features displayed 
#' (passed to feature plot module) as well as the R code to make plot
avgAbundance <- function(input, output, session, 
                         aggDat, 
                         featLevel,
                         featureSettings,
                         normalizedData,
                         reset) {
  ns <- session$ns
  
  ## stores the plotly plot (needed to set source correctly)
  paPlot <- reactiveVal(NULL)
  ## stores the R code needed to build the relative abundance plot
  repCode <- reactiveVal(NULL)
  width <- reactiveVal(650)
  
  
  observe({
    req(reset())
    shinyjs::disable("optionbutton_state")
    shinyjs::enable("avgAbChoice")
    paPlot(NULL)
    repCode(NULL)
  })
  
  observe({
    width(input$plotWidth)
  })

  
  ## debouncing number of features to avoid multiple redraws
  nof <- reactive(input$numofFeatures)
  numOfFeats <- debounce(nof,1000)
  

  ## calls the plotting function
  observe({
    req(aggDat(), featureSettings(),numOfFeats())
    ylab <- "Reads"
    if(isFALSE(normalizedData())){
      shinyWidgets::updateRadioGroupButtons(session = session,
                              inputId = "avgAbChoice",
                              selected = "Reads")
      shinyjs::removeClass(id = "pspan", class = "hide")
      shinyjs::disable("avgAbChoice")
    } else {
      ylab <- input$avgAbChoice
      shinyjs::addClass(id = "pspan", class = "hide")
      shinyjs::enable("avgAbChoice")
    }
    facet1 <- featureSettings()$facetby
    facet2 <- featureSettings()$facetby2
    plotTitle <- paste0("Top ",
                        numOfFeats(),
                        " feature ", 
                        tolower(gsub("Reads","abundance",ylab)),
                        " at ",
                        featLevel()," level")
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    pa <- plotAvgAbundance(aggDat(),
                        level = featLevel(),
                        facet1 = facet1,
                        facet2 = facet2,
                        ind = seq_len(numOfFeats()), 
                        plotTitle = plotTitle,
                        ylab = ylab
    ) 
    pa$x$source <- "avgabplot"
    paPlot(pa)
    shinyjs::enable("optionbutton_state")
  })
  
  output$avgabplot <- plotly::renderPlotly({
    paPlot()
  })
  
  observeEvent(width(), {
    req(paPlot())
    plotly::plotlyProxy("avgabplot", session) %>%
      plotly::plotlyProxyInvoke(
        "relayout",
        list(width = width())
      )
  })
  

  
  ## update R code if inputs change
  observe({
    req(aggDat(), featureSettings())
    ylab <- input$avgAbChoice
    facet1 <- featureSettings()$facetby
    facet2 <- featureSettings()$facetby2
    plotTitle <- paste0("Top ",
                        numOfFeats(),
                        " feature ", 
                        tolower(gsub("Reads","abundance",ylab)),
                        " at ",
                        featLevel()," level")
    if(!is.null(facet1)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet1)
    } else if(!is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " split by ",
                          facet2)
    }
    if(!is.null(facet1) & !is.null(facet2)){
      plotTitle <- paste0(plotTitle,
                          " and ",
                          facet2)
    }
    repCode(paste(
      paste0("#' ### Average relative abundance"),
      paste0("", "#- fig.width = 10"),  
      paste0("plotAvgAbundance(aggDat,"),
      paste0("\tlevel = \"", featLevel(), "\","),
      paste0(
        "\tfacet1 = ", 
        'if'(is.null(facet1),"NULL",paste0("\"", facet1, "\"")),
        ","
      ),
      paste0(
        "\tfacet2 = ", 
        'if'(is.null(facet2),"NULL",paste0("\"",facet2, "\"")),
        ","
      ),
      paste0("\tind = seq_len(", numOfFeats(), "),"),
      paste0("\tplotTitle = \"",plotTitle, "\","),
      paste0("\tylab = \"", ylab, "\")\n\n"),
      sep = "\n"
    ))
  })
  
  return(repCode = repCode)
}

## Module handling correlation analysis in the Shiny application

#' Feature correlation analysis module UI
#'
#' @param id namespace identifier
#' @author Janina Reeder
#'
#' @return box containing the UI elements
featureCorrUI <- function(id) {
    ns <- NS(id)
    
    box(title = "FEATURE CORRELATION", solidHeader = TRUE, 
        collapsible = TRUE, width = 12,
        fluidRow(
            column(width = 11,
                   shinycssloaders::withSpinner(
                       plotly::plotlyOutput(
                           ns("featcorrplot"), 
                           width = "auto", 
                           height = "auto"),
                       type = 3, color = "#424242", 
                       color.background = "#fdfdfc"),
                   br(),
                   shinyjs::disabled(
                       shinyWidgets::dropdownButton(
                           tags$h3("Plot Options"),
                           selectInput(
                               inputId = ns("fcol"),
                               label = "Color by",
                               choices = ""
                           ),
                           shinyWidgets::switchInput(
                               inputId = ns("logS"),
                               label = "Log Scale",
                               value = TRUE,
                               size = "mini",
                               labelWidth = "120px"
                           ),
                           shinyWidgets::switchInput(
                               inputId = ns("regL"),
                               label = "Regression Line",
                               value = TRUE,
                               size = "mini",
                               labelWidth = "120px"
                           ),
                           sliderInput(
                               inputId = ns("plotWidth"),
                               label = "Adjust plot width",
                               value = 550,
                               min = 250,
                               max = 1600,
                               round = TRUE
                           ),
                           circle = FALSE, 
                           status = "danger", 
                           icon = icon("gear"), 
                           width = "300px",
                           label = "Plot Options",
                           inputId = ns("optionbutton")
                       ))
            )
        ),
        fluidRow(
            column(width = 1),
            column(width = 10, class = "statsrow",
                   DT::DTOutput(ns("statsdatatable"), 
                                width = "90%", height = "auto")
            )
        )
    )
}

#' Feature correlation analysis server module
#'
#' @param input module input
#' @param output module output
#' @param session app session
#' @param aggDat aggregated MRExperiment
#' @param colorOptions reactive storing filters available via data input
#' @param corFeatBase first correlation feature
#' @param corFeat2 second correlation feature
#' @param corFacet1 first correlation facet
#' @param corFacet2 second correlation facet
#' @param corMethod correlation method to use
#' @param reset boolean reactive which resets the module if TRUE
#' 
#' @author Janina Reeder
#' @return R code used to do the correlation analysis (character)
featureCorr <- function(input, output, session,
                        aggDat,
                        colorOptions,
                        corFeatBase,
                        corFeat2,
                        corFacet1,
                        corFacet2,
                        corMethod,
                        reset) {
    ns <- session$ns
    
    observe({
        updateSelectInput(session, "fcol", 
                          choices = c("No Color", colorOptions()))
    })
    
    ## stores color selection
    colorChoice <- reactiveVal("No Color")
    ## stores featcorrplot
    corPlot <- reactiveVal(NULL)
    ## stores statsdatatable
    corStats <- reactiveVal(NULL)
    ## stores R code for reports
    repCode <- reactiveVal(NULL)
    width <- reactiveVal(550)
    
    observe({
        req(reset())
        shinyjs::disable("optionbutton_state")
        colorChoice("No Color")
        corPlot(NULL)
        corStats(NULL)
        repCode(NULL)
    })
    
    
    observeEvent(input$fcol, {
        colorChoice(input$fcol)
    })
    
    observe({
        width(input$plotWidth)
    })
    
    observeEvent(width(), {
        req(corPlot())
        plotly::plotlyProxy("featcorrplot", session) %>%
            plotly::plotlyProxyInvoke(
                "relayout",
                list(width = width())
            )
    })
    
    
    observe({
        req(aggDat(), corFeatBase(), corFeat2(),colorChoice())
        color <- as.character(colorChoice())
        if (stringr::str_detect(color, "No Color")) {
            color <- NULL
        }
        plotTitle <- paste0(stringr::str_to_sentence(corMethod()),
                            " correlation of ",
                            corFeatBase(), " vs ",
                            corFeat2())
        if(!is.null(corFacet1())){
            plotTitle <- paste0(plotTitle,
                                " split by ",
                                corFacet1())
        } else if(!is.null(corFacet2())){
            plotTitle <- paste0(plotTitle,
                                " split by ",
                                corFacet2())
        }
        if(!is.null(corFacet1()) & !is.null(corFacet2())){
            plotTitle <- paste0(plotTitle,
                                " and ",
                                corFacet2())
        }
        ## main function to perform feature-feature correlation
        withProgress({
            cf <- corrFeature(aggDat(),
                              feat1 = corFeatBase(),
                              feat2 = corFeat2(),
                              log = input$logS,
                              facet1 = corFacet1(),
                              facet2 = corFacet2(),
                              method = corMethod(),
                              addRegression = input$regL,
                              plotTitle = plotTitle,
                              col_by = color
            )
            corPlot(cf$plot)
            setProgress(0.9)
            
            statsframe <- dplyr::bind_rows(
                lapply(
                    names(cf$stats), 
                    function(n) {
                        s <- cf$stats[[n]]
                        list(
                            "facet" = n,
                            "method" = corMethod(),
                            "estimate" = round(s$estimate, 
                                               getOption("me.round_digits")),
                            "p" = round(s$p.value, 
                                        getOption("me.round_digits")),
                            "lower CI" = round(s$lower, 
                                               getOption("me.round_digits")),
                            "upper CI" = round(s$upper, 
                                               getOption("me.round_digits"))
                        )
                    }))
        }, message = "Analyzing correlation ...")
        
        ## code needed to do the analysis
        repCode(paste(
            paste0("#' ### Feature correlation"),
            paste0("", "#- fig.width = 10"),
            paste0("cf <- corrFeature(aggDat,"),
            paste0("\tfeat1 = \"", corFeatBase(), "\","),
            paste0("\tfeat2 = \"", corFeat2(), "\","),
            paste0("\tlog = ", input$logS, ","),
            paste0(
                "\tfacet1 = ", 'if'(is.null(corFacet1()),
                                    "NULL",
                                    paste0("\"", corFacet1(), "\"")
                ),
                ","
            ),
            paste0(
                "\tfacet2 = ", 'if'(is.null(corFacet2()),
                                    "NULL",
                                    paste0("\"", corFacet2(), "\"")
                ),
                ","
            ),
            paste0("\tmethod = \"", corMethod(), "\","),
            paste0("\taddRegression = ", input$regL, ","),
            paste0("\tplotTitle = \"", plotTitle, "\","),
            paste0("\tcol_by = ", 'if'(is.null(color), 
                                       "NULL", 
                                       paste0("\"", color, "\"")),","),
            paste0("\tallowWebGL = FALSE)\n\n"),
            paste0("cf$plot"),
            paste0("\n\n"),
            paste0("statsframe <- dplyr::bind_rows(lapply(names(cf$stats), 
                   function(n){"),
            paste0("\ts <- cf$stats[[n]]"),
            paste0("\tlist(\"facet\" = n,"),
            paste0("\t\t\"method\" = \"",corMethod(),"\","),
            paste0("\t\t\"estimate\" = round(s$estimate, getOption(\"me.round_digits\")),"),
            paste0("\t\t\"p\" = round(s$p.value, getOption(\"me.round_digits\")),"),
            paste0("\t\t\"lower CI\" = round(s$lower, getOption(\"me.round_digits\")),"),
            paste0("\t\t\"upper CI\" = round(s$upper, getOption(\"me.round_digits\")))"),
            paste0("}))\n\n"),
            paste0("if(doctype == \"html\"){"),
            paste0("\tDT::datatable(data = statsframe, 
                   class = \"stripe hover cell-border order-column\","),
            paste0("\t\tfilter = \"none\", style = \"bootstrap\","),
            paste0("\t\toptions = list(scrollX = TRUE, paging = TRUE, 
                   digits = 4,dom = \"<Bftp>\"),"),
            paste0("\t\trownames = FALSE, escape = FALSE)"),
            paste0("} else {"),
            paste0("\tkable(statsframe)"),
            paste0("}\n\n"),
            sep = "\n"
        ))
        corStats(statsframe)
    })
    
    
    output$featcorrplot <- plotly::renderPlotly({
        req(corPlot())
        shinyjs::enable("optionbutton_state")
        corPlot()
    })
    
    output$statsdatatable <- DT::renderDT({
        req(corStats())
        
        DT::datatable(
            data = corStats(), class = "stripe hover cell-border order-column",
            filter = "none", style = "bootstrap",
            options = list(
                scrollX = TRUE,
                paging = FALSE,
                digits = 4,
                dom = "<tp>"
            ),
            rownames = NULL,
            escape = FALSE
        )
    })
    
    return(repCode)
}



#' Phenotype correlation analysis module
#'
#' @param id namespace identifier
#' @author Janina Reeder
#'
#' @return box containing the UI element
phenotypeCorrUI <- function(id) {
    ns <- NS(id)
    
    box(title = "PHENOTYPE CORRELATION", solidHeader = TRUE, 
        collapsible = TRUE, width = 12,
        fluidRow(
            column(
                width = 11,
                shinycssloaders::withSpinner(
                    plotly::plotlyOutput(ns("phenocorrplot"), 
                                         width = "auto", 
                                         height = "auto"),
                    type = 3, color = "#424242", color.background = "#fdfdfc"),
                br(),
                shinyjs::disabled(
                    shinyWidgets::dropdownButton(
                        tags$h3("Plot Options"),
                        selectInput(
                            inputId = ns("pcol"),
                            label = "Color by",
                            choices = ""
                        ),
                        shinyWidgets::switchInput(
                            inputId = ns("logS"),
                            label = "Log Scale",
                            value = TRUE,
                            size = "mini",
                            labelWidth = "120px"
                        ),
                        shinyWidgets::switchInput(
                            inputId = ns("regL"),
                            label = "Regression line",
                            value = TRUE,
                            size = "mini",
                            labelWidth = "120px"
                        ),
                        sliderInput(
                            inputId = ns("plotWidth"),
                            label = "Adjust plot width",
                            value = 550,
                            min = 250,
                            max = 1600,
                            round = TRUE
                        ),
                        circle = FALSE, status = "danger", 
                        icon = icon("gear"), width = "300px",
                        label = "Plot Options",
                        inputId = ns("optionbutton")
                    ))
            )
        ),
        fluidRow(
            column(width = 1),
            column(width = 10, class = "statsrow",
                   column(
                       width = 10, class = "statsrow",
                       DT::DTOutput(ns("statsdatatable"), 
                                    width = "90%", height = "auto")
                   )
            )
        )
    )
}

#' Phenotype correlation analysis server module
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param aggDat aggregated MRExperiment
#' @param colorOptions reactive storing filters available via data input
#' @param corFeatBase first correlation feature
#' @param corPheno correlation phenotype
#' @param corFacet1 first correlation facet
#' @param corFacet2 second correlation facet
#' @param corMethod correlation method to use
#' @param reset boolean reactive which resets the module if TRUE
#' 
#' @author Janina Reeder
#' @return R code used to do the correlation analysis (character)
phenotypeCorr <- function(input, output, session,
                          aggDat,
                          colorOptions,
                          corFeatBase,
                          corPheno,
                          corFacet1,
                          corFacet2,
                          corMethod,
                          reset) {
    ns <- session$ns
    
    observe({
        updateSelectInput(session, "pcol", 
                          choices = c("No Color", colorOptions()))
    })
    
    ## stores color selection
    colorChoice <- reactiveVal("No Color")
    ## stores featcorrplot
    corPlot <- reactiveVal(NULL)
    ## stores statsdatatable
    corStats <- reactiveVal(NULL)
    ## stores R code for reports
    repCode <- reactiveVal(NULL)
    width <- reactiveVal(550)
    
    observe({
        req(reset())
        shinyjs::disable("optionbutton_state")
        colorChoice("No Color")
        corPlot(NULL)
        corStats(NULL)
        repCode(NULL)
    })
    
    observeEvent(input$pcol, {
        colorChoice(input$pcol)
    })
    
    observe({
        width(input$plotWidth)
    })
    
    observeEvent(width(), {
        req(corPlot())
        plotly::plotlyProxy("phenocorrplot", session) %>%
            plotly::plotlyProxyInvoke(
                "relayout",
                list(width = width())
            )
    })
    
    
    observe({
        req(aggDat(), corFeatBase(), corPheno(), colorChoice())
        color <- as.character(colorChoice())
        if (stringr::str_detect(color, "No Color")) {
            color <- NULL
        }
        plotTitle <- paste0(stringr::str_to_sentence(corMethod()),
                            " correlation of ",
                            corFeatBase(), " vs ",
                            corPheno())
        if(!is.null(corFacet1())){
            plotTitle <- paste0(plotTitle,
                                " split by ",
                                corFacet1())
        } else if(!is.null(corFacet2())){
            plotTitle <- paste0(plotTitle,
                                " split by ",
                                corFacet2())
        }
        if(!is.null(corFacet1()) & !is.null(corFacet2())){
            plotTitle <- paste0(plotTitle,
                                " and ",
                                corFacet2())
        }
        ## main function to perform feature-phenotype correlation
        withProgress({
            cp <- corrPhenotype(aggDat(),
                                feature = corFeatBase(),
                                phenotype = corPheno(),
                                log = input$logS,
                                facet1 = corFacet1(),
                                facet2 = corFacet2(),
                                method = corMethod(),
                                addRegression = input$regL,
                                plotTitle = plotTitle,
                                col_by = color
            )
            corPlot(cp$plot)
            setProgress(0.9)
            
            statsframe <- dplyr::bind_rows(
                lapply(
                    names(cp$stats), 
                    function(n) {
                        s <- cp$stats[[n]]
                        list(
                            "facet" = n,
                            "method" = corMethod(),
                            "estimate" = round(s$estimate, 
                                               getOption("me.round_digits")),
                            "p" = round(s$p.value, 
                                        getOption("me.round_digits")),
                            "lower CI" = round(s$lower, 
                                               getOption("me.round_digits")),
                            "upper CI" = round(s$upper, 
                                               getOption("me.round_digits"))
                        )
                    }))
            corStats(statsframe)
        }, message = "Analyzing correlation ...")
        
        ## code needed to do the analysis
        repCode(paste(
            paste0("#' ### Phenotype correlation"),
            paste0("", "#- fig.width = 10"),
            paste0("cp <- corrPhenotype(aggDat,"),
            paste0("\tfeature = \"", corFeatBase(), "\","),
            paste0("\tphenotype = \"", corPheno(), "\","),
            paste0("\tlog = ", input$logS, ","),
            paste0(
                "\tfacet1 = ", 'if'(is.null(corFacet1()),
                                    "NULL",
                                    paste0("\"", corFacet1(), "\"")
                ),
                ","
            ),
            paste0(
                "\tfacet2 = ", 'if'(is.null(corFacet2()),
                                    "NULL",
                                    paste0("\"", corFacet2(), "\"")
                ),
                ","
            ),
            paste0("\tmethod = \"", corMethod(), "\","),
            paste0("\taddRegression = ", input$regL, ","),
            paste0("\tplotTitle = \"", plotTitle, "\","),
            paste0("\tcol_by = ", 'if'(is.null(color), "NULL", 
                                       paste0("\"", color, "\"")),","),
            paste0("\tallowWebGL = FALSE)\n\n"),
            paste0("cp$plot"),
            paste0("\n\n"),
            paste0("statsframe <- dplyr::bind_rows(lapply(names(cp$stats), 
                   function(n){"),
            paste0("\ts <- cp$stats[[n]]"),
            paste0("\tlist(\"facet\" = n,"),
            paste0("\t\t\"method\" = \"",corMethod(),"\","),
            paste0("\t\t\"estimate\" = round(s$estimate, getOption(\"me.round_digits\")),"),
            paste0("\t\t\"p\" = round(s$p.value, getOption(\"me.round_digits\")),"),
            paste0("\t\t\"lower CI\" = round(s$lower, getOption(\"me.round_digits\")),"),
            paste0("\t\t\"upper CI\" = round(s$upper, getOption(\"me.round_digits\")))"),
            paste0("}))\n\n"),
            paste0("if(doctype == \"html\"){"),
            paste0("\tDT::datatable(data = statsframe, 
                   class = \"stripe hover cell-border order-column\","),
            paste0("\t\tfilter = \"none\", style = \"bootstrap\","),
            paste0("\t\toptions = list(scrollX = TRUE,paging = TRUE, 
                   digits = 4,dom = \"<Bftp>\"),"),
            paste0("\t\trownames = FALSE, escape = FALSE)"),
            paste0("} else {"),
            paste0("\tkable(statsframe, escape = FALSE)"),
            paste0("}"),
            sep = "\n"
        ))
    })
    
    output$phenocorrplot <- plotly::renderPlotly({
        req(corPlot())
        shinyjs::enable("optionbutton_state")
        corPlot()
    })
    
    output$statsdatatable <- DT::renderDT({
        req(corStats())
        
        DT::datatable(
            data = corStats(), class = "stripe hover cell-border order-column",
            filter = "none", style = "bootstrap",
            options = list(
                scrollX = TRUE,
                paging = FALSE,
                digits = 4,
                dom = "<tp>"
            ),
            rownames = NULL,
            escape = FALSE
        )
    })
    
    return(repCode)
}

#' Main feature analysis input module.
#' Set up to handle all analysis tabs in the app depending on given parameters
#'
#' @param id element identifier - namespace
#'
#' @author Janina Reeder
#'
#' @return box containing ui element
featureInputUI <- function(id) {
    ns <- NS(id)

    box(width = 12, id = ns("analysisbox"),
        h4("ANALYSIS PARAMETERS"),

        div(
          selectInput(ns("facetby"),
                      label = "Facet columns by", choices = "", 
                      multiple = FALSE, 
                      selectize = FALSE, width = "250px"
          ),
          selectInput(ns("facetby2"),
                      label = "Facet rows by", choices = "", 
                      multiple = FALSE, 
                      selectize = FALSE, width = "250px"
          )
        ),
        ## buttons are used to submit events. This ensures plots are not redrawn 
        ## while user still adjusts parameters
        div(id = ns("buttondiv"),
            fluidRow( width = 12, id = "actionbuttonrow",
                ## update analysis outputs (plots/tables)
                actionButton(ns("updatebutton"), 
                             icon = icon("far fa-chart-bar"),
                             label = HTML("&nbsp;UPDATE"), 
                             width = "120px"),
                actionButton(ns("reportButton"), 
                             label = HTML("<i class='far fa-bookmark'></i>&nbsp;&nbsp;REPORT"), 
                             width = "120px")
            )
        )
    )
}


#' Server side for the feature analysis input module
#'
#' @param input shiny input
#' @param output shiny output
#' @param session shiny session
#' @param meData MRExperiment object storing all data
#' @param facetOptions named vector of available facet choices
#' @param reset reactive boolean determining if all inputs should be reset 
#' @param aggDat aggregated MRExperiment object (default is NULL)
#'
#' @author Janina Reeder
#' 
#' @importFrom metagenomeSeq MRcounts
#' @importFrom Biobase pData fData
#'
#' @return list holding all chosen values and the selected feature
featureInput <- function(input, output, session, 
                          meData, facetOptions = NULL, 
                          reset, aggDat = reactive(NULL)) {
    ns <- session$ns

    ## reactive value storing choices made in the UI
    chosenValues <- reactiveVal(NULL)
    ## reactive storing sorted features at given level
    aggFeatures <- reactiveVal(NULL)

    observe({
        req(aggDat())
        aggFeatures(rownames(MRcounts(aggDat())))
        shinyjs::enable("savebutton")
    })

    ## reset all entries
    observe({
        req(reset())
        chosenValues(NULL)
        aggFeatures(NULL)
        shinyjs::disable("savebutton")
    })

    
    ## initialize input elements
    observe({
      req(meData())
      updateSelectInput(session, "facetby", 
                        choices = c("", facetOptions()), 
                        selected = chosenValues()$facetby)
      updateSelectInput(session, "facetby2", 
                        choices = c("", facetOptions()), 
                        selected = chosenValues()$facetby2)
    })

    ## react to first facet variable (phenotype variables should be distinct)
    observeEvent(input$facetby, {
      req(facetOptions())
      facetopts <- facetOptions()
      facetopts <- facetopts[!facetopts %in% input$facetby]
      updateSelectInput(session, "facetby2", choices = c("", facetopts), 
                        selected = input$facetby2)
    }, priority = 15, ignoreInit = TRUE)
    
    ## react to second facet variable (phenotype variables should be distinct)
    observeEvent(input$facetby2, {
      req(facetOptions())
      facetopts <- facetOptions()
      facetopts <- facetopts[!facetopts %in% input$facetby2]
      updateSelectInput(session, "facetby", choices = c("", facetopts), 
                        selected = input$facetby)
    }, priority = 15, ignoreInit = TRUE)


    
    ## main control button: store input choices in chosenValues
    observeEvent(input$updatebutton, {
      facet1 <- NULL
      if(!is.null(input$facetby)){
        if(input$facetby != "")
          facet1 <- input$facetby
      }
      facet2 <- NULL
      if(!is.null(input$facetby2)){
        if(input$facetby2 != "")
          facet2 <- input$facetby2
      }
      cV <- list("facetby" = facet1,
                 "facetby2" = facet2)
      chosenValues(cV)
    })
    

    return(list(chosenValues = chosenValues))
}

---
output: 
  html_document: 
    keep_md: yes
---
# Microbiome Explorer Tutorial

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE)
```

## Data Preparation

### File upload

Microbiome Explorer accepts several different file formats containing metagenome sequencing results which will be stored at a MRexperiment internally. If an MRexperiment is already available, the user can simply upload it as an rdata or RDS object (the application will attempt to guess the correct file format based on the file extension). Otherwise, Biological Observation Matrix (BIOM) formattes files produced by any program including qiime2 or mothur can also be provided. Or, a simple counts matrix in csv or tsv form can be uploaded. A counts matrix should have no row names and store the information on the features/OTUs in its first columns. All other columns names should correspond to sample names.
If the data does not contain phenotype information or in case the user wants to adjust the phenotype information included with an MRexperiment or biom file, they can upload an additional phenotype file and link it with the counts data. The required format is such that each sample is a row
with the names of the rows in the phenotype data corresponding to the names of the columns in the count data. Appending several phenotype files by subsequent uploads is possible. If no phenotype file is given, the names of the columns of the count data are used as the phenotype data.
Finally, if not already included with a given MRexperiment, a feature data file must be provided if aggregation to a particular phylogenetic level is desired. Here, each unique feature is a row which must correspond to the ones in the counts data feature column and each column is a taxonomy level.
All three data entities (counts, phenotypes and features) will be stored together internally as an MRexperiment. This can be downloaded (if desired) via the "Get Data" button on the Data input page of the Microbiome Explorer.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/data_input_overview.png")
```


### Data Filtering

Once the data has been uploaded, several QC plots give an overview of the number of features and the number of reads available in each of the samples. Here, the user can filter sample via several sliders requiring e.g. a minimum number of features present in a sample or a minimum number of samples that a feature needs to be present in. Any changes in slider settings are immediately shown in the QC plot to allow the user to see the effect such filtering would have. In addition to threshold based filtering, the user can also decide to subset by specific phenotypes. Here, any levels that are not of interest should be selected (these will not be reflected visually in the QC plot). Once the user presses the "filter" button, the data structure is subsetted accordingly. In order to get back to the original dataset, a "reset" button is available.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/data_input_filter.png")
```

In addition to the basic QC plots showing distriution of number of reads and number of features, a sample based barplot gives an overview of the number of base features (e.g. OTUs) available for each sample. This can be arranged based on frequency or phenotype levels as well as colored by phenotypes via the plot options.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/data_input_samplebars.png")
```

All of the plots will be included in the reports if the user presses the "report" button on this page. Otherwise, filtering will be included (to reflect the data that was analyzed), but the QC plots will be omitted.

### Phenotype modifications

The phenotype tab shows the available phenotype associated with the feature count set in an interactive table. The main purpose of this table is informational, but if desired the user can create new phenotype columns by comibining existing ones. This enables the creation of new data to center the analysis around: the values of the two columns are pasted together creating an interaction value. 
The user can furthermore select columns to show in the table and remove those irrelavant for the analysis. Finally, if desired, the data type of each column can be modified by opening the adjust datatype box via clicking on the "+" icon. This is usually not required, but might be of interest for specific analysis requests. Any modifications made are only stored in the internal MRexperiment for the analysis after pressing the "save" button. All saved modifications are automatically included in the reports.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/data_input_phenotype.png")
```

### Feature modifications

The feature tab shows the available feature annotation associated with each raw count feature (e.g. OTU). In case of missing values which may occur for features that cannot be annotated down to a specific taxonomy level, the user has the option to either mark them as unknown (to include "unkown" as an annotation value) or to roll down the taxonomy. In this case, the highest annotated taxonomy level is reused at more specific taxonomy level with the work "unkown_" prepended. For analysis at a more specific taxonomy level, this allows summary functions to distinguish between features of higher order taxonomy levels. These changes are calculated and shown in the table after clicking the "Assign" button and can be reverted via the "reset" button. Any modifications made are only stored in the internal MRexperiment for the analysis after pressing the "save" button. All saved modifications are automatically included in the reports.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/data_input_features.png")
```

## Analysis

After all required modifications to the counts, phenotype or feature annotation data have been completed, the user can switch to the analysis section of the application. The analysis workflow within the application is split into five different sections: intra sample, inter sample, correlation, differential and longitudinal. Each section will be described below. Each plot is generated after the user sets all required input elements and clicks the "update" button. All visualizations are implemented using the plotly R package which provides basic interactivity, including zooming or panning via its modebar. In addition, the user can export the plot in its current state (i.e. showing specific user interactions) as a svg file using the camera icon of the modebar. 
Every analysis section includes one or more "report" buttons associated with its respective input fields. In order to include the results of the current analysis, i.e. the currently visible plots, the user can click the "report" button which will include code to reproduce the visualization outside of the application. See below for more details on reports.

### Aggregation

Before any analyses can be done, the data has to be aggregated to a specific taxonomy level. In case of a missing feature annotation, this will be limited to the raw counts data. In those cases, aggregating doesn't change the underlying counts, but still needs to be performed. For all other taxonomy levels, features are grouped by their annotation and counts are summed up for each group. 
Normalization allows for the user to account for library size differences before analysis and the user is required to make a choice on normalization methods while aggregating the data. Certain app features are restricted if none is selected (e.g. percentage is unavailable if not normalized). Differential abundance testing also requires normalization, which we perform silently if the user does not choose to do so. The two available methods available here are based on either calculating proportions or by using cumulative sum scaling (CSS), Paulson, et al. Nat Meth 2013.
If desired the user can choose to download the counts data after aggregation by clicking the "get data" button. The aggregation box will automatically close after aggregation is complete and analysis input areas will become enabled. In order to switch to a different aggregation level, the user can open the aggregation section by clicking the "+" icon associated with the box.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_aggregation.png")
```

### Intra Analysis

Intra-sample analysis contains functions focus on investigating the microbial composition within a sample or a group of samples. Different functions are available to visualize the relative abundance of top features, the abundance of a specific feature as well as the alpha diversity within the sample. Within the application, one common set of input elements is used to generate all visualization. 

#### Relative Abundance

Relative abundance shows the most abundant features in a barplot summarized by a user-defined variable across the x-asis. 

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_intra_basic.png")
```

In addition, the user can choose to facet by phenotypes, adjust the number of features to show, switch between showing total numbers (Reads) and normalized value (if normalized), and modify the overall plot width. In general, main input elements must be defined before any visualization can be produced whereas the plot options only modify existing plots if deemed necessary.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_intra_faceted.png")
```
```{r, out.width="40%", out.height = "40%"}
knitr::include_graphics("www/img/analysis_intra_options.png")
```

Clicking on a specific feature in the plot, automatically opens a feature abundance plot for this feature.

#### Feature abundance

The feature abundance plot shows the individual abundance of a specific feature either as a boxplot or a categorical scatterplot depending on the x-axis variable chosen. The user can choose to employ a $\text{log}_2$ scale, define plot width and decide wether to show individual sample points or not. Feature plots can be opened by selecting a specific feature in the input section or by clicking on a feature in the relative abundance plot.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_intra_features.png")
```


#### Alpha Diversity

Alpha diversity is a measure of the complexity or diversity within a particular sample, eg. habitat or area. Alpha diversity is computed by 
functions in the vegan package and is visualized as a boxplot using the same input definitions by feature and relative abundance. The user can choose to color and thus split the boxes by a phenotype and set the overall plot width. Multiple diversity measures are offered with Shannon diversity provided as the default. We suggest users read up on the various measures and understand the differences in interpretation and nuances. 

Shannon diversity in particular measures how evenly the microbes are distributed in a sample and is defined by the following relationship where $p_i$, is the proportion of an individual feature:
$H = -\sum_{i=1}^T p_i \ln p_i$.  
The diversity measure can be changed via the plot options and the plots can be split by a specific phenotype coloring.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_intra_alpha.png")
```


### Inter Analysis

Inter-sample analyses focus on differences between samples or groups of samples via feature heatmaps and beta diversity calculations. Please note that these functions can be computationally intensive if there are many samples and a low aggregation level is chosen.

### Beta Diversity

Beta diversities are the measures of the complexity of communities between samples, as compared to within a sample (alpha diversity). Calculating beta diversity first requires the computation of a pairwise distance or similarity matrix. The user can select between different measures offered via the vegan package with Bray being the suggested default selection for microbiome analysis. We suggest users read up on the various measures and understand the differences in interpretation and nuances. 

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_inter_beta.png")
```

Principal component analysis, a dimension reduction method, is subsequently performed on the chosen distance matrix and visualized in a scatter plot. The user has the option to choose the principal components to display, add coloring and confidence ellipses based on a phenotype, define the shape based on a phenotype and adjust both the point size as well as the overall plot width.

```{r, out.width="50%", out.height = "50%"}
knitr::include_graphics("www/img/analysis_inter_ellipses.png")
```


PERMANOVA (permutational multivariate analysis of variance), from the vegan package is offered via the application (command line users will need to run this function independently and pass the results to the plotting function). Conceptually, a PERMANOVA analysis lets the user statistically determine if the centroids of a dissimilarity or distance matrix differ between groups of samples. Optionally, the user can select an phenotype as well as a strata variable with the results being shown, both within the visualization as well as in a table below it.

#### Heatmap

The heatmap offers another view on differences and similarities between the samples in a dataset. The user can either choose specific features or show the top 50 features sorted either by variance, Fano factor or median absolute deviation (MAD). The visualization is done with heatmaply which in turns relies on plotly to render the heatmap. The same options to interact with the plot are thus available. Once rendered, the user can change the number of features to include, turn of log scale, and add annotation to both rows (phenotypes) and columns (higher taxonomy levels) of the heatmap via the plot options. It is recommended to not use the heatmap functionality in datasets with many samples (5000+) as this can be quite slow to render.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_inter_heatmap.png")
```

### Correlation Analysis

Correlation allows the user to visualize the relationship between either two features or a feature and a numeric phenotype in a scatterplot enhanced with a linear regression statistic. Faceting and/or coloring by phenotypes is available in both correlation plots. The user is asked to choose between three different methods to aid in the evaluation of the association: Spearman (default), Pearson or Kendall.
The most common feature is selected by default as the base feature and the user has to set the correlation feature or phenotype and can opt to change the base feature. A regression line is added by default, but can be removed via the plot options.


```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_correlation.png")
```

```{r, out.width="50%", out.height = "50%"}
knitr::include_graphics("www/img/analysis_correlation_pheno.png")
```

### Differential Abundance Analysis

Differential abundance (DA) analysis is focused on testing the null hypothesis that the mean or mean ranks between groups are the same for a specific feature. DA analysis can help detect changes in feature abundance across two or more different levels of a phenotype. Four different methods can be chosen via the application: DESeq2, Kruskal-Wallis, limma, or a zero-inflated log normal model. DESeq2 and limma are widely used methods for comparisons in microarray and RNA-sequencing data which can easily be adapted for microbiome data. Kruskal-Wallis is a non-parametric test for any differences in distribution between groups. The zero-inflated log normal model is implemented in the metagenomeSeq package to account for zero-inflation in microbiome data. Typically, DESeq2 would is used with small (<=25) sample sizes. The results will be displayed in an interactive table (DT) within the application and the user can open feature plots showing the specific levels by clicking on a row of interest.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_differential.png")
```
```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_diff_details.png")
```

### Longitudinal Analysis

Longitudinal analysis allows the user to generate feature plots with more control over the data shown within the plot. For a specific feature, the user can choose a phenotype and specific levels of that phenotype to show in the plot. The chosen order of the levels will be kept within the visualization which allows sorting by specific dates or tissues among other things. All of these input elements are required to produce a visualization. In addtion, if desired and available, the user can choose a specific phenotype to summarize on which will then be connected by lines across the different levels. The resulting visualization is interactive and the user can then select and color specific IDs within the plot by clicking on the lines or selecting them via the input element above the plot. Several different IDs can be selected using different colors via the color select element next to the input element. In order to do this via mouse clicks on the plot, the user has to hold the shift key when adding additional line selections.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/analysis_longitudinal.png")
```

## Reports

Once an analysis is complete, a user might like to share the results with collaborators or download them for further analysis beyond the functionality provided via the Microbiome Explorer. To do this, we are providing the option to include any part of the analysis in a report which is fully reproducible outside of the application.

### Report Settings

In the report settings, the user can choose file name and add a project title, authors as well as any other text to be included in the output. Four different output formats are available by default. Please note that report generation does rely on the availability of external programs such as pandoc and latex (pdf). The available output formats can be restricted in global.R.

In addition to the basic settings, the user can also review any analysis made and choose which parts should be included in the report and which shouldn't. For each analysis element, the relevant R code is shown next to an icon illustrating the type of analysis. This is intended to help users without a computational background to identify which section is related to which part of their analysis. Those steps that are essential such as data loading or aggregation cannot be deselected.

```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/reports.png")
```

## Report Generation

In order to obtain the report, the user first clicks the "Generate" button. Then, the relevant R code collected during the analysis anytime the user clicks the "Report" button is written to a temporary file. The temporary R file is then knitted into an Rmd document using knitr::spin and subsequently rendered to the desired output format(s) with rmarkdown::render. R code chunks are enhanced with basic parameters to optimize the sizing of figures. This works reasonably well for HTML output. In order to produce well-designed output in other formats, more editing of the Rmarkdown document is recommended.

During the process any analysis is repeated for each output format while calling the render function. In future versions of the application, we are planning to do this as a background process with the results emailed to users. For now, the user needs to patiently wait until the render process has completed and then download the results by clicking "EXPORT". They will then obtain a zip folder which holds both the Rmd document as well as any output formats specified. The Rmd document can be edited and re-rendered outside of the application. The only modification required is to adjust the path to the input data which is unknown to the application.


```{r, out.width="70%", out.height = "70%"}
knitr::include_graphics("www/img/report_output.png")
```

## Offline use of the R package

All functionality available via the application is also accessible via the command line in a standard R environment. Please refer to package documentation and vignette for information on how to do this.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_input_module.R
\name{intraInput}
\alias{intraInput}
\title{Server side for the intra analysis input module}
\usage{
intraInput(
  input,
  output,
  session,
  meData,
  facetOptions = NULL,
  reset,
  aggDat = reactive(NULL)
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{meData}{MRExperiment object storing all data}

\item{facetOptions}{named vector of available facet choices}

\item{reset}{reactive boolean determining if all inputs should be reset}

\item{aggDat}{aggregated MRExperiment object (default is NULL)}
}
\value{
list holding all chosen values and the selected feature
}
\description{
Server side for the intra analysis input module
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_input_module.R
\name{dataInput}
\alias{dataInput}
\title{Main Data input server where the user selects files to upload to the app or 
connects to database}
\usage{
dataInput(
  input,
  output,
  session,
  dataSource,
  dataFilterRep,
  qcRep,
  addPheno,
  resetReports
)
}
\arguments{
\item{input}{module input}

\item{output}{module output}

\item{session}{app session}

\item{dataSource}{reactive Value storing commands for loading data}

\item{dataFilterRep}{reactive Value storing commands for filtering data}

\item{qcRep}{reactive Value storing commands for producing qc plots}

\item{addPheno}{reactive boolean keeping track of phenodata changes}

\item{resetReports}{reactive boolean indicating whether reports need to be reset}
}
\value{
list of reactives containing the uploaded and filtered data as well 
as the filterChoices on phenotypes
}
\description{
Main Data input server where the user selects files to upload to the app or 
connects to database
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{getFileType}
\alias{getFileType}
\title{Helper function assigning different file extensions to specific short texts 
identifying the types}
\usage{
getFileType(fileext)
}
\arguments{
\item{fileext}{the file extension found after '.'}
}
\value{
character string for the filetype
}
\description{
Helper function assigning different file extensions to specific short texts 
identifying the types
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long_input_module.R
\name{longInputUI}
\alias{longInputUI}
\title{Main diffanalysis input module.
Set up to handle diff analysis tabs in the app depending on given parameters}
\usage{
longInputUI(id)
}
\arguments{
\item{id}{element identifier - namespace}
}
\value{
box containing ui element
}
\description{
Main diffanalysis input module.
Set up to handle diff analysis tabs in the app depending on given parameters
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_functions.R
\name{plotAlpha}
\alias{plotAlpha}
\title{Plot alpha diversity}
\usage{
plotAlpha(
  aggdat,
  level,
  index = c("shannon", "simpson", "invsimpson", "richness"),
  x_var = "SAMPLE_ID",
  ylab = index,
  col_by = NULL,
  facet1 = NULL,
  facet2 = NULL,
  plotTitle = "",
  pwidth = 500,
  pheight = 150
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{level}{Feature level}

\item{index}{Diversity index, one of "shannon", "simpson", "invsimpson" or 
"richness" (=number of features). Default is "shannon".}

\item{x_var}{Phenotype to aggregate over on X-axis. Default by "SAMPLE_ID".}

\item{ylab}{Y-axis label. Default is "Reads".}

\item{col_by}{Phenotype for coloring.}

\item{facet1}{Phenotype for facet 1.}

\item{facet2}{Phenotype for facet 2.}

\item{plotTitle}{Plot title. By default, no title is used.}

\item{pwidth}{overall plot width; default is 650}

\item{pheight}{overall plot height; default is 150}
}
\value{
plotly plot object
}
\description{
This function plots the alpha diversity. See ?vegan::diversity for details
on the available index
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
plotAlpha(aggdat, level = "genus", index = "shannon", x_var = "diet")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_module.R
\name{betaDiversity}
\alias{betaDiversity}
\title{Beta Diversity module - server}
\usage{
betaDiversity(
  input,
  output,
  session,
  aggDat,
  aggLevel,
  colorOptions,
  shapeOptions,
  betadistance,
  betaSettings,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{MRExperiment storing data}

\item{aggLevel}{aggregation level}

\item{colorOptions}{phenotype selection options for color}

\item{shapeOptions}{phenotype selection options for shape}

\item{betadistance}{distance measured used for beta diversity analysis}

\item{betaSettings}{input choices for beta diversity}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
R code needed to generate the beta diversity plot
}
\description{
Beta Diversity module - server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_input_module.R
\name{dataInputUI}
\alias{dataInputUI}
\title{Main Data input UI where the user selects files to upload to the app or 
connects to database}
\usage{
dataInputUI(id)
}
\arguments{
\item{id}{module identifier}
}
\value{
fluidRow holding UI interface
}
\description{
Main Data input UI where the user selects files to upload to the app or 
connects to database
}
\examples{
dataInputUI("datainput_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_functions.R
\name{designPairs}
\alias{designPairs}
\title{Produce design matrix of pairwise comparisons}
\usage{
designPairs(levels)
}
\arguments{
\item{levels}{Character vector of the levels of a factor phenotype}
}
\value{
A model matrix
}
\description{
This function takes in the levels of a factor phenotype and outputs a
design matrix of all pairwise comparisons.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_analysis_module.R
\name{corrAnalysisUI}
\alias{corrAnalysisUI}
\title{corr Analysis Module - UI}
\usage{
corrAnalysisUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
fluidRow containing the ui code
}
\description{
corr Analysis Module - UI
}
\examples{
corrAnalysisUI("coranalysis_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_functions.R
\name{computeDistMat}
\alias{computeDistMat}
\title{Function to compute the distance matrix using vegdist from the vegan package}
\usage{
computeDistMat(aggdat, dist_method, log = TRUE, nfeatures = nrow(aggmat))
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{dist_method}{distance method from vegan package (See ?vegan::vegdist for details)}

\item{log}{transform count matrix to log2; default is TRUE}

\item{nfeatures}{number of features to use; default is all}
}
\value{
distance as dist
}
\description{
Function to compute the distance matrix using vegdist from the vegan package
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
computeDistMat(aggdat, dist_method = "bray")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenotype_module.R
\name{phenotypeTableUI}
\alias{phenotypeTableUI}
\title{Phenotype table UI module}
\usage{
phenotypeTableUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
fluidRow holding the ui code
}
\description{
Phenotype table UI module
}
\examples{
phenotypeTableUI("phenotype_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlation_functions.R
\name{computeCI_Interval}
\alias{computeCI_Interval}
\title{Helper function to calculate the confidence interval for a cor.test}
\usage{
computeCI_Interval(num, mS, method)
}
\arguments{
\item{num}{number of samples}

\item{mS}{results of cor.test}

\item{method}{statistical method used for cor.test}
}
\value{
named vector holding lower and upper thresholds
}
\description{
Helper function to calculate the confidence interval for a cor.test
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_module.R
\name{rollDownFeatures}
\alias{rollDownFeatures}
\title{Helper function which rolls down annotated from closest higher order with 
annotation}
\usage{
rollDownFeatures(featrow)
}
\arguments{
\item{featrow}{vector of entries to be replaced where needed (fData row)}
}
\value{
modified featurerow
}
\description{
Helper function which rolls down annotated from closest higher order with 
annotation
}
\examples{
data("mouseData", package = "metagenomeSeq")
featrow <- fData(mouseData)[5,]
rollDownFeatures(featrow)

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_module.R
\name{abundanceHeatmap}
\alias{abundanceHeatmap}
\title{Abundance Heatmap module - server}
\usage{
abundanceHeatmap(
  input,
  output,
  session,
  aggDat,
  featLevel,
  colorOptions,
  levelOpts,
  hmSort,
  hmFeatList,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{aggregated MRExperiment}

\item{featLevel}{chosen feature level (aggregation level)}

\item{colorOptions}{reactive storing filters selected via data input}

\item{levelOpts}{all available level choices for this dataset}

\item{hmSort}{reactive storing sorting method for heatmap}

\item{hmFeatList}{reactive storing list of features to include in heatmap}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
R code needed to generate the heatmap
}
\description{
Abundance Heatmap module - server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long_module.R
\name{longResultsUI}
\alias{longResultsUI}
\title{Longitudinal Analysis module UI}
\usage{
longResultsUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
row containing the UI elements
}
\description{
Longitudinal Analysis module UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_functions.R
\name{plotAbundance}
\alias{plotAbundance}
\title{Plot relative abundance}
\usage{
plotAbundance(
  aggdat,
  level,
  x_var = "SAMPLE_ID",
  ind = seq_len(10),
  plotTitle = "",
  ylab = "Reads",
  facet1 = NULL,
  facet2 = NULL,
  source = "A",
  pwidth = 650,
  pheight = 150
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment object}

\item{level}{Feature level.}

\item{x_var}{Phenotype to aggregate over on X-axis. Default by "SAMPLE_ID".}

\item{ind}{Indices of top abundant features to plot. Rest of features are
aggregated and displayed as "other".}

\item{plotTitle}{Plot title. Default shows no title.}

\item{ylab}{Y-axis label. Default is "Reads"}

\item{facet1}{Phenotype for facet 1.}

\item{facet2}{Phenotype for facet 2.}

\item{source}{name of the plot (needed for event handling); default is "A"}

\item{pwidth}{overall plot width; default is 650}

\item{pheight}{overall plot height; default is 150}
}
\value{
plotly plot
}
\description{
This function plots the relative abundance of the top abundant features.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
plotAbundance(aggdat, level = "genus", x_var = "diet")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_module.R
\name{getFeatSplitCode}
\alias{getFeatSplitCode}
\title{Helper function returning the fData modifications as strings for 
report generation}
\usage{
getFeatSplitCode(splittaxonomy)
}
\arguments{
\item{splittaxonomy}{name of column to split on}
}
\value{
String containing R code performing the modification
}
\description{
Helper function returning the fData modifications as strings for 
report generation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_module.R
\name{featAbundanceUI}
\alias{featAbundanceUI}
\title{Feature plot module - UI}
\usage{
featAbundanceUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box holding the UI code
}
\description{
Feature plot module - UI
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longitudinal_functions.R
\name{plotLongFeature}
\alias{plotLongFeature}
\title{Plot longitudinal features}
\usage{
plotLongFeature(
  aggdat,
  feature,
  x_var,
  id_var = "SAMPLE_ID",
  plotTitle = NULL,
  ylab = "Reads",
  log = FALSE,
  showLines = TRUE,
  fixedHeight = NULL,
  x_levels = NULL,
  pwidth = 650
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{feature}{Feature to plot.}

\item{x_var}{Phenotype to show along on X-axis.}

\item{id_var}{phenotype used to connect data points. Default is "SAMPLE_ID"}

\item{plotTitle}{Plot title. Default shows no title.}

\item{ylab}{Y-axis label. Default is "Reads"}

\item{log}{Log2 transform data. Default is FALSE.}

\item{showLines}{add lines between the points}

\item{fixedHeight}{sets a specific plot height (differential analysis)}

\item{x_levels}{restrict to specific levels of x_var (differential analysis)}

\item{pwidth}{overall plot width; default is 650}
}
\value{
plotly object holding long feature plot
}
\description{
This function plots the reads of a particular feature over different time points.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
plotLongFeature(aggdat, feature = "Prevotella", x_var = "diet", 
                id_var = "mouseID")

}
\author{
Janina Reeder, Mo Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_module.R
\name{reportListUI}
\alias{reportListUI}
\title{report tab ui}
\usage{
reportListUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
fluidRow holding ui elements
}
\description{
report tab ui
}
\examples{
reportListUI("reportlist_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_module.R
\name{relAbundance}
\alias{relAbundance}
\title{Relative abundance plot module - server}
\usage{
relAbundance(
  input,
  output,
  session,
  aggDat,
  featLevel,
  intraSettings,
  normalizedData,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{aggregated MRExperiment}

\item{featLevel}{chosen feature level (aggregation level)}

\item{intraSettings}{analysis input settings passed over to this module}

\item{normalizedData}{boolean indicating whether data has been normalized}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
list storing plot clicks and number of features displayed 
(passed to feature plot module) as well as the R code to make plot
}
\description{
Relative abundance plot module - server
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_module.R
\name{featureTable}
\alias{featureTable}
\title{Feature table module server code}
\usage{
featureTable(input, output, session, meData, featureModRep)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{meData}{MRExperiment storing the data}

\item{featureModRep}{reactiveValue storing modifications performed on fData}
}
\value{
feature table server fragment - no return value
}
\description{
Feature table module server code
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_module.R
\name{featureTableUI}
\alias{featureTableUI}
\title{Feature table UI module}
\usage{
featureTableUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
fluidRow containing the UI code for feature tables
}
\description{
Feature table UI module
}
\examples{
featureTableUI("feature_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{filterMEData}
\alias{filterMEData}
\title{Function to filter the MRexperiment data by numerical parameters}
\usage{
filterMEData(MRobj, minpresence = 1, minfeats = 2, minreads = 2)
}
\arguments{
\item{MRobj}{MRExperiment object to filter}

\item{minpresence}{minimum sample presence per feature}

\item{minfeats}{minimum number of features per sample}

\item{minreads}{minimum number of reads per sample}
}
\value{
the filtered MRobj
}
\description{
Function to filter the MRexperiment data by numerical parameters
}
\examples{
data("mouseData", package = "metagenomeSeq")
filterMEData(MRobj = mouseData, minpresence = 4, minfeats = 300)

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_module.R
\name{phenotypeCorrUI}
\alias{phenotypeCorrUI}
\title{Phenotype correlation analysis module}
\usage{
phenotypeCorrUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box containing the UI element
}
\description{
Phenotype correlation analysis module
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_functions.R
\name{plotHeatmap}
\alias{plotHeatmap}
\title{Plot heatmap}
\usage{
plotHeatmap(
  aggdat,
  features = NULL,
  log = TRUE,
  sort_by = c("Fano", "MAD", "Variance"),
  nfeat = 50,
  col_by = NULL,
  row_by = NULL,
  plotTitle = ""
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{features}{Vector of features to plot. If NULL, the top `nfeat`
features in terms of `sort_by` will be plotted.}

\item{log}{Log2 transform data. Default is TRUE.}

\item{sort_by}{Dispersion measure to sort features, one of "Fano", "MAD",
and "Variance"}

\item{nfeat}{Number of features to display. Default is 50.}

\item{col_by}{Vector of phenotypes for coloring.}

\item{row_by}{Name of feature level for coloring.}

\item{plotTitle}{Plot title. By default, no title.}
}
\value{
plotly heatmap
}
\description{
This function plots a heatmap of feature abundance.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
plotHeatmap(aggdat, sort_by = "Fano")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qc_functions.R
\name{makeQCPlot}
\alias{makeQCPlot}
\title{Plots sequencing statistics scatterplot}
\usage{
makeQCPlot(
  MRobj,
  col_by = NULL,
  log = "none",
  filter_feat = 0,
  filter_read = 0,
  allowWebGL = TRUE,
  pwidth = 550,
  pheight = 550
)
}
\arguments{
\item{MRobj}{metagenomeSeq object to be plotted}

\item{col_by}{factor by which to color the points}

\item{log}{character indicating which (if any) axes should be shown as log}

\item{filter_feat}{Numeric Y-coordinate to draw horizontal dashed line to
indicate feature filtering. If 0 (default), no line is drawn.}

\item{filter_read}{Numeric X-coordinate to draw vertical dashed line to
indicate read count filtering. If 0 (default), no line is drawn.}

\item{allowWebGL}{boolean indicating if webGL should be added}

\item{pwidth}{overall plot width; default is 550 (125 are added for legend)}

\item{pheight}{overall plot height; default is 550}
}
\value{
the plotly QC plot
}
\description{
This function makes a scatterplot of read and feature counts
for each sample. It was adjusted based on original work by Mo Huang
}
\examples{
data("mouseData", package = "metagenomeSeq")
makeQCPlot(mouseData)

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_module.R
\name{featAbundance}
\alias{featAbundance}
\title{Feature plot module - server}
\usage{
featAbundance(
  input,
  output,
  session,
  aggDat,
  featLevel,
  intraSettings,
  selectedFeat,
  featName,
  numOfFeats,
  ylabMode,
  normalizedData,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{aggregated MRExperiment}

\item{featLevel}{chosen feature level (aggregation level)}

\item{intraSettings}{analysis settings passed over from analysis input module}

\item{selectedFeat}{feature selected via drop down element of analysis input}

\item{featName}{plotly click event passed via relative abundance}

\item{numOfFeats}{number of features shown in relative abundance plot 
(affects plotly click data)}

\item{ylabMode}{character indication if raw \"Reads\" or \"Percentage\" 
should be shown}

\item{normalizedData}{boolean indicating whether data has been normalized}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
R code needed to build the feature plot
}
\description{
Feature plot module - server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{parseInteractionName}
\alias{parseInteractionName}
\title{Helper function used to build a correct interactionName based on the chosen 
columns}
\usage{
parseInteractionName(interactionName)
}
\arguments{
\item{interactionName}{as chosen by user. This may not be good to store 
internally}
}
\value{
updated interactionName or warning/error string
}
\description{
Helper function used to build a correct interactionName based on the chosen 
columns
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_functions.R
\name{runDiffTest}
\alias{runDiffTest}
\title{Performs differential abundance testing}
\usage{
runDiffTest(
  aggdat,
  level,
  phenotype,
  phenolevels = NULL,
  log = TRUE,
  coef = NULL,
  method = c("limma", "Kruskal-Wallis", "DESeq2")
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{level}{Feature level.}

\item{phenotype}{Phenotype to test.}

\item{phenolevels}{levels of the phenotype to restrict the comparison to}

\item{log}{Log2 transform data. Default is TRUE.}

\item{coef}{Numeric which indicates which pairwise comparison to analyze
when there are more than two groups. Corresponds to the column number of the
model matrix produced by \code{\link{designPairs}()}. If NULL, a test of any
difference between all groups is performed.}

\item{method}{Differential testing method. One of "limma" (default),
"Kruskal-Wallis", or "DESeq2".}
}
\value{
data.frame holding results of the differential analysis
}
\description{
This function performs differential abundance testing between groups of a
specified phenotype. Four methods are available: limma, Kruskal-Wallis, 
ZILN and DESeq2 (see details).
}
\details{
limma is a differential expression tool for microarray data using
linear models. It can also be applied to microbiome data.

The Kruskal-Wallis test is a non-parametric rank test which examines if 
groups come from the same distribution. A significant result indicates at 
least one group is distributionally different than another group.

ZILN is a zero-inflated log-normal model implemented in
\code{\link[metagenomeSeq]{fitFeatureModel}()} of the \code{metagenomeSeq}
package.

DeSeq2 performs differential gene expression analysis based on the negative 
binomial distribution
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
runDiffTest(aggdat = aggdat,level = "genus", 
            phenotype = "diet", method = "Kruskal-Wallis")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diff_module.R
\name{diffTableUI}
\alias{diffTableUI}
\title{Differential Analysis module UI}
\usage{
diffTableUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
row containing the UI elements
}
\description{
Differential Analysis module UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_functions.R
\name{plotSingleFeature}
\alias{plotSingleFeature}
\title{Plot features}
\usage{
plotSingleFeature(
  aggdat,
  feature = "other",
  x_var = "SAMPLE_ID",
  ind = seq_len(10),
  plotTitle = NULL,
  ylab = "Reads",
  xlab = NULL,
  facet1 = NULL,
  facet2 = NULL,
  log = FALSE,
  showPoints = FALSE,
  fixedHeight = NULL,
  x_levels = NULL,
  pwidth = 500
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{feature}{Feature to plot.}

\item{x_var}{Phenotype to aggregate over on X-axis. Default by "SAMPLE_ID".}

\item{ind}{Indices of top abundant features to plot. Needed to determine 
appropriate color}

\item{plotTitle}{Plot title. Default shows no title.}

\item{ylab}{Y-axis label. Default is "Reads"}

\item{xlab}{X-axis label. If NULL, x_var will be used as label.}

\item{facet1}{Phenotype for facet 1.}

\item{facet2}{Phenotype for facet 2.}

\item{log}{Log2 transform data. Default is FALSE.}

\item{showPoints}{add points for each sample on plot}

\item{fixedHeight}{sets a specific plot height (differential analysis)}

\item{x_levels}{restrict to specific levels of x_var (differential analysis)}

\item{pwidth}{overall plot width; default is 650}
}
\value{
plotly plot object
}
\description{
This function plots the reads of a particular feature or set of features.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
plotSingleFeature(aggdat, feature = "Prevotella", x_var = "diet")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_input_module.R
\name{betaInputUI}
\alias{betaInputUI}
\title{Main beta analysis input module.
Set up to handle all analysis tabs in the app depending on given parameters}
\usage{
betaInputUI(id)
}
\arguments{
\item{id}{element identifier - namespace}
}
\value{
box containing ui element
}
\description{
Main beta analysis input module.
Set up to handle all analysis tabs in the app depending on given parameters
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{runMicrobiomeExplorer}
\alias{runMicrobiomeExplorer}
\title{Main function to start the Microbiome Explorer Shiny app via a command line
call}
\usage{
runMicrobiomeExplorer()
}
\value{
the shiny application
}
\description{
Main function to start the Microbiome Explorer Shiny app via a command line
call
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{addPhenoData}
\alias{addPhenoData}
\title{Add phenotype data to object.}
\usage{
addPhenoData(MRobj, phenodata = NULL)
}
\arguments{
\item{MRobj}{An MRexperiment object.}

\item{phenodata}{Phenotype data frame or file path.}
}
\value{
An updated MRexperiment object.
}
\description{
This function adds phenotype data to the phenoData slot in an MRexperiment
object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggreg_norm_functions.R
\name{aggFeatures}
\alias{aggFeatures}
\title{Aggregates counts by level}
\usage{
aggFeatures(MRobj, level = NULL, sort = TRUE)
}
\arguments{
\item{MRobj}{An MRexperiment object.}

\item{level}{Level to aggregate over. If NULL, no aggregation occurs.}

\item{sort}{boolean determining if resulting aggregated MRexperiment should 
be sorted based on rowSums; default is TRUE}
}
\value{
Aggregated MRexperiment object or matrix depending on \code{out}.
}
\description{
This function aggregates counts by a level specified in the featureData slot
of the MRexperiment object.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggFeatures(mouseData, level = "genus")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_input_module.R
\name{corrInputUI}
\alias{corrInputUI}
\title{Main correlation analysis input module.
Handles correlation analysis tab in the app}
\usage{
corrInputUI(id, type)
}
\arguments{
\item{id}{element identifier - namespace}

\item{type}{determines if 'feature' or 'pheno' correlation}
}
\value{
box containing ui element
}
\description{
Main correlation analysis input module.
Handles correlation analysis tab in the app
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_module.R
\name{alphaDiversityUI}
\alias{alphaDiversityUI}
\title{Alpha Diversity module - UI}
\usage{
alphaDiversityUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box holding the UI code
}
\description{
Alpha Diversity module - UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_analysis_module.R
\name{intraAnalysis}
\alias{intraAnalysis}
\title{Intra Analysis Module - server}
\usage{
intraAnalysis(
  input,
  output,
  session,
  data,
  levelOpts,
  chosenLevel,
  resetInput,
  aggData,
  normalizedData
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{data}{the main data object returned from data_input_module}

\item{levelOpts}{available levels to aggregate on (depends on input data)}

\item{chosenLevel}{previously selected level (passed from different instance)}

\item{resetInput}{reactive boolean determining if reset is required}

\item{aggData}{the aggregated MRExperiment object}

\item{normalizedData}{boolean indicating if normalization was done}
}
\value{
reactive holding code to be used in reports
}
\description{
Intra Analysis Module - server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenotype_module.R
\name{phenotypeTable}
\alias{phenotypeTable}
\title{Phenotype table server module}
\usage{
phenotypeTable(input, output, session, meData, phenoModRep, addPheno)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{meData}{MRExperiment storing the data}

\item{phenoModRep}{reactive Value storing any phenotable modifications made}

\item{addPheno}{reactive boolean keeping track of pheno data modifications}
}
\value{
phenotype table server fragment - no return value
}
\description{
Phenotype table server module
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggreg_norm_functions.R
\name{normalizeData}
\alias{normalizeData}
\title{Calls appropriate normalization functions depending on input parameter
The two available methods included in the package are based on either 
calculating proportions or by using cumulative sum scaling (CSS), 
Paulson, et al. Nat Meth 2013.}
\usage{
normalizeData(MRobj, norm_method)
}
\arguments{
\item{MRobj}{the MRexperiment}

\item{norm_method}{method to use for normalization; CSS or Proportional}
}
\value{
the normalized MRobj
}
\description{
Calls appropriate normalization functions depending on input parameter
The two available methods included in the package are based on either 
calculating proportions or by using cumulative sum scaling (CSS), 
Paulson, et al. Nat Meth 2013.
}
\examples{
data("mouseData", package = "metagenomeSeq")
normalizeData(mouseData, norm_method = "CSS")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{add_plotly_config}
\alias{add_plotly_config}
\title{Adds a config call based on plotly::config}
\usage{
add_plotly_config(.data)
}
\arguments{
\item{.data}{plotly data object to apply the config call to}
}
\value{
plotly::config call
}
\description{
Adds a config call based on plotly::config
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{getLegendLevel}
\alias{getLegendLevel}
\title{Function to find a non-empty facet in the last row. This will be the
one to be connected to the plot legend to avoid duplicates within}
\usage{
getLegendLevel(df2, facets, facet2s)
}
\arguments{
\item{df2}{plotting data frame}

\item{facets}{column facets}

\item{facet2s}{row facets}
}
\value{
the name of the column-based facet which can be used as legend
}
\description{
Function to find a non-empty facet in the last row. This will be the
one to be connected to the plot legend to avoid duplicates within
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_input_module.R
\name{fileUploadUI}
\alias{fileUploadUI}
\title{Module handling file upload for the application: UI
In a deployed version this module should be replaced with database access}
\usage{
fileUploadUI(id)
}
\arguments{
\item{id}{module identifier}
}
\value{
div holding ui elements
}
\description{
Module handling file upload for the application: UI
In a deployed version this module should be replaced with database access
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intra_module.R
\name{alphaDiversity}
\alias{alphaDiversity}
\title{Alpha Diversity module - server}
\usage{
alphaDiversity(
  input,
  output,
  session,
  aggDat,
  featLevel,
  intraSettings,
  colorOptions,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{aggregated MRExperiment}

\item{featLevel}{chosen feature level (aggregation level)}

\item{intraSettings}{analysis settings as passed over from analysis 
input module}

\item{colorOptions}{phenotype selections: used for color choices}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
R code used to make the alpha diversity plot
}
\description{
Alpha Diversity module - server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_functions.R
\name{plotBeta}
\alias{plotBeta}
\title{Plot beta diversity}
\usage{
plotBeta(
  aggdat,
  dim = c(1, 2),
  log = TRUE,
  dist_method = "bray",
  pcas = NULL,
  nfeatures = nrow(aggdat),
  col_by = NULL,
  shape_by = NULL,
  plotTitle = "",
  xlab = NULL,
  ylab = NULL,
  pt_size = 8,
  plotText = NULL,
  confInterval = NULL,
  allowWebGL = TRUE,
  pwidth = 550,
  pheight = 550
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{dim}{Vector of length 2 specifying which dimensions to plot.}

\item{log}{Log2 transform data. Default is TRUE.}

\item{dist_method}{Which distance method to use. See ?vegan::vegdist for more
\code{\link[vegan]{vegdist}()} for options. Default is "bray".}

\item{pcas}{precalculated pcas to avoid recalculation via CalcPCs}

\item{nfeatures}{Number of top features in terms of standard deviation.
Default is all.}

\item{col_by}{Phenotype for coloring.}

\item{shape_by}{Phenotype for shape.}

\item{plotTitle}{Plot title. By default, becomes PCoA (code{dist.method}).}

\item{xlab}{X-axis label. By default, shows dimension and percent variance
explained.}

\item{ylab}{Y-axis label. By default, shows dimension and percent variance
explained.}

\item{pt_size}{the size of the markers}

\item{plotText}{adonis text to be added to plot}

\item{confInterval}{numeric value indicating confidence level for ellipses}

\item{allowWebGL}{boolean indicating if WebGL should be used}

\item{pwidth}{overall plot width; default is 550 (125 are added for legend)}

\item{pheight}{overall plot height; default is 550}
}
\value{
plotly plot object
}
\description{
This functions plots the beta diversity as a PCoA plot.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
plotBeta(aggdat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_main_module.R
\name{avgAbundanceUI}
\alias{avgAbundanceUI}
\title{Relative abundance plot module - UI}
\usage{
avgAbundanceUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box containing the ui code
}
\description{
Relative abundance plot module - UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_module.R
\name{featureCorrUI}
\alias{featureCorrUI}
\title{Feature correlation analysis module UI}
\usage{
featureCorrUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box containing the UI elements
}
\description{
Feature correlation analysis module UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_functions.R
\name{calculatePCAs}
\alias{calculatePCAs}
\title{Function to compute the PCAs for a given distance matrix}
\usage{
calculatePCAs(distmat, pcas)
}
\arguments{
\item{distmat}{the distance matrix}

\item{pcas}{2-element vector of PCAs to include in results}
}
\value{
the x slot limited to pcas after calling stats::prcomp on distmat
}
\description{
Function to compute the PCAs for a given distance matrix
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
distmat <- computeDistMat(aggdat, dist_method = "bray")
calculatePCAs(distmat, c(1,2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_module.R
\name{betaDiversityUI}
\alias{betaDiversityUI}
\title{Beta Diversity module - UI}
\usage{
betaDiversityUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box holding the ui code
}
\description{
Beta Diversity module - UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{getWidths}
\alias{getWidths}
\title{Helper function to account for issues plotly has with very small widths
(these end up being 1 and cover the entire plotting area)}
\usage{
getWidths(df2, facets, x_var, drop = TRUE)
}
\arguments{
\item{df2}{dataframe storing plotting data}

\item{facets}{column facets}

\item{x_var}{x variable}

\item{drop}{passed on as .drop to dplyr::group_by}
}
\value{
widths for each facet
}
\description{
Helper function to account for issues plotly has with very small widths
(these end up being 1 and cover the entire plotting area)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{add_plotly_layout}
\alias{add_plotly_layout}
\title{Adds a layout call based on plotly::layout}
\usage{
add_plotly_layout(.data, plotTitle, xaxis_text, ylab)
}
\arguments{
\item{.data}{plotly data object to apply the layout call to}

\item{plotTitle}{plot title to use}

\item{xaxis_text}{x axis label to use}

\item{ylab}{y axis label to use}
}
\value{
plotly::layout call
}
\description{
Adds a layout call based on plotly::layout
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlation_functions.R
\name{corrPhenotype}
\alias{corrPhenotype}
\title{Scatterplot of a feature and a phenotype}
\usage{
corrPhenotype(
  aggdat,
  feature,
  phenotype,
  log = TRUE,
  method = c("spearman", "pearson", "kendall"),
  addRegression = TRUE,
  col_by = NULL,
  facet1 = NULL,
  facet2 = NULL,
  plotTitle = "",
  xlab = NULL,
  ylab = NULL,
  allowWebGL = TRUE,
  pwidth = 550,
  pheight = 200
)
}
\arguments{
\item{aggdat}{aggregated MRExperiment}

\item{feature}{Feature input.}

\item{phenotype}{Phenotype input (must be numeric)}

\item{log}{Log2 transform data. Default is TRUE.}

\item{method}{Correlation coefficient. One of "spearman" (default),
"pearson", or "kendall".}

\item{addRegression}{boolean parameter indicating whether linear regression
line should be drawn; default: TRUE}

\item{col_by}{Phenotype for coloring.}

\item{facet1}{Phenotype for facet 1.}

\item{facet2}{Phenotype for facet 2. (WIP/TODO)}

\item{plotTitle}{Plot title. Default is no title.}

\item{xlab}{X-axis label. Default is \code{feat1}.}

\item{ylab}{Y-axis label. Default is \code{feat2}.}

\item{allowWebGL}{boolean indicating if WebGL should be used for large data}

\item{pwidth}{overall plot width; default is 550}

\item{pheight}{overall plot height; default is 200}
}
\value{
list holding plotly plot and lm fit
}
\description{
This function plots a scatterplot of a feature and a phenotype along with 
sample correlation statistics.
}
\examples{
data("mouseData", package = "metagenomeSeq")
aggdat <- aggFeatures(mouseData, level = "genus")
corrPhenotype(aggdat, feature = "Bacteroides", phenotype = "relativeTime")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{filterByPheno}
\alias{filterByPheno}
\title{Function to filter the MRexperiment by certain phenotype values}
\usage{
filterByPheno(MRobj, rm_phenovalues)
}
\arguments{
\item{MRobj}{the MRexperiment to subset}

\item{rm_phenovalues}{list of named vectors with names corresponding to 
column names in pData and values representing phenotypes within the column}
}
\value{
the filtered MRobj
}
\description{
Function to filter the MRexperiment by certain phenotype values
}
\examples{
data("mouseData", package = "metagenomeSeq")
filterByPheno(MRobj = mouseData, 
  rm_phenovalues = list("diet" = c("BK"),"mouseID" = c("PM1","PM10")))

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_input_module.R
\name{featureInputUI}
\alias{featureInputUI}
\title{Main feature analysis input module.
Set up to handle all analysis tabs in the app depending on given parameters}
\usage{
featureInputUI(id)
}
\arguments{
\item{id}{element identifier - namespace}
}
\value{
box containing ui element
}
\description{
Main feature analysis input module.
Set up to handle all analysis tabs in the app depending on given parameters
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reportRow_module.R
\name{reportRowUI}
\alias{reportRowUI}
\title{Report row module consisting of a checkbox, image and description/R code area}
\usage{
reportRowUI(id, type)
}
\arguments{
\item{id}{namespace identifier}

\item{type}{boolean indicating if a selector checkbox should be added}
}
\value{
div holding the UI code
}
\description{
Report row module consisting of a checkbox, image and description/R code area
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregation_module.R
\name{aggregationTabUI}
\alias{aggregationTabUI}
\title{Aggregation module ui function}
\usage{
aggregationTabUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box holding aggregation input elements
}
\description{
Aggregation module ui function
}
\examples{
aggregationTabUI("atu_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qc_functions.R
\name{plotlyHistogram}
\alias{plotlyHistogram}
\title{Function plotting a plotly histogram on the given histvalue}
\usage{
plotlyHistogram(
  histvalue,
  plotTitle,
  xaxisTitle = "",
  yaxisTitle = "",
  pwidth = 200,
  pheight = 200
)
}
\arguments{
\item{histvalue}{the value to plot as a histogram}

\item{plotTitle}{title of the plot}

\item{xaxisTitle}{name of xaxis; default is ""}

\item{yaxisTitle}{name of yaxis; default is ""}

\item{pwidth}{overall plot width; default is 200}

\item{pheight}{overall plot height; default is 200}
}
\value{
plotly plot object
}
\description{
Function plotting a plotly histogram on the given histvalue
}
\examples{
data("mouseData", package = "metagenomeSeq")
plotlyHistogram(histvalue = colSums(MRcounts(mouseData) > 0),
  plotTitle = "Feature distribution",
  xaxisTitle = "features", yaxisTitle = "frequency")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_module.R
\name{phenotypeCorr}
\alias{phenotypeCorr}
\title{Phenotype correlation analysis server module}
\usage{
phenotypeCorr(
  input,
  output,
  session,
  aggDat,
  colorOptions,
  corFeatBase,
  corPheno,
  corFacet1,
  corFacet2,
  corMethod,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{aggregated MRExperiment}

\item{colorOptions}{reactive storing filters available via data input}

\item{corFeatBase}{first correlation feature}

\item{corPheno}{correlation phenotype}

\item{corFacet1}{first correlation facet}

\item{corFacet2}{second correlation facet}

\item{corMethod}{correlation method to use}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
R code used to do the correlation analysis (character)
}
\description{
Phenotype correlation analysis server module
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{addFeatData}
\alias{addFeatData}
\title{Add feature data to MRobj.}
\usage{
addFeatData(MRobj, featdata = NULL)
}
\arguments{
\item{MRobj}{An MRexperiment object.}

\item{featdata}{Feature data frame or file path.}
}
\value{
An updated MRexperiment object.
}
\description{
This function adds feature data to the featureData slot in an MRexperiment
object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{buildPlottingDF}
\alias{buildPlottingDF}
\title{Sets up a dataframe used by several plotting functions by joining
the required data with relevant phenotype data}
\usage{
buildPlottingDF(
  df,
  phenoTable,
  x_var = NULL,
  facet1 = NULL,
  facet2 = NULL,
  col_by = NULL,
  col_name = col_by,
  id_var = NULL
)
}
\arguments{
\item{df}{dataframe storing plotting data values}

\item{phenoTable}{pData of the MRexperiment; all following parameters must be 
a column of the phenoTable}

\item{x_var}{main plotting variable}

\item{facet1}{column-based faceting (can be NULL)}

\item{facet2}{row-based faceting (can be NULL)}

\item{col_by}{coloring factor (can be NULL)}

\item{col_name}{character to be used as name for col_by}

\item{id_var}{variable used to connect samples longitudinally (can be NULL)}
}
\value{
dataframe obtained by joining df and relevant columns of phenoTable
}
\description{
Sets up a dataframe used by several plotting functions by joining
the required data with relevant phenotype data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inter_module.R
\name{abundanceHeatmapUI}
\alias{abundanceHeatmapUI}
\title{Abundance Heatmap module - UI}
\usage{
abundanceHeatmapUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
box holding the UI code
}
\description{
Abundance Heatmap module - UI
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{generateReport}
\alias{generateReport}
\title{Generates report}
\usage{
generateReport(
  rcode,
  filename = "report",
  dir = "out",
  title = "MicrobiomeExplorer Report",
  author = "",
  date = "`r format(Sys.time(), '\%d \%B, \%Y')`",
  data.source = "",
  output = c("html_document"),
  toc = TRUE,
  intro_text = NULL
)
}
\arguments{
\item{rcode}{A named list where each element corresponds to a different
analysis (Alpha diversity, Beta diversity). The name of the list is used to
denote the first part of the code chunks in each analysis section
(alpha, beta). Each element is itself a list of R commands corresponding to
a code chunk.}

\item{filename}{Name of output files. Default is "report".}

\item{dir}{Directory of output. Default is "out".}

\item{title}{Title of the report.}

\item{author}{Author of the report.}

\item{date}{Date of the report.}

\item{data.source}{R code used to obtain the dataset}

\item{output}{Output of Rmarkdown file. Options defined in global.R}

\item{toc}{Table of contents. Default is TRUE.}

\item{intro_text}{Introductory text to include with the report (optional)}
}
\value{
A character vector where each element is a line in the R script.
}
\description{
This function generates the pieces of the report, which includes the R
script, Rmarkdown file, and any Rmarkdown outputs.
}
\details{
Adapted from https://yihui.name/knitr/demo/stitch/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregation_module.R
\name{aggregationTab}
\alias{aggregationTab}
\title{Aggregation module server function}
\usage{
aggregationTab(
  input,
  output,
  session,
  resetInput,
  levelOpts,
  chosenLevel,
  meData
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{resetInput}{boolean updated to TRUE if new data is available}

\item{levelOpts}{available levels to aggregate on (depends on input data)}

\item{chosenLevel}{previously selected level (passed from different instance)}

\item{meData}{the main MRexperiment object}
}
\value{
reactive list holding aggregated object, aggregation code and 
boolean on normalization
}
\description{
Aggregation module server function
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{extendPhenoData}
\alias{extendPhenoData}
\title{Extends existing phenodata for an object}
\usage{
extendPhenoData(MRobj, phenodata = NULL)
}
\arguments{
\item{MRobj}{An MRexperiment object.}

\item{phenodata}{Phenotype data frame or file path.}
}
\value{
An updated MRexperiment object.
}
\description{
This function adds phenotype data to the phenoData slot in an MRexperiment
object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_input_module.R
\name{fileUpload}
\alias{fileUpload}
\title{Module handling file upload for the application: server}
\usage{
fileUpload(
  input,
  output,
  session,
  meData,
  meName,
  initializeData,
  addPheno,
  dataSource,
  resetFile = reactive(NULL)
)
}
\arguments{
\item{input}{module input}

\item{output}{module output}

\item{session}{app session}

\item{meData}{main reactive storing the MRexperiment data}

\item{meName}{main reactive storing the filename uploaded}

\item{initializeData}{reactiveVal keeping track of new uploads to reset data}

\item{addPheno}{reactiveVal keeping track of phenodata changes}

\item{dataSource}{reactive Value storing commands for loading data}

\item{resetFile}{indicating if module should be reset}
}
\value{
boolean denoting successful upload of a file
}
\description{
Module handling file upload for the application: server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{createHeader}
\alias{createHeader}
\title{Makes header for R script}
\usage{
createHeader(
  title = "MicrobiomeExplorer Report",
  author = "",
  date = "",
  data.source = "",
  output = getOption("me.reportformat"),
  toc = TRUE
)
}
\arguments{
\item{title}{Title of the report.}

\item{author}{Author of the report.}

\item{date}{Date of the report.}

\item{data.source}{R code used to obtain the dataset}

\item{output}{Output of Rmarkdown file.}

\item{toc}{Table of contents. Default is TRUE.}
}
\value{
A character vector where each element is a line in the R script.
}
\description{
This function makes the header for the report R script to be rendered by
knitr into Rmarkdown and rendered into HTML, PDF, or Word.
}
\details{
This was adapted from https://yihui.name/knitr/demo/stitch/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenotype_module.R
\name{getPhenoChanges}
\alias{getPhenoChanges}
\title{Helper function returning the code used to modify the data types of the
pheno table}
\usage{
getPhenoChanges(phenotype, datatype)
}
\arguments{
\item{phenotype}{name of the phenotype column header}

\item{datatype}{variable type to assign to column}
}
\value{
String storing code to perform modification
}
\description{
Helper function returning the code used to modify the data types of the
pheno table
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_analysis_module.R
\name{featureAnalysisUI}
\alias{featureAnalysisUI}
\title{feature Analysis Module - UI}
\usage{
featureAnalysisUI(id)
}
\arguments{
\item{id}{namespace identifier}
}
\value{
fluidRow containing the ui code
}
\description{
feature Analysis Module - UI
}
\examples{
featureAnalysisUI("featureanalysis_id")

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reportRow_module.R
\name{reportRow}
\alias{reportRow}
\title{Report Row}
\usage{
reportRow(input, output, session, type, content)
}
\arguments{
\item{input}{module input}

\item{output}{module output}

\item{session}{app session}

\item{type}{boolean indicating whether checkbox should be included}

\item{content}{R code to show}
}
\value{
reactive boolean indicating whether row is selected
}
\description{
Report Row
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_module.R
\name{replaceWithUnknown}
\alias{replaceWithUnknown}
\title{Helper function to replace any un-annotated features with the term unknown}
\usage{
replaceWithUnknown(featcol)
}
\arguments{
\item{featcol}{vector of entried to be replaced where needed (fData column)}
}
\value{
modified featcol
}
\description{
Helper function to replace any un-annotated features with the term unknown
}
\examples{
data("mouseData", package = "metagenomeSeq")
featcol <- fData(mouseData)[["genus"]]
featcol[featcol == "NA"] <- NA
replaceWithUnknown(featcol)

}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_input_module.R
\name{corrInput}
\alias{corrInput}
\title{Server side for the analysis input module handling analysis control}
\usage{
corrInput(
  input,
  output,
  session,
  type,
  meData,
  facetOptions = NULL,
  reset,
  aggDat = reactive(NULL)
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{type}{of the correlation (feature vs phenotype)}

\item{meData}{MRExperiment object storing all data}

\item{facetOptions}{named vector of available facet choices}

\item{reset}{reactive boolean determining if all inputs should be reset}

\item{aggDat}{aggregated MRExperiment object (default is NULL)}
}
\value{
list holding all chosen values and the selected feature
}
\description{
Server side for the analysis input module handling analysis control
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenotype_module.R
\name{getPhenoModCode}
\alias{getPhenoModCode}
\title{Helper function returning the code used to modify the phenotable as a string}
\usage{
getPhenoModCode(name, pheno1, pheno2)
}
\arguments{
\item{name}{interaction name}

\item{pheno1}{first interaction phenotype}

\item{pheno2}{second interaction phenotype}
}
\value{
String storing code to perform modification
}
\description{
Helper function returning the code used to modify the phenotable as a string
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{readData}
\alias{readData}
\title{Reads in data}
\usage{
readData(filepath, type = "RDS")
}
\arguments{
\item{filepath}{Relative or absolute file path of data object.}

\item{type}{The type of file to be read; default is "RDS", other options are
"RDATA", "BIOM", "TAB", "CSV"}
}
\value{
An MRexperiment object.
}
\description{
This function reads in an MRexperiment object saved as an RDS file, a Biom
file, or a tab - delimited count matrix with features as rows
and samples as columns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_input_module.R
\name{getFilterChoices}
\alias{getFilterChoices}
\title{Helper function to filter phenodata for interesting phenotypes to be used 
for filtering or subsetting}
\usage{
getFilterChoices(MRobj)
}
\arguments{
\item{MRobj}{the MRexperiment storing the data}
}
\value{
list of named vectors with names being pData column headers and 
values being unique entries; columns with only one entry or those with 
different values for each samples are omitted
}
\description{
Helper function to filter phenodata for interesting phenotypes to be used 
for filtering or subsetting
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{buildEmptyPlotlyPlot}
\alias{buildEmptyPlotlyPlot}
\title{Creates an empty plotly plot using the given labels on the x and y axis}
\usage{
buildEmptyPlotlyPlot(xaxis_text, ylab)
}
\arguments{
\item{xaxis_text}{x axis label}

\item{ylab}{y axis label}
}
\value{
call to plotly_empty
}
\description{
Creates an empty plotly plot using the given labels on the x and y axis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long_module.R
\name{longResults}
\alias{longResults}
\title{Longitudinal analysis module server code}
\usage{
longResults(
  input,
  output,
  session,
  aggDat,
  featLevel,
  longSettings,
  normalizedData,
  reset
)
}
\arguments{
\item{input}{shiny input}

\item{output}{shiny output}

\item{session}{shiny session}

\item{aggDat}{aggregated MRExperiment}

\item{featLevel}{chosen feature level (aggregation level)}

\item{longSettings}{reactive storing values selected in analysis 
input interface}

\item{normalizedData}{reactive boolean indicating if data has been normalized}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
list containing R code for analysis and for feature plots
}
\description{
Longitudinal analysis module server code
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_module.R
\name{reportList}
\alias{reportList}
\title{Report tab module server}
\usage{
reportList(
  input,
  output,
  session,
  dataSource,
  preprocessRep,
  qcRep,
  analysisRep,
  aggIndex,
  reset
)
}
\arguments{
\item{input}{module input}

\item{output}{module output}

\item{session}{app session}

\item{dataSource}{R code to obtain data for rendering}

\item{preprocessRep}{R code containing preprocessing steps of data}

\item{qcRep}{R Code to generate QC plots}

\item{analysisRep}{R Code to generate all analyses saved to reports}

\item{aggIndex}{boolean value representing aggregation steps in analysisRep}

\item{reset}{boolean reactive which resets the module if TRUE}
}
\value{
report list server fragment - no return value
}
\description{
Report tab module server
}
\author{
Janina Reeder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qc_functions.R
\name{plotlySampleBarplot}
\alias{plotlySampleBarplot}
\title{Function plotting a barplot showing number of OTUs per samples}
\usage{
plotlySampleBarplot(
  MRobj,
  col_by = NULL,
  xaxisTitle = "",
  yaxisTitle = "",
  pwidth = 600,
  pheight = 450,
  sortbyfreq = FALSE,
  pheno_sort = NULL,
  x_levels = NULL
)
}
\arguments{
\item{MRobj}{containing data to plot}

\item{col_by}{phenotype to color bars by; default is NULL}

\item{xaxisTitle}{name of xaxis; default is ""}

\item{yaxisTitle}{name of yaxis; default is ""}

\item{pwidth}{overall plot width; default is 600}

\item{pheight}{overall plot height; default is 450}

\item{sortbyfreq}{boolean determining if bars should be sorted by frequency; 
default is FALSE}

\item{pheno_sort}{order of pheno levels to sort by; 
ignored if sortbyfreq is TRUE}

\item{x_levels}{character vector holding x values in order to be shown}
}
\value{
plotly plot object
}
\description{
Function plotting a barplot showing number of OTUs per samples
}
\examples{
data("mouseData", package = "metagenomeSeq")
plotlySampleBarplot(mouseData)

}
